/*
 * Copyright (C) Global Graphics Software Ltd 2007-2019. All rights reserved
 */
 
/**
 * @file    edlerrors.cpp
 * @ingroup ErrorMessages
 * @brief   Publishes a look-up table that maps integer error codes,
 *          as returned by many of the EDL APIs, into a corresponding (English) string.
 *          It is expected that alternate language translations are implemented
 *          by creating alternate versions of this file.
 */

#include <edl/edlerrors.h>

namespace EDL {

    // must sorted by numeric error code for getErrorString() to work correctly
    static struct EDL_ERR_LIST errorList[] = {
        /*   1*/ { EDL_ERR_PANIC, L"Panic - EDL in unstable state!" },
        /*   2*/ { EDL_ERR_UNDEFINED, L"Undefined" },
        /*   3*/ { EDL_ERR_LICENSE, L"Invalid Serial Number or License provided" },
        /*   4*/ { EDL_ERR_FILENOTFOUND, L"File %1 not found"},
        /*   5*/ { EDL_ERR_CORRUPT_PNG_IMAGE, L"Corrupt PNG image %1"},
        /*   6*/ { EDL_ERR_OUTOFMEMORY, L"Out of memory"},
        /*   7*/ { EDL_ERR_READ, L"Read error"},
        /*   8*/ { EDL_ERR_LARGE_FILE, L"File too large"},
        /*   9*/ { EDL_ERR_OPENFORWRITE, L"Open for write failed" },
        /*  10*/ { EDL_ERR_WRITE, L"Write error" },
        /*  11*/ { EDL_ERR_IOERROR, L"General IO Error" },
        /*  12*/ { EDL_ERR_COMPRESS, L"Compression error %1" },
        /*  13*/ { EDL_ERR_CONFIG, L"Configuration error" },
        /*  14*/ { EDL_ERR_RESTRICTED_FONT, L"Restricted font encountered"},
        /*  15*/ { EDL_ERR_ABORTED, L"Job aborted by user"},
        /*  16*/ { EDL_ERR_BAD_ARGUMENTS, L"Bad arguments passed to an API function"},
        /*  17*/ { EDL_ERR_VECTOR_ERROR, L"An error occurred attempting to access the content of a vector"},
        /*  18*/ { EDL_ERR_BAD_FONT_MAP, L"An error was found in a FontMap file"},
        /*  19*/ { EDL_ERR_UNICODE, L"An invalid Unicode string was encountered"},
        /*  20*/ { EDL_ERR_PAGE_TOO_LARGE, L"A page was too large to be used"},

        /*  99*/ { JM_ERR_GENERAL, L"A general error occurred: %1"},
        /* 100*/ { JM_ERR_UNSUPPORTED, L"An attempt was made to use an unsupported, unimplemented feature"},
        /* 101*/ { JM_ERR_REVERT_FAILED, L"An attempt was made to revert a page that could not be performed"},
        /* 102*/ { JM_ERR_TARGET_NOT_FOUND, L"A target with the given DOMid was not found"},
        /* 103*/ { JM_ERR_RANGE_ERROR, L"An attempt was made to request or add an item from the API with an out-of-bounds index"},
        /* 104*/ { JM_ERR_PAGE_NOT_FOUND, L"The given page was not found in a document"},
        /* 105*/ { JM_ERR_DOCUMENT_NOT_FOUND, L"The given document was not found in an assembly"},
        /* 106*/ { JM_DUPLICATE_FIELD_PARTIAL_NAME, L"A field with a duplicate partial name was found"},
        /* 107*/ { JM_ERR_FORM_FIELD_NOT_FOUND, L"The given form field was not found"},
        /* 108*/ { JM_ERR_DUPLICATE_WIDGET, L"A widget with a duplicate reference was not found"},
        /* 109*/ { JM_ERR_WIDGET_NOT_FOUND, L"The given widget annotation or annotation reference could not be found"},
        /* 110*/ { JM_ERR_BAD_CONFIGURATION, L"General configuration error"},
        /* 111*/ { JM_ERR_APPEARANCE_NOT_FOUND, L"The required annotation appearance was not found"},
        /* 112*/ { JM_ERR_ANNOTATION_NOT_FOUND, L"The required annotation was not found"},
        /* 113*/ { JM_ERR_ASSEMBLY_WRITE_FORBIDDEN, L"The given assembly cannot be written due to security permissions"},
        /* 114*/ { JM_ERR_FONT_NOT_FOUND, L"The requested font was not found"},
        /* 115*/ { JM_ERR_TOO_MANY_PDFOUT_WRITERS, L"There are too many PDF Out writers currently in progress"},
        /* 116*/ { JM_ERR_INFORMATION_NOT_AVAILABLE, L"The requested information is not yet available" },
        /* 117*/ { JM_ERR_DIRECTORY_DOESNT_EXIST, L"A required directory does not exist on disk: %1" },
        /* 118*/ { JM_ERR_RESOURCE_NOT_FOUND, L"The requested resource could not be found" },
        /* 119*/ { JM_ERR_BAD_UNICODE_CMAP, L"A bad ToUnicode CMap was encountered" },
        /* 120*/ { JM_ERR_OPTIONAL_CONTENT_GROUP_NOT_FOUND, L"The requested optional content group was not found" },
        /* 121*/ { JM_ERR_INVALID_OPTIONAL_CONTENT, L"Invalid optional content information encountered" },
        /* 122*/ { JM_ERR_PCLXL, L"PCL/XL Processing Error: %1 at stream position %2" },
        /* 123*/ { JM_ERR_PCL, L"PCL5 Processing Error: %1 at stream position %2" },
        /* 124*/ { JM_ERR_PJL, L"PJL Processing Error: %1 at stream position %2" },
        /* 125*/ { JM_ERR_SYMBOLSET_NOT_FOUND, L"The requested PCL or PCL/XL symbol set was not found" },
        /* 126*/ { JM_ERR_PDF_OBJECT_NOT_FOUND, L"An expected PDF object was not found (Object %1, Generation %2)" },

        /* 503*/ { EDL_ERR_BAD_BRUSH, L"A bad brush was encountered"},
        /* 504*/ { EDL_ERR_DOM_CREATION_FAILED, L"Failure while attempting to create a DOM object"},
        /* 505*/ { EDL_ERR_INVALID_DOM_ID, L"An invalid DOMid was encountered"},
        /* 506*/ { EDL_ERR_INVALID_TYPE3_GLYPH, L"An invalid Type 3 Glyph was encountered"},
        /* 507*/ { EDL_ERR_INVALID_GLYPHS, L"An invalid Glyphs node was encountered."},

        // XPS specific error codes
        /*1001*/ { EDL_ERR_INVALID_ZIP, L"Invalid ZIP" },
        /*1002*/ { EDL_ERR_NO_CONTENT_TYPE, L"Error getting content type for %1 part" },
        /*1003*/ { EDL_ERR_PROCESS_PART, L"Error processing part %1" },
        /*1004*/ { EDL_ERR_XPS_PROVIDER, L"XPS Provider Error" },
        /*1005*/ { EDL_ERR_MISSING_RESOURCE, L"Missing resource %1" },
        /*1006*/ { EDL_ERR_DTD_CONTENT, L"DTD Content detected" },
        /*1007*/ { EDL_ERR_MISSING_REQ_ATTRIBUTE, L"Missing required atribute %1 in %2 element" },
        /*1008*/ { EDL_ERR_INVALID_ATTRIBUTE, L"Invalid attribute %1 in %2 element" },
        /*1009*/ { EDL_ERR_PROPERTY_ALREADY_SET, L"Attempt to set property %1 more than once" },
        /*1010*/ { EDL_ERR_REMOTERESOURCE_REF, L"Remote resource dictionary references another remote resource dictionary" },
        /*1011*/ { EDL_ERR_KEY_SET_NON_RESOURCE, L"Key attribute is set for an element that is not in resource dictionary" },
        /*1012*/ { EDL_ERR_UNSUPPORTED_GLYPHS, L"Glyphs unsupported : %1" },
        /*1013*/ { EDL_ERR_INVALID_URI, L"Encountered invalid URI : %1" },
        /*1014*/ { EDL_ERR_DEFLATE, L"Deflate error" },
        /*1015*/ { EDL_ERR_NO_KEY_RESDICT_OBJECT, L"Resource dictionary object missing the key attribute" },
        /*1016*/ { EDL_ERR_ADD_OBJECT_RESDICT, L"Error adding %1 element to resource dictionary" },
        /*1017*/ { EDL_ERR_INVALID_ABBR_PATH, L"Error processing abbreviated path" },
        /*1018*/ { EDL_ERR_NAMESPACE_NOT_UNDERSTOOD, L"Namespace %1 is not understood" },
        /*1019*/ { EDL_ERR_XML_PARSER, L"XML Parser Error in %1 at line:%2, column:%3 %4"},
        /*1020*/ { EDL_ERR_NO_REQ_ELEMENT, L"%1 requires %2 sub-element"},
        /*1021*/ { EDL_ERR_MORE_THAN_ONE_ELEMENT, L"%1 contains more than one instance of %2 sub-element"},
        /*1022*/ { EDL_ERR_INVALID_ELEMENT, L"%1 contains invalid %2 sub-element"},
        /*1023*/ { EDL_ERR_FALLBACK_BEFORE_CHOICE, L"Fallback element seen before choice"},
        /*1024*/ { EDL_ERR_PREFIX_NOT_DEFINED, L"%1 prefix is not defined"},
        /*1025*/ { EDL_ERR_UNPREFIXED_ATTRIBUTE, L"%1 contains unprefixed attribute %2"},
        /*1026*/ { EDL_ERR_BROKEN_ELEMENT_SEQUENCE, L"%1 contains broken element sequence"},
        /*1027*/ { EDL_ERR_PREFIXED_ATTRIBUTE, L"%1 contains prefixed attribute %2"},
        /*1028*/ { EDL_ERR_INVALID_IMAGE, L"Invalid image %1"},
        /*1029*/ { EDL_ERR_NUMBER_REQ_ELEMENT, L"%1 must have no less than %2 %3 elements"},
        /*1030*/ { EDL_ERR_MISSING_XPS_PART, L"Missing XPS part %1"},
        /*1031*/ { EDL_ERR_PRINTTICKET_EXISTS, L"PrintTicket is already found for %1"},
        /*1032*/ { EDL_ERR_MULTIPLE_STARTPARTS_RELS, L"Multiple StartPart relationships"},
        /*1033*/ { EDL_ERR_DICT_ITEM_PRESENT, L"Item with the same key (%1) is already present"},
        /*1034*/ { EDL_ERR_MULTIPLE_COREPROPERTIES_RELS, L"Multiple CoreProperty relationships"},
        /*1035*/ { EDL_ERR_MULTIPLE_DOCSTRUCTURE_RELS, L"Multiple CoreProperty relationships"},
        /*1036*/ { EDL_ERR_MULTIPLE_THUMBNAIL_RELS, L"Multiple Thumbnail relationships"},
        /*1037*/ { EDL_ERR_INVALID_THUMBNAIL_TYPE, L"Invalid thumbnail type: %1"},
        /*1038*/ { EDL_ERR_PRESERVED_ITEMS_NOT_DECLARED_IGNORABLE,  L"Unable to preserve items, namespace %1 is not declared as ignorable"},
        /*1039*/ { EDL_ERR_MORE_THAN_ONE_GROUP_ELEMENT, L"%1 contains more than one sub-element"},
        /*1040*/ { EDL_ERR_NO_REQ_GROUP_ELEMENT, L"%1 requires one sub-element"},
        /*1041*/ { EDL_ERR_INVALID_COLOR_SPECIFICATION, L"Invalid color specification in %1"},
        /*1042*/ { EDL_ERR_PROCESS_CONTENT_NOT_DECLARED_IGNORABLE,  L"Namespace %1 in ProcessContent attribute is not declared as ignorable"},
        /*1043*/ { EDL_ERR_DUPLICATE_URI,  L"Duplicate URI encountered %1"},

        // XPS out specific error codes
        /*1300*/ { EDL_ERR_UNEXPECTED_RESOURCE, L"Unexpected resource %1"},
        /*1301*/ { EDL_ERR_DEVICE_OUTOFMEMORY, L"Device has not enough memory to process page"},
        /*1302*/ { EDL_ERR_XPS_FROM_ENCRYPTED_SOURCE, L"Attempt to create XPS from an encrypted document"},

        /*1700*/ { EDL_ERR_DELETE_OUTLINE_WITH_CLOSED_PARENT, L"Attempt to delete outline entry with closed parent"},
        /*1701*/ { EDL_ERR_APPEND_OUTLINE_TO_CLOSED_NODE, L"Attempt to append outline entry to closed outline entry"},

        /*1600*/ { EDL_ERR_APPEND_EMPTY_ANNOTATION, L"Attempt to append empty annotaion"},
        /*1601*/ { EDL_ERR_APPEND_ANNOTATION_WITHOUT_RECT, L"Attempt to append annotation with empty Rect property"},

        /*2000*/ { EDL_ERR_INTERNAL_RIP, L"Internal RIP error: %1"},

        // PDF in specific errors
        /*2300*/ { EDL_ERR_OPEN_PDF, L"Error opening input PDF stream"},
        /*2301*/ { EDL_ERR_INVALID_PDF_PASSWORD, L"Error opening encrypted PDF; No valid password provided"},
        
        // PDF out specific errors
        /*2400*/ { EDL_ERR_PSOUT_GENERAL_FAILURE, L"General error in PS out"},
        /*2401*/ { EDL_ERR_PDFOUT_GENERAL_FAILURE, L"General error in PDF out"},
        /*2402*/ { EDL_ERR_INVALID_FONT, L"Error attempting to process a font"},
        /*2403*/ { EDL_ERR_INCOMPATIBLE_PDFA, L"PDF/A output: Incompatible content found: %1"},
        /*2404*/ { EDL_ERR_INCOMPATIBLE_PDFX, L"PDF/X output: Incompatible content found: %1"},

                
        // Color/Color space specific errors
        /*2500*/ { EDL_ERR_BAD_COLOR_SPACE, L"A bad color space was encountered"},
        /*2501*/ { EDL_ERR_COLOR_CONVERSION_FAILURE, L"Color conversion failed"},


        // Image specific errors
        /*2600*/ { EDL_ERR_INCOMPATIBLE_IMAGE_PARAMETER, L"An incompatible parameter was given to an encoder"},
        /*2601*/ { EDL_ERR_IMAGE_PARAMETER_OUT_OF_RANGE, L"An out-of-range parameter was given to an image encoder: %1"},
        /*2602*/ { EDL_ERR_IMAGE_DECODE_FAILURE, L"An image file could not be decoded"},
        /*2603*/ { EDL_ERR_IMAGE_ENCODE_FAILURE, L"An error occurred during image encoding: %1"},

        // ZIP specific errors
        /*2700*/ { EDL_ERR_ZIP_WRITE_ONLY, L"An attempt was made to retrieve a stream from a write-only ZIP file"},
        /*2701*/ { EDL_ERR_ZIP_PATH_NOT_FOUND, L"An expected entry in a ZIP package was not found"},
    };

    /**
     * @ingroup ErrorMessages
     * @brief      Implements a simple numeric-error-code-to-corresponding-language-specific-error-message-string look-up
     *                using the "errorList" array/table
	 * @param	errorCode The error code number
	 * @return	EDLString  Error message text
     */

    EDLString getEDLErrorString(uint32 errorCode)
    {
        EDLString errorString;

        for(int i=0;i<sizeof(errorList)/sizeof(struct EDL_ERR_LIST);i++)
        {
            if (errorList[i].errCode > errorCode)
                break;
            else if (errorList[i].errCode == errorCode) {
                errorString = errorList[i].errStr;
                break;
            }
        }
        return errorString;
    };

}; /* end of namespace EDL */
