/*
 * Copyright (C) 2009 Global Graphics Software Ltd. All rights reserved.
 */

/**
 * @file    edlvector.h
 * @ingroup EDL
 * @brief   Simple buffer class which preserves the allocation context
 *          where the buffers were created.
 */

#ifndef edlsimplebuffer_h
#define edlsimplebuffer_h

#include <stdlib.h>
#include <edl/edltypes.h>
#include <edl/edlnamespaces.h>

_BEGIN_EDL_DOM_NAMESPACE

class CEDLSimpleBuffer
{
    public:
    
        // Copy constructor
        CEDLSimpleBuffer(const CEDLSimpleBuffer &incoming)
        {
            init();
            assign(incoming.m_buffer, incoming.m_bufferSize);
        }
                
        // Allocate from a pre-existing block
        CEDLSimpleBuffer(const uint8 *src, uint32 size)
        {
            init();
            assign(src, size);
        }
                
        // Allocate an empty block
        CEDLSimpleBuffer(uint32 size = 0, bool clearBuff = false)
        {
            init();
            resize(size);
            if (clearBuff)
                clear();
        }
        
        ~CEDLSimpleBuffer()
        {
            release();
        }

        // Get size
        uint32 size()
        {
            return m_bufferSize;
        }
        
        // Dereference via index
        uint8& operator[](uint32 index)
        {
            if (index >= m_bufferSize)
            {
                throw;
            }
            return m_buffer[index];
        }
        
        CEDLSimpleBuffer& operator=(const CEDLSimpleBuffer &incoming)
        {
            release();
            assign(incoming.m_buffer, incoming.m_bufferSize);
            return *this;
        }
            
        void resize(uint32 newSize)
        {
            // Need to free?
            if (!newSize && m_buffer)
            {
                release();
            }
            // Need to allocate?
            else if (!m_buffer && newSize != 0)
            {
                m_buffer = (uint8 *) m_allocate(newSize);
                if (!m_buffer)
                {
                    throw;
                }
            }
            // Need to resize?
            else if (newSize != m_bufferSize)
            {
                uint8 *newBuffer = (uint8 *) m_reallocate(m_buffer, newSize);
                if (!newBuffer)
                {
                    throw;
                }
                m_buffer = newBuffer;
            }
            m_bufferSize = newSize;

        }
        
        void clear()
        {
            if (m_buffer && m_bufferSize)
            {
                memset(m_buffer, 0, m_bufferSize);
            }
        }
        
    private:
        
        void init()
        {
            m_buffer = NULL;
            m_bufferSize = 0;
            m_allocate = malloc;
            m_deallocate = free;
            m_reallocate = realloc;
        }

        void assign(const uint8 *src, uint32 size)
        {
            resize(size);
            if (size)
            {
                memcpy(m_buffer, src, size);
            }
        }
                
        void release()
        {
            if (m_buffer)
                m_deallocate(m_buffer);
            m_buffer = NULL;
            m_bufferSize = 0;
        }
        
        uint8 *m_buffer;
        uint32 m_bufferSize;

        typedef void *(*AllocatorFn)(size_t n);
        typedef void (*DeallocatorFn)(void *p);
        typedef void *(*ReallocatorFn)(void *p, size_t newSize);

        AllocatorFn   m_allocate;
        DeallocatorFn m_deallocate;
        ReallocatorFn m_reallocate;        
};

_END_EDL_DOM_NAMESPACE

#endif

