/*
 * Copyright (C) 2007-2019 Global Graphics Software Ltd. All rights reserved.
 */

/**
 * @file edlstream.h
 * @ingroup IController
 * @brief EDL provides a collection of "stream" classes
 * that are supplied to, and returned by, EDL APIs that access files or data streams
 */

#ifndef __EDLSTREAM_H__
#define __EDLSTREAM_H__

#include <fstream>
#include <edl/edltypes.h>
#include <edl/edlstream.h>
#include <edl/iedlobject.h>
#include <edl/idomhashable.h>
#include <edl/edlvector.h>

#ifdef _WIN32
#include <stdarg.h>
#endif

_BEGIN_EDL_NAMESPACE

/**
 * @class IEDLStream
 * @brief Generic stream. Abstract base class for EDL stream subsystem.
 */

class IEDLStream : virtual public IEDLObject
{
public:
    /**
	 *  @brief Determine stream valididy
	 *  @return @b bool True if stream is valid for operations (no fatal problems)
	 */
	virtual bool isValid() const = 0;

    /**
	 *  @brief Opens the stream
	 *  @return @b bool True if stream was successfully opened
	 */
	virtual bool open() = 0;

    /**
	 *  @brief Closes the stream
	 */
	virtual void close() = 0;

    /**
	 *  @brief Get current stream position
	 *  @return @b int64 The current stream position
	 */
	virtual int64 getPos() = 0;
};
DECL_SMART_PTR(IEDLStream);
#define edlobj2IEDLStream(src) edl_cast((IEDLStream *)NULL, src)

class IInputStream;
DECL_SMART_PTR(IInputStream);
typedef CEDLVector<IInputStreamPtr> CIInputStreamVect;
class IRAInputStream;
DECL_SMART_PTR(IRAInputStream);
class IInputPushbackStream;
DECL_SMART_PTR(IInputPushbackStream);

/**
 * @brief Type definition of a callback function to receive streamed output.
 *
 * Pointers to functions of this type can be passed to EDL via the @c outputcallback
 * parameter. Any implementations of this type are expected to return zero on success,
 * and a non-zero error code if the call fails.  They should also be able to handle
 * the special case where @c pBuff is set to NULL and @c len is set to 0. The stream
 * writer passes these values to signal an aborted output stream. That is, no more
 * data will be written to and incomplete stream, due to an error upstream in the process.
 * @param pPriv A private pointer that is passed with each invocation.
 * @param pBuff The buffer containing the data to be written.
 * @param len The length of the data in the buffer. 0 indicates that the stream is ending.
 * @return @b int Zero on success, non-zero on failure.
 */
typedef int (*UserStreamWriteFunc) (void *pPriv, void *pBuff, unsigned int len);

/**
 * @brief Callback typedef for streaming input.
 *
 * @param pPriv A private pointer that is passed with each invocation
 * @param pBuff The buffer to read data into
 * @param len   The number of bytes that are requested for the read.
 * @param pLenRead A pointer to the number of bytes actually read.
 * @param pEof A pointer to a int value that indicates whether the
 *        end of file (EOF) was read. *pEof = 1 if end of file, *pEof = 0 otherwise
 * @return @b int Zero on success, non-zero on failure.
 */
typedef int (*UserStreamReadFunc) (void *pPriv, void *pBuff, unsigned int len,
unsigned int *pLenRead, int *pEof);

/**
 * @brief Callback typedef for user defined random access input.
 *
 * For correct operation, this callback function must be thread safe as it
 * is possible that multiple threads may call this function simultaneously.
 *
 * @param pPriv A private pointer that is passed with each invocation
 * @param pBuff The buffer to read data into
 * @param offset The position within the input from which the data will be read.
 * @param len   The number of bytes that are requested for the read.
 * @return @b int Zero on success, non-zero on failure.
 */
typedef int (*UserRAReadFunc) (void *pPriv, void *pBuff, int64 offset, int32 length);


/**
 * @class IInputStream
 * @brief Generic input stream. Abstract base class for all input streams.
 */

class IInputStream : virtual public IDOMHashable, virtual public IEDLStream
{
public:
    /**
    * @brief    Creation function for an IInputStream for a file on disk.
    *           Throws an IEDLError exception on failure.
    * @param    pFactory The class factory
    * @param    path Path to the file
    * @return   @b IRAInputStreamPtr The new input stream
    */
    static EDL_API IRAInputStreamPtr createFromFile(IEDLClassFactory *pFactory, const EDLSysString &path);

    /**
    * @brief    Creation function for an IInputStream for a file on disk.
    *           Throws an IEDLError exception on failure.
    * @param    pFactory The class factory
    * @param    path Path to the file
    * @return   @b IRAInputStreamPtr The new input stream
    */
    static EDL_API IRAInputStreamPtr createFromFile(IEDLClassFactory *pFactory, const EDLString &path);
    
    /**
    * @brief    Creation function for an IInputStream for a file on disk.
    *           Similar to createFromFile, but if this file is cloned, only
    *           one file handle will be open at a time. This is slower, but
    *           is useful if many users are likely to want to have this file
    *           open at the same file. On some systems, having too many files
    *           open may result in errors. Performance should be acceptable
    *           providing there is not a great deal of contention and
    *           simultaneous access.
    * @param    pFactory The class factory
    * @param    path Path to the file
    * @return   @b IRAInputStreamPtr The new input stream
    */
    static EDL_API IRAInputStreamPtr createFromFileShared(IEDLClassFactory *pFactory, const EDLSysString &path);
    
    /**
    * @brief    Creation function for an IInputStream for a file on disk.
    *           Similar to createFromFile, but if this file is cloned, only
    *           one file handle will be open at a time. This is slower, but
    *           is useful if many users are likely to want to have this file
    *           open at the same file. On some systems, having too many files
    *           open may result in errors. Performance should be acceptable
    *           providing there is not a great deal of contention and
    *           simultaneous access.
    * @param    pFactory The class factory.
    * @param    path Path to the file
    * @return   @b IRAInputStreamPtr The new input stream
    */
    static EDL_API IRAInputStreamPtr createFromFileShared(IEDLClassFactory *pFactory, const EDLString &path);
    
    /**
    * @brief    Creation function for an IInputStream for data in memory.
    *           Throws an IEDLError exception on failure.
    * @param    pFactory The class factory.
    * @param    mem A pointer to the memory to be used as the source data for the stream.
    * @param    length The length of the buffer pointed to by mem.
    * @param   copy If true, a copy of the memory pointed to by mem will be made and will
    *           be managed and released by the stream.
    * @param   free If copy is false and free is true, then when the stream is destroyed, mem
    *           will be deallocated using using stdlib free(). Ignored if copy is true.
    * @return   @b IInputStreamPtr The new input stream
    */
    static EDL_API IRAInputStreamPtr createFromMemory(IEDLClassFactory *pFactory, const void *mem, uint32 length, bool copy = false, bool free = true);
    
    /**
     * @brief    Creation function for an IInputStream from a user function that provides data.
     *           Throws an IEDLError exception on failure.
     * @param    pFactory The class factory.
     * @param    readFunc A function that when called, will provide the data for the stream.
     * @param    priv An opaque private pointer that is passed to the readFunc on each call.
     * @return   @b IInputStreamPtr The new input stream
     */
    static EDL_API IInputStreamPtr createFromUserFunc(IEDLClassFactory *pFactory, UserStreamReadFunc readFunc, void *priv);
    
    /**
     * @brief    Creation function for an IRAInputStream from a user function that provides random access.
     *           Throws an IEDLError exception on failure.
     * @param    pFactory The class factory.
     * @param    length The stream length
     * @param    readFunc A function that when called, will provide the data for the stream.
     * @param    priv An opaque private pointer that is passed to the readFunc on each call.
     * @return   @b IInputStreamPtr The new input stream.
     */
    static EDL_API IRAInputStreamPtr createFromRAUserFunc(IEDLClassFactory *pFactory, int64 length, UserRAReadFunc readFunc, void *priv);
    
    /**
    * @brief    Creation function for an IInputStream for a file on disk created with the
    *           contents of an existing stream.
    *           Throws an IEDLError exception on failure.
    * @param    pFactory The class factory.
    * @param    path Path to the file
    * @param    stream The stream to populate the new file with.
    * @return   @b IInputStreamPtr The new input stream
    */
    static EDL_API IRAInputStreamPtr createFromNewFileWithContents(IEDLClassFactory *pFactory, const EDLSysString &path, const IInputStreamPtr &stream);

    /**
    * @brief    Creation function for an IInputStream for a file on disk created with the
    *           contents of an existing stream.
    *           Throws an IEDLError exception on failure.
    * @param    pFactory The class factory.
    * @param    path Path to the file
    * @param    stream The stream to populate the new file with.
    * @return   @b IInputStreamPtr The new input stream
    */
    static EDL_API IRAInputStreamPtr createFromNewFileWithContents(IEDLClassFactory *pFactory, const EDLString &path, const IInputStreamPtr &stream);
    
    /**
    * @brief    Creation routine for a stream representing a portion of a file on disk.
    *           If the source file is random access, then the created file shall be also.
    *           Throws an IEDLError exception on failure.
    * @param    pFactory The class factory
    * @param    stream The stream to use for source.
    * @param    offset The offset within the source stream of the start of the data to use.
    * @param    length The length of source data to use.
    * @return   @b IInputStreamPtr The new input stream
    */
    static EDL_API IInputStreamPtr createSubFile(IEDLClassFactory *pFactory, const IInputStreamPtr &stream, int64 offset, int64 length);

    /**
    * @brief    Creation routine for a input stream for decompressing a flate stream.
    *           Throws an IEDLError exception on failure.
    * @param    pFactory The class factory
    * @param    stream The compressed stream.
    * @param    raw Pass to true if the flate stream has no zlib header.
    * @param    ignoreChecksums Pass true if checksum errors should be ignored.
    * @return   @b IInputStreamPtr The new input stream
    */
    static EDL_API IInputStreamPtr createFromFlateCompressed(IEDLClassFactory *pFactory, const IInputStreamPtr &stream, bool raw = true, bool ignoreChecksums = false);
    
    /**
    * @brief    Creation routine for a input stream for decompressing an lz4 block compressed stream.
    *           Throws an IEDLError exception on failure.
    *           Note: This is not intended for interoperability with other LZ4 formats, but is useful
    *           for things like temporary storage.
    * @param    pFactory The class factory
    * @param    stream The compressed stream.
    * @return   @b IInputStreamPtr The new input stream
    */
    static EDL_API IInputStreamPtr createFromLz4Compressed(IEDLClassFactory *pFactory, const IInputStreamPtr &stream);
    
    /**
    * @brief    Creation routine for creating a composite input stream representing the
    *           concatenation of a series of streams.
    * @param    pFactory The class factory.
    * @param    streams The vector of streams to use.
    * @return   @b IInputStreamPtr The new input stream.
    */
    static EDL_API IInputStreamPtr createCompositeStream(IEDLClassFactory *pFactory, const CIInputStreamVect &streams);

    /**
    * @brief    Creation routine for creating a push-back stream using a non-pushback
    *           stream as a data source.
    *           If the source stream is random access, the resulting stream will also be
    *           random access.
    * @param    pFactory The class factory.
    * @param    sourceStream The source stream to use.
    * @param    clone If true, the input stream is cloned. If false, the existing stream is used in place.
    * @return   @b IInputStreamPtr The new input stream.
    */
    static EDL_API IInputPushbackStreamPtr createPushbackStream(IEDLClassFactory *pFactory, const IInputStreamPtr &sourceStream, bool clone = true);

	/**
	 *  @brief Read specified number of bytes from a stream into buffer
	 *  @param buffer Buffer to accept the data
	 *  @param count Number of bytes to read
	 *  @return @b int32 The number of bytes actually read
	 *  
	 */
	virtual int32 read(void * buffer, int32 count) = 0;
    
	/**
	 *  @brief Read single byte from a stream
	 *  @return @b int8 Number of bytes actually read
	 */
	virtual int8 read() = 0;
    
    /**
	 *  @brief Determine if the stream has exhausted
	 *  @return @b bool True if the stream has exhausted
	 */
	virtual bool eof() const = 0;

	/**
	 *  @brief Skip a specified number of bytes
	 *  @param count Number of bytes to skip
	 *  @return @b int64 The number of bytes actually skipped
	 */
	virtual int64 skip(int64 count)
    { 
        static const int32 kBufSize = 1024;
        char buf[kBufSize];
        int64 rc = 0;
        int32 n_read;
        while (count > 0) {
                if ((n_read = read((void *)buf, (int32)((count > kBufSize)? kBufSize : count))) <= 0)
                {
                    break;
                }
                else
                        rc += n_read;
                count -= n_read;
        }
        return rc;
    }
    
	/**
	 *  @brief Perform a complete read
	 *  @param buffer Buffer to accept the data
	 *  @param count Number of bytes to be read into the buffer
	 *  @return @b bool True if the read operation was successful, or false if the read could not be completely fulfilled
	 */
	virtual bool completeRead(void * buffer, int32 count);
    
    /**
     * @brief  Obtain a 64-bit hash of the stream.
     *         Please note that this requires reading the stream and is therefore
     *         not thread safe. If thread safety is desired, make a clone of the
     *         stream first.
	 *  @param hash A reference to receive the hash value
	 *  @return @b bool True if method succeeded
	 */
	virtual bool hash(uint64 &hash);
};
#define edlobj2IInputStream(src) edl_cast((IInputStream *)NULL, src)

/**
 * @class IRAStream
 * @brief Abstract base class for "Random-Access" streams
 *        i.e. streams that can be arbitrarily re-positioned
 */
 
class IRAStream
{
public:
    /**
     *  @brief Get length of the stream
     *  @return @b int64 The length of the stream
     */
	virtual int64 length() = 0;

	/**
	 *  @brief Set stream position
	 *  @param newPos The desired new position
	 *  @return @b bool True if method succeeded
	 */
	virtual bool setPos(int64 newPos) = 0;
};

/**
 * @class IPushbackStream
 * @brief Abstract base class (for input streams only) that
 *        provides a "push back" mechanism.
 *        When used with random access streams, the pushback buffer
 *        is invalidated by setPos().
 */
class IPushbackStream
{
  public:
	/**
	 *  @brief Push back a byte
	 *  @param byte Byte to be pushed back
	 *  @return @b bool True if method succeeded
	 */
	virtual bool pushBack(uint8 byte) = 0;
    
	/**
	 *  @brief Push back from a buffer
	 *  @param buffer Buffer to push back from
	 *  @param count  Number of bytes to push back
	 *  @return @b bool True if method succeeded
	 */
	virtual bool pushBack(const void *buffer, int32 count) = 0;
};

/**
 * @class IRAInputStream
 * @brief Random Access Input Stream
 */
class IRAInputStream : virtual public IInputStream, virtual public IRAStream
{
};

DECL_SMART_PTR(IRAInputStream);
#define edlobj2IRAInputStream(src) edl_cast((IRAInputStream *)NULL, src)

/**
 * @class IInputPushbackStream
 * @brief Input Stream with pushback support
 */

class IInputPushbackStream : virtual public IInputStream, virtual public IPushbackStream
{
};

#define edlobj2IInputPushbackStream(src) edl_cast((IInputPushbackStream *)NULL, src)

/**
 * @class IRAInputPushbackStream
 * @brief Random-access Input Stream with pushback support
 */

class IRAInputPushbackStream : virtual public IRAInputStream, virtual public IPushbackStream
{
};

DECL_SMART_PTR(IRAInputPushbackStream);
#define edlobj2IRAInputPushbackStream(src) edl_cast((IRAInputPushbackStream *)NULL, src)

#define FILE_STREAM_GUID         0x89ad895, 0x48f1685b, 0x8dab9c1a, 0xe8e3b18b
#define SHARED_STREAM_GUID       0xe88a727d, 0xe53140da, 0x9bbff26d, 0x79a42d81
#define OUTPUTFILE_STREAM_GUID   0x50a914fc, 0xe81446b9, 0x8e420d48, 0x09454eff
#define OUTPUTUSER_STREAM_GUID   0x789AEE52, 0xD0384136, 0x8B4584B0, 0x942E55CA
#define INPUTUSER_STREAM_GUID    0xd55cc022, 0xa6844749, 0x9414d265, 0xeeab7cea
#define INPUTRAUSER_STREAM_GUID  0x9895EBEB, 0xe2654b86, 0xa527a102, 0x3c3d36cd
#define INPUT_MEMORY_STREAM_GUID 0x26CAC8F2, 0x726A432c, 0xB8399237, 0xEA12D95F
#define FILE_STREAM_WITH_CALLBACK_GUID 0X7FB63540, 0XC0684D87, 0XB513F53A, 0X4925CF1C

#define USERINPUT_RASTREAM_GUID  0x776b2562, 0x87e141b0, 0xa3d4c121, 0xdf3588ba
#define USEROUTPUT_STREAM_GUID   0x1d0a1b45, 0xb2694fd2, 0xa1b28077, 0xee815f6a

/**
  * @brief Callback typedef for CFileStreamWithCallback that is used to notify a client of reads from stream.
  *
  * @param pPriv A private pointer that is passed with each invocation
  * @param pos   Starting position for this read.
  * @param len   The number of bytes that were read.
  */
typedef void (*FileStreamReadFunc) (void *pPriv, int64 pos, int32 len);

class CFileStreamParams : public CClassParams
{
public:
    CFileStreamParams(const EDLSysString & sName, bool append = false) : m_sName(sName), m_bAppend(append) {}

    EDLSysString m_sName;
    bool         m_bAppend;
};

class CFileStreamWithCallbackParams : public CFileStreamParams
{
public:
    CFileStreamWithCallbackParams(const EDLSysString & sName, FileStreamReadFunc pReadFunc, void *pPriv) : 
        CFileStreamParams(sName), 
        m_pReadFunc(pReadFunc), 
        m_pPriv(pPriv) 
    {
    }

    FileStreamReadFunc m_pReadFunc;
    void *m_pPriv;
};

// Forward declaration
class ISession;


class CUserStreamWriteParams : public CClassParams
{
public:
    CUserStreamWriteParams(UserStreamWriteFunc pWriteFunc, void *pPriv) : 
      m_pWriteFunc(pWriteFunc), m_pPriv(pPriv) 
    {}

    UserStreamWriteFunc m_pWriteFunc;
    void *m_pPriv;
};

class CUserStreamReadParams : public CClassParams
{
public:
    CUserStreamReadParams(UserStreamReadFunc pReadFunc, void *pPriv) : 
      m_pReadFunc(pReadFunc), m_pPriv(pPriv) 
    {}

    UserStreamReadFunc m_pReadFunc;
    void *m_pPriv;
};

class CRAUserStreamReadParams : public CClassParams
{
public:
    CRAUserStreamReadParams(UserRAReadFunc pReadFunc, int64 length, void *pPriv) :
      m_pReadFunc(pReadFunc), m_pPriv(pPriv), m_length(length)
    {}

    UserRAReadFunc m_pReadFunc;
    void          *m_pPriv;
    int64          m_length;
};

class CInputMemoryParams : public CClassParams
{
public:
    // If makeCopy is false and free is true, the memory input stream will take ownership of the memory
    // (which must be able to be freed by free()).
    // If makeCopy is true, then a copy is made and will be managed by the stream. In this case,
    // free is ignored.
    CInputMemoryParams(const uint8 *mem, int32 length, bool makeCopy, bool free = true) : m_memory(mem), m_length(length), m_makeCopy(makeCopy), m_free(free) {}
    const uint8 * m_memory;
    int32         m_length;
    bool          m_makeCopy;
    bool          m_free; // Ignored if m_makeCopy is true
};

class IOutputStream;
DECL_SMART_PTR(IOutputStream);

/**
 * @class IOutputStream
 * @brief Generic output stream. Abstract base class for output streams
 */
 
class IOutputStream : virtual public IEDLStream
{
public:
    /**
    * @brief    Creation function for an IOutputStream for a file on disk.
    *           Throws an IEDLError exception on failure.
    * @param    pFactory The EDL class factory
    * @param    path Path to the file 
    * @param    append Specify what to do if the file exists. Optional
	*           - If false the file will be overwritten by new content. Default
    *           - If true new content will be added to the end of the file 
    * @return   @b IOutputStreamPtr The new output stream
    */
    static EDL_API IOutputStreamPtr createToFile(IEDLClassFactory *pFactory, const EDLSysString &path, bool append = false);

    /**
    * @brief    Creation function for an IOutputStream for a file on disk.
    *           Throws an IEDLError exception on failure.
    * @param    pFactory The EDL class factory
    * @param    path Path to the file
    * @param    append Specify what to do if the file exists. Optional
	*           - If false the file will be overwritten by new content. Default
    *           - If true new content will be added to the end of the file 
    * @return   @b IOutputStreamPtr The new output stream
    */
    static EDL_API IOutputStreamPtr createToFile(IEDLClassFactory *pFactory, const EDLString &path, bool append = false);
    
    /**
     * @brief    Creation function for an IOutputStream from a user function that provides data.
     *           Throws an IEDLError exception on failure.
     * @param    pFactory The class factory.
     * @param    writeFunc A function that when called, will write the data for the stream.
     * @param    priv An opaque private pointer that is passed to the readFunc on each call.
     * @return   @b IOutputStreamPtr The new input stream
     */
    static EDL_API IOutputStreamPtr createFromUserFunc(IEDLClassFactory *pFactory, UserStreamWriteFunc writeFunc, void *priv);

    /**
     * @brief    Creation routine for an output stream for compressing a flate stream.
     *           Throws an IEDLError exception on failure.
     * @param    pFactory The class factory
     * @param    stream The compressed stream.
     * @param    compressionLevel Values are in the range 1 (fastest, least compression) through
     *           9 (slowest, most compression)
     * @param    raw Pass true if the flate stream should have no zlib header.
     * @return   @b IOutputStreamPtr The new output stream
     */
    static EDL_API IOutputStreamPtr createToFlateCompressed(IEDLClassFactory *pFactory, const IOutputStreamPtr &stream, uint32 compressionLevel, bool raw = true);
    
    /**
     * @brief    Creation routine for an output stream for compressing an lz4 stream.
     *           Throws an IEDLError exception on failure.
     *           Note: This is not intended for interoperability with other LZ4 formats, but is useful
     *           for things like temporary storage.
     * @param    pFactory The class factory
     * @param    stream The compressed stream.
     * @return   @b IOutputStreamPtr The new output stream
     */
    static EDL_API IOutputStreamPtr createToLz4Compressed(IEDLClassFactory *pFactory, const IOutputStreamPtr &stream);

    /**
    * @brief    Copy a source stream to a destination stream
    *           Opens and closes both the input and output streams.
    *           Throws an IEDLError exception on failure.
    * @param    inStream The source stream
    * @param    outStream The destination stream
    * @return   @b int64 The number of bytes copied
    */
    static EDL_API int64 copy(const IInputStreamPtr &inStream, const IOutputStreamPtr &outStream);

    /**
    * @brief    Write the contents of the given stream to an output stream.
    *           Opens and closes the input, but does not open or close the output.
    *           Throws an IEDLError exception on failure.
    * @param    inStream The source stream
    * @param    outStream The destination stream
    * @return   @b int64 The number of bytes written.
    */
    static EDL_API int64 writeStream(const IInputStreamPtr &inStream, const IOutputStreamPtr &outStream);

    virtual int32 write(const void *buffer, int32 count) = 0;

    /**
	 *  @brief Perform a write
	 *  @param str C-style string that contains the text to be written to the output
	 *  @return @b int32 On success, the total number of characters written
	 *  
	 */
	virtual int32 write(const char *str)
    {
        return this->write(str, (int32) strlen(str));
    }

    virtual bool flush() = 0;
    
    /**
     * @brief    Perform a formatted write as per fprintf()
	 * @param fmt C-style string that contains the text to be written to the output 
	 * @param ... Additional arguments as specified by @a fmt
	 * @return @b int32 On success, the total number of characters written
	 *  
	 */
	virtual EDL_API int32 writeFormatted (const char *fmt, ...);

    /**
	 *  @brief Perform a complete write
	 *  @param buffer Address of buffer
	 *  @param count Number of bytes to be written
	 *  @return @b bool True if the write operation was successful, or false if the write could not be completely fulfilled
	 *  
	 */
	virtual bool completeWrite(const void * buffer, int32 count);
};
#define edlobj2IOutputStream(src) edl_cast((IOutputStream *)NULL, src)

/**
 * @class IRAOutputStream
 * @brief Random Access Output Stream
 */

class IRAOutputStream : public IOutputStream, virtual public IRAStream
{
};

DECL_SMART_PTR(IRAOutputStream);
#define edlobj2IRAOutputStream(src) edl_cast((IRAOutputStream *)NULL, src)

class IRAInputOutputStream : public IRAInputStream, public IRAOutputStream
{
};

DECL_SMART_PTR(IRAInputOutputStream);
#define edlobj2IRAInputOutputStream(src) edl_cast((IRAInputOutputStream *)NULL, src)

/**
 * @class EDLIFStream
 * @brief An ifstream that can deal with UTF8 file names on all platforms.
 */
class EDLIFStream : public std::ifstream
{
  public:
    EDLIFStream() : std::ifstream() {}
    explicit EDLIFStream(const char* filename, std::ios_base::openmode mode = std::ios_base::out) : std::ifstream()
    {
        open(filename, mode);
    }
#ifdef _WIN32
    void open(const char* filename, std::ios_base::openmode mode = std::ios_base::in)
    {
        EDLString wFilename = EDLSysStringToEDLString(EDLSysString(filename));
        std::ifstream::open(wFilename.c_str(), mode);
    }
#endif
};

/**
 * @class EDLOFStream
 * @brief An ofstream that can deal with UTF8 file names on all platforms.
 */
class EDLOFStream : public std::ofstream
{
  public:
    EDLOFStream() : std::ofstream() {}
    explicit EDLOFStream(const char* filename, std::ios_base::openmode mode = std::ios_base::out) : std::ofstream()
    {
        open(filename, mode);
    }
#ifdef _WIN32
    void open(const char* filename, std::ios_base::openmode mode = std::ios_base::out)
    {
        EDLString wFilename = EDLSysStringToEDLString(EDLSysString(filename));
        std::ofstream::open(wFilename.c_str(), mode);
    }
#endif
};



_END_EDL_NAMESPACE

#endif /* __EDLSTREAM_H__ */
