/*
* Copyright (C) 2009-2019 Global Graphics Software Ltd. All rights reserved.
*/

#ifndef ICOLORMANAGER_H
#define ICOLORMANAGER_H

#include <edl/iedlfactory.h>
#include <edl/iedlobject.h>
#include <edl/idomresources.h>
#include <edl/idomcolor.h>
#include <edl/idomcolorspace.h>
#include <edl/edlvector.h>
#include <edl/edlsimplebuffer.h>

_BEGIN_EDL_NAMESPACE

/**
* @file     icolormanager.h
* @ingroup  IDom
* @brief    Public interface to the EDL color manager.
*/

#define EDL_CMM_MAX_COMPONENTS 32

class IColorManager;
DECL_SMART_PTR(IColorManager);

/**
* @class    IColorManager
* @ingroup  IDom
* @brief    Public interface to the EDL color manager.
*           There is only one instance of the color manager for each factory.
*           It can be retrieved using the IEDLFactory::getSingleton method, or
*           by using the get() static function.
*
*           Exceptions of type IEDLError are thrown on errors.
*/
class IColorManager : public IEDLObject
{
#define IColorManagerClassID 0x573cc514, 0x821611dc, 0x83140800, 0x200c9a66
public:
    DECLARE_CLASS_ID(IColorManagerClassID);
    IMPLEMENT_CLASS_ID(IColorManager);
    
    virtual ~IColorManager() {};
    
	/**
	 *  @brief Get the color manager singleton.
     *         Throws an IEDLError on failure.
	 *  
	 *  @param pFactory The EDL Class factory to use
	 *  @return @b IColorManagerPtr Pointer to the color manager
	 *  
	 */
	static EDL_API IColorManagerPtr get(IEDLClassFactory *pFactory);
    
    /**
    * @brief    Convert colors from one color space to another, using floating point samples.
    * @details  Currently, only base color spaces are supported; Indexed and DeviceN spaces
    *           cannot be used. For conversion of multiple samples, samples are interleaved,
    *           with any alpha being last. For example, sRGB with alpha must be presented as
    *           r, g, b, a, r, g, b, a, etc.
    * @param    numColors The number of colors to convert.
    * @param    hasAlpha Set to true if the color data contains alpha.
    * @param    sourceSpace The color space describing the input samples.
    * @param    destSpace The desired color space for the output samples.
    * @param    intent The desired rendering intent for the conversion.
    * @param    bpc The desired handling of black point compensation. If in doubt, use eBPCDefault.
    * @param    in Pointer to the input sample buffer.
    * @param    out Pointer to the output sample buffer.
    */
    virtual void convertColors(int numColors, bool hasAlpha, IDOMColorSpacePtr &sourceSpace, IDOMColorSpacePtr &destSpace, eRenderingIntent intent, eBlackPointCompensation bpc, float *in, float *out) = 0;
	
    /**
    * @brief    Convert colors from one color space to another, using integer samples.
    *           Currently, only base color spaces are supported; Indexed and DeviceN spaces
    *           cannot be used. For conversion of multiple samples, samples are interleaved,
    *           with any alpha being last. For example, sRGB with alpha must be presented as
    *           r, g, b, a, r, g, b, a, etc.
    *
    * @param    numColors The number of colors to convert.
    * @param    hasAlpha Set to true if the color data contains alpha.
    * @param    sourceSpace The color space describing the input samples.
    * @param    destSpace The desired color space for the output samples.
    * @param    intent The desired rendering intent for the conversion.
    * @param    bpc The desired handling of black point compensation. If in doubt, use eBPCDefault.
    * @param    in Pointer to the input sample buffer.
    * @param    out Pointer to the output sample buffer.
    * @param    inBPS The bits per sample of the input samples. Currently 1, 2, 4, 8, 12 and 16 bits are supported. 
    * @param    outBPS The desired bits per sample of the output samples. Currently 1, 2, 4, 8, 12 and 16 bits are supported. 
    */
    virtual void convertColors(int numColors, bool hasAlpha, IDOMColorSpacePtr &sourceSpace, IDOMColorSpacePtr &destSpace, eRenderingIntent intent, eBlackPointCompensation bpc, uint8 *in, uint8 *out, int inBPS, int outBPS) = 0;
    
    /**
    * @brief Find the number of components for an ICCBased color space
    * @param space The ICC space to interrogate
    * @return @b uint8 The number of components in the ICC space
    */
    virtual uint8 getNumComponentsForICCBasedSpace(const IDOMColorSpaceICCBasedPtr &space) = 0;
    
    /**
     * @brief    Find the number of components for an ICC profile
     * @param    profile The ICC profile to interrogate
     * @return   @b uint8 The number of components in the ICC space
     */
    virtual uint8 getNumComponentsForICCProfile(const IDOMICCProfilePtr &profile) = 0;
    
    /**
    * @brief    Find the default rendering intent for an ICCBased color space
    * @param    space The ICC space to interrogate
    * @return   @b eRenderingIntent The default intent
    */
	virtual eRenderingIntent getDefaultIntentForICCBasedSpace(const IDOMColorSpaceICCBasedPtr &space) = 0;
    
    /**
    * @brief    Get the version number of the given ICC profile.
    * @param    space The ICC color space to interrogate.
    * @param    majorVersion Reference to receive the major version
    * @param    minorVersion Reference to receive the minor version
    */
    virtual void getProfileVersionForICCBasedSpace(const IDOMColorSpaceICCBasedPtr &space, uint8 &majorVersion, uint8 &minorVersion) = 0;
    
    /**
    * @brief    Get the name of the profile for an ICCBased color space
    * @param    space The ICC color space to interrogate.
    * @param    name Reference to receive the profile name
    */
    virtual void getProfileNameForICCBasedSpace(const IDOMColorSpaceICCBasedPtr &space, EDLSysString &name) = 0;
    
	/**
	* @brief Retrieve the built-in CMYK SWOP profile.
	* @return @b IDOMICCProfilePtr A smart pointer to the resulting profile.
	*/   
	virtual IDOMICCProfilePtr getCMYKSWOPProfile() = 0;

    /**
    * @brief    Retrieve the built-in sGray profile.
	* @return @b IDOMICCProfilePtr A smart pointer to the resulting profile.
    */
    virtual IDOMICCProfilePtr getsGrayProfile() = 0;

    /**
    * @brief    Retrieve the built-in sRGB profile.
	* @return  @b IDOMICCProfilePtr A smart pointer to the resulting profile.
    */
    virtual IDOMICCProfilePtr getsRGBProfile() = 0;
    
    /**
    * @brief    Retrieve the built in scRGB profile.
	* @return @b IDOMICCProfilePtr A smart pointer to the resulting profile.
    */
    virtual IDOMICCProfilePtr getscRGBProfile() = 0;
    
    /**
    * @brief    Create a profile for a calibrated gray color space with
    *           the given parameters. Such a profile will be analogous
    *           to the PDF CalGray color space.
    * @param    whitePoint A three entry vector describing the white point
    *           of the color space, in the CIE XYZ color space.
    * @param    gamma Optional. The gamma for the color space, 1.0 if not provided
    * @param    blackPoint Optional. A three entry vector describing the
    *           black point of the color space, in the CIE XYZ color space.
    *           If not supplied, the values 0,0,0 will be used.
    * @return   @b IDOMICCProfilePtr A smart pointer to the resulting profile.
    */
	virtual IDOMICCProfilePtr createCalibratedGrayProfile(const CEDLVector<double> &whitePoint, double gamma = 1.0, const CEDLVector<double> &blackPoint = CEDLVector<double>()) = 0;

    /**
    * @brief    Create a profile for a gray color space with
    *           the given parameters, including the ability to specify an arbitrary
    *           gamma function.
    * @param    whitePoint A three entry vector describing the white point
    *           of the color space, in the CIE XYZ color space.
    * @param    gamma A gamma function consisting of a table of evenly-
    *           spaced, output values. For example, a 10 entry vector would
    *           describe the output of the gamma function for 0.1 increments
    *           between 0.0 and 1.0 inclusive. There must be at least two
    *           entries (which would describe a linear gamma function).
    * @param    blackPoint Optional. A three entry vector describing the
    *           black point of the color space, in the CIE XYZ color space.
    *           If not supplied, the values 0,0,0 will be used.
    * @return   @b IDOMICCProfilePtr A smart pointer to the resulting profile.
    */
    virtual IDOMICCProfilePtr createGrayProfile(const CEDLVector<double> &whitePoint, CEDLVector<float> &gamma, const CEDLVector<double> &blackPoint = CEDLVector<double>()) = 0;

    /**
    * @brief    Create a profile for a calibrated RGB color space with
    *           the given parameters. Such a profile will be analogous
    *           to the PDF CalRGB color space.
    * @param    whitePoint A three entry vector describing the white point
    *           of the color space, in the CIE XYZ color space.
    * @param    gamma Optional. A vector containing the desired gamma for the
    *           color space, one value for R, G and B restpectively. 1 1 1 is
    *           used if not provided.
    * @param    matrix Optional. A nine entry vector representing a the
    *           linear transformation of the gamma corrected RGB values into
    *           CIE XYZ space. If not provided, then an identit matrix
    *           (1, 0, 0,   0, 1, 0,  0, 0, 1) will be used.
    * @param    blackPoint Optional. A three entry vector describing the
    *           black point of the color space, in the CIE XYZ color space.
    *           If not supplied, the values 0,0,0 will be used.
    * @return   @b IDOMICCProfilePtr A smart pointer to the resulting profile.
    */
    virtual IDOMICCProfilePtr createCalibratedRGBProfile(const CEDLVector<double> &whitePoint, const CEDLVector<double> &gamma = CEDLVector<double>(), const CEDLVector<double> &matrix = CEDLVector<double>(), const CEDLVector<double> &blackPoint = CEDLVector<double>()) = 0;
    
    /**
     * @brief Configure how DeviceGray colors are converted to DeviceCMYK.
     *
     * @details Normally, all color conversions in JawsMako are achieved using
     *        color management. In particular, this includes the DeviceGray,
     *        DeviceCMYK and DeviceRGB color spaces, which are converted using
     *        the intercept color spaces configured in the functions below.
     *
     *        However this means that conversions between DeviceGray and DeviceCMYK
     *        generally pass through ICC conversion. Gray values may not map
     *        exclusively to DeviceCMYK K values, resulting in non-zero values
     *        in the Cyan, Magenta and Yellow components.
     *
     *        This may be unsuitable in some environments, in particular when
     *        dealing with overprint simulation.
     *
     * @brief This behavior can be changed with this member. If the argument is
     *        - @a true: All color conversions from DeviceGray to DeviceCMYK 
     *        will simply map and invert the gray value to the K channel.
     *        This will also apply to any rendering operations performed by JawsMako.
     *        - @a false (default): The intercept color spaces will be used to 
     *        convert from DeviceGray to DeviceCMYK.
     *  
     * @param mapGrayDirectly Determine how DeviceGray colors are converted to DeviceCMYK
     */
	virtual void setMapDeviceGrayToCMYKBlack(bool mapGrayDirectly) = 0;
    
    /**
     * @brief    Setting that determines how the color manager will convert DeviceGray
     *           colors to DeviceCMYK
     *           @see setMapDeviceGrayToCMYKBlack() for details.
	 * @return @b bool The current setting
	 *  
	 */
	virtual bool getMapDeviceGrayToCMYKBlack() const = 0;

    /**
    * @brief    Set the intercept color space for DeviceGray. This is the color
    *           space that will be used for conversion if the color manager is
    *           asked to convert from or to DeviceGray.
    * @param    space The desired DeviceGray intercept color space.
    */
    virtual void setDeviceGrayIntercept(const IDOMColorSpacePtr &space) = 0;

    /**
    * @brief    Get the intercept color space for DeviceGray. This is the color
    *           space that will be used for conversion if the color manager is
    *           asked to convert from or to DeviceGray.
    *           The default DeviceGray intercept is sGray.
    * @return   @b IDOMColorSpacePtr A smart pointer to the intercept color space
    */
    virtual IDOMColorSpacePtr getDeviceGrayIntercept() = 0;
    
    /**
    * @brief    Set the intercept color space for DeviceRGB. This is the color
    *           space that will be used for conversion if the color manager is
    *           asked to convert from or to DeviceRGB.
    * @param    space The desired DeviceRGB intercept colorspace.
    */
    virtual void setDeviceRGBIntercept(const IDOMColorSpacePtr &space) = 0;

    /**
    * @brief    Get the intercept color space for DeviceRGB. This is the color
    *           space that will be used for conversion if the color manager is
    *           asked to convert from or to DeviceRGB.
    *           The default DeviceRGB intercept is sRGB.
    * @return   @b IDOMColorSpacePtr A smart pointer to the intercept color space.
    */
    virtual IDOMColorSpacePtr getDeviceRGBIntercept() = 0;

    /**
    * @brief    Set the intercept color space for DeviceCMYK. This is the color
    *           space that will be used for conversion if the color manager is
    *           asked to convert from or to DeviceCMYK.
    * @param    space The desired DeviceCMYK intercept color space.
    */
    virtual void setDeviceCMYKIntercept(const IDOMColorSpacePtr &space) = 0;

    /**
    * @brief    Get the intercept color space for DeviceCMYK. This is the color
    *           space that will be used for conversion if the color manager is
    *           asked to convert from or to DeviceCMYK.
    *           The default DeviceCMYK intercept is a CMYK SWOP ICCBased color space.
    * @return   @b IDOMColorSpacePtr A smart pointer to the intercept color space.
    */
    virtual IDOMColorSpacePtr getDeviceCMYKIntercept() = 0;
    
    /**
    * @brief    Gets the intercept associated with the given space if any,
    *           or return the original space if not. It is safe to pass any
    *           color space to this member.
    * @param    space The color space to find the intercept for.
    * @return   @b IDOMColorSpacePtr Returns the intercepted space, if applicable.
    *           Otherwise the original space is returned.
    */
    virtual IDOMColorSpacePtr interceptSpace(const IDOMColorSpacePtr &space) = 0;
    
    /**
    * @brief    Get a profile for the given space, if possible.
    *
    *           Convenience.
    *
    *           For ICCBased color spaces, this simply returns the profile.
    *           For DeviceGray, DeviceRGB and DeviceCMYK color spaces, this gets
    *           the profile for the intercept color space associated with the
    *           respective intercept. For sRGB, scRGB and sGray, this returns
    *           the built-in profiles.
    *
    *           For all other colour space types, NULL will be returned.
    */
    virtual IDOMICCProfilePtr getProfileForSpace(const IDOMColorSpacePtr &space) = 0;
    
    /**
    * @brief    Get an equivalent PostScript CSA for an ICC based colour space. This
    *           will be a CIEBased color space in the form of PostScript code.
    * @param    csaMemory A reference to a simple buffer to store the generated csa.
    * @param    space The ICC Based color space to process.
    * @param    intent The intent for which the CSA should be generated.
    */
    virtual void getPostScriptCSAForICCBasedSpace(CEDLSimpleBuffer &csaMemory, const IDOMColorSpaceICCBasedPtr &space, eRenderingIntent intent) = 0;
    
    /**
    * @brief    Get any PostScript CSA present in an ICC Based color space. These
    *           are found under the 'ps2s' tag in an ICC profile, and are optional.
    * @param    csaMemory A reference to a simple buffer to store the given csa.
    * @param    space The ICC Based color space to process.
    * @return   @b bool True if an embedded CSA was found
    */
    virtual bool getEmbeddedPSCSAForICCBasedSpace(CEDLSimpleBuffer &csaMemory, const IDOMColorSpaceICCBasedPtr &space) = 0;

};
#define edlobj2IColorManager(src) edl_cast((IColorManager *)NULL, src)

_END_EDL_NAMESPACE

#endif

