/*
* Copyright (C) 2007-2019 Global Graphics Software Ltd. All rights reserved.
*/

/**
* @file     idomcatalog.h
* @ingroup  IDom
* @brief    IDOMCatalog Interface
*
* CDOM Catalog provides association of addressable CDOM Nodes with CDOM Node IDs.
* Provides address space for the CDOMs in the context of a single EDL job.
*/

#ifndef __EDLIDOMCATALOG_H__
#define __EDLIDOMCATALOG_H__

#include <edl/edltypes.h>
#include <edl/edlstring.h>
#include <edl/iedlobject.h>
#include <edl/idomid.h>
#include <edl/idomnode.h>

_BEGIN_EDL_DOM_NAMESPACE

/**
* @class    IDOMCatalog
* @ingroup  IDom
* @brief    IDOMCatalog interface
* The IDOMCatalog serves as a catalog for addressable DOM nodes, where a DOM node ID is 
* used as the address of the node.
*/
class IDOMCatalog : public IEDLObject
{
#define IDOMCatalogClassID 0xA4109342, 0x680849b3, 0xA42D5AF3, 0x8D59ADAF
public:
    /**
    * @brief    Retrieves the class id of IDOMCatalog.
    * @return   @b CClassID Class id of the catalog.
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMCatalogClassID);
        return id;
    }
    /**
    * @brief    Creates a new DOM id from a URI string.
    * @param    uri A string uniquely addressing a resource.
    * @return   @b DOMid. The newly created DOM id. The newly allocated ID will be unique across the
    *           the current process.
    */
    virtual DOMid createNewDOMid(const EDLSysString &uri) = 0;

    /**
    * @brief    Creates a new DOM id based on an integer identifier
    * @param    id1 A numeric identifier.
    * @param    id2 A second numeric identifier (optional.  Presently not used, and set to zero.)
    * @return   @b DOMid. The newly created DOM id. The newly allocated ID will be unique across the
    *           the current process.
    */
    virtual DOMid createNewDOMid(uint32 id1, uint32 id2 = 0) = 0;

    /**
    * @brief    Registers the IEDLObject specified by ptrObject in the catalog, under the ID specified by id.
    * @param    id The ID to register the object under.
    * @param    ptrObject A smart pointer to the IEDLObject to register.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool registerObject(DOMid id, const IEDLObjectPtr &ptrObject) = 0;

    /**
    * @brief    Registers the number pait in the catalog, under the ID specified by id.
    * @param    id The ID to register the object under.
    * @param    id1 A numeric identifier.
    * @param    id2 A second numeric identifier.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool registerNumbers(DOMid id, uint32 id1, uint32 id2) = 0;
    
    /**
    * @brief    Unregisters the IEDLObject currently registered under the specified ID in the catalog.
    * @param    id The DOM id of the object to unregister.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool unregisterObject(DOMid id) = 0;

    /**
    * @brief    Retrieves an IEDLObject by using its DOM id
    * @param    id The DOM ID of the IEDLObject to retrieve.
    * @param    ptrObject A Smart pointer to receive a reference to the retrieved IEDLObject.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool getObject(DOMid id, IEDLObjectPtr &ptrObject) = 0;

    /**
    * @brief    Retrieves the URI of a registered object by using its DOM ID.
    * @param    id DOM ID of the resource
    * @param    uri Pointer to receive the URI for the resource
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool getURI(DOMid id, EDLSysString &uri) = 0;

    /**
    * @brief    Retrieve the DOM ID of a registered resource by using its URI,
    *           optionally creating it
    * @param    uri URI of the resource whose DOM ID is required.
    * @param    createIfNotExist If set to true, create a new DOM ID if it doesn't already exist 
    * @return   @b DOMid The DOM ID of the resource.
    */
    virtual DOMid getIdByURI(const EDLSysString &uri, bool createIfNotExist = false) = 0;

    /**
    * @brief    Retrieve the DOM ID of a registered resource by using the integer identifier(s)
    *           used to create the DOM ID, optionally creating it
    * @param    id1 A numeric identifier.
    * @param    id2 A second numeric identifier (optional. Presently not used, and set to zero.)
	* @param    createIfNotExist If set to true, create a new DOM ID if it doesn't already exist
    * @return   @b DOMid The DOM id of the identified resource. If a new id has been created
    *           it will be unique across the process.
    */
    virtual DOMid getIdByNumbers(uint32 id1, uint32 id2 = 0, bool createIfNotExist = false) = 0;

    /**
    * @brief    Retrieve the DOM ID of a registered resource by its index in the internal vector.
    * @param    index The registered resource index
    * @return   @b DOMid Returns the DOM ID of the resource.
    */
    virtual DOMid getIdByIndex(uint32 index) const = 0;

    /**
    * @brief    Retrieve count of registered resource objects.
    * @return   @b uint32 Returns the count of catalog entries.
    */
    virtual uint32 getCount() const = 0;
};

DECL_SMART_PTR(IDOMCatalog);
#define edlobj2IDOMCatalog(src) edl_cast((IDOMCatalog *)NULL, src) 

_END_EDL_DOM_NAMESPACE

#endif /* __EDLIDOMCATALOG_H__ */
