/*
* Copyright (C) 2007-2019 Global Graphics Software Ltd. All rights reserved.
*/

/**
* @file     idomcolor.h
* @ingroup  IDom
* @brief    IDomColor Interface
*/

#ifndef __EDLIDOMCOLOR_H__
#define __EDLIDOMCOLOR_H__

#include <edl/edlrenderingintent.h>
#include <edl/edlblackpointcompensation.h>
#include <edl/edltypes.h>
#include <edl/iedlobject.h>
#include <edl/idomcolorspace.h>
#include <edl/idomresources.h>
#include <edl/idomhashable.h>
#include <edl/edlvector.h>

_BEGIN_EDL_DOM_NAMESPACE

class IDOMColor;
DECL_SMART_PTR(IDOMColor);


/**
* @class    IDOMColor
* @ingroup  IDom
* @brief    Holds a single color value. The color values themselves are held
*           as floating point values for all color spaces. For some
*           spaces (such as indexed color spaces) the values will be
*           integral, but still stored as floats. 
*
*           The color values themselves cannot be interpreted without reference
*           to the referenced color space.
*
*            IDOMColor nodes are used to specify the colors of lines
*            and strokes, not of images. Hence if an ArcSegment is drawn with a blue line, the blue is
*            specified by an IDOMColor node.
*/
class IDOMColor : public IEDLObject, public IDOMHashable
{
#define IDOMColorClassID 0x4a33f930, 0xc5d64967, 0xaa600a12, 0x78eb3212
public:

    virtual ~IDOMColor ()
    {}

    /**
    * @brief    Simplified color creation routine.
    *           Throws an IEDLError on failure.
    * @param    pFactory The EDL Class factory to use.
    * @param    space The color space to use.
    * @param    alpha The alpha to use.
    * @param    ... The color components to use, as floats or doubles. The number of
    *           arguments MUST match the number of components expected of the color space.
    * @return   @b IDOMColorPtr The new color.
    */
    static EDL_API IDOMColorPtr create(IEDLClassFactory *pFactory, const IDOMColorSpacePtr &space, float alpha, ...);

    /**
    * @brief    Simplified color creation routine.
    *           Throws an IEDLError on failure.
    * @param    pFactory The EDL Class factory to use.
    * @param    space The color space to use.
    * @param    alpha The alpha to use.
    * @param    components A vector of color components to use, as floats. The number of
    *           components MUST match the number of components expected of the color space.
    * @return   @b IDOMColorPtr The new color.
    */
    static EDL_API IDOMColorPtr createFromVect(IEDLClassFactory *pFactory, const IDOMColorSpacePtr &space, float alpha, const CEDLVector<float> &components);

    /**
    * @brief    Simplified color creation routine.
    *           Throws an IEDLError on failure.
    * @param    pFactory The EDL Class factory to use.
    * @param    space The color space to use.
    * @param    alpha The alpha to use.
    * @param    components An array of color components to use, as floats or doubles. The number of
    *           floats MUST match the number of components expected of the color space.
    * @return   @b IDOMColorPtr The new color.
    */
    static EDL_API IDOMColorPtr createFromArray(IEDLClassFactory *pFactory, const IDOMColorSpacePtr &space, float alpha, const float *components);

    /**
    * @brief    Retrieves the class id of IDOMColor.
    * @return   @b CClassID Returns the class id of the element.
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMColorClassID);
        return id;
    };

    /**
    * @brief    Retrieves the alpha channel value.
    * The alpha channel value specifies the transparency of the color.
    *
    * Alpha values are between zero and 1.  A value of 0 means that the color does not
    * have any coverage information and is fully transparent; a value of 1 means that the
    * color is fully opaque because the geometry completely overlapped the pixel.
    *
    * @return   @b float The alpha channel value
    */
    virtual float getAlpha() const = 0;

    /**
    * @brief    Sets the alpha channel value.
    * @param    a Desired new alpha channel value.
    * @return   @ bool True on success, false if the call fails.
    */
    virtual bool setAlpha(float a) = 0;

    /**
    * @brief    Retrieves the color space.
    * @details  A device color space simply describes the range of colors, or gamut, that a camera can see, a
    *           printer can print, or a monitor can display. Editing color spaces, on the other hand, such as Adobe
    *           RGB or sRGB, are device-independent. They also determine a color range you can work in. Their
    *           design allows you to edit images in a controlled, consistent manner. A device color space is tied
    *           to the device it describes. An editing space, on the other hand, is gray-balanced colors with
    *           equal amounts of red, green, and blue appear neutral. Editing spaces also are perceptually uniform;
    *           that is, changes to lightness, hue, or saturation are applied equally to all the colors in the
    *           image.
    * @param    colorSpace Reference parameter to receive a smart pointer to the color space.
    * @return   @b bool True on success
    */
    virtual bool getColorSpace (IDOMColorSpacePtr &colorSpace) = 0;

    /**
    * @brief    Set the color space.
    * @details  If the new color space has has the same number of
    *           components as the old, the values will be retained and clipped
    *           to the range of the new color space, if applicable. Otherwise
    *           if setDefaultColor is true, a default value will be set based
    *           on the minima of the range of the new color space.
    * @param    colorSpace Smart pointer to the desired new color space
    * @param    setDefaultColor If true, and the new space has a different number of
    *           components than the previous space, a default color will be set. 
    * @return   @b bool True on success
    */
    virtual bool setColorSpace(const IDOMColorSpacePtr &colorSpace, bool setDefaultColor = true) = 0;

    /** 
    * @brief    Set the color space, converting color values from the previous color space to the new.
    * 
    * The rendering intent refers to the way the CMM (Color Management Module) will handle out-of-gamut 
    * colors during a conversion from one color space to another. The ICC specification includes four 
    * different rendering intents: perceptual, relative colorimetric, absolute colorimetric, and saturation.
    * @see       eRenderingIntent
    * @param    colorSpace Smart pointer to the desired new color space.
    * @param    intent The rendering intent to be used for conversion.
    * The rendering intent refers to the way the CMM (Color Management Module) will handle out-of-gamut 
    * colors during a conversion from one color space to another. The ICC specification includes four 
    * different rendering intents: perceptual, relative colorimetric, absolute colorimetric, and saturation.
    * @param    bpc Black point compensation handling.
    * Black point compensation refers to the way the CMM (Color Managemenr Module) will handle shadows
    * in color conversion where the black point of the input and output color spaces differ.
    * If in doubt, use eBPCDefault.
    * @param    pFactory The EDL class factory.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setColorSpace(const IDOMColorSpacePtr &colorSpace, eRenderingIntent intent, eBlackPointCompensation bpc, IEDLClassFactory *pFactory) = 0;

    /**
    * @brief    Retrieves the value of a component.
    * @param    component Index of the component.
    * @param    value Reference to receive the value.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool getComponentValue(uint32 component, float &value) = 0;

    /**
    * @brief    Sets a component value. 
    *           The value will be clipped to the range of the color space, if known.
    * @param    component Index of the component.
    * @param    value New value of the component.
    * @return   @b bool Ttrue on success, false if the call fails.
    */
    virtual bool setComponentValue(uint32 component, float value) = 0;

};
#define edlobj2IDOMColor(src) edl_cast((IDOMColor *)NULL, src) 


_END_EDL_DOM_NAMESPACE

#endif /* __EDLIDOMCOLOR_H__ */
