/*
* Copyright (C) 2007-2019 Global Graphics Software Ltd. All rights reserved.
*/

/**
* @file     idomcolorspace.h
* @ingroup  IDom
* @brief   idomcolorspace Interface
*/

#ifndef EDLIDOMCOLORSPACE_H
#define EDLIDOMCOLORSPACE_H

#include <stdarg.h>
#include <edl/edlrenderingintent.h>
#include <edl/edlblackpointcompensation.h>
#include <edl/edlmath.h>
#include <edl/edltypes.h>
#include <edl/iedlobject.h>
#include <edl/idomresources.h>
#include <edl/idomfunction.h>
#include <edl/edlvector.h>
#include <edl/idomhashable.h>

_BEGIN_EDL_DOM_NAMESPACE

class IDOMColorSpace;
DECL_SMART_PTR(IDOMColorSpace);


/**
* @class    IDOMColorSpace
* @ingroup  IDom
* @brief    IDOMColorSpace interface
* 
* A color space describes a range of colors appropriate to a given context.
* A device color space describes the range of colors that a camera can see, a printer can print,
* or a monitor can display, and is tied to the device it describes. 
* Editing color spaces, on the other hand, such as Adobe RGB or
* sRGB, are device-independent, determining a color range you can work in. They are designed to 
* be gray balanced, such that colors with equal amounts of red, green, and blue appear neutral, and 
* allow you to edit images in a controlled, consistent manner. Editing spaces also are perceptually uniform;
* that is, changes to lightness, hue, or saturation are applied equally to all the colors in the image.
*
* Instances of these objects may throw IEDLError exceptions on failure.
*/
class IDOMColorSpace : public IDOMHashable, public IEDLObject
{
public:
    virtual ~IDOMColorSpace() {}

    /**
    * @enum     eColorSpaceType
    * @ingroup  IDom
    * @brief    Color spaces type enumeration
    */
    typedef enum {
        eDeviceRGB,     //!< Device RGB color space
        eDeviceGray,    //!< Device Gray color space
        eDeviceCMYK,    //!< Device CMYK color space
        esRGB,          //!< XPS sRGB color space with input range scaled to between 0 and 1
        esGray,         //!< Gray with sRGB gamma color space
        escRGB,         //!< XPS scRGB color space
        eICCBased,      //!< Color space with an ICC profile
        eIndexed,       //!< A color space analogous to the PostScript language/PDF Indexed color space, representing a mapping from an integral component value to components of an underlying color space. See section 4.8.4 of the PostScript Language Reference, 3rd Edition, and section 4.5.5 of the PDF Reference, version 1.7
        eDeviceN,       //!< A color space analogous to the PostScript language/PDF DeviceN/Separation color spaces as described in section 4.5.5 of the PDF 1.7 Reference Manual
        eLAB,           //!< Lab color space as described in section 4.5.4 of the PDF 1.7 Reference Manual
        eDeviceCMY,     //!< Device CMY color space to support PCL input. Internal use only.
        eNumColorSpaceTypes = 11
    } eColorSpaceType;
    
    /**
    * @brief    Retrieves the color space type
    * @see      eColorSpaceType
    * @return   @b eColorSpaceType.  Retrieves the color space type
    */
    virtual eColorSpaceType getColorSpaceType() = 0;
    
    /**
    * @brief    Retrieves the number of components that are in colors in this color space.
    * @return   @b int The number of components for colors in this color space.
    */
    virtual int getNumComponents() = 0;
    
    /**
    * @brief    Checks if this color space has the same range for all its components.
    *           Some color spaces (such as Lab and ICC) may not have the same range for 
    *           all its components. If a color space does have the same range for all its
    *           components, you only need to call getComponentRange() once to find the
    *           range for all components.
    * @return   @b bool  Returns true if all the components in the color space have the same range, false if they do not.
    */
    virtual bool getComponentsHaveSameRange() = 0;
    
    /**
    * @brief    Retrieves the expected range of component values for a given channel,
    *           if applicable.
    * @param    component The index of the component whose range is being queried
    * @param    low Reference parameter to receive the low bound of the component's value range.
    * @param    high Reference parameter to receive the high bound of the component's value range.
    * @return   @b bool Returns true if the given component has a range, false otherwise. 
    */
    virtual bool getComponentRange(int component, float &low, float &high) = 0;
    
    /**
    * @brief   Retrieves the default rendering intent for this color space.
    * @return  @b eRenderingIntent The default rendering intent.
    */
    virtual eRenderingIntent getDefaultRenderingIntent() = 0;
    
    /**
    * @brief  Determines if the given color space is equivalent to this color space.
    * @param  colorSpace The color space to compare with.
    * @return @b bool True if equal, false otherwise.
    */
    virtual bool equals(const IDOMColorSpacePtr &colorSpace) = 0;
    
    /**
    * @brief  Determines if the given color space is very similar to this color space
    *         for the given rendering intent. Color spaces are tested via selective
    *         sampling and are considered similar if the samples show the same results
    *         to a 10 bit accuracy. This can be an expensive operation for complicated
    *         color spaces.
    * @param  colorSpace The color space to compare with.
    * @param  intent The rendering intent to use.
    * @param  bpc The black point compensation treatment. If in double, use eBPCDefault.
    * @return @b bool True if similar, false otherwise.
    */
    virtual bool similar(const IDOMColorSpacePtr &colorSpace, eRenderingIntent intent, eBlackPointCompensation bpc) = 0;
};
#define edlobj2IDOMColorSpace(src) edl_cast((IDOMColorSpace *)NULL, src) 


class IDOMColorSpacesRGB;
DECL_SMART_PTR(IDOMColorSpacesRGB);

/**
* @class    IDOMColorSpacesRGB
* @ingroup  IDom
* @brief    Represents the RGB color space.
*/
class IDOMColorSpacesRGB : public IDOMColorSpace
{
#define IDOMColorSpacesRGBClassID 0xbc9883fc, 0x15d34522, 0xa339ae75, 0xba783f1a
public:
    virtual ~IDOMColorSpacesRGB() {}

    /**
     * @brief    Create an sRGB Color Space.
     *           Throws an IEDLError on failure.
     * @param    pFactory The EDL Class factory to use.
     * @return   @b IDOMColorSpacesRGBPtr The new color space.
     */
    static EDL_API IDOMColorSpacesRGBPtr create(IEDLClassFactory *pFactory);

    /**
    * @brief    Retrieves the class id of IDOMColorSpacesRGB
    * @return   @b CClassID Class id of the element
    */
    static const CClassID &classID()
    {
       static CClassID id(IDOMColorSpacesRGBClassID);
       return id;
    };
};
#define edlobj2IDOMColorSpacesRGB(src) edl_cast((IDOMColorSpacesRGB *)NULL, src) 


class IDOMColorSpacesGray;
DECL_SMART_PTR(IDOMColorSpacesGray);

/**
* @class    IDOMColorSpacesGray
* @ingroup  IDom
* @brief    Represents a gray color space using the sRGB gamma and WhitePoint.
* 
* Gray colors for vector graphics can be specified as sRGB or scRGB colors with the red, green and
* blue components set to the same value. Gray colors for raster images can be specified without
* using any image format.
*/
class IDOMColorSpacesGray : public IDOMColorSpace
{
#define IDOMColorSpacesGrayClassID 0x21164cec, 0x039345e2, 0xbd7d2a20, 0x91116c09
public:
    virtual ~IDOMColorSpacesGray() {}

    /**
     * @brief    Create an sGray Color Space.
     *           Throws an IEDLError on failure.
     * @param    pFactory The EDL Class factory to use.
     * @return   @b IDOMColorSpacesGrayPtr The new color space.
     */
    static EDL_API IDOMColorSpacesGrayPtr create(IEDLClassFactory *pFactory);

    /**
    * @brief    Retrieves class id of IDOMColorSpacesGray
    * @return   @b CClassID Class id of the element
    */
    static const CClassID &classID()
    {
       static CClassID id(IDOMColorSpacesGrayClassID);
       return id;
    };

};
#define edlobj2IDOMColorSpacesGray(src) edl_cast((IDOMColorSpacesGray *)NULL, src) 


class IDOMColorSpacescRGB;
DECL_SMART_PTR(IDOMColorSpacescRGB);

/**
* @class    IDOMColorSpacescRGB
* @ingroup  IDom
* @brief    Represents the scRGB color space.
* 
* The scRGB color space allows you to specify a color using the full scRGB color space, which is
* much larger than the sRGB color space and can represent the entire range of colors perceivable by
* the human eye.
*/
class IDOMColorSpacescRGB : public IDOMColorSpace
{
#define IDOMColorSpacescRGBClassID 0xdf8fb57c, 0xd20e11db, 0x9fdeecb6, 0x55d89593
public:
    virtual ~IDOMColorSpacescRGB() {}

    /**
     * @brief    Create an scRGB Color Space. 
     *           Throws an IEDLError on failure.
     * @param    pFactory The EDL Class factory to use
     * @return   @b IDOMColorSpacescRGBPtr The new color space
     */
    static EDL_API IDOMColorSpacescRGBPtr create(IEDLClassFactory *pFactory);

    /**
    * @brief    Retrieves the class id of IDOMColorSpacescRGB
    * @return   @b CClassID Class id of the element
    */
    static const CClassID &classID()
    {
       static CClassID id(IDOMColorSpacescRGBClassID);
       return id;
    };

};
#define edlobj2IDOMColorSpacescRGB(src) edl_cast((IDOMColorSpacescRGB *)NULL, src) 


class IDOMColorSpaceDeviceRGB;
DECL_SMART_PTR(IDOMColorSpaceDeviceRGB);

/**
* @class    IDOMColorSpaceDeviceRGB
* @ingroup  IDom
* @brief    IDOMColorSpaceDeviceRGB interface
*/
class IDOMColorSpaceDeviceRGB : public IDOMColorSpace
{
#define IDOMColorSpaceDeviceRGBClassID 0x5A615EA5, 0xFBF2440D, 0xAE04C3BD, 0xF0758C02
public:
    virtual ~IDOMColorSpaceDeviceRGB() {}

    /**
     * @brief    Create a DeviceGray Color Space.
     *           Throws an IEDLError on failure.
     * @param    pFactory The EDL Class factory to use.
     * @return   @b IDOMColorSpaceDeviceRGB The new color space.
     */
    static EDL_API IDOMColorSpaceDeviceRGBPtr create(IEDLClassFactory *pFactory);

    /**
    * @brief    Retrieves class id of IDOMColorSpaceDeviceRGB
    * @return   @b CClassID Class id of the element
    */
    static const CClassID &classID()
    {
       static CClassID id(IDOMColorSpaceDeviceRGBClassID);
       return id;
    };

};
#define edlobj2IDOMColorSpaceDeviceRGB(src) edl_cast((IDOMColorSpaceDeviceRGB *)NULL, src) 


class IDOMColorSpaceDeviceGray;
DECL_SMART_PTR(IDOMColorSpaceDeviceGray);

/**
* @class    IDOMColorSpaceDeviceGray
* @ingroup  IDom
* @brief    IDOMColorSpaceDeviceGray interface
*/
class IDOMColorSpaceDeviceGray : public IDOMColorSpace
{
#define IDOMColorSpaceDeviceGrayClassID 0x8FB5ECD1, 0x18C2447E, 0x91ABB50F, 0xE4E70A0D
public:
    virtual ~IDOMColorSpaceDeviceGray() {}

    /**
     * @brief    Create a DeviceGray Color Space.
     *           Throws an IEDLError on failure.
     * @param    pFactory The EDL Class factory to use.
     * @return   @b IDOMColorSpaceDeviceGrayPtr The new color space.
     */
    static EDL_API IDOMColorSpaceDeviceGrayPtr create(IEDLClassFactory *pFactory);

    /**
    * @brief    Retrieves class id of IDOMColorSpaceDeviceGray
    * @return   @b CClassID Class id of the element
    */
    static const CClassID &classID()
    {
       static CClassID id(IDOMColorSpaceDeviceGrayClassID);
       return id;
    };

};
#define edlobj2IDOMColorSpaceDeviceGray(src) edl_cast((IDOMColorSpaceDeviceGray *)NULL, src) 


class IDOMColorSpaceDeviceCMYK;
DECL_SMART_PTR(IDOMColorSpaceDeviceCMYK);

/**
* @class    IDOMColorSpaceDeviceCMYK
* @ingroup  IDom
* @brief    Represents the default CMYK color space.
*/
class IDOMColorSpaceDeviceCMYK : public IDOMColorSpace
{
#define IDOMColorSpaceDeviceCMYKClassID 0x6F31A7BC, 0x4B994A3C, 0x9FFE836F, 0xE5C07FBB
public:
    virtual ~IDOMColorSpaceDeviceCMYK() {}

    /**
     * @brief    Create a DeviceCMYK Color Space.
     *           Throws an IEDLError on failure.
     * @param    pFactory The EDL Class factory to use.
     * @return   @b IDOMColorSpaceDeviceCMYKPtr The new color space.
     */
    static EDL_API IDOMColorSpaceDeviceCMYKPtr create(IEDLClassFactory *pFactory);

    /**
    * @brief    Retrieves class id of IDOMColorSpaceDeviceCMYK
    * @return   @b CClassID Class id of the element
    */
    static const CClassID &classID()
    {
       static CClassID id(IDOMColorSpaceDeviceCMYKClassID);
       return id;
    };

};
#define edlobj2IDOMColorSpaceDeviceCMYK(src) edl_cast((IDOMColorSpaceDeviceCMYK *)NULL, src) 


class IDOMColorSpaceDeviceCMY;
DECL_SMART_PTR(IDOMColorSpaceDeviceCMY);

/**
* @class    IDOMColorSpaceDeviceCMY
* @ingroup  IDom
* @brief    Represents the default CMY color space.
*           NOTE: Currently for internal use only; Do not use this color space in
*           your own applications.
*/
class IDOMColorSpaceDeviceCMY : public IDOMColorSpace
{
#define IDOMColorSpaceDeviceCMYClassID 0x27849B58, 0x9E584782, 0x876CE042, 0x5A91C168

public:
    virtual ~IDOMColorSpaceDeviceCMY() {}

    /**
     * @brief    Create a DeviceCMY Color Space.
     *           Throws an IEDLError on failure.
     * @param    pFactory The EDL Class factory to use.
     * @return   @b IDOMColorSpaceDeviceCMYPtr The new color space.
     */
    static EDL_API IDOMColorSpaceDeviceCMYPtr create(IEDLClassFactory *pFactory);

    /**
    * @brief    Retrieves class id of IDOMColorSpaceDeviceCMY
    * @return   @b CClassID Class id of the element
    */
    static const CClassID &classID()
    {
       static CClassID id(IDOMColorSpaceDeviceCMYClassID);
       return id;
    };

};
#define edlobj2IDOMColorSpaceDeviceCMY(src) edl_cast((IDOMColorSpaceDeviceCMY *)NULL, src) 



class IDOMColorSpaceICCBased;
DECL_SMART_PTR(IDOMColorSpaceICCBased);

/**
* @class    IDOMColorSpaceICCBased
* @ingroup  IDom
* @brief    Represents a color space described by an ICC profile.
*/
class IDOMColorSpaceICCBased : public IDOMColorSpace
{
#define IDOMColorSpaceICCBasedClassID 0xd67937e4, 0x7a1d4f5f, 0xb950bc30, 0x38474418
public:
    virtual ~IDOMColorSpaceICCBased() {}

    /**
    * @brief    Creation function for an IDOMColorspaceICCBased color space.
    *           Throws an IEDLError exception on failure.
    * @param    pFactory The class factory.
    * @param    profile The profile to use
    * @param    alternate The alternate color space, if any.
    * @return   @b IDOMColorSpaceICCBasedPtr The new color space.
    */
    static EDL_API IDOMColorSpaceICCBasedPtr create(IEDLClassFactory *pFactory, const IDOMICCProfilePtr &profile, const IDOMColorSpacePtr &alternate = IDOMColorSpacePtr(NULL));

    /**
    * @class    Data
    * @ingroup  IDOMColorSpaceICCBased
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        Data(IDOMICCProfilePtr &pICCProfile):
             m_pICCProfile(pICCProfile), m_AlternateSpace(0)
        {}
        Data(IDOMICCProfilePtr &pICCProfile, IDOMColorSpacePtr &pAlt):
             m_pICCProfile(pICCProfile), m_AlternateSpace(pAlt)
        {}
        IDOMICCProfilePtr m_pICCProfile;
        IDOMColorSpacePtr m_AlternateSpace;  /* PDF alternate color space */
    };

    /**
    * @brief    Retrieves class id of IDOMColorSpaceICCBased
    * @return   @b CClassID Class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMColorSpaceICCBasedClassID);
        return id;
    };

    /**
    * @brief    Retrieve the ICC profile for this color space.
    *           As a profile is required and should have been available
    *           at initialisation time, an exception of type IEDLError
    *           is thrown on failure.
    * @return   @b IDOMICCProfilePtr The profile.
    */
    virtual IDOMICCProfilePtr getICCProfile() = 0;

    /**
    * @brief    Set the ICC profile for the color space.
    * @param    profile A reference to the new ICC profile.
    */
    virtual void setICCProfile(const IDOMICCProfilePtr &profile) = 0;


    /**
    * @brief    Gets the PDF alternate color space for this ICC-based color space. Useful for PDF input filter context
    * @return   @b IDOMColorSpacePtr The alternate space, or NULL if no present.
    */
    virtual IDOMColorSpacePtr getAlternateColorSpace() = 0;
};

#define edlobj2IDOMColorSpaceICCBased(src) edl_cast((IDOMColorSpaceICCBased *)NULL, src) 


class IDOMColorSpaceIndexed;
DECL_SMART_PTR(IDOMColorSpaceIndexed);

/**
* @class    IDOMColorSpaceIndexed
* @ingroup  IDom
* @brief    This color space is analogous to the PostScript/PDF Indexed color space.
* @details  Instances of this type represent a mapping from an integral component value
*           to components of an underlying color space using a Function.
*           - Indexed color spaces cannot be used as the underlying space.
*           - See section 4.8.4 of the PostScript Language Reference, 3rd Edition,
*           and section 4.5.5 of the PDF Reference, version 1.7
*/
class IDOMColorSpaceIndexed : public IDOMColorSpace
{
#define IDOMColorSpaceIndexedClassID 0x0ef4de20, 0x5f914784, 0x9b607760, 0x6ae02137
public:
    virtual ~IDOMColorSpaceIndexed() {}

    /**
    * @brief    Create an Indexed Color Space.
    *           Throws an IEDLError on failure.
    * @param    pFactory The EDL Class factory to use.
    * @param    underlying The underlying color space.
    * @param    mappingFunction A function that maps indexes to colors in the
    *           underlying color space.
    * @return   @b IDOMColorSpaceIndexedPtr The new color space.
    */
    static EDL_API IDOMColorSpaceIndexedPtr create(IEDLClassFactory *pFactory, const IDOMColorSpacePtr &underlying, const IDOMFunctionPtr &mappingFunction);

    /**
    * @brief    Create an Indexed Color Space from an 8bpc table.
    *           Throws an IEDLError on failure.
    * @param    pFactory The EDL Class factory to use.
    * @param    underlying The underlying color space.
    * @param    table An 8-bit lookup table consisting of a color values for each index.
    *           For example, for an RGB underlying color space, the values are packed as
    *           [ <R for index 0>, <G for index 0>, <B for index 0>, <R for index 1>, <G for index 1> ... ]
    * @return   @b IDOMColorSpaceIndexedPtr The new color space.
    */
    static EDL_API IDOMColorSpaceIndexedPtr create(IEDLClassFactory *pFactory, const IDOMColorSpacePtr &underlying, const CEDLVector<uint8> &table);

    /**
    * @class    Data
    * @ingroup  IDOMColorSpaceIndexed
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        Data() : pUnderlyingSpace(NULL), pMappingFunction(NULL) {} 
        Data(IDOMColorSpacePtr &pUnderlying, IDOMFunctionPtr &pMapping):
             pUnderlyingSpace(pUnderlying), pMappingFunction(pMapping)
        {}
        IDOMColorSpacePtr pUnderlyingSpace;
        IDOMFunctionPtr   pMappingFunction;
    };

    /**
    * @brief    Retrieves class id of IDOMColorSpaceICCBased
    * @return   @b CClassID Class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMColorSpaceIndexedClassID);
        return id;
    };

    /**
    * @brief    Get the underlying color space for this color space, which must
    *           be present. Throws an exception of type IEDLError on failure.
    * @return   @b IDOMColorSpacePtr The underlying color space.
    */
    virtual IDOMColorSpacePtr getUnderlyingColorSpace() = 0;

    /**
    * @brief    Get the function that maps an index value to component(s) in
    *           the underlying color space. Must always be present. An exception
    *           of type IEDLError is thrown if it is missing.
    * @return   @b IDOMFunctionPtr The mapping function.
    */
    virtual IDOMFunctionPtr getMappingFunction() = 0;

};
#define edlobj2IDOMColorSpaceIndexed(src) edl_cast((IDOMColorSpaceIndexed *)NULL, src) 


/**
* @class    IDOMDeviceNColorant
* @ingroup  IDOM
* @brief    This class enables the specification of colorant information for
*           PDF style NChannel variants of DeviceN color spaces.
*/
class IDOMDeviceNColorant : public IDOMHashable, public IEDLObject
{
#define IDOMDeviceNColorantClassID 0x78fad1cd, 0x07cd43c8, 0x96f7e4cf, 0xd1f98d63
public:
    virtual ~IDOMDeviceNColorant() {}

    /**
    * @class    Data
    * @ingroup  IDOMDeviceNColorant
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        Data() : hasSolidity(false), solidity(0.0),
                 pDotGainFunction(NULL), pAlternateSpace(NULL),
                 pTintTransform(NULL)
        {}
        EDLSysString      name;
        bool              hasSolidity;
        float             solidity;
        IDOMFunctionPtr   pDotGainFunction;
        IDOMColorSpacePtr pAlternateSpace;
        IDOMFunctionPtr   pTintTransform;
    };

    /**
    * @brief    Retrieves class id fo IDOMDeviceNColorant
    * @return   @b CClassID Class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMDeviceNColorantClassID);
        return id;
    };

    /**
    * @brief    Get the name of the colorant, which must be present.
    *           An exception of type IEDLError will be thrown if it is missing.
    * @return   @b EDLSysString The name of the colorant.
    */
    virtual const EDLSysString& getName() = 0;

    /**
    * @brief    Get the solidity of the colorant
    * @param    solidity Reference to receive the solidity
    * @return   @b bool Returns true on success, false if the colorant has no solidity specified.
    */
    virtual bool getSolidity(float &solidity) = 0;

    /**
    * @brief    Get the function that maps the colorant to component(s)
    *           of the alternate color space. Optional.
    * @return   @b IDOMFunctionPtr The function if present, NULL otherwise.
    */
    virtual IDOMFunctionPtr getTintTransform() = 0;

    /**
    * @brief    Get the function that describes the dot gain of the colorant
    *           on the intended output device. Optional
    * @return   @b IDOMFunctionPtr The function if present, NULL otherwise.
    */
    virtual IDOMFunctionPtr getDotGainFunction() = 0;

    /**
    * @brief    Get the alternate color space for this colorant. Optional.
    * @return   @b IDOMColorSpacePtr The alternate space if present, NULL otherwise.
    */
    virtual IDOMColorSpacePtr getAlternateColorSpace() = 0;
};

DECL_SMART_PTR(IDOMDeviceNColorant);
#define edlobj2IDOMDeviceNColorant(src) edl_cast((IDOMDeviceNColorant *) NULL, src)
typedef CEDLVector<IDOMDeviceNColorantPtr> CDeviceNColorantVect;

class IDOMColorSpaceDeviceN;
DECL_SMART_PTR(IDOMColorSpaceDeviceN);

/**
* @class    IDOMColorSpaceDeviceN
* @ingroup  IDom
* @brief This color space is analogous to the PostScript/PDF DeviceN/Separation color spaces.
*
*        See section 4.5.5 of the PDF Reference, version 1.7
*/
class IDOMColorSpaceDeviceN : public IDOMColorSpace
{
#define IDOMColorSpaceDeviceNClassID 0x30404cc4, 0xa1d940ea, 0xb396c3b3, 0xa84d4bf7
public:
    virtual ~IDOMColorSpaceDeviceN() {}

    /**
    * @brief    Create a simple DeviceN Color Space.
    *           Throws an IEDLError on failure.
    * @param    pFactory The EDL Class factory to use.
    * @param    colorants A vector of colorant names for the device inks
    * @param    alternate The alternate color space.
    * @param    tintTransform A function that maps colors in the space to
    *           the alternate color space.
    * @param    processColorSpace The PDF process color space associated with 
    *           this DeviceN color space. Optional.
    * @return   @b IDOMColorSpaceDeviceN The new color space.
    */
    static EDL_API IDOMColorSpaceDeviceNPtr create(IEDLClassFactory *pFactory, const CEDLVector<EDLSysString> &colorants,
                                                   const IDOMColorSpacePtr &alternate, const IDOMFunctionPtr &tintTransform,
                                                   const IDOMColorSpacePtr &processColorSpace = IDOMColorSpacePtr(NULL));

    /**
     * @brief    Create a DeviceN Color Space.
     *           Throws an IEDLError on failure.
     * @param    pFactory The EDL Class factory to use.
     * @param    colorants A vector of smart pointers to IDOMDeviceNColorant objects
     * @param    alternate The alternate color space.
     * @param    tintTransform A function that maps colors in the space to
     *           the alternate color space.
     * @param    processColorSpace The PDF process color space associated with
     *           this DeviceN color space. Optional.
     * @param    processComponents A vector containing a list of process components. Optional
     * @param    printingOrder A vector containing a list defining the printing order. Optional
     * @param    isNChannel Specifies if the new color space is NChannel. Optional
     * @return   @b IDOMColorSpaceDeviceN The new color space.
     */
	static EDL_API IDOMColorSpaceDeviceNPtr create(IEDLClassFactory *pFactory, const CDeviceNColorantVect &colorants,
                                                   const IDOMColorSpacePtr &alternate, const IDOMFunctionPtr &tintTransform,
                                                   const IDOMColorSpacePtr &processColorSpace = IDOMColorSpacePtr(NULL),
                                                   const CEDLVector<EDLSysString> &processComponents = CEDLVector<EDLSysString>(),
                                                   const CEDLVector<EDLSysString> &printingOrder = CEDLVector<EDLSysString>(),
                                                   bool isNChannel = false);

    class CColorantInfo
    {
      public:
        CColorantInfo() {}
        CColorantInfo(const EDLSysString &_name, uint32 numComponents, ...)
        {
            va_list argPtr;
            name = _name;
            components.reserve(numComponents);

            va_start(argPtr, numComponents);
            for (uint32 i = 0; i < numComponents; i++)
            {
                components.append((float) va_arg(argPtr, double));
            }
            va_end(argPtr);
        }
        EDLSysString      name;
        CEDLVector<float> components;
        
    };
    typedef CEDLVector<CColorantInfo> CColorantInfoVect;
    
    /**
    * @brief    Create a simple DeviceN Color Space, with automatic generation of
    *           the tint transform function.
    *           Throws an IEDLError on failure.
    * @param    pFactory The EDL Class factory to use.
    * @param    colorants A vector of CColorantInfo classes, specifying the name
    *           of each colorant, and the color components of that colorant in the alternate
    *           color space.
    * @param    alternate The alternate color space.
    * @param    processColorSpace The PDF process color space associated with
    *           this DeviceN color space. Optional.
    * @return   @b IDOMColorSpaceDeviceN The new color space.
    */
    static EDL_API IDOMColorSpaceDeviceNPtr create(IEDLClassFactory *pFactory, const CColorantInfoVect &colorants,
                                                   const IDOMColorSpacePtr &alternate,
                                                   const IDOMColorSpacePtr &processColorSpace = IDOMColorSpacePtr(NULL));

    
    /**
    * @class    Data
    * @ingroup  IDOMColorSpaceDeviceN
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
       public:
        Data() : nChannel(false) {}
        bool                     nChannel;
        IDOMColorSpacePtr        pAlternateSpace;
        IDOMFunctionPtr          pTintTransform;
        CDeviceNColorantVect     pColorants;
        IDOMColorSpacePtr        pProcessColorSpace;
        CEDLVector<EDLSysString> pProcessComponentNames;
        CEDLVector<EDLSysString> pPrintingOrder;
    };

    /**
    * @brief    Retrieves class id of IDOMColorSpaceDeviceN
    * @return   @b CClassID Class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMColorSpaceDeviceNClassID);
        return id;
    };
    
    /**
    * @brief    Get if the color space represents an NChannel color space
	* @return   @b bool True if an NChannel color space
    */
    virtual bool getIsNChannel() const = 0;

    /**
    * @brief    Get the alternate color space for this color space, which
    *           must be present. An exception of type IEDLError is thrown if
    *           it is missing.
    * @return   @b IDOMColorSpacePtr The alternate space.
    */
    virtual IDOMColorSpacePtr getAlternateColorSpace() = 0;

    /**
    * @brief    Get the function that maps components of this space to component(s)
    *           of the alternate color space. Must be present. An exception
    *           of type IEDLError is thrown if it is missing.
    * @return   @b IDOMFunctionPtr The function.
    */
    virtual IDOMFunctionPtr getTintTransform() = 0;

    /**
    * @brief    Get the colorant information for a component at the given index.
    *           An exception of type IEDLError is thrown on failure.
    * @param    component The 0 base component index of the desired colorant
    * @return   @b IDOMDeviceNColorantPtr The colorant information.
    */
    virtual IDOMDeviceNColorantPtr getColorant(uint8 component) = 0;

    /**
    * @brief    Get the Process color space associated with this DeviceN space.
    *           (See Table 4.2.2 of the PDF 1.7 Reference). Optional
    * @return   @b IDOMColorSpacePtr The process color space if present, NULL otherwise.
    */
    virtual IDOMColorSpacePtr getProcessColorSpace() = 0;

    /**
    * @brief    Get the vector of Process component names to those in the associated
    *           process color space (see Table 4.2.2 of the PDF 1.7 reference).
    *           An exception of IEDLError is thrown on failure, which should not happen.
    * @return   @b CEDLVector<EDLSysString> The component names vector.
    */
    virtual const CEDLVector<EDLSysString>& getProcessComponentNames() = 0;

    /**
    * @brief    Get the list of component names describing the printing order of
    *           the device N colorants.
    *           An exception of IEDLError is thrown on failure, which should not happen.
    * @return   @b CEDLVector<EDLSysString> The printing order component names vector.
    */
    virtual const CEDLVector<EDLSysString> &getPrintingOrder() = 0;

};
#define edlobj2IDOMColorSpaceDeviceN(src) edl_cast((IDOMColorSpaceDeviceN *)NULL, src) 


class IDOMColorSpaceLAB;
DECL_SMART_PTR(IDOMColorSpaceLAB);

/**
* @class    IDOMColorSpaceLAB
* @ingroup  IDom
* @brief This color space is as described in section 4.5.4 of the PDF 1.7 Reference Manual.
*/
class IDOMColorSpaceLAB : public IDOMColorSpace
{
#define IDOMColorSpaceLABClassID 0x0d9f95d3, 0x4f2d4882, 0xb4293528, 0xea287840
public:
    virtual ~IDOMColorSpaceLAB() {}

    /**
    * @brief    Create a LAB Color Space.
    *           Throws an IEDLError on failure.
    * @param    pFactory The EDL Class factory to use.
    * @param    whitePointX The CIE X value for the white point.
    * @param    whitePointY The CIE Y value for the white point.
    * @param    whitePointZ The CIE V value for the white point.
    * @param    blackPointX The CIE X value for the black point.
    * @param    blackPointY The CIE Y value for the black point.
    * @param    blackPointZ The CIE V value for the black point.
    * @param    rangeALo The low bound of the A parameter
    * @param    rangeAHi The high bound of the A parameter
    * @param    rangeBLo The low bound of the B parameter
    * @param    rangeBHi The high bound of the B parameter
    * @return   @b IDOMColorSpaceLABPtr The new color space.
    */
    static EDL_API IDOMColorSpaceLABPtr create(IEDLClassFactory *pFactory, float whitePointX, float whitePointY, float whitePointZ,
                                               float blackPointX = 0.0f, float blackPointY = 0.0f, float blackPointZ = 0.0f,
                                               float rangeALo = -100.0f, float rangeAHi = 100.0f, float rangeBLo = -100.0f, float rangeBHi = 100.0f);

    /**
    * @class    Data
    * @ingroup  IDOMColorSpaceLAB
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
        public:
        Data()
        {
            rangeAB[0] = rangeAB[2] = -100.0f;
            rangeAB[1] = rangeAB[3] =  100.0f;
            blackPoint[0] = blackPoint[1] = blackPoint[2] = 0.0f;
        } 
        float rangeAB[4];
        float whitePoint[3];
        float blackPoint[3];
    };

    /**
    * @brief    Retrieves class id of IDOMColorSpaceLAB
    * @return   @b CClassID Class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMColorSpaceLABClassID);
        return id;
    };

    /**
    * @brief    Get the input range for this color space.
    *           The range of the L component is always implicitly
    *           0 to 100.
    * @param    lowA  Reference to receive the low bound of the A parameter
    * @param    highA Reference to receive the high bound of the A parameter
    * @param    lowB  Reference to receive the low bound of the B parameter
    * @param    highB Reference to receive the high bound of the B parameter
    */
    virtual void getRangeAB(float &lowA, float &highA,
                            float &lowB, float &highB) = 0;

    /**
    * @brief    Get the White Point in CIE 1931 XYZ space.
    * @param    x Reference to receive the white point X value
    * @param    y Reference to receive the white point Y value
    * @param    z Reference to receive the white point Z value
    */
    virtual void getWhitePoint(float &x, float &y, float &z) = 0;

    /**
    * @brief    Get the Black Point in CIE 1931 XYZ space.
    * @param    x Reference to receive the black point X value
    * @param    y Reference to receive the black point Y value
    * @param    z Reference to receive the black point Z value
    */
    virtual void getBlackPoint(float &x, float &y, float &z) = 0;

};
#define edlobj2IDOMColorSpaceLAB(src) edl_cast((IDOMColorSpaceLAB *)NULL, src) 




_END_EDL_DOM_NAMESPACE

#endif /* EDLIDOMCOLORSPACE_H */
