/*
* Copyright (C) 2007 Global Graphics Software Ltd. All rights reserved.
*/

/**
* @file     idomcolorspace.h
* @ingroup  IDom
* @brief    idomcolorspace Interface
*/

#ifndef __EDLIDOMFUNCTION_H__
#define __EDLIDOMFUNCTION_H__

#include <edl/edltypes.h>
#include <edl/iedlobject.h>
#include <edl/edlstream.h>
#include <edl/isession.h>
#include <edl/edlvector.h>
#include <edl/idomhashable.h>

_BEGIN_EDL_DOM_NAMESPACE

/**
* @class    IDOMFunction
* @ingroup  IDom
* @brief    Base class for PDF/PS Style functions
*/
class IDOMFunction : public IDOMHashable, public IEDLObject
{
public:

    virtual ~IDOMFunction() {}

    /**
    * An enum for the various types of functions.
    */
    typedef enum
    {
        eSampled,
        eExponential,
        eStitching,
        eGrouping,
        ePostScriptCalculator
    } eFunctionType;

    /**
    * @brief    Retrieves function type
    * @return   eFunctionType The function type
    */
    virtual eFunctionType getFunctionType() = 0;

    /**
    * @brief    Get the number of input values that this function will operate on.
    * @return   int The number of input values.
    */
    virtual int getNumInputValues() = 0;

    /**
    * @brief    Get the number of output values that this function will produce.
    * @return   int The number of output values.
    */
    virtual int getNumOutputValues() = 0;

    /**
    * @brief    Get the input domain for a given input to the function.
    * @param    inputNum The 0-indexed input number
    * @param    low A reference to receive the low domain bound for the given inputNum
    * @param    high A reference to receive the high domain bound for the given inputNum
    * @return   bool Returns true on success
    */
    virtual bool getInputDomain(int inputNum, float &low, float &high) = 0;

    /**
    * @brief    Get the output range for a given input to the function.
    * @param    outputNum The 0-indexed output number
    * @param    low A reference to receive the low range bound for the given outputNum
    * @param    high A reference to receive the high range bound for the given outputNum
    * @return   bool Returns true on success. False if there is no output range (which is not
    *           required for all function types) or other error.
    */
    virtual bool getOutputRange(int outputNum, float &low, float &high) = 0;


    /**
    * @brief    Evaluate the input through the function and return the result.
    * @param    inputValues  An array of floats that are input into the function. The size of
    *           the array must be the same as the required number of inputValues
    * @param    outputValues  An array of floats that are the result of evaluating the input through the function. 
    *           The size of the array must be the same as the required number of inputValues
    * @return   bool Returns true on success. False if there is an error evaluting the input.
    */
    virtual bool evaluate (float  *inputValues, float *outputValues) = 0;

    /**
    * @brief    Evaluate the input through the function and return the result.
    * @param    inputValues  An array of integers that are input into the function. The size of
    *           the array must be the same as the required number of inputValues
    * @param    outputValues  An array of floats that are the result of evaluating the input through the function. 
    *           The size of the array must be the same as the required number of inputValues
    * @return   bool Returns true on success. False if there is an error evaluting the input.
    */
    virtual bool evaluate (int    *inputValues, float *outputValues) = 0;
};
DECL_SMART_PTR(IDOMFunction);
#define edlobj2IDOMFunction(src) edl_cast((IDOMFunction *)NULL, src)

typedef CEDLVector<IDOMFunctionPtr> CFunctionVector;


/**
* @class    IDOMSampledFunction
* @ingroup  IDom
* @brief    Interface for sampled functions. See section 3.9.1 of the PDF 1.7 Reference.
*           Default values are as per described in that reference.
*/
class IDOMSampledFunction : public IDOMFunction
{
#define IDOMSampledFunctionClassID 0x4f107645, 0x3bad4e88, 0xb28abc9d, 0x77fdc5ac
public:

    virtual ~IDOMSampledFunction() {}

    /**
    * @brief    Retrieves class id of IDOM
    * @return   CClassID class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMSampledFunctionClassID);
        return id;
    };

    /**
    * @brief    Enum for interpolation types
    */
    typedef enum
    {
        eLinearInterpolation,
        eCubicInterpolation
    } eInterpolationMethod;

    /**
    * @class    Data
    * @ingroup  IDOMFunction
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        Data(): numInputs(0), numOutputs(0),bitsPerSample(0),
            interpolationMethod(eLinearInterpolation)
        {}
        uint32               numInputs;
        CEDLVector<float>    domain;
        uint32               numOutputs;
        CEDLVector<float>    range;
        CEDLVector<uint32>   size;
        uint8                bitsPerSample;
        eInterpolationMethod interpolationMethod;
        CEDLVector<float>    encode;
        CEDLVector<float>    decode;
        CEDLVector<uint8>    table;
    };

    /**
    * @brief    Get the dimension of the sample table for a given input number.
    * @param    inputNum The 0-indexed input number
    * @param    dimension A reference to receive the dimension.
    * @return   bool Returns true on success
    */
    virtual bool getTableDimension(int inputNum, uint32 &dimension) = 0;

    /**
    * @brief    Get the bits per sample for the sample table
    * @param    bitsPerSample A reference to receive the value.
    * @return   bool Returns true on success
    */
    virtual bool getBitsPerSample(uint32 &bitsPerSample) = 0;

    /**
    * @brief    Get the interpolation method used for sample lookup.
    * @param    interpolationMethod A reference to receive the value.
    * @return   bool Returns true on success
    */
    virtual bool getInterpolationMethod(eInterpolationMethod &interpolationMethod) = 0;

    /**
    * @brief    Get the input encode range for a given input to the function.
    * @param    inputNum The 0-indexed input number
    * @param    low A reference to receive the low encode bound for the given inputNum
    * @param    high A reference to receive the high encode bound for the given inputNum
    * @return   bool Returns true on success
    */
    virtual bool getInputEncode(int inputNum, float &low, float &high) = 0;

    /**
    * @brief    Get the output range for a given input to the function.
    * @param    outputNum The 0-indexed output number
    * @param    low A reference to receive the low decode bound for the given outputNum
    * @param    high A reference to receive the high decode bound for the given outputNum
    * @return   bool Returns true on success.
    */
    virtual bool getOutputDecode(int outputNum, float &low, float &high) = 0;

    /**
    * @brief    Get a pointer to the sample table.
    *           Values in this table are stored in the same format as described
    *           by the PostScript and PDF references. The memory is owned by the
    *           instance and must not be modified or freed.
    * @return   CEDLVector<uint8> A reference to the table data.
    */
    virtual const CEDLVector<uint8> &getTableData() = 0;

    /**
    * @brief    Evaluate the input through the function and return the result.
    * @param    inputValues  An array of floats that are input into the function. The size of
    *           the array must be the same as the required number of inputValues
    * @param    outputValues  An array of floats that are the result of evaluating the input through the function. 
    *           The size of the array must be the same as the required number of inputValues
    * @return   bool Returns true on success. False if there is an error evaluting the input.
    */
    virtual bool evaluate (float  *inputValues, float *outputValues) = 0;

    /**
    * @brief    Evaluate the input through the function and return the result.
    * @param    inputValues  An array of integers that are input into the function. The size of
    *           the array must be the same as the required number of inputValues
    * @param    outputValues  An array of floats that are the result of evaluating the input through the function. 
    *           The size of the array must be the same as the required number of inputValues
    * @return   bool Returns true on success. False if there is an error evaluting the input.
    */
    virtual bool evaluate (int    *inputValues, float *outputValues) = 0;

};

DECL_SMART_PTR(IDOMSampledFunction);
#define edlobj2IDOMSampledFunction(src) edl_cast((IDOMSampledFunction *)NULL, src)

/**
* @class    IDOMExponentialFunction
* @ingroup  IDom
* @brief    Interface for exponential functions. See section 3.9.2 of the PDF 1.7 Reference.
*           Default values are as per described in that reference.
*           There can be only one input for this function type.
*/
class IDOMExponentialFunction : public IDOMFunction
{
#define IDOMExponentialFunctionClassID 0x2daf61c4, 0x7e1e11dc, 0x83140800, 0x200c9a66
public:

    virtual ~IDOMExponentialFunction() {}

    /**
    * @brief    Retrieves class id of IDOM
    * @return   CClassID Class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMExponentialFunctionClassID);
        return id;
    };

    /**
    * @class    Data
    * @ingroup  IDOMFunction
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        Data(): numOutputs(0), exponent(1.0)
        {
        }
        CEDLVector<float> domain;
        uint32            numOutputs;
        CEDLVector<float> range;
        CEDLVector<float> c0;
        CEDLVector<float> c1;
        float             exponent;
    };

    /**
    * @brief    Get the output C0 value for a given output to the function.
    * @param    outputNum The 0-indexed output number
    * @param    c0 A reference to receive c0 value for the given outputNum
    * @return   bool Returns true on success
    */
    virtual bool getOutputC0(int outputNum, float &c0) = 0;

    /**
    * @brief    Get the C1 value for a given output to the function.
    * @param    outputNum The 0-indexed input number
    * @param    c1 A reference to receive c1 value for the given outputNum
    * @return   bool Returns true on success
    */
    virtual bool getOutputC1(int outputNum, float &c1) = 0;

    /**
    * @brief    Get the exponent (N) value for the function.
    * @param    n A reference to receive the exponent value
    * @return   bool Returns true on success.
    */
    virtual bool getExponent(float &n) = 0;

    /**
    * @brief    Evaluate the input through the function and return the result.
    * @param    inputValues  An array of floats that are input into the function. The size of
    *           the array must be the same as the required number of inputValues
    * @param    outputValues  An array of floats that are the result of evaluating the input through the function. 
    *           The size of the array must be the same as the required number of inputValues
    * @return   bool Returns true on success. False if there is an error evaluting the input.
    */
    virtual bool evaluate (float  *inputValues, float *outputValues) = 0;

    /**
    * @brief    Evaluate the input through the function and return the result.
    * @param    inputValues  An array of integers that are input into the function. The size of
    *           the array must be the same as the required number of inputValues
    * @param    outputValues  An array of floats that are the result of evaluating the input through the function. 
    *           The size of the array must be the same as the required number of inputValues
    * @return   bool Returns true on success. False if there is an error evaluting the input.
    */
    virtual bool evaluate (int    *inputValues, float *outputValues) = 0;


};

DECL_SMART_PTR(IDOMExponentialFunction);
#define edlobj2IDOMExponentialFunction(src) edl_cast((IDOMExponentialFunction *)NULL, src)


/**
* @class    IDOMStitchingFunction
* @ingroup  IDom
* @brief    Interface for stitching functions. See section 3.9.3 of the PDF 1.7 Reference.
*           Default values are as per described in that reference.
*           There can only be one input for this function, and the functions contained therein
*           must also handle one input.
*/
class IDOMStitchingFunction : public IDOMFunction
{
#define IDOMStitchingFunctionClassID 0x565924fc, 0x7e2011dc, 0x83140800, 0x200c9a66
public:

    virtual ~IDOMStitchingFunction() {}


    /**
    * @brief    Retrieves class id of IDOM
    * @return   CClassID class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMStitchingFunctionClassID);
        return id;
    };

    /**
    * @class    Data
    * @ingroup  IDOMFunction
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        Data(): domain(0), numOutputs(0), range(0),
            bounds(0), encode(0)
        {
        }
        CEDLVector<float> domain;
        uint32            numOutputs;
        CEDLVector<float> range;
        CEDLVector<float> bounds;
        CEDLVector<float> encode;
        CFunctionVector   functions;
    };

    /**
    * @brief    Get the number of functions that are stitched.
    * @param    numFunctions A reference to receive the number of functions.
    * @return   bool Returns true on success
    */
    virtual bool getNumFunctions(uint32 &numFunctions) = 0;

    /**
    * @brief    Get the function for a given function index.
    * @brief    index A zero based index for the desired function.
    * @brief    function A reference to receive the function.
    * @return   bool Returns true on success
    */
    virtual bool getFunctionAtIndex(uint32 index, IDOMFunctionPtr &function) = 0;

    /**
    * @brief    Get a reference to the bounds vector for this function.
    * @return   CEDLVector<float> Reference to a constant CEDLBuffer<float> containing the bounds.
    */
    virtual const CEDLVector<float> &getBoundsVector() = 0;

    /**
    * @brief    Get a reference to the encode vector for this function.
    * @return   CEDLVector<float> Reference to a constant CEDLBuffer<float> containing the encode.
    */
    virtual const CEDLVector<float> &getEncodeVector() = 0;

    /**
    * @brief    Evaluate the input through the function and return the result.
    * @param    inputValues  An array of floats that are input into the function. The size of
    *           the array must be the same as the required number of inputValues
    * @param    outputValues  An array of floats that are the result of evaluating the input through the function. 
    *           The size of the array must be the same as the required number of inputValues
    * @return   bool Returns true on success. False if there is an error evaluting the input.
    */
    virtual bool evaluate (float  *inputValues, float *outputValues) = 0;

    /**
    * @brief    Evaluate the input through the function and return the result.
    * @param    inputValues  An array of integers that are input into the function. The size of
    *           the array must be the same as the required number of inputValues
    * @param    outputValues  An array of floats that are the result of evaluating the input through the function. 
    *           The size of the array must be the same as the required number of inputValues
    * @return   bool Returns true on success. False if there is an error evaluting the input.
    */
    virtual bool evaluate (int    *inputValues, float *outputValues) = 0;


};

DECL_SMART_PTR(IDOMStitchingFunction);
#define edlobj2IDOMStitchingFunction(src) edl_cast((IDOMStitchingFunction *)NULL, src)

/**
* @class    IDOMGroupingFunction
* @ingroup  IDom
* @brief    Interface to encapsulate an array of x-input-1-output functions.
*
*           This allows an array of \e n x-input-1-output functions to be treated as a
*           single x-input-n-output function. This is used in particular to represent
*           arrays of functions present in certain shading patterns.
*/
class IDOMGroupingFunction : public IDOMFunction
{
#define IDOMGroupingFunctionClassID 0x77077047, 0x5d4a4d47, 0xb4b08719, 0x25b1b24f
public:

    virtual ~IDOMGroupingFunction() {}

    /**
    * @brief    Retrieves class id of IDOM
    * @return   CClassID class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMGroupingFunctionClassID);
        return id;
    };

    /**
    * @class    Data
    * @ingroup  IDOMFunction
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        CFunctionVector functions;
    };

    /**
    * @brief    Get the number of functions in the group.
    * @param    numFunctions A reference to receive the number of functions.
    * @return   bool Returns true on success.
    */
    virtual bool getNumFunctions(uint32 &numFunctions) = 0;

    /**
    * @brief    Get the function for a given function index.
    * @param    index A zero based index for the desired function.
    * @param    function A reference to receive the function pointer.
    * @return   bool Returns true on success
    */
    virtual bool getFunctionAtIndex(uint32 index, IDOMFunctionPtr &function) = 0;

    /**
    * @brief    Evaluate the input through the function and return the result.
    * @param    inputValues  A pointer to a single float value that is the input to this grouped function.
    * @param    outputValues  An array of floats that are the result of evaluating the input through the function. 
    *           The size of the array must be the same as the required number of output values
    * @return   bool Returns true on success. False if there is an error evaluting the input.
    */
    virtual bool evaluate (float  *inputValues, float *outputValues) = 0;

    /**
    * @brief    Evaluate the input through the function and return the result.
    * @param    inputValues  A pointer to a single float value that is the input to this grouped function.
    * @param    outputValues  An array of floats that are the result of evaluating the input through the function. 
    *           The size of the array must be the same as the required number of output values
    * @return   bool Returns true on success. False if there is an error evaluting the input.
    */
    virtual bool evaluate (int    *inputValues, float *outputValues) = 0;
};
DECL_SMART_PTR(IDOMGroupingFunction);
#define edlobj2IDOMGroupingFunction(src) edl_cast((IDOMGroupingFunction *)NULL, src)


/**
* @class    IDOMPostScriptCalculatorFunction
* @ingroup  IDom
* @brief    Interface for PostScript calculator functions. See section 3.9.4 of the PDF 1.7 Reference.
*           Default values are as per described in that reference.
*/
class IDOMPostScriptCalculatorFunction : public IDOMFunction
{
#define IDOMPostScriptCalculatorFunctionClassID 0x220a9a6c, 0x7e2511dc, 0x83140800, 0x200c9a66
public:

    virtual ~IDOMPostScriptCalculatorFunction() {}

    /**
    * @brief    Retrieves class id of IDOM
    * @return   CClassID Class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMPostScriptCalculatorFunctionClassID);
        return id;
    };

    /**
    * @class    Data
    * @ingroup  IDOMFunction
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        Data(): numInputs(1), domain(0), numOutputs(0), range(0),
            priv(NULL)
        {
        }
        CEDLVector<float>  domain;
        uint32             numInputs;
        uint32             numOutputs;
        CEDLVector<float>  range;
        IEDLObjectPtr      priv;
        IEDLClassFactory  *pFactory;
    };

    /**
    * @brief    Get the PostScript Calculator function as a PostScript Procedure
    * @param    procStream A reference to receive the function as an input Stream
    * @return   bool Returns true on success.
    */
    virtual bool getCalculatorAsPostScriptStream(IInputStreamPtr &stream) = 0;

    /**
    * @brief    Get the PostScript Calculator function as an executable PS object
    * @param    psArray A reference to receive the function
    * @return   bool Returns true on success.
    */
    virtual bool getCalculator (IEDLObjectPtr &psObj) = 0;

    /**
    * @brief    Evaluate the input through the function and return the result.
    * @param    inputValues  An array of floats that are input into the function. The size of
    *           the array must be the same as the required number of inputValues
    * @param    outputValues  An array of floats that are the result of evaluating the input through the function. 
    *           The size of the array must be the same as the required number of inputValues
    * @return   bool Returns true on success. False if there is an error evaluting the input.
    */
    virtual bool evaluate (float  *inputValues, float *outputValues) = 0;

    /**
    * @brief    Evaluate the input through the function and return the result.
    * @param    inputValues  An array of integers that are input into the function. The size of
    *           the array must be the same as the required number of inputValues
    * @param    outputValues  An array of floats that are the result of evaluating the input through the function. 
    *           The size of the array must be the same as the required number of inputValues
    * @return   bool Returns true on success. False if there is an error evaluting the input.
    */
    virtual bool evaluate (int    *inputValues, float *outputValues) = 0;

};

DECL_SMART_PTR(IDOMPostScriptCalculatorFunction);
#define edlobj2IDOMPostScriptCalculatorFunction(src) edl_cast((IDOMPostScriptCalculatorFunction *)NULL, src)


_END_EDL_DOM_NAMESPACE

#endif /* __EDLIDOMFUNCTION_H__ */
