/*
* Copyright (C) 2007-2019 Global Graphics Software Ltd. All rights reserved.
*/

/**
* @file    idomglyph.h
* @ingroup IDom
* @brief   IDOMGlyph Interface
*/

#ifndef EDLIDOMGLYPH_H
#define EDLIDOMGLYPH_H

#include <edl/edltypes.h>
#include <edl/edlgeom.h>
#include <edl/idomnode.h>

_BEGIN_EDL_DOM_NAMESPACE

class IDOMGlyphName;
DECL_SMART_PTR(IDOMGlyphName);

/**
* @class   IDOMGlyphName
* @ingroup IDOM
*          Reference-counted name for a string.
*          Used to reduce duplication of glyph names.
*/
class IDOMGlyphName : public IRCObject
{
  public:
    virtual ~IDOMGlyphName() {}

    static EDL_API IDOMGlyphNamePtr create(IEDLClassFactory *pFactory, const EDLSysString &glyphName);

    virtual const EDLSysString &getNameString() const = 0;
};

/**
* @class   IDOMGlyph
* @ingroup IDom
* @brief   The IDOMGlyph class is an abstract class modelling a single character from a font.
* 
* All fonts contain glyphs. TrueType fonts describe each glyph as a set of paths. A path is a closed
* curve specified using points and particular mathematics. A lower case 'i' has two paths, one for
* the dot and one for the stem. The paths are filled with pixels to create the final letter form. This
* set of paths is called an outline.
* 
* In general, glyphs within a text run are either base glyphs or combining marks that may be
* attached to base glyphs. A glyph describes a single character. The IDOMGlyphs node describes a
* run of characters, each of the characters in the run is described by an IDOMGlyph node.
* 
* Another way that a glyph may be specified is in terms of other component glyphs. A glyph may
* consist of references to other glyphs which are combined to make the new compound glyph. An
* 'acute e'(é) could be composed of the glyphs for 'e' and 'acute'. In such compound glyphs, each
* component glyph has placement and optional transformation data associated with it.
* 
* All measurements are relative to the font rendering em size, which is considered to be 1.
* 
* If children are present, they are the marking nodes that represent the glyph
*/
class IDOMGlyph : public IDOMNode
{
#define IDOMGlyphClassID 0x0dcbbdbd, 0xa6ee4a4a, 0x93843fd9, 0xcd134e05
public:

    /**
    * @typedef GlyphID
    * @ingroup IDom
    * @brief Holds the glyph ID of the glyph.
    *
    * A glyf table contains the data describing each glyph in the font. A particular glyph is identified
    * by a glyph id, which is used exclusively throughout the font to identify that glyph. For further
    * information on using glyf tables see http://www.microsoft.com/typography/otspec/glyf.htm.
    *
    * Note: There are some special glyph ids which are reserved by convention as described in the
    * eGlyphIDSpecial type. It is advisable to follow these conventions when modifying fonts.
    * Glyph 0 is normally an open rectangle and is used by rendering systems to substitute for characters 
    * that are not present in the font.
    */
    typedef uint16 GlyphID;

    /**
    * @typedef eGlyphIDSpecial
    * @ingroup IDom
    * @brief Enumeration type used to define known, special Glyph IDs
    */
    typedef enum {

        eGlyphIDNotdef  = 0     //!< The glyph ID to subsitute for font undefined glyphs

    } eGlyphIDSpecial;

    /**
    * @brief    Retrieves the class id for IDOMGlyph.
    * @return   @b CClassID The class id of IDOMGlyph.
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMGlyphClassID);
        return id;
    };

    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        Data(): hasCustomAdvance(false), advanceX(0.0), advanceY(0.0),
            uOffset(0.0), vOffset(0.0), glyphID(0), unicodeID(-1), unicodeIDValid(true), 
            originalAdvanceX(0.0)
        {
            colored = false;
            bounds.setEmpty();
        }
        bool             hasCustomAdvance;
        double           advanceX;
        double           advanceY;
        double           uOffset;
        double           vOffset;
        bool             unicodeIDValid;
        uint32           unicodeID;
        GlyphID          glyphID;
        IDOMGlyphNamePtr glyphName;
        FRect            bounds;
        bool             colored;
        double           originalAdvanceX;
    };
    
    typedef CEDLVector<Data> CDOMGlyphDataVect;

    /**
    * @brief    Determines whether or not a custom value has been set for the glyph's advance.
    * @details  The advance of a glyph is the distance from the glyph's origin to the origin of the 
    *            next glyph along the baseline, which can be either vertical or horizontal.
    *            
    *            A "custom" advance is one that has not been extracted from the font. A glyph has a custom
    *            advance if the advance information has been determined by the IndicesString
    *            or has been adjusted manually in some way.
    * @return   @b bool True if the glyph is using a custom advance, and false otherwise.
    */
    virtual bool getHasCustomAdvance() const = 0;

    /**
    * @brief    Informs whether a custom value has been set for the glyph's advance.
    * @details   The advance of a glyph is the distance from the glyph's origin to the origin of the 
    *            next glyph along the baseline, which can be either vertical or horizontal.
    *            
    *            A "custom" advance is one that has not been extracted from the font. A glyph has a custom
    *            advance if the advance information has been determined by the IndicesString
    *            or has been adjusted manually in some way.
	* @param    bHasCustomAdvance Set to true if the glyph has a custom advance
    */
    virtual void setHasCustomAdvance(bool bHasCustomAdvance) = 0;

    /**
    * @brief    Retrieves the glyph id for this glyph.
    * @return   @b GlyphID GlyphID value
    */
    virtual GlyphID getGlyphID() const = 0;
    
    /**
    * @brief    Retrieves the Unicode code point for the glyph.
    * @param    codePoint A reference to receive the code point.
    * @return   @b bool True on success.
    */
    virtual bool  getGlyphUnicode(uint32 &codePoint) const = 0;

    /**
    * @brief    Retrieves the glyph name for this glyph, if present.
    *           An empty string is still a valid glyph name
    * @param    glyphName Reference to receive the glyph name 
    * @return   @b bool True on success, false if the call fails in any way.
    */
    virtual bool getGlyphName(EDLSysString &glyphName) = 0;
    
    /**
    * @brief    Retrieves the glyph name for this glyph, if present, as a IDOMGlyphNamePtr.
    *           Returns NULL if the glyph has no name.
    * @return   @b IDOMGlyphNamePtr The glyph name
    */
    virtual IDOMGlyphNamePtr getGlyphName() const = 0;

    /* All measurements are as if the font rendering em size is '1'
    * So, you need to scale them yourself!
    */

    /**
    * @brief    Retrieves the x-component of the advance of the glyph.
    * The advance indicates the positioning of the next glyph in the sequence, relative to the
    * origin of the current glyph. Base glyphs generally have a non-zero advance and combining
    * glyphs have a zero advance.
    *
    * Advance width is measured in ems.
    * @return   @b double The value of the x-component of the advance of the glyph
    */
    virtual double getAdvanceX() const = 0;

    /**
    * @brief    Retrieves the y-component of the advance of the glyph.
    * The advance indicates the positioning of the next glyph in the sequence, relative to the
    * origin of the current glyph. Base glyphs generally have a non-zero advance and combining
    * glyphs have a zero advance.
    * @return   @b double The value of the y-component of the advance of the glyph, in ems.
    */
    virtual double getAdvanceY() const = 0;

    /**
    * @brief    Retrieves the UOffset value for the glyph.
    * @details  Offsets are used to attach marks to base characters, and define how to move a glyph, relative
    *           to its origin, to place it correctly relative to the base glyph it is attached to.
    *           The offset value is added to the nominal glyph origin calculated
    *           using the advance to generate the actual origin for the glyph. 
    *           
    *           UOffset defines the x-component of the offset for a glyph; VOffset defines the y-component of 
    *           the offset. UOffset and VOffset are real numbers, specified in ems.
    *           
    *           By default, the offset values are (0.0, 0.0).
    *           
    *           Base glyphs generally have a glyph offset of (0.0, 0.0). Combining glyphs generally have an
    *           offset that places them correctly on top of the nearest preceding base glyph.
    *           For left-to-right text, a positive UOffset value points to the right, for right-to-left text, a positive
    *           UOffset value points to the left.
    * @return   @b double The value of UOffset.
    */
    virtual double getUOffset() const = 0;

    /**
    * @brief    Retrieves the VOffset value for the glyph.
    * @details  Offsets are used to attach marks to base characters, and define how to move a glyph, relative
    *           to its origin, to place it correctly relative to the base glyph it is attached to.
    *           The offset value is added to the nominal glyph origin calculated
    *           using the advance to generate the actual origin for the glyph. 
    *           
    *           UOffset defines the x-component of the offset for a glyph; VOffset defines the y-component of 
    *           the offset. UOffset and VOffset are real numbers, specified in ems.
    *           
    *           By default, the offset values are (0.0, 0.0).
    *           
    *           Base glyphs generally have a glyph offset of (0.0, 0.0). Combining glyphs generally have an
    *           offset that places them correctly on top of the nearest preceding base glyph.
    *           For left-to-right text, a positive UOffset value points to the right, for right-to-left text, a positive
    *           UOffset value points to the left.
    * @return   @b double Returns the value of VOffset.
    */
    virtual double getVOffset() const = 0;

    /**
    * @brief    Retrieves the glyph bounding box.
    * 
    * @details  The bounding box of a glyph is an imaginary box that encloses the glyph as tightly as possible.
    *           The bounding box is described as an FRect representing the 
    *           origin of the bounding box relative to the glyph origin and the bounding box's width and height.
    *           
    *           Note that it is possible for the values of the bounding box origin to be negative. For example, the origin of
    *           the glyph rests on the baseline for the font, so any glyph with a descender will have a negative y-component
    *           in the origin of its bounding box.
    *           
    *           The bounding box values are not adjusted by any of the offsets that have been set.
    *
    * @return   @b FRect Bounding box value
    */
    virtual FRect getBounds() const = 0;

    /**
    * @brief    Sets a flag whereby the glyph specifies its own colors or not.
    * 
    * @details  For most glyphs, the color is specified before the glyph proc is invoked
    *           and the current color is used in drawing the glyph. This type of glyph can
    *           be considered uncolored.
    *           
    *           If some part of the glyph proc specifies its own colors then the glyph
    *           can be considered colored as that part of the glyph is always drawn in that color.
    *           
    *           This is used in PostScript output to determine whether or not the glyph proc
    *           should use setcachedevice or setcharwidth in the PS proc
    * @param    bColored Set to true if the glyph specifies its own colors
    *
    */
    virtual void setColored (bool bColored) = 0;

    /**
    * @brief    Retrieves the flag which indicates if this glyph contains color information.
    * 
    * @details  For most glyphs, the color is specified before the glyph proc is invoked
    *           and the current color is used in drawing the glyph. This type of glyph can
    *           be considered uncolored.
    *           
    *           If some part of the glyph proc specifies its own colors then the glyph
    *           can be considered colored as that part of the glyph is always drawn in that color.
    *           
    *           This is used in PostScript output to determine whether or not the glyph proc
    *           should use setcachedevice or setcharwidth in the PS proc.
    *           
    * @return   @b bool Colored flag
    */
    virtual bool getColored () const = 0;

    /**
    * @brief    Retrieves the x-component of the original advance of the glyph.
    * @details  The advance indicates the positioning of the next glyph in the sequence, relative to the
    *           origin of the current glyph. Base glyphs generally have a non-zero advance and combining
    *           glyphs have a zero advance. The original advance is the one specified in the font.
    *           
    *           Advance width is measured in hundredths of the font em size.
    * @return   @b double The value of the x-component of the original advance of the glyph
    */
    virtual double getOriginalAdvanceX() const = 0;
};

DECL_SMART_PTR(IDOMGlyph);
#define edlobj2IDOMGlyph(src) edl_cast((IDOMGlyph *)NULL, src) 


/**
* @class    IDOMGlyphIDEnumerator.
* @brief    DOM GlyphID Enumerator
*
* The IDOMGlyphIDEnumerator provides a generalized interface to a list of glyphs by ID.
* The list is designed to be read from beginning to end in sequence.  Glyphs can only
* be added to the end of the list.
*
*/
class IDOMGlyphIDEnumerator  :  public IEDLObject
{
#define IDOMGlyphIDEnumeratorClassID 0x637f8550, 0xdb1511dc, 0x95ff0800, 0x200c9a66
public:

    /**
    * @brief    Retrieves the class id of IDOMGlyphIDEnumerator.
    * @return   @b CClassID Class id of IDOMGlyphIDEnumerator.
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMGlyphIDEnumeratorClassID);
        return id;
    };

    /**
    * @brief    Adds a glyph to the end of the list.
    * @param    glyphID The glyph ID to add to the end of the list.
    */
    virtual void add(IDOMGlyph::GlyphID glyphID) = 0;

    /**
    * @brief    Returns the ID of the glyph at the current point in the list
    * @return   @b IDOMGlyph::GlyphID The ID of the glyph at the current position in the list.
    */
    virtual IDOMGlyph::GlyphID getGlyphID() = 0;

    /**
    * @brief    Moves the current list point to the start of the list.
    * @return   @b bool True on success, false otherwise.
    */
    virtual bool beginEnumeration() = 0;

    /**
    * @brief    Indicates whether or not the end of the list has been reached.
    * @return   @b bool True if the end of the list has been reached, false otherwise.
    */
    virtual bool endEnumeration() = 0;

    /**
    * @brief    Moves the current list position on to the next item in the list, unless the end of the list has been reached.
    * @return   @b bool True if the current list position was successfully advanced.  In the case where the end of 
    * the list had already been reached prior to the call, the current list position remains on the last item and the 
    * function returns false.
    */
    virtual bool nextEnumerationItem() = 0;

};

DECL_SMART_PTR(IDOMGlyphIDEnumerator);


typedef IInputEnumRC<IDOMGlyphPtr> IDOMGlyphCollectionEnum;
DECL_SMART_PTR(IDOMGlyphCollectionEnum);

_END_EDL_DOM_NAMESPACE

#endif /* __EDLIDOMGLYPH_H__ */
