/*
* Copyright (C) 2007-2019 Global Graphics Software Ltd. All rights reserved.
*/

/**
* @file     idomimageresource.h
* @ingroup  IDom
* @brief    IDOMResources interface
*/

#ifndef __IDOMIMAGERESOURCE_H__
#define __IDOMIMAGERESOURCE_H__

#include <edl/edltypes.h>
#include <edl/edlfwd.h>
#include <edl/edlstream.h>
#include <edl/idomnode.h>
#include <edl/iimagecodec.h>
#include <edl/idomcolor.h>
#include <edl/idomhashable.h>
#include <jawsmako/optionalcontent.h>

_BEGIN_EDL_DOM_NAMESPACE

/**
* @class    IDOMImageProperties
* @ingroup  IDom
* @brief    The IDOMImageProperties interface provides access to an underlying implementation which stores
            miscellaneous information about the associated image.
*            
*           IDOMImageProperties holds a collection of name-value pairs which can be used to 
*           store arbitrary information about the image. The name must be unique within the collection, as
*           it provides a key into it.
*
*           Methods are provided to allow the storage of integer, Boolean and string values.
*
*           A typical use would be to store information which would be expensive 
*           to compute-for example, whether or not the image is a noisy mask.
*/
class IDOMImageProperties : public IEDLObject
{
#define IDOMImagePropertiesClassID 0x3E76E62E, 0x11414364, 0xA36BEF96, 0xE81EB47B
public:

    /**
    * @brief    Retrieves class id of IDOMImageProperties
    * @return   CClassID Class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMImagePropertiesClassID);
        return id;
    };


    /**
    * @brief    Registers the name of a property and a string value for the property.
    * @param    name Property name.
    * @param    value The string value for the property.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setProperty (const EDLString &name, const EDLString &value) = 0; 

    /**
    * @brief    Retrieves the value of a named property as a string value.
    * @param    name Property name.
    * @param    value Reference parameter to receive the property value.
    * @return   @b bool True on success, false if the call fails. The value parameter will contain the property value. 
    */
    virtual bool getProperty (const EDLString &name, EDLString &value) = 0; 

    /**
    * @brief    Registers the name of a property and a Boolean value for the property.
    * @param    name Property name.
    * @param    value The Boolean value for the property.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setBoolProperty (const EDLString &name, bool value) = 0;

    /**
    * @brief    Retrieves the value of a named property as a Boolean value.
    * @param    name Property name.
    * @param    value Reference parameter to receive the property value.
    * @return   @b bool True on success, false if the call fails. The value parameter will contain the property value. 
    */
    virtual bool getBoolProperty (const EDLString &name, bool &value) = 0;

    /**
    * @brief    Registers the name of a property and an integer value for the property.
    * @param    name Property name.
    * @param    value The integer value for the property.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setIntProperty (const EDLString &name, int value) = 0;

    /**
    * @brief    Retrieves the value of a named property as an integer value.
    * @param    name Property name.
    * @param    value Reference parameter to receive the property value.
    * @return   @b bool True on success, false if the call fails. The value parameter will contain the property value. 
    */
    virtual bool getIntProperty (const EDLString &name, int &value) = 0;
    
    /**
     * @brief    Registers the name of a property and an IRCObject value for the property.
     * @param    name Property name.
     * @param    object An IRCObject reference to the value for the property.
     * @return   @b bool True on success, false if the call fails.
     */
    virtual bool setObjectProperty (const EDLString &name, const IRCObjectPtr &object) = 0;
    
    /**
     * @brief    Retrieves the value of a named property as an IRCObject value.
     * @param    name Property name.
     * @param    value An IRCObject reference to receive the property value.
     * @return   @b bool True on success, false if the call fails. The value parameter will contain the property value.
     */
    virtual bool getObjectProperty (const EDLString &name, IRCObjectPtr &value) = 0;
    

};
DECL_SMART_PTR(IDOMImageProperties);
#define edlobj2IDOMImageProperties(src) edl_cast((IDOMImageProperties *)NULL, src) 

    /**
    * @enum     eDOMImageType
    * @ingroup  IDom
    * @brief    Image type enumeration
    */
typedef enum
{
	eDOMImageTypeUnknown = 0, //!< Unknown image type
    eDOMImageTypeJPEG,        //!< Discrete cosine transform (DCT), a lossy form of compression used in JPEG images
    eDOMImageTypeJBIG2,       //!< Compression for bi-level (monochrome) images
    eDOMImageTypeJPEG2000,    //!< Image compression that uses a wavelet-based method
    eDOMImageTypePNG,         //!< Portable Network Graphics, a format that supports lossless data compression and transparency
    eDOMImageTypeCCITTFax,    //!< A Comité Consultatif International Téléphonique et Télégraphique (ITU-T) standard for compressing monochrome images
    eDOMImageTypeFlate,       //!< Flate (zip) compressed
    eDOMImageTypeLZW,         //!< Lempel-Ziv-Welch compressed
    eDOMImageTypeRunLength,   //!< Run-length encoded
    eDOMImageTypeRendered,    //!< A rendered image, therefore uncompressed
} eDOMImageType;

/**
* @class    IDOMImage
* @ingroup  IDom
* @brief    The base class describing an image. This class is subclassed to create a number of more specific image types.
*/
class IDOMImage : public IDOMHashable, public IDOMResource
{
#define IDOMImageClassID 0xf1050c98, 0x1bee40e7, 0xaa604df2, 0x82e822c4
public:
    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        Data() : streamLength(0), imageType (eDOMImageTypeUnknown)
        { }
        EDLSysString    uri;
        IInputStreamPtr stream;
        uint64          streamLength;
        eDOMImageType   imageType;
    };

    /**
    * @brief    Creates a properly initialized image decoder object that reads from an inputstream that is specific to that image format
    * @param    factory Pointer to the EDL class factory
    * @param    imageDecoder Smart pointer to the image decoder interface
    * @param    imageProperties Smart pointer to the DOM image properties interface
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool createImageDecoder (IEDLClassFactory *factory, IImageDecoderPtr &imageDecoder, IDOMImagePropertiesPtr &imageProperties) = 0;
    
    /**
    * @brief    Convenience routine to retrieve the first frame in an image.
    * @param   factory Pointer to the EDL class factory
    * @param    frame Smart pointer to receive the image frame.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool getFirstImageFrame (IEDLClassFactory *factory, IImageFramePtr &frame);

    /**
    * @brief    Creates a properly initialized image encoder object that writes to an outputstream that is specific to that image format
    * @param    session Pointer to the EDL session
	* @param    imageDest Smart pointer to the output stream
    * @param    imageEncoder Smart pointer to the image encoder interface
    * @param    imageProperties Smart pointer to the DOM image properties interface
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool createImageEncoder (ISessionPtr &session, IOutputStreamPtr &imageDest, IImageEncoderPtr &imageEncoder, IDOMImagePropertiesPtr &imageProperties) = 0; 
    
    /**
    * @brief    Returns an object that stores the properties for this image object. The properties
    *           can then be inspected (or more added) by clients that need to manipulate the image resource
    * @param    imageProperties Smart pointer to receive the DOM image properties interface. Image properties can be a boolean, an integer or a string. Atpresent it is used to store information such as whether the image has an alpha value.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool getImageProperties (IDOMImagePropertiesPtr &imageProperties) = 0;

    /**
     *  @brief Retrieves the image type
     *  @return @b eDOMImageType The image type
     */
    virtual eDOMImageType getImageType () = 0;
};

DECL_SMART_PTR(IDOMImage);
#define edlobj2IDOMImage(src) edl_cast((IDOMImage *)NULL, src) 


class IDOMJPEGImage;
DECL_SMART_PTR(IDOMJPEGImage);

/**
* @class    IDOMJPEGImage
* @ingroup  IDom
* @brief    Interface to a class representing a JPEG (.jpg or .jpeg) image
*/
class IDOMJPEGImage : public virtual IDOMImage
{
#define IDOMJPEGImageClassID 0x07c50584, 0xf4d44316, 0xa3de2ddb, 0x83f3e7cf
public:

    /**
     * @brief    Create a JPEG Image resource with the given JPEG stream.
     *           Throws an IEDLError on failure.
     * @param    pFactory The EDL Class factory to use.
     * @param    stream The stream containing the JPEG image.
     * @return   @b IDOMImagePtr The new image.
     */
    static EDL_API IDOMJPEGImagePtr create(IEDLClassFactory *pFactory, const IInputStreamPtr &stream);

    /**
     * @brief    Encode an image as a JPEG stream.
     *           This routine may convert the source image into a form that may
     *           be encoded as JPEG, such as by stripping alpha channels or
     *           converting to a supported colour space.
     *           Throws an IEDLError on failure.
     * @param    pSession The relevant EDL session
     * @param    image The image to be encoded
     * @param    stream The stream to use to store the image data.
     * @param    quality The desired quality in the range 1 through 5, with
     *           1 being lowest and 5 being highest. Default is 3.
     */
    static EDL_API void encode(const ISessionPtr &pSession, const IDOMImagePtr &image, const IOutputStreamPtr &stream, uint8 quality = 3);

    /**
     * @brief    Encode the contents of an IImageFrame as a JPEG stream, returning the stream.
     *           This routine may convert the source image into a form that may
     *           be encoded as JPEG, such as by stripping alpha channels or
     *           converting to a supported colour space.
     * @param    pSession The relevant EDL session
     * @param    frame The frame providing the source image data
     * @param    stream The stream to use to store the image data.
     * @param    quality The desired quality in the range 1 through 5, with
     *           1 being lowest and 5 being highest. Default is 3.
     */
    static EDL_API void encode(const ISessionPtr &pSession, const IImageFramePtr &frame, const IOutputStreamPtr &stream, uint8 quality = 3);

    /**
    * @brief    Retrieves class id of IDOMJPEGImage
    * @return   @b CClassID Class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMJPEGImageClassID);
        return id;
    };

};
#define edlobj2IDOMJPEGImage(src) edl_cast((IDOMJPEGImage *)NULL, src) 


class IDOMPNGImage;
DECL_SMART_PTR(IDOMPNGImage);

/**
* @class    IDOMPNGImage
* @ingroup  IDom
* @brief    Interface to a class representing a PNG (.png) image.
*/
class IDOMPNGImage : public virtual IDOMImage
{
#define IDOMPNGImageClassID 0x7792c068, 0x67a94eaf, 0x9563973a, 0x910c6b61
public:
    /**
     * @brief    Create a PNG Image resource with the given PNG stream.
     *           Throws an IEDLError on failure.
     * @param    pFactory The EDL Class factory to use.
     * @param    stream The stream containing the PNG image.
     * @return   @b IDOMImagePtr The new image.
     */
    static EDL_API IDOMPNGImagePtr create(IEDLClassFactory *pFactory, const IInputStreamPtr &stream);

    /**
     * @brief    Encode an image as a PNG stream, returning the stream.
     *           This routine may convert the image samples into a form that may
     *           be encoded as PNG if required, such as by converting to a supported
     *           color space.
     *           Throws an IEDLError on failure.
     * @param    pSession The relevant EDL session
     * @param    image The image to be encoded
     * @param    stream The stream to use to store the image data.
     */
    static EDL_API void encode(const ISessionPtr &pSession, const IDOMImagePtr &image, const IOutputStreamPtr &stream);
    
    /**
     * @brief    Encode the contents of an IImageFrame as a PNG stream, returning the stream.
     *           This routine may convert the image samples into a form that may
     *           be encoded as PNG if required, such as by converting to a supported
     *           color space.
     *           Throws an IEDLError on failure.
     * @param    pSession The relevant EDL session
     * @param    frame The frame providing the source image data
     * @param    stream The stream to use to store the image data.
     */
    static EDL_API void encode(const ISessionPtr &pSession, const IImageFramePtr &frame, const IOutputStreamPtr &stream);
    
    /**
     * @brief    Encode a frame buffer as a PNG stream, returning the stream.
     *           The source image must be compatible with PNG.
     *           Throws an IEDLError on failure.
     * @param    pSession The relevant EDL session
     * @param    colorSpace The color space of the frame buffer
	 * @param    dpi The image resolution in dots-per-inch
	 * @param    bpc Bits per component (pixel)
     * @param    frameBuffer The frame buffer to be encoded
     * @param    width The width of the frame buffer
     * @param    height The height of the frame buffer
     * @param    stride The offset in bytes in the frameBuffer from one scanline to the next. May be negative.
     * @param    hasAlpha Set true if the frame buffer has an alpha channel.
     * @param    stream The stream to use to store the image data.
     */
    static EDL_API void encode(const ISessionPtr &pSession, const IDOMColorSpacePtr &colorSpace, float dpi, uint8 bpc,
                               void *frameBuffer, uint32 width, uint32 height, int32 stride, bool hasAlpha,
                               const IOutputStreamPtr &stream);


    /**
    * @brief    Retrieves class id of IDOM
    * @return   @b CClassID Class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMPNGImageClassID);
        return id;
    };
};
#define edlobj2IDOMPNGImage(src) edl_cast((IDOMPNGImage *)NULL, src) 


class IDOMTIFFImage;
DECL_SMART_PTR(IDOMTIFFImage);

/**
* @class    IDOMTIFFImage
* @ingroup  IDom
* @brief    IDOMTIFFImage interface
*/
class IDOMTIFFImage : public IDOMImage
{
#define IDOMTIFFImageClassID 0x1469a59c, 0x78114e29, 0xb2fc0e45, 0x8aa62afe
public:

    /**
     * @brief    Create a TIFF Image resource with the given TIFF stream.
     *           Throws an IEDLError on failure.
     * @param    pFactory The EDL Class factory to use.
     * @param    stream The stream containing the JPEG image.
     * @return   @b IDOMImagePtr The new image.
     */
    static EDL_API IDOMTIFFImagePtr create(IEDLClassFactory *pFactory, const IInputStreamPtr &stream);

    /**
    * @brief Available TIFF compression schemes.
    */
    typedef enum
    {
        eTCAuto, /// Choose an appropriate scheme for the incoming image
        eTCNone,
        eTCCCITT,
        eTCCCITT3,
        eTCCCITT4,
        eTCPackBits,
        eTCLZW,
    } eTIFFCompression;

    /**
     * @brief    Encode an image as a TIFF stream, returning the stream.
     *           This routine may convert the image samples into a form that may
     *           be encoded as TIFF if required, such as by converting to a supported
     *           color space.
     *           Throws an IEDLError on failure.
     * @param    pSession The relevant EDL session
     * @param    image The image to be encoded
     * @param    scheme The TIFF compression scheme to be used.
     * @param    stream The stream to use to store the image data.
     */
    static EDL_API void encode(const ISessionPtr &pSession, const IDOMImagePtr &image, const IOutputStreamPtr &stream, eTIFFCompression scheme = eTCAuto);

    /**
     * @brief    Encode the contents of an IImageFrame as a TIFF stream, returning the stream.
     *           This routine may convert the image samples into a form that may
     *           be encoded as TIFF if required, such as by converting to a supported
     *           color space.
     *           Throws an IEDLError on failure.
     * @param    pSession The relevant EDL session
     * @param    frame The frame providing the source image data
     * @param    scheme The TIFF compression scheme to be used.
     * @param    stream The stream to use to store the image data.
     */
    static EDL_API void encode(const ISessionPtr &pSession, const IImageFramePtr &frame, const IOutputStreamPtr &stream, eTIFFCompression scheme = eTCAuto);

    /**
    * @brief    Retrieves class id of IDOMTIFFImage
    * @return   @b CClassID Class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMTIFFImageClassID);
        return id;
    };
};
#define edlobj2IDOMTIFFImage(src) edl_cast((IDOMTIFFImage *)NULL, src) 


/**
* @class    IDOMWMPImage
* @ingroup  IDom
* @brief    IDOMWMPImage interface
*/
class IDOMWMPImage : public IDOMImage
{
#define IDOMWMPImageClassID 0x09bda5e5, 0x58cf4912, 0xbee4617d, 0x2c201426
public:

    /**
    * @brief    Retrieves class id of IDOMWMPImage
    * @return   @b CClassID Class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMWMPImageClassID);
        return id;
    };
};

DECL_SMART_PTR(IDOMWMPImage);
#define edlobj2IDOMWMPImage(src) edl_cast((IDOMWMPImage *)NULL, src) 


/**
* @class    IDOMRawImage
* @ingroup  IDom
* @brief    Interface to a class representing a raw image.

* In EDL, a raw image is an image which is represented by raw image pixels, capable of handling
* common bit depths and any colour space.
*/
class IDOMRawImage : public IDOMImage
{
#define IDOMRawImageClassID 0x373a5278, 0x136411dc, 0x83140800, 0x200c9a66
public:

    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public IDOMImage::Data
    {
    public:
        Data(): isSynthetic(false)
        {
            imageType = eDOMImageTypeUnknown;
        }
        bool                isSynthetic;
        IDOMColorSpacePtr   colorSpace;
        eDOMImageType       imageType;
    };

    /**
    * @brief    Retrieves class id of IDOMRawImage
    * @return   @b CClassID Class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMRawImageClassID);
        return id;
    };

    /**
    * @brief    Create an IDOMRawImage and frame that can be used to populate same.
    * @param    session The session to use
    * @param    frame On exit, this is populated with a frame ready to receive image data via
    *           frame->writeScanLine(). Use frame->flushData() to complete the encoding
    *           process.
    * @param    colorSpace The color space to use. Any valid color space may be used for
    *           IDOMRawImages.
    * @param    width The width of the image, in pixels.
    * @param    height The height of the image, in pixels.
    * @param    bitsPerComponent The bits per component to use. 1, 2, 4, 8, 12 and 16 bits per
    *           sample are supported.
    * @param    xResolution The x resolution, in pixels-per-inch.
    * @param    yResolution The y resolution, in pixels-per-inch.
    * @param    extraChannel The type of extra channel, if provided.
    * @param    inStream Optional. The first in a pair of streams used to read and write the
    *           raw image data if an external stream is desired. If NULL, a temporary store
    *           stream will be created. If non NULL, outStream must also be provided.
    * @param    outStream Optional. The second in a pair of streams used to read and write the
    *           raw image data if an external stream is desired. If NULL, a temporary store
    *           stream will be created. If non NULL, inStream must also be provided.
    * @return   @b IDOMImagePtr The resulting image. Not valid until the frame is flushed.
    */
    static EDL_API IDOMImagePtr createWriterAndImage(const ISessionPtr &session,
                                                     IImageFrameWriterPtr &frame,
                                                     const IDOMColorSpacePtr &colorSpace,
                                                     int32 width, int32 height,
                                                     uint8 bitsPerComponent = 8,
                                                     double xResolution = 96.0, double yResolution = 96.0,
                                                     ImageExtraChannel_t extraChannel = ImageExtraChannel_None,
                                                     const IInputStreamPtr &inStream = IInputStreamPtr(),
                                                     const IOutputStreamPtr &outStream = IOutputStreamPtr());

    /**
    * @brief    Returns a Boolean value indicating whether or not the image is synthetic.
    * @return   @b bool Returns true if the image is synthetic, false otherwise.
    */
    virtual bool getSynthetic() const = 0;
};

DECL_SMART_PTR(IDOMRawImage);
#define edlobj2IDOMRawImage(src) edl_cast((IDOMRawImage *)NULL, src) 



class IDOMPDFImage;
DECL_SMART_PTR(IDOMPDFImage);

/**
* @class    IDOMPDFAlternateImage
* @ingroup  IDom
* @brief    An encapsulation of a PDF alternate image, which may be referenced from
*           an image or mask brush.
*/
class IDOMPDFAlternateImage : public IEDLObject, public IDOMHashable
{
#define IDOMPDFAlternateImageClassID 0x09cfB9ef, 0xdf084a30, 0x963979B6, 0x6f01eee2
  public:

    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
      public:
        Data() : defaultForPrinting(false) {}
        virtual ~Data() {}
        IEDLClassFactory                    *pFactory;
        IDOMImageBrushPtr                    alternateBrush;
        JawsMako::IOptionalContentDetailsPtr optionalContentDetails;
        bool                                 defaultForPrinting;
    };
    
    /**
    * @brief   Creator for an alternate image.
    * @detail  Although regular, soft mask and masked images are allowed, there are several
    *          restrictions on the brushes that can be used for alternate images in order
    *          to ensure correct operation.
    *          - The render transform of the alternate brush must be identity.
    *          - The viewport of the alternate brush must match the viewport of the host brush.
    *            Note - these two restrictions are in place as the alternate image must be the same
    *            final dimensions as the host brush.
    *          - The viewbox of the alternate brush must select the entire image.
    *          - If the alternate brush is a masked brush using an image sub brush, the sub
    *            brush is subject to all the restrictions above.
    *          - Alternate images are ignored if the host image brush has tiling enabled
    *          - Optional content and alternate image information must not be present in the
    *            alternate brush.
    * @param   pFactory The class factory to use
    * @param   alternateBrush The alternate image brush itself. It must not have any alternate images
    *          of its own
    * @param   defaultForPrinting Set true if this image should be the default image when printing
    * @param   optionalContentDetails The optional content details to use, or NULL if not optional
    * @return  @b IDOMPDFAlternateImagePtr The new alternate image
    */
    static IDOMPDFAlternateImagePtr create(IEDLClassFactory *pFactory,
                                           const IDOMImageBrushPtr &alternateBrush,
                                           bool defaultForPrinting = false,
                                           const JawsMako::IOptionalContentDetailsPtr &optionalContentDetails = JawsMako::IOptionalContentDetailsPtr());
    
    /**
     *  @brief   Retrieves class id of IDOMPDFAlternateImage
     *  @return  @b CClassID Class id of the element
     */
    static const CClassID &classID()
    {
        static CClassID id(IDOMPDFAlternateImageClassID);
        return id;
    };

    /**
    * @brief Get the image brush for this alternate image
    * @return @b IDOMImageBrushPtr The image, or NULL on failure
    */
    virtual IDOMImageBrushPtr getImageBrush() const = 0;
    
    /**
    * @brief Get if the alternate image is marked as default for printing
    * @return @b bool True if the image is default for printing, false otherwise
    */
    virtual bool getDefaultForPrinting() const = 0;
    
    /**
    * @brief Get the optional content properties, if present
    * @return @b JawsMako::IOptionalContentDetailsPtr The optional content defails, or NULL if not optional
    */
    virtual JawsMako::IOptionalContentDetailsPtr getOptionalContentDetails() const = 0;
};


/**
* @class    IDOMPDFImage
* @ingroup  IDom
* @brief    Interface to a class representing an image extracted from
*           a PDF file. Intended to be only used with the JawsMako APIs
*/
class IDOMPDFImage : public IDOMImage
{
#define IDOMPDFImageClassID 0x865d5367, 0x8fee4078, 0xadd975ad, 0x5106ee38
public:
    
    /**
    * @class    IDecodeParams
    * @ingroup  IDom
    * @brief    Abstract interface for per-image decoding filter parameters
    */
    class IDecodeParams : public IRCObject
    {
      public:
        virtual ~IDecodeParams() {}
    };
    DECL_SMART_PTR(IDecodeParams);
    
    
    /**
    * @class    FlateLZWParams
    * @ingroup  IDom
    * @brief    Class to hold filter parameters for Flate or LZW-compressed
    *           image data. Please see the PDF specification for the meaning
    *           of these parameters.
    */
    class FlateLZWParams;
    DECL_SMART_PTR(FlateLZWParams);
    class FlateLZWParams : public IDecodeParams
    {
      public:
        static EDL_API FlateLZWParamsPtr create(uint32 predictor = 1, uint32 colors = 1,
                                                uint32 bpc = 8, uint32 columns = 1, uint32 earlyChange = 1);
        uint32 predictor;
        uint32 colors;
        uint32 bitsPerComponent;
        uint32 columns;
        uint32 earlyChange;
    };
    #define obj2FlateLZWParams(obj) IDOMPDFImage::FlateLZWParamsPtr(dynamic_cast<IDOMPDFImage::FlateLZWParams *>((IRCObject *) obj), true)
    
    /**
    * @class    CCITTFaxParams
    * @ingroup  IDom
    * @brief    Class to hold filter parameters for CCITTFax-compressed
    *           image data. Please see the PDF specification for the meaning
    *           of these parameters.
    */
    class CCITTFaxParams;
    DECL_SMART_PTR(CCITTFaxParams);
    class CCITTFaxParams : public IDecodeParams
    {
      public:
        static EDL_API CCITTFaxParamsPtr create(int32 k = 0, bool endOfLine = false, bool encodedByteAlign = false,
                                                uint32 columns = 1728, uint32 rows = 0, bool endOfBlock = true,
                                                bool blackIs1 = false);
        int32  k;
        bool   endOfLine;
        bool   encodedByteAlign;
        uint32 columns;
        uint32 rows;
        bool   endOfBlock;
        bool   blackIs1;
    };
    #define obj2CCITTFaxParams(obj) IDOMPDFImage::CCITTFaxParamsPtr(dynamic_cast<IDOMPDFImage::CCITTFaxParams *>((IRCObject *) obj), true)
    
    /**
    * @class    JBIG2Params
    * @ingroup  IDom
    * @brief    Class to hold filter parameters for JBIG2-compressed
    *           image data. Please see the PDF specification for the meaning
    *           of these parameters.
    */
    class JBIG2Params;
    DECL_SMART_PTR(JBIG2Params);
    class JBIG2Params : public IDecodeParams
    {
      public:
        static EDL_API JBIG2ParamsPtr create(const IInputStreamPtr &globals = IInputStreamPtr());
        IInputStreamPtr globals;
    };
    #define obj2JBIG2Params(obj) IDOMPDFImage::JBIG2ParamsPtr(dynamic_cast<IDOMPDFImage::JBIG2Params *>((IRCObject *) obj), true)
    
    /**
    * @class    DCTParams
    * @ingroup  IDom
    * @brief    Class to hold filter parameters for DCT-compressed
    *           image data. Please see the PDF specification for the meaning
    *           of these parameters.
    */
    class DCTParams;
    DECL_SMART_PTR(DCTParams);
    class DCTParams : public IDecodeParams
    {
      public:
        static EDL_API DCTParamsPtr create(int32 colorTransform = 0);
        uint32 colorTransform;
    };
    #define obj2DCTParams(obj) IDOMPDFImage::DCTParamsPtr(dynamic_cast<IDOMPDFImage::DCTParams *>((IRCObject *) obj), true)
    
    typedef enum
    {
        eIANone,
        eIAAlpha,
        eIAPremultipliedAlpha
    } eImageAlpha;
    
    /**
     * @class    Data
     * @ingroup  IDom
     * @brief    Initialization data
     */
    class Data : public IDOMImage::Data
    {
      public:
        Data() : imageId(INVALID_DOM_ID) {}
        virtual ~Data() {}
        IDecodeParamsPtr                     decodeParams;
        IDOMColorSpacePtr                    colorSpace;
        int32                                width;
        int32                                height;
        uint32                               bitsPerComponent;
        eImageAlpha                          alpha;
        CEDLVector<float>                    decode;
        CEDLVector<uint16>                   colorKeyForMask;
        DOMid                                imageId; // Note: Excluded from hash()
    };
    
    static IDOMPDFImagePtr create(IEDLClassFactory *pFactory,
                                  const IInputStreamPtr &stream,
                                  eDOMImageType imageType,
                                  const IDecodeParamsPtr &decodeParams,
                                  const IDOMColorSpacePtr &colorSpace,
                                  int32 width, int32 height,
                                  uint32 bitsPerComponent,
                                  const CEDLVector<float> &decode = CEDLVector<float>(),
                                  const CEDLVector<uint16> &colorKeyForMask = CEDLVector<uint16>(),
                                  eImageAlpha alpha = eIANone);
    
    /**
     *  @brief   Retrieves class id of IDOMPDFImage
     *  @return  @b CClassID Class id of the element
     */
    static const CClassID &classID()
    {
        static CClassID id(IDOMPDFImageClassID);
        return id;
    };
    
    /**
     *  @brief    Retrieves the decode parameters. May be NULL.
	 *  @return   @b IDecodeParamsPtr The decode parameters
     */
    virtual IDecodeParamsPtr getDecodeParameters() const = 0;

    /**
     *  @brief    Retrieves the decode array used to interpret the color samples.
     *            An empty vector will be returned if there is no decode.
     *  @return   @b CEDLVector<float> The decode array
     */
    virtual CEDLVector<float> getDecode() const = 0;

    /**
     *  @brief   Retrieves the bits per component of the source data.
     *           For JPEG2000, this information is available only in the
     *           image stream and this function will return 0. In this case
     *           please use the @c IImageStream interface.
     *  @return  @b uint32 The number of bits per component
     */
    virtual uint32 getBitsPerComponent() const = 0;

    /**
     *  @brief    Retrieves the color key for mask generation.
     *            An empty vector will be returned if there is no color key.
     *  @return   @b CEDLVector<uint16> The color key
     */
    virtual CEDLVector<uint16> getColorKey() const = 0;

    /**
     *  @brief    Retrieves the color space to be used with this image.
     *            For JPEG2000, this may be NULL if the colorspace needs
     *            to be determined from the image data. In this case
     *            please use the IImageStream interface.
     *  @return   @b IDOMColorSpacePtr The color space
     */
    virtual IDOMColorSpacePtr getColorSpace() const = 0;
    
    /**
     *  @brief     Returns if the image has alpha, and if the colour samples
     *             are premultiplied.
     *  @return    @b eImageAlpha The image alpha details
     */
    virtual eImageAlpha getAlphaDetails() const = 0;
};
#define edlobj2IDOMPDFImage(src) edl_cast((IDOMPDFImage *)NULL, src)

class IDOMPCLImage;
DECL_SMART_PTR(IDOMPCLImage);

/**
* @class    IDOMPCLImage
* @ingroup  IDom
* @brief    Interface to a class representing an image extracted from
*           a PCLXL file
*/
class IDOMPCLImage : public IDOMImage
{
#define IDOMPCLImageClassID 0x1444956E, 0xec014100, 0x9bec3968, 0xfe976935
public:
    
    /**
    * @brief    Retrieves class id of IDOMCLImage
    * @return   @b CClassID Class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMPCLImageClassID);
        return id;
    };

    /**
    * @brief The image compression mode.
    */
    typedef enum
    {
        eCMNoCompression = 0,
        eCMRleCompression = 1,
        eCMDeltaRowCompression = 3
    } eCompressMode;
    
    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public IDOMImage::Data
    {
      public:
        Data() : compressionMode(eCMNoCompression), bitsPerComponent(8), padding(0), width(0), height(0) {}
        eCompressMode     compressionMode;
        IDOMColorSpacePtr colorSpace;
        IInputStreamPtr   stream;
        uint8             bitsPerComponent;
        int32             width;
        int32             height;
        uint8             padding;
    };
    
    static IDOMPCLImagePtr create(IEDLClassFactory *pFactory,
                                  const IInputStreamPtr &stream,
                                  const IDOMColorSpacePtr &colorSpace,
                                  uint8 bitsPerComponent,
                                  int32 width, int32 height,
                                  uint8 padding,
                                  eCompressMode compressionMode);
};
#define edlobj2IDOMPCLImage(src) edl_cast((IDOMPCLImage *)NULL, src)

/**
* @class    IDOMRecombineImage
* @ingroup  IDom
* @brief    Interface to a class representing a image made up of separate single channel images
*           (each with the same bps, dimensions and resolution) each representing a single
*           component of the entire image, or a mask channel.
*/
class IDOMRecombineImage : public IDOMImage
{
#define IDOMRecombineImageClassID 0xe91e20cc, 0xbe2d469e, 0x8bad9d10, 0xca68Ad90
public:

    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public IDOMImage::Data
    {
    public:
        Data()
        {
            imageType = eDOMImageTypeUnknown;
            extraChannelIsMask = false;
        }
        IDOMColorSpacePtr        colorSpace;
        CEDLVector<IDOMImagePtr> componentImages;
        IDOMImagePtr             extraChannel;
        bool                     extraChannelIsMask; /** If true, any extra channel image is treated as a mask,
                                                      *  otherwise it is treated as an alpha channel.
                                                      */
        
    };

    /**
    * @brief    Retrieves class id of IDOMRecombineImage
    * @return   @b CClassID Class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMRecombineImageClassID);
        return id;
    };

    /**
     *  @brief   This image type does not allow direct access to the
     *           underlying streams.
     *  @param   stream A smart pointer to the stream
     *  @return  @b bool Always false
     */
    virtual bool getStream(IInputStreamPtr &stream) const
    {
        UNREFERENCED_PARAMETER(stream);
        return false;
    }

    /**
     *  @brief   This image type does not allow direct access to the
     *           underlying streams.
     *  @param   stream A smart pointer to the stream
     *  @return  @b bool Always false
     */
    virtual bool setStream(const IInputStreamPtr &stream)
    {
        UNREFERENCED_PARAMETER(stream);
        return false;
    }
};

DECL_SMART_PTR(IDOMRecombineImage);
#define edlobj2IDOMRecombineImage(src) edl_cast((IDOMRecombineImage *)NULL, src) 

class IDOMRecombineAlphaImage;
DECL_SMART_PTR(IDOMRecombineAlphaImage);

/**
* @class    IDOMRecombineAlpha
* @ingroup  IDom
* @brief    Similar to IDOMRecombineImage, but instead combines an image comprising
*           the colour components of the image, with a single-channel image that
*           represents the mask or alpha channel. The images must have the same,
*           dimensions, but may have different dimensions. The resolution information
*           will be taken from the colour image. Images with Indexed colour spaces
*           will be converted to the base spaces.
*/
class IDOMRecombineAlphaImage : public IDOMImage
{
#define IDOMRecombineAlphaImageClassID 0x4270168b, 0xf1384655, 0x86cbb981, 0x2d47dbc0
public:
    
    /**
     * @brief    Simplified creator for a IDOMRecombineAlphaImage.
     *           Throws an IEDLError on failure.
     * @param    pFactory The EDL Class Factory
     * @param    colorImage The image with the colour componens
     * @param    extraChannel The image with the mask/alpha channel
     * @param    extraChannelIsMask If true, the extra channel will be noted as a mask rather than alpha
     * @return   @b IDOMRecombineAlphaImagePtr The combined result.
     */
    static EDL_API IDOMRecombineAlphaImagePtr create(IEDLClassFactory *pFactory, const IDOMImagePtr &colorImage, const IDOMImagePtr &extraChannel, bool extraChannelIsMask = false);

    /**
     * @class    Data
     * @ingroup  IDom
     * @brief    Initialization data
     */
    class Data : public IDOMImage::Data
    {
    public:
        Data()
        {
            imageType = eDOMImageTypeUnknown;
            extraChannelIsMask = false;
        }
        IDOMImagePtr             colorImage;
        IDOMImagePtr             extraChannel;
        bool                     extraChannelIsMask; /** If true, any extra channel image is treated as a mask,
                                                      *  otherwise it is treated as an alpha channel.
                                                      */
        
    };
    
    /**
     * @brief    Retrieves class id of IDOMRecombineAlphaImage
     * @return   @b CClassID Class id of the element
     */
    static const CClassID &classID()
    {
        static CClassID id(IDOMRecombineAlphaImageClassID);
        return id;
    };
    
    /**
     *  @brief   This image type does not allow direct access to the
     *           underlying streams.
     *  @param   stream A smart pointer to the stream
     *  @return  @b bool Always false
     */
    virtual bool getStream(IInputStreamPtr &stream) const
    {
        UNREFERENCED_PARAMETER(stream);
        return false;
    }
    
    /**
     *  @brief   This image type does not allow direct access to the
     *           underlying streams.
     *  @param   stream A smart pointer to the stream
     *  @return  @b bool Always false
     */
    virtual bool setStream(const IInputStreamPtr &stream)
    {
        UNREFERENCED_PARAMETER(stream);
        return false;
    }
};

#define edlobj2IDOMRecombineAlphaImage(src) edl_cast((IDOMRecombineAlphaImage *)NULL, src)

class IDOMCompositeImage;
DECL_SMART_PTR(IDOMCompositeImage);

/**
* @class    IDOMCompositeImage
* @ingroup  IDom
* @brief    Interface to a class representing a image made up of separate images joined
*           together vertically, appearing as a single image. All images must use the
*           same color space, depth, width, and the same number of channels.
*/
class IDOMCompositeImage : public IDOMImage
{
#define IDOMCompositeImageClassID 0x95f3e000, 0xc9224e4a, 0x8c31b784, 0xe2e5b860
public:

    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public IDOMImage::Data
    {
    public:
        Data()
        {
            imageType = eDOMImageTypeUnknown;
        }
        CEDLVector<IDOMImagePtr> componentImages;
    };

    /**
     * @brief    Simplified creator for a IDOMCompositeImage.
     *           Throws an IEDLError on failure.
     * @param    pFactory The EDL Class Factory
     * @param    componentImages The images to be concatenated
     * @return   @b IDOMCompositeImagePtr The combined result.
     */
    static EDL_API IDOMCompositeImagePtr create(IEDLClassFactory *pFactory, const CEDLVector<IDOMImagePtr> componentImages);

    /**
    * @brief    Retrieves class id of IDOMRecombineImage
    * @return   @b CClassID Class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMCompositeImageClassID);
        return id;
    };

    /**
     *  @brief   This image type does not allow direct access to the
     *           underlying streams.
     *  @param   stream A smart pointer to the stream
     *  @return  @b bool Always false
    */
    virtual bool getStream(IInputStreamPtr &stream) const
    {
        UNREFERENCED_PARAMETER(stream);
        return false;
    }

    /**
     *  @brief   This image type does not allow direct access to the
     *           underlying streams.
     *  @param   stream A smart pointer to the stream
     *  @return  @b bool Always false
     */
    virtual bool setStream(const IInputStreamPtr &stream)
    {
        UNREFERENCED_PARAMETER(stream);
        return false;
    }
};

DECL_SMART_PTR(IDOMCompositeImage);
#define edlobj2IDOMCompositeImage(src) edl_cast((IDOMCompositeImage *)NULL, src)


/**
* @class    IDOMImageFilter
*/

class IDOMImageFilter : public IEDLObject
{
public:
    /**
     * @brief Get a filtered Image frame, based on an input frame.
     * @param pFactory The edl class factory.
     * @param sourceFrame The image frame the filter will source image data from.
     * @param filteredFrame A reference to receive the newly created frame.
     * @return @b bool True on success, false if the call fails.
     */
    virtual bool createFilteredImageFrame(IEDLClassFactory *pFactory, IImageFramePtr &sourceFrame, IImageFramePtr &filteredFrame) = 0;

    /**
    * @brief    Calculates a 64-bit hash for the filter parameters. Used
    *           to check for likely identical filter chains.
    * @param    hash A reference to receive the hash value
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool hashParameters (uint64 &hash) = 0;
};
DECL_SMART_PTR(IDOMImageFilter);
#define edlobj2IDOMImageFilter(src) edl_cast((IDOMImageFilter *)NULL, src) 

class IDOMImageChannelSelectorFilter;
DECL_SMART_PTR(IDOMImageChannelSelectorFilter);

/**
* @class IDOMImageChannelSelectorFilter
* @brief An image filter that presents optionally an image stripped of alpha,
*        or alternatively a Gray image representing the extra channel (ie Alpha or Mask).
*/
class IDOMImageChannelSelectorFilter : public IDOMImageFilter
{
#define IDOMImageChannelSelectorFilterClassID 0x014F52D9, 0x36284F76, 0xA2ED4585, 0xA0AC95CD
public:
    /**
    * @brief  Enum for choosing which channel(s) to select during filtering.
    */
    typedef enum
    {
        eSelectExtraChannel,        // Device Gray with alpha/mask values as gray values
        eSelectExtraChannelAsAlpha, // Presents a gray image with alpha retained as alpha
        eSelectColorChannels        // Presents a plain image with the alpha/mask stripped
    } eSelection;

    /**
     * @brief    Simplified creator for a color converter filter.
     *           Throws an IEDLError on failure.
     * @param    pFactory The EDL Class Factory
     * @param    selection Which channel(s) to retain during filtering.
     * @return   @b IDOMImageChannelSelectorFilterPtr The new filter.
     */
    static EDL_API IDOMImageChannelSelectorFilterPtr create(IEDLClassFactory *pFactory, eSelection selection);

    /**
    * @brief    Retrieves class id of IDOMImageChannelSelectorFilter
    * @return   @b CClassID Class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMImageChannelSelectorFilterClassID);
        return id;
    };
    
    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        eSelection selection;
    };
    
};
#define edlobj2IDOMImageChannelSelectorFilter(src) edl_cast((IDOMImageChannelSelectorFilter *)NULL, src) 


class IDOMImageColorSpaceSubstitutionFilter;
DECL_SMART_PTR(IDOMImageColorSpaceSubstitutionFilter);

/**
* @class IDOMImageColorSpaceSubstitutionFilter
* @brief An image filter that presents an identical image, just with the colourspace substituted.
*/
class IDOMImageColorSpaceSubstitutionFilter : public IDOMImageFilter
{
#define IDOMImageColorSpaceSubstitutionFilterClassID 0x70CF33A5, 0xF8D24BB2, 0x97B3A27A, 0xEB957EF3
public:
    /**
     * @brief    Simplified creator for a color space substitutor filter.
     *           Throws an IEDLError on failure.
     * @param    pFactory The EDL Class Factory
     * @param    colorSpace The color space to substitute.
     * @return   @b IDOMImageColorSpaceSubstitutionFilterPtr The new filter.
     */
    static EDL_API IDOMImageColorSpaceSubstitutionFilterPtr create(IEDLClassFactory *pFactory, const IDOMColorSpacePtr &colorSpace);

    /**
    * @brief    Retrieves class id of IDOMImageColorSpaceSubstitutionFilterClassID
    * @return   @b CClassID Class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMImageColorSpaceSubstitutionFilterClassID);
        return id;
    };
    
    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        IDOMColorSpacePtr substituteColorSpace;
    };
    
};
#define edlobj2IDOMImageColorSpaceSubstitutionFilter(src) edl_cast((IDOMImageColorSpaceSubstitutionFilter *)NULL, src) 

class IDOMImageColorConverterFilter;
DECL_SMART_PTR(IDOMImageColorConverterFilter);

/**
* @class IDOMImageColorConverterFilter
* @brief An image filter that presents a colour converted version of an image.
*/
class IDOMImageColorConverterFilter : public IDOMImageFilter
{
#define IDOMImageColorConverterFilterClassID 0xCF41C4A8, 0xB63A46A0, 0x8F1648BC, 0xB266EC94
public:
    /**
     * @brief    Simplified creator for a color converter filter.
     *           Throws an IEDLError on failure.
     * @param    pFactory The EDL Class Factory
     * @param    space The target color space. Must be a base space; ie no DeviceN or Indexed spaces.
     * @param    intent The rendering intent to use
     * @param    bpc The desired black point compensation treatment. If in doubt, use eBPCDefault.
     * @return   @b IDOMImageColorConverterFilterPtr The new filter.
     */
    static EDL_API IDOMImageColorConverterFilterPtr create(IEDLClassFactory *pFactory, const IDOMColorSpacePtr &space, eRenderingIntent intent, eBlackPointCompensation bpc);


    /**
    * @brief    Retrieves class id of IDOMImageColorConverterFilter
    * @return   @b CClassID Class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMImageColorConverterFilterClassID);
        return id;
    };
    
    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        Data() : renderingIntent(ePerceptual) {}
        IDOMColorSpacePtr       destinationColorSpace;
        eRenderingIntent        renderingIntent;
        eBlackPointCompensation bpc;
    };
    
};
#define edlobj2IDOMImageColorConverterFilter(src) edl_cast((IDOMImageColorConverterFilter *)NULL, src) 

class IDOMImageBleederFilter;
DECL_SMART_PTR(IDOMImageBleederFilter);

/**
* @class IDOMImageBleederFilter
* @brief An image filter that presents an image with the edge pixels repeated. Useful for
*        cases where consumers may interpolate pixels at the edge, creating unwanted artifacts.
*/
class IDOMImageBleederFilter : public IDOMImageFilter 
{
#define IDOMImageBleederFilterClassID 0xB31F7AA7, 0x38F043B8, 0xAC94C932, 0xCE38D04C
public:
    /**
     * @brief    Simplified creator for an image bleeder.
     *           Throws an IEDLError on failure.
     * @param    pFactory The EDL Class Factory
     * @param    bleedPixels The number of pixels to bleed the image.
     * @return   @b IDOMImageBleederFilterPtr The new filter.
     */
    static EDL_API IDOMImageBleederFilterPtr create(IEDLClassFactory *pFactory, uint16 bleedPixels = 1);

    /**
    * @brief    Retrieves class id of IDOMImageBleederFilter
    * @return   @b CClassID Class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMImageBleederFilterClassID);
        return id;
    };
    
    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        Data() : bleedPixels(1) {}
        uint16 bleedPixels;
    };
    
};
#define edlobj2IDOMImageBleederFilter(src) edl_cast((IDOMImageBleederFilter *)NULL, src) 


class IDOMImageDownsamplerFilter;
DECL_SMART_PTR(IDOMImageDownsamplerFilter);

/**
* @class IDOMImageDownsamplerFilter
* @brief An image filter that presents a downsampled version of an image.
*/
class IDOMImageDownsamplerFilter : public IDOMImageFilter 
{
#define IDOMImageDownsamplerFilterClassID 0x4DAECA20, 0x882B4B32, 0xA136C61B, 0xAA0F626A
public:
    /**
    * @brief    The type of downsampling to be performed.
    */
    typedef enum
    {
        eSubsample,
        eAverage,
        eBicubic,
    } eDownsamplingMethod;
        

    /**
     * @brief    Simplified creator for an image downsampler filter.
     *           Throws an IEDLError on failure.
     * @param    pFactory The EDL Class Factory
     * @param    xRes The desired x resolution of the result.
     * @param    yRes The desired y resolution of the result. If 0, the x resolution will be used.
     * @param    method The desired downsampling method to use.
     * @return   @b IDOMImageDownsamplerFilterPtr The new filter.
     */
    static EDL_API IDOMImageDownsamplerFilterPtr create(IEDLClassFactory *pFactory, double xRes, double yRes = 0.0, eDownsamplingMethod method = eSubsample);

    /**
    * @brief    Retrieves class id of IDOMImageDownsamplerFilter
    * @return   @b CClassID Class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMImageDownsamplerFilterClassID);
        return id;
    };
    
    
    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        Data() : desiredXRes(0.0), desiredYRes(0.0), method(eSubsample) {}
        double              desiredXRes;
        double              desiredYRes;
        eDownsamplingMethod method;
    };
    
};
#define edlobj2IDOMImageDownsamplerFilter(src) edl_cast((IDOMImageDownsamplerFilter *)NULL, src) 


class IDOMImageMaskExpanderFilter;
DECL_SMART_PTR(IDOMImageMaskExpanderFilter);

/**
* @brief An image filter that presents a image source and color combination as a plain
*        image with an alpha channel, with all pixels colored with the given color.
*        Useful for simplifying a IDOMMaskedBrush where the brush masked by the image
*        is a solid color.
*/
class IDOMImageMaskExpanderFilter : public IDOMImageFilter 
{
#define IDOMImageMaskExpanderFilterClassID 0xC1262ED1, 0x5AB2451C, 0x9F921341, 0xA89262FF
public:

    /**
     * @brief    Simplified creator for a mask expander filter.
     *           Throws an IEDLError on failure.
     * @param    pFactory The EDL Class Factory
     * @param    color The color to use.
     * @return   @b IDOMImageMaskExpanderFilterPtr The new filter.
     */
    static EDL_API IDOMImageMaskExpanderFilterPtr create(IEDLClassFactory *pFactory, const IDOMColorPtr &color);

    /**
    * @brief    Retrieves class id of IDOMImageMaskExpanderFilter
    * @return   @b CClassID Class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMImageMaskExpanderFilterClassID);
        return id;
    };
    
    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        IDOMColorPtr color;
    };
    
};
#define edlobj2IDOMImageMaskExpanderFilter(src) edl_cast((IDOMImageMaskExpanderFilter *)NULL, src) 

class IDOMImageDeindexerFilter;
DECL_SMART_PTR(IDOMImageDeindexerFilter);

/**
* @brief An image filter that presents an image with an Indexed colour space as a simple
*        eight bit image.
*/
class IDOMImageDeindexerFilter : public IDOMImageFilter 
{
#define IDOMImageDeindexerFilterClassID 0x5FD82F65, 0xAC16414C, 0xB98FDA8F, 0xB0A5A5BA
public:

    /**
     * @brief    Simplified creator for a deindexer filter.
     *           Throws an IEDLError on failure.
     * @param    pFactory The EDL Class Factory
     * @return   @b IDOMImageDeindexerFilterPtr The new filter.
     */
    static EDL_API IDOMImageDeindexerFilterPtr create(IEDLClassFactory *pFactory);

    /**
    * @brief    Retrieves class id of IDOMImageDeindexerFilter
    * @return   @b CClassID Class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMImageDeindexerFilterClassID);
        return id;
    };
    
    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
    };
    
};
#define edlobj2IDOMImageDeindexerFilter(src) edl_cast((IDOMImageDeindexerFilter *)NULL, src) 


class IDOMImageDeviceNToBaseFilter;
DECL_SMART_PTR(IDOMImageDeviceNToBaseFilter);

/**
* @brief An image filter that presents an image with a DeviceN colour space as a simple
*        image in the alternate space.
*/
class IDOMImageDeviceNToBaseFilter : public IDOMImageFilter 
{
#define IDOMImageDeviceNToBaseFilterClassID 0xA8BFA7A4, 0x3896470E, 0xBFF879DB, 0xE83F507A
public:
    /**
     * @brief    Simplified creator for this filter
     *           Throws an IEDLError on failure.
     * @param    pFactory The EDL Class Factory
     * @return   @b IDOMImageDeviceNToBaseFilterPtr The new filter.
     */
    static EDL_API IDOMImageDeviceNToBaseFilterPtr create(IEDLClassFactory *pFactory);

    /**
    * @brief    Retrieves class id of IDOMImageDeindexerFilter
    * @return   @b CClassID Class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMImageDeviceNToBaseFilterClassID);
        return id;
    };
    
    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
    };
    
};
#define edlobj2IDOMImageDeviceNToBaseFilter(src) edl_cast((IDOMImageDeviceNToBaseFilter *)NULL, src) 

class IDOMImageInverterFilter;
DECL_SMART_PTR(IDOMImageInverterFilter);

/**
* @class IDOMImageInverterFilter
* @brief An image filter that presents a bitwise inverted form of the source image.
*/
class IDOMImageInverterFilter : public IDOMImageFilter
{
#define IDOMImageInverterFilterClassID 0x46147105, 0x981F496D, 0xB1F4910F, 0xCBB59B52
public:

    /**
     * @brief    Simplified creator for this filter
     *           Throws an IEDLError on failure.
     * @param    pFactory The EDL Class Factory
     * @return   @b IDOMDePremultiplierFilterPtr The new filter.
     */
    static EDL_API IDOMImageInverterFilterPtr create(IEDLClassFactory *pFactory);

    /**
    * @brief    Retrieves class id of IDOMImageInverterFilter
    * @return   @b CClassID Class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMImageInverterFilterClassID);
        return id;
    };
    
    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
    };
    
};
#define edlobj2IDOMImageInverterFilter(src) edl_cast((IDOMImageInverterFilter *)NULL, src) 

class IDOMDePremultiplierFilter;
DECL_SMART_PTR(IDOMDePremultiplierFilter);

/**
* @class IDOMDePremultiplyFilter
* @brief An image filter that presents an image with premultiplied alpha as a plain
*        image with plain alpha. It can be applied to any source image, and will do
*        nothing if not required.
*/
class IDOMDePremultiplierFilter : public IDOMImageFilter
{
#define IDOMDePremultiplierFilterClassID 0x0fc321ce, 0xf33e4fa3, 0xb79dbab3, 0xe93e388f
public:
    /**
     * @brief    Simplified creator for this filter
     *           Throws an IEDLError on failure.
     * @param    pFactory The EDL Class Factory
     * @return   @b IDOMDePremultiplierFilterPtr The new filter.
     */
    static EDL_API IDOMDePremultiplierFilterPtr create(IEDLClassFactory *pFactory);

    /**
    * @brief    Retrieves class id of IDOMDePremultiplierFilter
    * @return   @b CClassID Class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMDePremultiplierFilterClassID);
        return id;
    };
    
    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
    };
    
};
#define edlobj2IDOMDePremultiplierFilter(src) edl_cast((IDOMDePremultiplierFilter *)NULL, src) 

class IDOMImageMatteRemoverFilter;
DECL_SMART_PTR(IDOMImageMatteRemoverFilter);

/**
 * @class IDOMMatteRemoverFilter
 * @brief An image filter that removes a Matte and undoes premultiplication for
 *        a PDF Matte'd image and soft mask. The resulting image does not have
 *        alpha, and can be used with the mask to generate the desired result.
 */
class IDOMImageMatteRemoverFilter : public IDOMImageFilter
{
#define IDOMImageMatteRemoverFilterClassID 0x3d883344, 0x1c134c8e, 0xb40f88bd, 0xaa427fd6
public:
    /**
     * @brief    Simplified creator for this filter
     *           Throws an IEDLError on failure.
     * @param    pFactory The EDL Class Factory
     * @return   @b IDOMMatteRemoverFilterPtr The new filter.
     */
    static EDL_API IDOMImageMatteRemoverFilterPtr create(IEDLClassFactory *pFactory,
                                                         const IDOMColorPtr &matteColor,
                                                         const IDOMImagePtr &maskImage);
    
    /**
     * @brief    Retrieves class id of IDOMMatteRemoverFilter
     * @return   @b CClassID Class id of the element
     */
    static const CClassID &classID()
    {
        static CClassID id(IDOMImageMatteRemoverFilterClassID);
        return id;
    };
    
    /**
     * @class    Data
     * @ingroup  IDom
     * @brief    Initialization data
     */
    class Data : public CClassParams
    {
      public:
        IDOMColorPtr matteColor;
        IDOMImagePtr maskImage;
    };
    
};
#define edlobj2IDOMImageMatteRemoverFilter(src) edl_cast((IDOMImageMatteRemoverFilter *)NULL, src)


class IDOMImageBitScalerFilter;
DECL_SMART_PTR(IDOMImageBitScalerFilter);

/**
* @brief An image filter that presents an image as an image with a different bits per sample.
*/
class IDOMImageBitScalerFilter : public IDOMImageFilter
{
#define IDOMImageBitScalerFilterClassID 0x4CCE45A6, 0xA1C54615, 0x9E477ADB, 0xED305706
public:
    /**
     * @brief    Simplified creator for a bit scaling filter.
     *           Throws an IEDLError on failure.
     * @param    pFactory The EDL Class Factory
     * @param    bps The desired bits per sample (1, 3, 4, 8, 12 or 16 bit)
     * @return   @b IDOMBitScalerFilterPtr The new filter.
     */
    static EDL_API IDOMImageBitScalerFilterPtr create(IEDLClassFactory *pFactory, uint8 bps);

    /**
    * @brief    Retrieves class id of IDOMImageBitScalerFilterClassID
    * @return   @b CClassID Class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMImageBitScalerFilterClassID);
        return id;
    };
    
    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        uint8 desiredBPS; /// 1, 2, 4, 8, 12 or 16 bits per sample
    };
    
};
#define edlobj2IDOMImageBitScalerFilter(src) edl_cast((IDOMImageBitScalerFilter *)NULL, src) 

class IDOMImageColorKeyFilter;
DECL_SMART_PTR(IDOMImageColorKeyFilter);

/**
* @brief An image filter that presents a masked image where colours within
*        a given range are masked out, analogous to a green screen.
*        The source image must not have a mask or alpha channel.
*/
class IDOMImageColorKeyFilter : public IDOMImageFilter
{
#define IDOMImageColorKeyFilterClassID 0x46129cf8, 0xff2c481c, 0xa7e0c9c7, 0x8865835b
public:
    /**
    * @brief    Simplified creator for a color key scaling filter.
    *           Throws an IEDLError on failure.
     * @param    pFactory The EDL Class Factory
    * @param    key A vector that describes the range of colours to be masked out.
    *           In the form low1, hi1, low2, hi2, ... lowN, hiN where N is the number of
    *           color components in the source image. The values are expressed in the range of
    *           the samples in the image. For example, if the image is 4 bits per component
    *           then the values in this vector need to be between 0 and 15 inclusive.
    * @return   @b IDOMImageColorKeyFilterPtr The new filter.
    */
    static EDL_API IDOMImageColorKeyFilterPtr create(IEDLClassFactory *pFactory, const CEDLVector<uint16> &key);
    
    /**
    * @brief    Retrieves class id of IDOMImageColorKeyFilter
    * @return   @b CClassID Class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMImageColorKeyFilterClassID);
        return id;
    };
    
    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        CEDLVector<uint16> key;
    };
    
};
#define edlobj2IDOMImageColorKeyFilter(src) edl_cast((IDOMImageColorKeyFilter *)NULL, src)

class IDOMImageDecodeFilter;
DECL_SMART_PTR(IDOMImageDecodeFilter);

/**
* @brief An image filter that applies a PDF/PS style Decode array to the image contents.
*        For details on decode arrays, please see "Decode Arrays" on page 344 of the
*        PDF Reference, version 1.7. The bit depth of the result may be promoted to
*        eight or 16 bits per component depending on the situation.
*/
class IDOMImageDecodeFilter : public IDOMImageFilter
{
#define IDOMImageDecodeFilterClassID 0x29a2d3a8, 0xe92b4752, 0xa6f44fa2, 0x65aba989
public:
    /**
    * @brief    Simplified creator for a decode filter.
    *           Throws an IEDLError on failure.
    * @param    pFactory The EDL Class Factory
    * @param    decode The decode vector.
    * @return   @b IDOMImageDecodeFilterPtr The new filter.
    */
    static EDL_API IDOMImageDecodeFilterPtr create(IEDLClassFactory *pFactory, const CEDLVector<float> &decode);
    
    /**
    * @brief    Retrieves class id of IDOMImageDecodeFilter
    * @return   @b CClassID Class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMImageDecodeFilterClassID);
        return id;
    };
    
    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        CEDLVector<float> decode;
    };
    
};
#define edlobj2IDOMImageDecodeFilter(src) edl_cast((IDOMImageDecodeFilter *)NULL, src)



class IDOMFilteredImage;
DECL_SMART_PTR(IDOMFilteredImage);

/**
* @class    IDOMFilteredImage
* @ingroup  IDom
* @brief    IDOMFilteredImage interface. Provides a method for filtering
*           of an underlying image without requiring converted image data
*           to be stored. It maintains a list of filters that are successively
*           applied.
*/
class IDOMFilteredImage : public IDOMImage
{
#define IDOMFilteredImageClassID 0xC4ACE9A2, 0xC45F4060, 0x98F9788A, 0x5D84C9F0
public:
    /**
     * @brief    Simplified creator for a filtered image with a single filter.
     *           Throws an IEDLError on failure.
     * @param    pFactory The EDL Class Factory
     * @param    sourceImage The source image.
     * @param    filter The filter to use.
     * @return   @b IDOMFilteredImagePtr The filtered image.
     */
    static EDL_API IDOMFilteredImagePtr create(IEDLClassFactory *pFactory, const IDOMImagePtr &sourceImage, const IDOMImageFilterPtr &filter);

    /**
     * @brief    Simplified creator for a filtered image with a vector of filters
     *           Throws an IEDLError on failure.
     * @param    pFactory The EDL Class Factory
     * @param    sourceImage The source image.
     * @param    filters The vector of filters to instantiate with.
     * @return   @b IDOMFilteredImagePtr The filtered image.
     */
    static EDL_API IDOMFilteredImagePtr create(IEDLClassFactory *pFactory, const IDOMImagePtr &sourceImage, const CEDLVector<IDOMImageFilterPtr> &filters);
    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public IDOMImage::Data
    {
    public:
        CEDLVector<IDOMImageFilterPtr> filters;
        IDOMImagePtr sourceImage;
    };

    /**
    * @brief    Retrieves class id of IDOMFilteredImage
    * @return   @b CClassID Class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMFilteredImageClassID);
        return id;
    };

    /**
    * @brief   Push a filter onto the end of the internal filter chain.
    * @param   filter A pointer to the image filter to be added.
    * @return  @b bool True on success.
    */
    virtual bool pushFilter(IDOMImageFilterPtr &filter) = 0;
    
    /**
    * @brief   Get the number of individual filters in the internal filter chain.
    * @return  uint32 The number of filters.
    */
    virtual uint32 getNumFilters() const = 0;
    
    /**
    * @brief   Get a filter from the filter chain at the given index.
    * @param   index Index of the desired filter in the chain (beginning at 0)
    * @param   filter Smart pointer to receive the filter.
    * @return  @b bool True on success, false otherwise.
    */
    virtual bool getFilterAtIndex(uint32 index, IDOMImageFilterPtr &filter) const = 0;
    
    /**
    * @brief   Get the source image.
    * @return  @b IDOMImagePtr Smart pointer to the source image.
    */
    virtual IDOMImagePtr getSourceImage() const = 0;
    
    
    /**
     *  @brief   This image type does not allow direct access to the
     *           underlying streams.
     *  @param   stream A smart pointer to the stream
     *  @return  @b bool Always false
     */
    virtual bool getStream(IInputStreamPtr &stream) const
    {
        UNREFERENCED_PARAMETER(stream);
        return false;
    }

    /**
     *  @brief   This image type does not allow direct access to the
     *           underlying streams.
     *  @param   stream A smart pointer to the stream
     *  @return  @b bool Always false
     */
    virtual bool setStream(const IInputStreamPtr &stream)
    {
        UNREFERENCED_PARAMETER(stream);
        return false;
    }
};
#define edlobj2IDOMFilteredImage(src) edl_cast((IDOMFilteredImage *)NULL, src) 

_END_EDL_DOM_NAMESPACE

#endif /* __IDOMIMAGERESOURCE_H__ */
