/*
* Copyright (C) 2007-2019 Global Graphics Software Ltd. All rights reserved.
*/

/**
*
* @file     idomjobtk.h
* @ingroup  IDom
* @brief    DOM Job Ticket Interface
*/

#ifndef IDOMJOBTK_H
#define IDOMJOBTK_H

#include <edl/edltypes.h>
#include <edl/edlqname.h>
#include <edl/edlgeom.h>
#include <edl/idomnode.h>
#include <edl/idomresources.h>
#include <edl/iedlfactory.h>
#include <edl/edlvector.h>

_BEGIN_EDL_DOM_NAMESPACE

#define edlobj2IDOMJobTkNode(src) edl_cast((IDOMJobTkNode *)NULL, src)
class IDOMJobTkNode;
DECL_SMART_PTR(IDOMJobTkNode);

#define edlobj2IDOMJobTkValue(src) edl_cast((IDOMJobTkValue *)NULL, src)
class IDOMJobTkValue;
DECL_SMART_PTR(IDOMJobTkValue);

#define edlobj2IDOMJobTkContent(src) edl_cast((IDOMJobTkContent *)NULL, src)
class IDOMJobTkContent;
DECL_SMART_PTR(IDOMJobTkContent);

#define edlobj2IDOMJobTk(src) edl_cast((IDOMJobTk *)NULL, src)
class IDOMJobTk;
DECL_SMART_PTR(IDOMJobTk);

#define PSK_NAMESPACE "http://schemas.microsoft.com/windows/2003/08/printing/printschemakeywords"
#define PSF_NAMESPACE "http://schemas.microsoft.com/windows/2003/08/printing/printschemaframework"
#define XSD_NAMESPACE "http://www.w3.org/2001/XMLSchema"
#define XSI_NAMESPACE "http://www.w3.org/2001/XMLSchema-instance"
#define MAKO_NAMESPACE "http://schemas.globalgraphics.com/mako/2018"
#define EDL_PIPELINE_NAMESPACE "http://schemas.globalgraphics.com/edl/pipeline/2008"

/**
* @enum     eDOMJobTkLevel
* @ingroup  IDom
* @brief    DOMJobTk Job Ticket Level (docseq, doc, page)
*/
typedef enum
{       
    eDOMJobTkLevelNotValid  = 0,    //!< Level value not initialized or invalid 
    eDOMJobTkLevelDefault   = 1,    //!< The default level of JobTicket
    eDOMJobTkLevelJob       = 2,    //!< The document sequence level of JobTicket
    eDOMJobTkLevelDoc       = 3,    //!< The document level of JobTicket
    eDOMJobTkLevelPage      = 4,    //!< The page level of JobTicket
    eDOMJobTkLevelCnt       = 5,    //!< The number of levels

} eDOMJobTkLevel;

/**
* @class    IDOMJobTkOwner
* @ingroup  IDom
* @brief    Interface to the IDOMJobTkOwner node.
*
* IDOMJobTkOwner is the immediate ancestor of the node types which can own a JobTicket. 
* These are the IDOMContentRoot, IDOMDocumentSequence, IDOMFixedDocument and IDOMFixedPage classes.
*/
class IDOMJobTkOwner : public IDOMNode
{
#define IDOMJobTkOwnerClassID 0xc67ada74, 0x105b455c, 0xbacd2c3b, 0xa3381713
public:

    /**
    * @brief    Retrieves the class id of IDOMJobTkOwner.
    * @return   CClassID. Class id of the element.
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMJobTkOwnerClassID);
        return id;
    }

    /**
    * @brief    Retrieves the JobTicket held by this node.
    * @param    ptrJobTicket Smart pointer receive a reference to the JobTicket.
    * @return   bool. Returns true on success, false if the call fails.
    */
    virtual bool getJobTicket(IDOMJobTkPtr &ptrJobTicket) const = 0;

    /**
    * @brief    Sets the JobTicket for the node.
    * @param    ptrJobTicket Smart pointer to the new JobTicket.
    * @return   bool. Returns true on success, false if the call fails.
    */
    virtual bool setJobTicket(const IDOMJobTkPtr &ptrJobTicket) = 0;

};

DECL_SMART_PTR(IDOMJobTkOwner);
#define edlobj2IDOMJobTkOwner(src) edl_cast((IDOMJobTkOwner *)NULL, src)

/**
* @class    IDOMJobTkGenericNode
* @ingroup  IDom
* @brief    Interface to the IDOMJobTkGenericNode node.
*
* IDOMJobTkGenericNode is a print ticket node designed to represent
* unhandled XML data in a print ticket. This allows such markup to
* be retained, interrogated, modified and subsequently written
* faithfully in a output print ticket. During XPS input processing,
* a node of this type is created for each tag that is not natively
* understood by EDL. The QName is the name of the tag (including
* namespace information) and the attributes are stored within
* the object.
*
* Nodes of this type may have IDOMJobTkGenericCharacterData children
* which represent CDATA owned by the tag.
*/
class IDOMJobTkGenericNode : public IDOMNode
{
#define IDOMJobTkGenericNodeClassID 0xAF6FD278, 0xC7C848CA, 0xA85C88FD, 0xD2773FD5
public:
    /**
    * @brief    Retrieves the class id of IDOMJobTkGenericNode.
    * @return   CClassID. Class id of the element.
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMJobTkGenericNodeClassID);
        return id;
    }

    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        EDLQName       name;
    };
    
    /**
    * @brief    Retrieves the node qname
    * @param    qname Qualified name value
    * @return   bool. Returns true on success, false if the call fails.
    */
    virtual bool getQName(EDLQName &qname) const = 0;
    
    /**
    * @brief    Adds an attribute
    * @param    qname Qualified name of the attribute
    * @param    value The value of the attribute as a string
    * @return   bool. Returns true on success, false if the call fails.
    */
    virtual bool addAttribute(const EDLQName &name, const EDLSysString &value) = 0;

    /**
    * @brief    Gets the number of attributes
    * @return   uint32 The number of attributes
    */
    virtual uint32 getNumAttributes() const = 0;
    
    /**
    * @brief    Get an attribute at the given index
    * @param    index The index of the attribute, with 0 representing the first attribute
    * @param    qname Reference to receive the qualified name of the attribute
    * @param    value Reference to receive the value of the attribute as a string
    * @return   bool. Returns true on success, false if the call fails.
    */
    virtual bool getAttributeAtIndex(uint32 index, EDLQName &name, EDLSysString &value) const = 0;
};
DECL_SMART_PTR(IDOMJobTkGenericNode);
#define edlobj2IDOMJobTkGenericNode(src) edl_cast((IDOMJobTkGenericNode *)NULL, src)

/**
* @class    IDOMJobTkGenericCharacterData
* @ingroup  IDom
* @brief    Interface to the IDOMJobTkGenericCharacterData node.
*
* Objects of type IDOMJobTkGenericCharacterData store CDATA associated with an
* IDOMJobTkGenericNode. Objects of this type are stored as children of 
* IDOMJobTkGenericNodes.
*/
class IDOMJobTkGenericCharacterData : public IDOMNode
{
#define IDOMJobTkGenericCharacterDataClassID 0xA9A7CE3E, 0x0927418C, 0xBAFAA86C, 0xD49324DC
public:
    /**
    * @brief    Retrieves the class id of IDOMJobTkGenericNode.
    * @return   CClassID. Class id of the element.
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMJobTkGenericCharacterDataClassID);
        return id;
    }

    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        EDLString  charData;
    };
    
    /**
    * @brief    Get the node's CDATA
    * @param    charData Reference to receive the CDATA
    * @return   bool. Returns true on success, false if the call fails.
    */
    virtual bool getCharacterData(EDLString &charData) const = 0;

    /**
    * @brief    Set the node's CDATA
    * @param    charData The CDATA as an EDLString
    * @return   bool. Returns true on success, false if the call fails.
    */
    virtual bool setCharacterData(const EDLString &charData) = 0;
    
    /**
    * @brief    Append to the node's CDATA
    * @param    charData The additional CDATA as an EDLString
    * @return   bool. Returns true on success, false if the call fails.
    */
    virtual bool appendCharacterData(const EDLString &charData) = 0;
};
DECL_SMART_PTR(IDOMJobTkGenericCharacterData);
#define edlobj2IDOMJobTkGenericCharacterData(src) edl_cast((IDOMJobTkGenericCharacterData *)NULL, src)

/**
* @class    IDOMJobTkNode
* @ingroup  IDom
* @brief    Represents a Job Ticket Node.
*/
class IDOMJobTkNode : public IDOMNode
{
#define IDOMJobTkNodeClassID 0x2AE77EA2, 0x14E111DC, 0x83140800, 0x200C9A66
public:

    /**
    * @enum     eDOMJobTkNodeType
    * @ingroup  IDom
    * @brief    DOMJobTk node (Property, Feature, Option, InitParam, ScoredProperty, ParamRef)
    */  
    typedef enum
    {
        eDOMJobTkNodeInValid,
        eDOMJobTkPTNodePrintTicket,
        eDOMJobTkPTNodePrintCapabilities,
        eDOMJobTkPTNodeParameterInit,
        eDOMJobTkPTNodeParameterRef,
        eDOMJobTkPTNodeParameterDef,
        eDOMJobTkPTNodeProperty,
        eDOMJobTkPTNodeFeature,
        eDOMJobTkPTNodeOption,
        eDOMJobTkPTNodeScoredProperty,
        eDOMJobTkPTNodeValue,
        eDOMJobTkPJLNodeName,
        eDOMJobTkPJLNodeValue
    } eDOMJobTkNodeType;

    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        EDLQName           name;           //!< Name of the node
        eDOMJobTkNodeType  jobTkNodeType;  //!< Job Ticket node type
    };

    /**
    * @brief    Retrieves the class id of IDOMJobTk.
    * @return   CClassID. Class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMJobTkNodeClassID);
        return id;
    }

    /**
    * @brief    Sets the qualified name of the node.
    *
    * URI references can contain characters not allowed in names, and are
    * often inconveniently long, so expanded names are not used directly to
    * name elements and attributes in XML documents. Instead qualified
    * names are used. A qualified name is a name subject to namespace interpretation.
    * @param    qname Qualified name value
    * @return   bool. Returns true on success, false if the call fails.
    */
    virtual bool setQName(const EDLQName &qname)  = 0;

    /**
    * @brief    Retrieves the node qname
    * @param    qname Qualified name value
    * @return   bool. Returns true on success, false if the call fails.
    */
    virtual bool getQName(EDLQName &qname) const = 0;

    /**
    * @brief    Retrieves the full qualified name of the node as a string.
    * @param    strQName Reference parameter to receive the node's qualified name as a string.
    * @return   bool. Returns true on success, false if the call fails.
    */
    virtual bool getQNameAsString(EDLString &strQName) const = 0;

    /**
    * @brief    Sets the node type.
    * @param    nodeType The node type. See eDOMJobTkNodeType.
    * @return   bool. Returns true on success, false if the call fails.
    */
    virtual bool setJobTkNodeType(eDOMJobTkNodeType nodeType) = 0;

    /**
    * @brief    Retrieves the node type.
    * @return   eDOMJobTkNodeType. Returns the node type.
    */
    virtual eDOMJobTkNodeType getJobTkNodeType() = 0;

    /**
    * @brief    Returns the JobTicket content of this node, by following up the parent node chain.
    * @return   IDOMJobTkContentPtr. Smart pointer to the JobTicket content interface
    */
    virtual IDOMJobTkContentPtr getJobTkContent() = 0;

    /**
    * @brief    If JobTicket node has child with type eDOMJobTkPTNodeValue then returns this child, otherwise returns NULL
    * @return   IDOMJobTkValuePtr. Smart pointer to the JobTicket value node interface
    */
    virtual IDOMJobTkValuePtr getChildValue() = 0;

    /**
    * @brief    Searches through the node's direct child set for a node matching the provided node type, name and namespace.
    * @param    nodeType Job ticket node type.
    * @param    name Name of JobTicket node.
    * @param    nmspace Namespace of JobTicket node.
    * @return   IDOMJobTkNodePtr. Smart pointer to the found JobTicket node
    */
    virtual IDOMJobTkNodePtr findChild(eDOMJobTkNodeType nodeType, const EDLString &name, const EDLSysString &nmspace) = 0;    

    /**
    * @brief    Searches through the node's direct child set for a node matching the provided node type and qname.
    * @param    nodeType Job ticket node type.
    * @param    qname Qualified name of JobTicket node.
    * @return   IDOMJobTkNodePtr. Smart pointer to the found JobTicket node
    */    
    virtual IDOMJobTkNodePtr findChild(eDOMJobTkNodeType nodeType, const EDLQName &qname) = 0;

};


/**
* @class    IDOMJobTkValue
* @ingroup  IDom
* @brief    Represents a Job Ticket value element.
*/
class IDOMJobTkValue : public IDOMNode
{
#define IDOMJobTkValueClassID 0x04EC0CCA, 0x151A11DC, 0x83140800, 0x200C9A66
public:

    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        PValue value;
    };

    /**
    * @brief    Retrieves the class id of IDOMJobTk.
    * @return   CClassID. The class id of the element.
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMJobTkValueClassID);
        return id;
    }

    /**
    * @brief    Returns an indicator of the validity of the data.
    * @return   bool. Returns true if the data has been initialised, false if it has not.
    */
    virtual bool isValid() const = 0;

    /**
    * @brief    Sets the node value.
    * @param    value The new node value.
    * @return   bool Returns true on success.
    */
    virtual bool setValue(const PValue &value) = 0;

    /**
    * @brief    Retrieves the node value.
    * @return   bool Returns true on success.
    */
    virtual bool getValue(PValue &value) const = 0;
};


/**
* @class    IDOMJobTkContent
* @ingroup  IDom
* @brief    Represents the content element of the JobTicket.
*/
class IDOMJobTkContent : public IDOMNode
{
#define IDOMJobTkContentClassID 0xfde5a46a, 0xfd474247, 0x99d7957d, 0x9a185393
public:

    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:      
        Data(): modified(false), level(eDOMJobTkLevelNotValid) 
        {}
        eDOMJobTkLevel  level;      //!< The level of the JobTicket content (DocumentSequence, FixedDocument, Fixedpage).
        double         version;     //!< Version of the JobTicket content.
        bool           modified;    //!< User should set this flag if job ticked content was modified.
    };

    /**
    * @brief    Retrieves the class id of IDOMJobTkContent.
    * @return   CClassID. Returns the class id of the element.
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMJobTkContentClassID);
        return id;
    }

    /**
    * @brief   Returns an indicator of the validity of the JobTicket content-that is, whether or not the JobTicket content has been initialised.
    * @return  bool Returns true if the content has been initialised, false otherwise.
    */ 
    virtual bool isValid() const = 0;

    /**
    * @brief   Sets the level of the JobTicket content, corresponding to DocumentSequence, FixedDocument or
    * FixedPage (see eDOMJobTkLevel). The default level is eDOMJobTkLevelDefault.  The uninitialised value of 
    * the level is eDOMJobTkLevelNotValid.
    * @param   level Job ticket level.
    * @return  bool. Returns true on success, or false if the call fails.
    */ 
    virtual bool setLevel(eDOMJobTkLevel level)  = 0;

    /**
    * @brief    Retrieves the level of the JobTicket content, corresponding to DocumentSequence, FixedDocument or
    * FixedPage (see eDOMJobTkLevel). The default level is eDOMJobTkLevelDefault.  The uninitialised value of 
    * the level is eDOMJobTkLevelNotValid.
    * @return   eDOMJobTkLevel. Returns the level of the JobTicket content.
    */
    virtual eDOMJobTkLevel getLevel() const = 0;

    /**
    * @brief   Sets the version of the JobTicket content.
    * @param   version Job ticket version.
    * @return  bool. Returns true on success, or false if the call fails.
    */ 
    virtual bool setVersion(double version)  = 0;

    /**
    * @brief    Retrieves the version of the JobTicket content.
    * @return   double. Returns the version of the JobTicket content.
    */
    virtual double getVersion() const = 0;

    /**
    * @brief   Sets the "modified" flag.
    * @param   modified New value of the "modified" flag.
    * @return  bool. Returns true on success, or false if the call fails.
    */ 
    virtual bool setModified(bool modified)  = 0;

    /**
    * @brief    Retrieves the value of the "modified" flag.
    * @return   bool. Value of the "modified" flag.
    */
    virtual bool getModified() const = 0;

    /**
    * @brief   Retrieves the root node of the JobTicket content.
    * @param   rootNode Smart pointer to receive a reference to the root node.
    * @return  bool. Returns true on success, or false if the call fails.
    */
    virtual bool getRootNode(IDOMJobTkNodePtr &rootNode) = 0;

    /**
    * @brief    Retrieves the enumerator of the namespace collection.
    * @return   IEDLNamespaceCollectionEnumPtr. Returns a pointer to the enumerator.
    */
    virtual IEDLNamespaceCollectionEnumPtr getNamespaceCollectionEnum() = 0;

    /**
    * @brief    Retrieves the number of namespaces in the namespace collection.
    * @return   uint32 Returns the number of namespaces in the collection.
    */
    virtual uint32 getNamespacesCount() = 0;

    /**
    * @brief    Clears the collection of namespaces 
    */
    virtual void clearNamespaceCollection() = 0; 

    /**
    * @brief    Appends a namespace to the collection of namespaces.
    * @param    ptrNamespace Smart pointer to the new namespace.
    * @return  bool. Returns true on success, or false if the call fails.
    */
    virtual bool addNamespace(const IEDLNamespacePtr &ptrNamespace) = 0;

    /**
    * @brief    Appends a namespace to the collection of namespaces.
    * @param    prefix Value of the new namespace prefix
    * @param    name Value of the new namespace name
    * @return   IEDLNamespacePtr. Smart pointer to the new namespace which has been added.
    */
    virtual IEDLNamespacePtr addNamespace(const EDLSysString &prefix, const EDLSysString &name) = 0;

    /**
    * @brief    Finds a namespace in the namespace collection by name.
    * @param    nmspace Value of namespace name.
    * @param    ptrNamespace Smart pointer to receive a reference to the found namespace.
    * @return  bool. Returns true on success, or false if the call fails.
    */
    virtual bool findNamespaceByName(const EDLSysString &nmspace, IEDLNamespacePtr &ptrNamespace) = 0;

    /**
    * @brief    Finds a namepace in the namespace collection by prefix.
    * @param    prefix Value of namespace prefix.
    * @param    ptrNamespace Smart pointer to receive a reference to the found namespace.
    * @return  bool. Returns true on success, or false if the call fails.
    */
    virtual bool findNamespaceByPrefix(const EDLSysString &prefix, IEDLNamespacePtr &ptrNamespace) = 0;

    /**
    * @brief    Appends standard print ticket namespaces to the JobTicket content
    * The following standard namespaces are appended:
    * psk="http://schemas.microsoft.com/windows/2003/08/printing/printschemakeywords"
    * xsd="http://www.w3.org/2001/XMLSchema"
    * xsi="http://www.w3.org/2001/XMLSchema-instance"
    * psf="http://schemas.microsoft.com/windows/2003/08/printing/printschemaframework"
    */
    virtual void addStandardNamespaces() = 0;

    /**
    * @brief    Constructs a qualified name based on a name and namespace provided. The namespace must be present in the namespace collection.
    * @param    name Name of JobTicket content element
    * @param    nmspace Namespace of JobTicket content element
    * @param    qname reference parameter to receive the constructed qualified name.
    * @return  bool. Returns true on success, or false if the call fails.
    */
    virtual bool convertToQName(const EDLString &name, const EDLSysString &nmspace, EDLQName &qname) = 0;

    /**
    * @brief    Searches through the node's direct child set for a node matching the provided node type, name and namespace.
    * @param    nodeType Job ticket node type.
    * @param    name Name of JobTicket node.
    * @param    nmspace Namespace of JobTicket node.
    * @return   IDOMJobTkNodePtr. Returns smart pointer to the retrieved JobTicket node.
    */
    virtual IDOMJobTkNodePtr findChild(IDOMJobTkNode::eDOMJobTkNodeType nodeType, const EDLString &name, const EDLSysString &nmspace) = 0;

    /**
    * @brief    Searches through the node's direct child set for a node matching provided the node type and qualified name.
    * @param    nodeType Job ticket node type.
    * @param    qname Qualified name of JobTicket node.
    * @return   IDOMJobTkNodePtr. Returns smart pointer to the retrieved JobTicket node.
    */    
    virtual IDOMJobTkNodePtr findChild(IDOMJobTkNode::eDOMJobTkNodeType nodeType, const EDLQName &qname) = 0;

    /**
    * @brief    Loads a JobTicket from a named XML file.
    * @param    filename Path to and name of the JobTicket file to load
    * @return  bool. Returns true on success, or false if the call fails.
    */
    virtual bool loadFromFile(const EDLSysString &filename) = 0;

    /**
    * @brief    Loads a JobTicket from an XML stream.
    * @param    xmlStream Stream for xml data
    * @return  bool. Returns true on success, or false if the call fails.
    */
    virtual bool loadFromStream(const IRAInputStreamPtr &xmlStream) = 0;

    /**
    * @brief    Fills the JobTicket using input from the initialisation string.
    * @param    sInit Initialisation string.
    * @return  bool. Returns true on success, or false if the call fails.
    */
    virtual bool loadFromInitString(const EDLSysString & sInit) = 0;
    
    /**
    * @brief    Static helper to add a key/value pair to an initialisation string.
    *           Takes care of escaping as required.
    * @param    paramName The parameter name.
    * @param    initString The init string to add to
    * @param    paramValue The optional parameter value.
    * @param    overwrite Determines the behaviour if the parameter is already present
    *           in the initString. If true, the existing value will be replaced. If false
    *           the value will not be inserted.
    * @return  EDLSysString Returns the new init string.
    */
    static EDL_API EDLSysString addToInitString(const EDLSysString &paramName, const EDLSysString &initString, const EDLSysString &paramValue = "", bool overwrite = true);

    /**
    * @brief    Adds a <psf:ParameterInit> element to the JobTicket.
    * @param    parameterNamespace Smart pointer to the parameter namespace.
    * @param    parameterName Name of parameter.
    * @param    value Value of parameter.
    * @return  bool. Returns true on success, or false if the call fails.
    */
    virtual bool addParameterInit(const IEDLNamespacePtr &parameterNamespace, const EDLSysString &parameterName, const PValue &value) = 0;

    /**
    * @brief    Adds Feature/Option elements to the JobTicket in the format:
    *
    *           <psf:Feature name=parameterName>
    *
    *               <psf:Option name=value/> 
    *
    *           </psf:Feature>
    * @param    featureNamespace Smart pointer to the feature name namespace.
    * @param    featureName Name of feature.
    * @param    option Qualified name of feature option.
    * @return  bool. Returns true on success, or false if the call fails.
    */
    virtual bool addFeature(const IEDLNamespacePtr &featureNamespace, const EDLSysString &featureName, const EDLQName &option) = 0;

};


/**
* @class    IDOMJobTk
* @ingroup  IDom
* @brief    Represents an EDL JobTicket.
* 
* EDL JobTickets provide user intent and device configuration information to printers.
* JobTickets can be attached only to DocumentSequence, FixedDocument and FixedPage 
* nodes. JobTickets can provide override settings to be used when printing the node 
* to which they are attached.
*/
class IDOMJobTk : public IEDLObject
{
#define IDOMJobTkClassID 0x2AE77EA3, 0x14E111DC, 0x83140800, 0x200C9A66
public:

    /**
    * @brief    Retrieves the class id of IDOMJobTk.
    * @return   CClassID. Class id of the element.
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMJobTkClassID);
        return id;
    }

    /**
    * @brief    Sets the owner node, that is, the level of the DOMtree with which the JobTicket is associated.
    * @param    ptrOwner Smart pointer to the new owner node.
    * @return   bool. Returns true on success, false if the call fails.
    */
    virtual bool setOwner (IDOMJobTkOwnerPtr &ptrOwner) = 0;

    /**
    * @brief    Retrieves the owner node, that is, the level of the DOMtree with which the JobTicket is associated.
    * @return   IDOMNodePtr. Returns a smart pointer to the owner node.
    */
    virtual IDOMJobTkOwnerPtr getOwner() const = 0;

    /**
    * @brief    Sets the JobTicket content node.
    * @param    ptrOwner Smart pointer to the new JobTicket content node.
    * @return   bool. Returns true on success, false if the call fails.
    */
    virtual bool setContent (const IDOMJobTkContentPtr &ptrContent) = 0;

    /**
    * @brief    Retrieves the JobTicket content node.
    * @return   IDOMJobTkContentPtr Returns smart pointer to the JobTicket content node
    */
    virtual IDOMJobTkContentPtr getContent() const = 0;

    /**
    * @brief    Searches through the node's direct child set for the node type, name and namespace matching the information provided.
    * @param    nodeType Type of the node's child to search for.
    * @param    name Name of the node's child to search for.
    * @param    nmspace Namespace of the node's child to search for.
    * @return   IDOMJobTkNodePtr Smart pointer to the found child node.
    */
    virtual IDOMJobTkNodePtr findChild(IDOMJobTkNode::eDOMJobTkNodeType nodeType, const EDLString &name, const EDLSysString &nmspace) = 0;

    /**
    * @brief    Searches through the node's direct child set for the node type and qname matching the information provided.
    * @param    nodeType Node type of the the node's child to search for.
    * @param    qname Qualified name of the the node's child to search for.
    * @return   IDOMJobTkNodePtr Smart pointer to the found child node.
    */
    virtual IDOMJobTkNodePtr findChild(IDOMJobTkNode::eDOMJobTkNodeType nodeType, const EDLQName &qname) = 0;

    /**
    * @brief    Generate combined JobTicket content.
    * Combined content is generated by combining the current JobTicket with all upper level JobTickets.
    * @param    ptrCombinedContent A smart pointer to receive the resulting combined content.
    * @param    addDefaultJobTk If this parameter is set to "true" then combine the default level of JobTicket as well as document, job and page levels.
    * @return   bool. Returns true on success, false if the call fails.
    */
    virtual bool getCombinedContent(IDOMJobTkContentPtr &ptrCombinedContent, bool addDefaultJobTk = false) = 0;
};

IDOMJobTkPtr getJobTicket(const IDOMNodePtr &ptrNode);


/**
* @brief    Clone the JobTicket and the whole JobTicket content tree.
* @param    jobTk Smart pointer to the source JobTicket.
* @param    factory Pointer to the class factory interface.
* @return   IDOMJobTkPtr. Returns the cloned JobTicket, with cloned JobTicket content set.
*/
IDOMJobTkPtr cloneJobTkTree(IDOMJobTkPtr &jobTk, IEDLClassFactory *factory);


/**
* @brief    Clone the whole JobTicket content tree.
* @param    jobTkContent Smart pointer to the source JobTicket content.
* @param    factory Pointer to the class factory interface.
* @return   IDOMJobTkContentPtr. Returns the cloned JobTicket content.
*/
IDOMJobTkContentPtr cloneJobTkContentTree(IDOMJobTkContentPtr &jobTkContent, IEDLClassFactory *factory);


_END_EDL_DOM_NAMESPACE

#endif /* IDOMJOBTK_H */

