/*
* Copyright (C) 2007-2019 Global Graphics Software Ltd. All rights reserved.
*/

/**
* @file     idommetadata.h
* @ingroup  IDom
*/

#ifndef __IDOMMETADATA_H__
#define __IDOMMETADATA_H__

#include <edl/edltypes.h>
#include <edl/iedlenum.h>
#include <edl/edlproperty.h>

_BEGIN_EDL_DOM_NAMESPACE

/**
* @class    IDOMMetadata
* @ingroup  IDom
* @brief    The IDOMMetadata interface provides access to the metadata attached to the DocumentSequence node.
* The IDOMMetadata interface is designed to be flexible enough to represent different types of metadata. 
*
* The metadata set on IDOMDocumentSequence was designed to hold general information about
* the document, such as document author or creation/modification date.  
* Metadata can also describe the sort of information that can be found in the document information
* dictionary for a PDF document. The closest analogue for the XPS world would be the information
* contained in the CoreProperties part.
*
* Several groups of metadata properties are supported by EDL, as described by the eType enumeration.  
* The properties in each group are as follows:
*
* <B>eCoreProperties</B>
*<UL>
*    <LI> <B>category</B>              - EDLSysString
*    <LI> <B>contentStatus</B>         - EDLSysString
*    <LI> <B>contentType</B>           - EDLSysString
*    <LI> <B>keywords</B>              - EDLSysString
*    <LI> <B>lastModifiedBy</B>        - EDLSysString
*    <LI> <B>lastPrinted</B>           - IEDLTime
*    <LI> <B>revision</B>              - EDLSysString
*    <LI> <B>version</B>               - EDLSysString
*    <LI> <B>creator</B>               - EDLSysString
*    <LI> <B>description</B>           - EDLSysString
*    <LI> <B>identifier</B>            - EDLSysString
*    <LI> <B>language</B>              - EDLSysString
*    <LI> <B>subject</B>               - EDLSysString
*    <LI> <B>title</B>                 - EDLSysString
*    <LI> <B>created</B>               - IEDLTime
*    <LI> <B>modified</B>              - IEDLTime
*</UL>
*
* <B>eViewerPreferences</B>
*<UL>
*    <LI> <B>HideToolbar</B>           - bool
*    <LI> <B>HideMenubar</B>           - bool
*    <LI> <B>HideWindowUI</B>          - bool
*    <LI> <B>FitWindow</B>             - bool
*    <LI> <B>CenterWindow</B>          - bool
*    <LI> <B>DisplayDocTitle</B>       - bool
*    <LI> <B>PickTrayByPDFSize</B>     - bool
*    <LI> <B>NumCopies</B>             - int32
*    <LI> <B>NonFullScreenPageMode</B> - EDLSysString
*    <LI> <B>Direction</B>             - EDLSysString
*    <LI> <B>ViewArea</B>              - EDLSysString
*    <LI> <B>ViewClip</B>              - EDLSysString
*    <LI> <B>PrintArea</B>             - EDLSysString
*    <LI> <B>PrintClip</B>             - EDLSysString
*    <LI> <B>PrintScaling</B>          - EDLSysString
*    <LI> <B>Duplex</B>                - EDLSysString
*</UL>
*
* <B>eDocumentInfo</B>
*<UL>
*    <LI> <B>Title</B>                 - EDLSysString
*    <LI> <B>Author</B>                - EDLSysString or CEDLStringVect with variant value (one of eXmpContainerType)
*    <LI> <B>Subject</B>               - EDLSysString or CEDLStringVect with variant value (one of eXmpContainerType)
*    <LI> <B>Keywords</B>              - EDLSysString or CEDLStringVect with variant value (one of eXmpContainerType)
*    <LI> <B>Creator</B>               - EDLSysString
*    <LI> <B>Producer</B>              - EDLSysString
*    <LI> <B>CreationDate</B>          - IEDLTime
*    <LI> <B>ModDate</B>               - IEDLTime
*    <LI> <B>Trapped</B>               - EDLSysString
*</UL>
*
*
* <B>ePDFInfo</B>  - read-only PDF document properties
*<UL>
*    <LI> <B>Version</B>               - EDLSysString  property in a string form <B>M.m</B>,
*                                             where M is the major and m
*                                             is the minor version numbers
*    <LI> <B>ExtensionLevel</b>             - int32 representing the extension level
*    <LI> <B>Linearized</B>            - bool          indicator of original PDF document linearization
*    <LI> <B>FileIdentifier1</B>       - EDLSysString property representing the first part of PDF file ID
*    <LI> <B>FileIdentifier2</B>       - EDLSysString property representing second part of PDF file ID
*    <LI> <B>Standard</B>              - EDLSysString property representing the PDF/A, PDF/E or PDF/X standard the PDF purports to conform to
*    <LI> <B>Marked</B>                - bool         true if the PDF document claims to be a tagged PDF
*    <LI> <B>UserProperties</B>        - bool         true if the PDF document is tagged and structure elements that contain user properties are present
*    <LI> <B>Suspects</B>              - bool         true if the PDF contains tagged suspects
*</UL>
*
* <B>ePageView</B>
*<UL>
*    <LI> <B>PageMode</B>              - EDLSysString
*    <LI> <B>PageLayout</B>            - EDLSysString
*</UL>
*/

class IDOMMetadata;
DECL_SMART_PTR(IDOMMetadata);

class IDOMMetadata : public IEDLObject
{
#define IDOMMetadataClassID 0x4bc2067b, 0x6b5644dc, 0xa6dd49d9, 0xc2c77c5b
public:

    DECLARE_CLASS_ID(IDOMMetadataClassID);

    /**
    * @enum     eType
    * @ingroup  IDom
    * @brief    Metadata types data type
    */
    typedef enum 
    {
        eCoreProperties = 0,    //!< Core properties metadata (XPS core properties)
        eDocumentInfo,          //!< Document information metadata (PDF document information dictionary)
        eViewerPreferences,     //!< Viewer Preferences entry of PDF catalog dictionary
        ePageView,              //!< PageLayout and PageMode entries of PDF catalog dictionary
        ePDFInfo,               //!< Version and Linearization entries defined for PDF document
        eMetadataTypeCnt        //!< The number of types defined by eType
    } eType;

    /**
    * @enum     eXmpContainerType
    * @ingroup  IDom
    * @brief    The variant values passed to PValue when storing a CEDLStringVect
    */
    typedef enum 
    {
        eXmpContainer_None = 0,    //!< The PValue variant that indicates it is not an Xmp container
        eXmpContainer_Alt  = 1,    //!< The PValue variant for storing a CEDLStringVect representing an Xmp Alt container
        eXmpContainer_Bag  = 2,    //!< The PValue variant for storing a CEDLStringVect representing an Xmp Bag container
        eXmpContainer_Seq  = 3     //!< The PValue variant for storing a CEDLStringVect representing an Xmp Seq container
    } eXmpContainerType;
    
    /**
    * @brief    Create a new IDOMMetadata object
    */
    static EDL_API IDOMMetadataPtr create(IEDLClassFactory *pFactory);

    /**
    * @brief    Retrieves the value of a named property.
    * @param    propertyType The type of the property whose value is to be retrieved.
    * @param    propertyName The name of the property whose value is to be retrieved.
    * @param    propertyValue A smart pointer to receive the retrieved property value.
    * @return   bool. Returns true on success, false if the call fails.
    */
    virtual bool getProperty(eType propertyType, const EDLSysString &propertyName, PValue &propertyValue) const = 0;

    /**
    * @brief    Sets value of a property
    * @param    propertyType The type of the property whose value is to be set.
    * @param    propertyName The name of the property whose value is to be set.
    * @param    propertyValue The new value to be set for the property.
    * @return   bool. Returns true on success, false if the call fails.
    */
    virtual bool setProperty(eType propertyType, const EDLSysString &propertyName, const PValue &propertyValue) = 0;

    /**
    * @brief    Removes a property, if it exists.
    * @param    propertyType The type of the property to be removed.
    * @param    propertyName The name of the property to be removed.
    * @return   bool. Returns true on success, false if the call fails.
    */
    virtual bool removeProperty(eType propertyType, const EDLSysString &propertyName) = 0;

    /**
    * @brief    Retrieves a navigable list of the property names in the metadata.
    * @return   IEDLSysStringCollectionEnumPtr. Returns a smart pointer to the list of property names.
    */
    virtual IEDLSysStringCollectionEnumPtr getPropertyCollectionEnum(eType propertyType) = 0;

};

#define edlobj2IDOMMetadata(src) edl_cast((IDOMMetadata *)NULL, src)


_END_EDL_DOM_NAMESPACE

#endif // __IDOMMETADATA_H__
