/*
* Copyright (C) 2008 Global Graphics Software Ltd. All rights reserved.
*/

/**
* @file    idomopi.h
* @ingroup IDom
* @brief   Interface to EDL DOM OPI
*/

#ifndef __EDLIDOMOPI_H__
#define __EDLIDOMOPI_H__

#include <edl/edltypes.h>
#include <edl/edlstring.h>
#include <edl/iedlobject.h>
#include <edl/ifilespec.h>

_BEGIN_EDL_DOM_NAMESPACE


/**
* @class    IDOMOPI
* @ingroup  IDom
* @brief    Base class representing OPI proxy.
*           Has two descendant interfaces IDOMOPI13 and IDOMOPI20
*/
class IDOMOPI : public IEDLObject
{
public:
    /**
    * @brief Retrieves the version of the OPI supported by the interface in the form: major.minor
    *
    * @param    major integer representing the major version of OPI.
    * @param    minor integer representing the minor version of OPI.
    */
    virtual void getVersion(int32 &major, int32 &minor) const = 0;
};
DECL_SMART_PTR(IDOMOPI);
#define edlobj2IDOMOPI(src) edl_cast((IDOMOPI *)NULL, src)

/**
* @class    IDOMOPI13
* @ingroup  IDom
* @brief    The interface representing OPI proxy.with accordance to
*           Open Prepress Interface Specification 1.3
*/
class IDOMOPI13 : public IDOMOPI
{
#define IDOMOPI13ClassID 0x73318038, 0x39a44c2d, 0x9469a6ca, 0xac82fbee
public:
    enum eOPIColorType {
        eProcess,
        eSpot,
        eSeparation
    };
    static const CClassID &classID()
    {
        static CClassID id(IDOMOPI13ClassID);
        return id;
    };
    virtual void getVersion(int32 &major, int32 &minor) const
    {
        major = 1;
        minor = 3;
    }
    class Data : public CClassParams
    {
    public:
        Data() 
        {
            colorType = eSpot;
            tint = 1.0;
            overPrint = false;
            transparency = true;
        }
        IFileSpecPtr    ptrF;           // required
        EDLSysString    strID;
        EDLSysString    strComments;
        IntPoint        size;           // required
        IntRect         cropRect;       // required
        FRect           cropFixed;      // default: cropFixed = cropRect
        double          position[8];    // required
        FPoint          resolution;
        eOPIColorType   colorType;
        double          colorValue[4];
        EDLSysString    strColorName;
        double          tint;
        bool            overPrint;      // default: false
        IntPoint        imageType;
        bool            transparency;   // default: true
    };
    
    /**
    * @brief Retrieves the external file containing the image
    *
    * corresponds to %ALDImageFilename in OPI1.3 specs
    *
    * @param    ptrF Smart pointer to file specification
    * @return   bool Returns true on success
    */
    virtual bool getF(IFileSpecPtr &ptrF) const = 0;
    /**
    * @brief Retrieves the ID string
    *
    * corresponds to %ALDImageID in OPI1.3 specs
    *
    * @param    strID ID string
    * @return   bool Returns true on success
    */
    virtual bool getID(EDLSysString &strID) const = 0;
    /**
    * @brief Retrieves the Comments in human readable form
    *
    * corresponds to %ALDObjectComments in OPI1.3 specs
    *
    * @param    strComments Comments string
    * @return   bool Returns true on success
    */
    virtual bool getComments(EDLSysString &strComments) const = 0;
    /**
    * @brief Retrieves the dimensions of the image in pixels
    *
    * corresponds to %ALDImageDimensions in OPI1.3 specs, x represents width and y height
    *
    * @param    size image dimensions
    * @return   bool Returns true on success
    */
    virtual bool getSize(IntPoint &size) const = 0;
    /**
    * @brief Retrieves the CropRect (portion of the image to be used)
    *
    * corresponds to %ALDImageCropRect in OPI1.3 specs
    *
    * @param    cropRect CropRect of the image
    * @return   bool Returns true on success
    */
    virtual bool getCropRect(IntRect &cropRect) const = 0;
    /**
    * @brief Retrieves the CropFixed (same as CropRect but in real numbers)
    *
    * corresponds to %ALDImageCropFixed in OPI1.3 specs
    *
    * @param    cropFixed CropFixed of the image
    * @return   bool Returns true on success
    */
    virtual bool getCropFixed(FRect &cropFixed) const = 0;
    /**
    * @brief Retrieves the position on the page of the cropped image
    *
    * corresponds to %ALDImagePosition in OPI1.3 specs
    * Position is an array of 8 doubles in the form [ llx lly ulx uly urx ury lrx lry ]
    *
    * @param    position array sufficient to hold 8 doubles
    * @return   bool Returns true on success
    */
    virtual bool getPosition(double position[]) const = 0;
    /**
    * @brief Retrieves the resolution of the image in samples per inch
    *
    * corresponds to %ALDImageResolution in OPI1.3 specs
    *
    * @param    resolution image resolution. x - horizontal and y - vertical resolution
    * @return   bool Returns true on success
    */
    virtual bool getResolution(FPoint &resolution) const = 0;
    /**
    * @brief Retrieves the color type of the image
    *
    * corresponds to %ALDImageColorType in OPI1.3 specs
    *
    * @return   eOPIColorType Returns color type of the image
    */
    virtual eOPIColorType getColorType() const = 0;
    /**
    * @brief Retrieves the values of the image color
    *
    * corresponds to the color values in %ALDImageColor in OPI1.3 specs
    *
    * @param    value an array of four numbers in the form [C M Y K]
    * @return   bool Returns true on success
    */
    virtual bool getColorValue(double value[]) const = 0;
    /**
    * @brief Retrieves the name of the image color
    *
    * corresponds to the color name in %ALDImageColor in OPI1.3 specs
    *
    * @param    colorName image color name
    * @return   bool Returns true on success
    */
    virtual bool getColorName(EDLSysString &colorName) const = 0;
    /**
    * @brief Retrieves the concentration of the color
    *
    * corresponds to %ALDImageTint in OPI1.3 specs
    *
    * @return   double Returns number in range 0.0 to 1.0 (default: 1.0)
    */
    virtual double getTint() const = 0;
    /**
    * @brief Retrieves the flag whether the image is to overprint or knock out 
    * underlying marks on other separations
    *
    * corresponds to %ALDImageOverprint in OPI1.3 specs
    *
    * @return   bool Returns true if overprint (default: false)
    */
    virtual bool getOverPrint() const = 0;
    /**
    * @brief Retrieves the image type as a set of two numbers, first specifying the 
    * number of samples per pixel and second specifying the number of bits per sample
    *
    * corresponds to %ALDImageType in OPI1.3 specs
    *
    * @param    imageType image type. x represents number of samples, y - number of bits
    * @return   bool Returns true on success
    */
    virtual bool getImageType(IntPoint &imageType) const = 0;
    /**
    * @brief Retrieves the flag specifying whether white pixels
    * in the image are to be treated as transparent
    *
    * corresponds to %ALDImageTransparency in OPI1.3 specs
    *
    * @return   bool Returns true if transparent (default: true)
    */
    virtual bool getTransparency() const = 0;
    
    /**
    * @brief Retrieves the size of the GrayMap
    *
    * @return   int32 Returns size of the GrayMap
    */
    virtual int32 getGrayMapSize() const = 0;
    /**
    * @brief Retrieves the GrayMap as the array of integers
    * in the ranage 0 to 65535
    *
    * corresponds to %ALDImageGrayMap in OPI1.3 specs
    *
    * @param    grayMap array of integers to hold the GrayMap values
    * @return   bool Returns true on success
    */
    virtual bool getGrayMap(int32 grayMap[]) const = 0;
    /**
    * @brief Sets the GrayMap as the array of integers
    * in the ranage 0 to 65535
    *
    * corresponds to %ALDImageGrayMap in OPI1.3 specs
    *
    * @param    size the new size of the GrayMap. Size 0 removes the GrayMap
    * @param    grayMap array of integers holding the GrayMap values
    * @return   bool Returns true on success
    */
    virtual bool setGrayMap(int32 size, int32 grayMap[]) = 0;

    /**
    * @brief Retrieves the size of the Tags array
    *
    * @return   int32 Returns size of the Tags array
    */
    virtual int32 getTagsSize() const = 0;
    /**
    * @brief Retrieves the set of Tags (as the array of integers)
    * supported by the OPI proxy
    *
    * corresponds to %ALDImageAsciiTag in OPI1.3 specs
    *
    * @param    tags array of integers to hold the values of the Tags supported
    * @return   bool Returns true on success
    */
    virtual bool getTags(int32 tags[]) const = 0;
    /**
    * @brief Retrieves the Tag text for a given tag number
    *
    * corresponds to %ALDImageAsciiTag in OPI1.3 specs
    *
    * @param    tag tag number for which the text is to be retrieved
    * @param    tagText tag text retrieved
    * @return   bool Returns true on success
    */
    virtual bool getTagText(int32 tag, EDLSysString &tagText) const = 0;
    /**
    * @brief Adds the new Tag to the set of Tags
    * supported by the OPI proxy
    *
    * @param    tag tag number to add
    * @param    tagText tag text corresponding to the tag number to add
    * @return   bool Returns true on success
    */
    virtual bool addTag(int32 tag, const EDLSysString &tagText) = 0;
    /**
    * @brief Deletes the Tag from the set of Tags
    * supported by the OPI proxy
    *
    * @param    tag tag number to delete
    * @return   bool Returns true on success
    */
    virtual bool deleteTag(int32 tag) = 0;
};
DECL_SMART_PTR(IDOMOPI13);
#define edlobj2IDOMOPI13(src) edl_cast((IDOMOPI13 *)NULL, src) 

/**
* @class    IDOMOPI20
* @ingroup  IDom
* @brief    The interface representing OPI proxy.with accordance to
*           Open Prepress Interface Specification 2.0
*/
class IDOMOPI20 : public IDOMOPI
{
#define IDOMOPI20ClassID 0x03d527e0, 0xb72b4c3d, 0x8bca0f8f, 0xfdc75c98
public:
    enum eInksType {
        eUndefined,
        eMonochrome,
        eRegistration,
        eFull_Color
    };

    static const CClassID &classID()
    {
        static CClassID id(IDOMOPI20ClassID);
        return id;
    };
    virtual void getVersion(int32 &major, int32 &minor) const
    {
        major = 2;
        minor = 0;
    }
    class Data : public CClassParams
    {
    public:
        Data() 
        {
            overPrint = false;
            includedImageQuality = 0;
        }
        IFileSpecPtr ptrF;                      //required
        EDLSysString mainImage;
        FPoint       size;
        FRect        cropRect;
        bool         overPrint;
        IntPoint     includedImageDimensions;
        int32        includedImageQuality;      // 1, 2 or 3 are valid
    };
    virtual bool getF(IFileSpecPtr &ptrF) const = 0;
    virtual bool getMainImage(EDLSysString &mainImage) const = 0;
    virtual bool getSize(FPoint &size) const = 0;
    virtual bool getCropRect(FRect &cropRect) const = 0;
    virtual bool getOverPrint() const = 0;
    virtual bool getIncludedImageDimensions(IntPoint &dimensions) const = 0;
    virtual bool getIncludedImageQuality(int32 &quality) const = 0;

    virtual int32 getTagsSize() const = 0;
    virtual bool getTags(int32 tags[]) const = 0;
    virtual bool getTagText(int32 tag, EDLSysString &tagText) const = 0;
    virtual bool addTag(int32 tag, const EDLSysString &tagText) = 0;
    virtual bool deleteTag(int32 tag) = 0;

    virtual eInksType getInksType() const = 0;
    virtual bool setInksType(eInksType inksType) = 0;
    virtual bool addInksColorant(const EDLSysString &colorantName, double tint) = 0;
    virtual int32 getInksColorantsCount() const = 0;
    virtual bool getInksColorant(int32 colorantNo, EDLSysString &colorantName, double &tint) const = 0;
    virtual bool clearInksColorants() = 0;
};
DECL_SMART_PTR(IDOMOPI20);
#define edlobj2IDOMOPI20(src) edl_cast((IDOMOPI20 *)NULL, src) 

_END_EDL_DOM_NAMESPACE

#endif // __EDLIDOMOPI_H__
