/*
* Copyright (C) 2007-2019 Global Graphics Software Ltd. All rights reserved.
*/

/**
*
* @file     idompage.h
* @ingroup  IDom
* @brief    Page and Fixed page interfaces
*/

#ifndef __IDOMPAGE_H__
#define __IDOMPAGE_H__

#include <edl/edltypes.h>
#include <edl/edlfwd.h>
#include <edl/edlgeom.h>
#include <edl/idomid.h>
#include <edl/idomnode.h>
#include <edl/idomjobtk.h>
#include <edl/edlfwd.h>
#include <edl/idomresources.h>
#include <edl/idomimageresource.h>
#include <edl/idomgroup.h>

_BEGIN_EDL_DOM_NAMESPACE


/**
* @class    IDOMPage
* @ingroup  IDom
* @brief    The base class for DOM page classes such as IDOMFixedPage.
*/
class IDOMPage : public IDOMNode
{
#define IDOMPageClassID 0x1BFEC534, 0x7D5548bc, 0xA1BB081F, 0xE0D8BE2A
public:

    /**
    * @brief    Retrieves the class id of IDOMPage
    * @return   @b CClassID The class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMPageClassID);
        return id;
    }

};

DECL_SMART_PTR(IDOMPage);
#define edlobj2IDOMPage(src) edl_cast((IDOMPage *)NULL, src)

class IDOMFixedPage;
DECL_SMART_PTR(IDOMFixedPage);

/**
* @class    IDOMFixedPage
* @ingroup  IDom
* @brief    Represents \<FixedPage\> element.
*
* A fixed page describes the contents of a page. A fixed page contains all of the visual elements on
* each page. Each page has a fixed size and orientation. The layout of the visual elements on a page
* is determined by the fixed page markup. This applies to both graphics and text. The fixed page
* contains the elements that together form the basis for all markings rendered on the page: that is,
* Paths, Glyphs and Canvases.
* 
* The fixed page specifies a height, width and a default language.
*/
class IDOMFixedPage : public IDOMJobTkOwner
{
#define IDOMFixedPageClassID 0xE3258A9B, 0x97194760, 0x854C790B, 0x48B724CF
public:
    /**
     *  @brief  Simplified creation function for IDOMFixedPage
     *          Throws an IEDLError exception on failure
     *  @param  pFactory The EDL class factory
	 *  @param  width    Description for width (defaults to 210mm, width of A4)
	 *  @param  height   Description for height  (defaults to 297mm, height of A4)
	 *  @return @b IDOMFixedPagePtr A smart pointer to the fixed page
     */
	static EDL_API IDOMFixedPagePtr create(IEDLClassFactory *pFactory, double width = 793.7, double height = 1122.5);

    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        Data() : width(0.0), height(0.0), degenerated(false) {}
        double width;
        double height;
        FRect bleedBox;
        FRect contentBox;
        FRect cropBox;
        FRect trimBox;
        EDLString language;
        bool degenerated;
        IDOMResourceDictionaryPtr ptrResources;
        IDOMJobTkPtr ptrJobTicket;
        IDOMImagePtr ptrThumbnail;
        IDOMTransparencyGroupPtr ptrPageGroup;
    };

    /**
    * @brief    Retrieves the class id of IDOMFixedPage
    * @return   @b CClassID Class id of the IDOMFixedPage
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMFixedPageClassID);
        return id;
    }

    /**
    * @brief    Sets the fixed page width
    *
    * The width of the page is expressed as a real number in units of 1/96th of an inch.
    * If either the width or height of the page (or both) is set to greater than the original 
    * page width or height, a larger page will result. This would have more white space toward 
    * the right and/or bottom of the page when the document is viewed in an XPS viewer,
    * since the document origin is in the top-left corner. Changing the page dimensions has no 
    * effect on the coordinates of the page content.
    * @param    width New fixed page width
    * @return   @b bool True on success
    */
    virtual bool setWidth(double width) = 0;

    /**
    * @brief    Retrieves the fixed page width.
    * @return   @b double Returns the fixed page width.
    */
    virtual double getWidth() const = 0;

    /**
    * @brief    Sets the fixed page height.
    *
    * The height of the page is expressed as a real number in units of 1/96th of an inch.
    * If either the width or height of the page (or both) is set to greater than the original 
    * page width or height, a larger page will result. This would have more white space toward 
    * the right and/or bottom of the page when the document is viewed in an XPS viewer,
    * since the document origin is in the top-left corner. Changing the page dimensions has no 
    * effect on the coordinates of the page content.
    * @param    height New fixed page height.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setHeight(double height) = 0;

    /**
    * @brief    Retrieves the fixed page height.
    * @return   @b double  Returns the fixed page height
    */
    virtual double getHeight() const = 0;

    /**
    * @brief    Retrieves the fixed page's content box.
    * The content bos is the area of the page containing imageable content that 
    * is to fit within the imageable area when printing or viewing
    *
    * These values are specified in units of 1/96 inch.
    * @param    bb A reference parameter to receive the fixed page's content box 
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool getContentBox (FRect &bb) const = 0;

    /**
    * @brief    Sets the content box for the fixed page.
    * The content bos is the area of the page containing imageable content that 
    * is to fit within the imageable area when printing or viewing. If omitted, the default value is a content
    * box with an origin of (0,0) (that is, the top left hand corner of the page), with a width set to 
    * the width of the page and a height the same as that of the page.
    *
    * These values are specified in units of 1/96 inch.
    * @param    bb The new content box settings
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setContentBox (const FRect &bb) = 0;

    /**
    * @brief    Retrieves the fixed page's bleed box.
    * The bleed box is the area, inclusive of crop marks, that extends outside of the physical
    * page. See Figure 8.11.
    *
    * These values are specified in units of 1/96 inch.
    * @param    bb A reference parameter to receive the fixed page's bleed box.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool getBleedBox (FRect &bb) const = 0;

    /**
    * @brief    Sets the fixed page's bleed box.
    * The bleed box is the area, inclusive of crop marks, that extends outside of the physical
    * page. See Figure 8.11.
    *
    * These values are specified in units of 1/96 inch.
    * @param    bb The new bleed box settings.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setBleedBox (const FRect &bb) = 0;

    /**
    * @brief    Retrieves the fixed page's crop box - the region to which the contents of the page are to be clipped (cropped) when displayed or printed
    * @param    cropBox A reference parameter to receive the fixed page's crop box.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool getCropBox (FRect &cropBox) const = 0;

    /**
    * @brief    Sets the fixed page's crop box.
    * @param    cropBox The new crop box settings.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setCropBox (const FRect &cropBox) = 0;

    /**
    * @brief    Retrieves the fixed page's trim box.
    * The trim box defines the intended dimensions of the finished page after trimming
    *
    * These values are specified in units of 1/96 inch.
    * @param    trimBox A reference parameter to receive the fixed page's trim box.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool getTrimBox (FRect &trimBox) const = 0;

    /**
    * @brief    Sets the fixed page's trim box.
    * The trim box defines the intended dimensions of the finished page after trimming
    *
    * These values are specified in units of 1/96 inch.
    * @param    trimBox The new trim box settings.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setTrimBox (const FRect &trimBox) = 0;

    /**
    * @brief    Retrieves default language of the fixed page and any of its children.
    *
    * The language is specified according to RFC 3066. English is defined as en_GB and
    * American English as en_US. There is no default setting.
    * If the language is not known it is set to und (undetermined). For further information see
    * http://www.w3.org/International/articles/language-tags/
    *
    * @param    lang A reference parameter which will recieve the language of the fixed page.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool getLanguage (EDLString &lang) const = 0;
    
    /**
    * @brief    Retrieves the page transparency group, if present.
    *           Generally this is used to dictate the color space in which
    *           transparency blending will be performed.
    * @return   @b IDOMTransparencyGroupPtr Smart pointer to the page group, or
    *           NULL if no page group is supplied.
    */
    virtual IDOMTransparencyGroupPtr getPageGroup() const = 0;
    
    /**
    * @brief    Set the page transparency group.
    * @param    pageGroup A reference to the desired transparency
    *           group, or NULL.
    */
    virtual void setPageGroup(const IDOMTransparencyGroupPtr &pageGroup) = 0;

    /**
    * @brief    Sets the language of the fixed page and any of its children.
    *
    * The language is specified according to RFC 3066. English is defined as en_GB and
    * American English as en_US. There is no default setting.
    * For further information see
    * http://www.w3.org/International/articles/language-tags/
    *
    * @param    lang The new language setting.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setLanguage (const EDLString &lang) = 0;

    /**
    * @brief    Retrieves a smart pointer to the resource dictionary.
    * The resource dictionary contains the IDs of resources stored at this level of the document.
    * @param    ptrResources Reference parameter to be populated with a smart pointer to the resource dictionary
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool getResourceDictionary(IDOMResourceDictionaryPtr &ptrResources) const = 0;

    /**
    * @brief    Sets the resource dictionary.
    * @param    ptrResources Smart pointer to the new resource dictionary
    * The resource dictionary contains the IDs of resources stored at this level of the document.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setResourceDictionary(const IDOMResourceDictionaryPtr &ptrResources) = 0;

    /**
    * @brief    Retrieves a smart pointer to the fixed document thumbnail image.
    * @param    ptrThumbnail Smart pointer to recieve a reference to the thumbnail image.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool getThumbnail(IDOMImagePtr &ptrThumbnail) const = 0;

    /**
    * @brief    Sets a new thumbnail image for the document sequence.
    * The image must be in either JPEG or PNG format. Setting thumbnail to NULL deletes the document sequence thumbnail.
    * @param    ptrThumbnail Smart pointer to new thumbnail image.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setThumbnail(const IDOMImagePtr &ptrThumbnail) = 0;

    /**
    * @brief    Retrieves the page's links manager
    * @return   @b IDOMPageLinkManagerPtr The page's link manager
    */
    virtual IDOMPageLinkManagerPtr getLinkManager() = 0;

    /**
    * @brief    Retrieves the page's annotations manager
    * @return   @b IDOMPageAnnotationManagerPtr A smart pointer to the page's annotations manager.
    */
    virtual IDOMPageAnnotationManagerPtr getAnnotationManager() = 0;

    /**
    * @brief    Retrieves the page's denerated flag value
    * @return   @b bool True if page is denerated
    */
    virtual bool isDegenerated() const = 0;

};
#define edlobj2IDOMFixedPage(src) edl_cast((IDOMFixedPage *)NULL, src)

_END_EDL_DOM_NAMESPACE

#endif /* __IDOMPAGE_H__ */
