/*
* Copyright (C) 2007-2019 Global Graphics Software Ltd. All rights reserved.
*/

/**
* @file     idompath.h
* @ingroup  IDom
* @brief    Path related interfaces of the DOM
*/

#ifndef EDLIDOMPATH_H
#define EDLIDOMPATH_H

#include <edl/edltypes.h>
#include <edl/edlgeom.h>
#include <edl/iedlobject.h>
#include <edl/idomnode.h>
#include <edl/idomshape.h>
#include <edl/idombrush.h>
#include <edl/idomtarget.h>
#include <edl/edlblend.h>
#include <edl/idompathgeometry.h>
#include <edl/idomedgemode.h>

_BEGIN_EDL_DOM_NAMESPACE

class IDOMPathNode;
DECL_SMART_PTR(IDOMPathNode);

/**
* @class    IDOMPathNode
* @ingroup  IDom
* @brief    Interface to an EDL path node. A path node specifies a geometry that can be filled with a brush.
* 
* Vector graphics are created using the PathNode class. A full set of properties is available to
* describe the visual characteristics of the graphic. The description of the geometry of the path is
* described by the Data property. Raster images are included in fixed page markup by specifying a
* PathNode filled with an ImageBrush.
*
* A PathNode is the sole means of adding vector graphics and images to a fixed page. It defines a
* single vector graphic to be rendered on a page. Some properties of the PathNode are composable,
* meaning that the markings rendered to the page are determined by a combination of the property
* and all of the like-named properties of its parent and ancestor elements.
*/
class IDOMPathNode : public IDOMNode
{
public:
#define IDOMPathNodeClassID 0xF0E21B7F, 0x89174e0e, 0x90DD88A5, 0x89D6F6BE
    /**
    * @brief    Retrieves class id of IDOMPathNode
    * @return   @b CClassID Class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMPathNodeClassID);
        return id;
    }
    /**
    * @enum     eStrokeLineJoin
    * @ingroup  IDom
    * @brief    Specifies the different ways in which the lines in the path could be joined.
    */
    typedef enum
    {
        eMiterJoin,        //!< Use mitered joins
        eBevelJoin,        //!< Use bevelled joins
        eRoundJoin         //!< Use rounded joins
    } eStrokeLineJoin;
    
    /**
    * @enum      eStrokeMiterLimitTreatment
    * @ingroup	 IDom
    * @brief     Chooses how miters that extend beyond the miter limit should
    *            be treated. ClipLongMiters specifies XPS style behaviour, where
    *            miters extending beyond the limit are clipped to the limit.
    *            BevelLongMiters specifies PDF/PS style behaviour where miters
    *            longer than the limit are instead replaced with a bevel join.
    */
    typedef enum
    {
        eClipLongMiters,	//!< Clip long miters
        eBevelLongMiters	//!< Bevel long miters
    } eStrokeMiterLimitTreatment;

    /**
    * @enum     eStrokeLineCap
    * @ingroup  IDom
    * @brief    Specifies the different types of line end caps available.
    */
    typedef enum
    {
        eFlatCap,          //!< Flat end caps
        eSquareCap,        //!< Square end caps
        eRoundCap,         //!< Round end caps
        eTriangleCap       //!< Triangular end caps
    } eStrokeLineCap;

    /**
    * @brief    Simplified creator for a filled path.
    *           Throws an IEDLError on failure.
    * @param    pFactory The factory to use.
    * @param    geometry The path data/geometry to fill.
    * @param    brush The fill brush to use.
    * @param    renderTransform The render transform to use.
    * @param    clip The geometry to use for clipping. NULL if no clip.
    * @param    opacity The opacity to use.
    * @param    blendMode The blend mode to use.
    * @param    edgeMode The edge mode to use.
    * @return   @b IDOMPathNodePtr The new path.
    */
    static EDL_API IDOMPathNodePtr createFilled(IEDLClassFactory *pFactory,
                                                const IDOMPathGeometryPtr &geometry,
                                                const IDOMBrushPtr &brush,
                                                const FMatrix renderTransform = FMatrix(),
                                                const IDOMPathGeometryPtr &clip = IDOMPathGeometryPtr(),
                                                float opacity = 1.0f,
                                                eBlendMode blendMode = eBlendModeNormal,
                                                eEdgeMode edgeMode = eEMDefault);
    
    /**
    * @brief    Simplified creator for a stroked path.
    *           Throws an IEDLError on failure.
    * @param    pFactory The factory to use.
    * @param    geometry The path data/geometry to fill.
    * @param    brush The fill brush to use.
    * @param    renderTransform The render transform to use.
    * @param    clip The geometry to use for clipping. NULL if no clip.
    * @param    strokeThickness The stroke thickness. If this value is 0 and zeroWidthLinesAreVisible then treat the stroke as a hairline stroke.
    * @param    miterLimit The miter limit to be used, if applicable
    * @param    join The line join to use
    * @param    startCap The starting line cap
    * @param    endCap The ending line cap
    * @param    dashCap The line cap to be used for dashed sections
    * @param    miterTreatment How to treat miters that exceed the miter limit (see setStrokeMiterLimit() for details)
    * @param    dashOffset The dash offset to use if dashing is to be used
    * @param    dashPattern The dash pattern to be used. Pass an empty vector is no dashing is required. If strokeThickness is 0 then this is in user units, otherwise this is scaled up by the strokeThickness.
    * @param    zeroWidthLinesAreVisible Set to true if zero width lines should mark (see getShouldZeroWidthLinesBeVisible() for details)
    * @param    opacity The opacity to use.
    * @param    edgeMode The edge mode to use.
    * @param    blendMode The blend mode to use.
    * @return   @b IDOMPathNodePtr The new path.
    */
    static EDL_API IDOMPathNodePtr createStroked(IEDLClassFactory *pFactory,
                                                 const IDOMPathGeometryPtr &geometry,
                                                 const IDOMBrushPtr &brush,
                                                 const FMatrix renderTransform = FMatrix(),
                                                 const IDOMPathGeometryPtr &clip = IDOMPathGeometryPtr(),
                                                 double strokeThickness = 1.0,
                                                 double miterLimit = 10.0,
                                                 eStrokeLineJoin join = eMiterJoin,
                                                 eStrokeLineCap startCap = eFlatCap,
                                                 eStrokeLineCap endCap = eFlatCap,
                                                 eStrokeLineCap dashCap = eFlatCap,
                                                 eStrokeMiterLimitTreatment miterTreatment = eClipLongMiters,
                                                 double dashOffset = 0.0,
                                                 const CEDLVector<double> dashPattern = CEDLVector<double>(),
                                                 bool zeroWidthLinesAreVisible = false,
                                                 float opacity = 1.0f,
                                                 eBlendMode blendMode = eBlendModeNormal,
                                                 eEdgeMode edgeMode = eEMDefault);
    
    /**
    * @brief    Convenience creator for a path containing a rectangular image.
    *           Throws an IEDLError on failure.
    *
    *           The resulting path will have an image brush which paints the entire
    *           image scaled to the given destination rectangle.
    *
    * @param    pFactory The factory to use.
    * @param    image The image to use.
    * @param    viewPort The destination rectangle.
    * @param    renderTransform The render transform to apply.
    * @return   @b IDOMPathNodePtr The new path.
    */
    static EDL_API IDOMPathNodePtr createImage(IEDLClassFactory *pFactory,
                                               const IDOMImagePtr &image,
                                               const FRect &viewPort,
                                               const FMatrix &renderTransform = FMatrix());
    
    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        Data(): opacity(1.0), strokeThickness(1.0), miterLimit(10.0), strokeLineJoin(eMiterJoin),
                strokeStartLineCap(eFlatCap), strokeEndLineCap(eFlatCap), strokeDashLineCap(eFlatCap),
                strokeDashOffset(0.0), blendMode(eBlendModeNormal), strokeMiterLimitTreatment(eClipLongMiters),
                zeroWidthLinesAreVisible(false), edgeMode(eEMDefault), snapsToDevicePixels(false)
        {}
        float opacity;
        double strokeThickness;
        double miterLimit;
        eStrokeLineJoin strokeLineJoin;
        eStrokeLineCap strokeStartLineCap;
        eStrokeLineCap strokeEndLineCap;
        eStrokeLineCap strokeDashLineCap;
        eStrokeMiterLimitTreatment strokeMiterLimitTreatment;
        double strokeDashOffset;
        bool snapsToDevicePixels;
        EDLString language;
        EDLString automationPropertiesName;
        EDLString automationPropertiesHelpText;
        FMatrix renderTransform;
        IDOMTargetPtr ptrNavigateLink;
        IDOMBrushPtr ptrFill;
        IDOMBrushPtr ptrStroke;
        IDOMBrushPtr ptrOpacityMask;
        IDOMPathGeometryPtr ptrPathData;
        IDOMPathGeometryPtr ptrClip;
        eBlendMode          blendMode;
        eEdgeMode           edgeMode;
        bool zeroWidthLinesAreVisible;
    };

    /**
    * @brief    Retrieves the render transform matrix.
    * The render transform matrix establishes a new coordinate frame for all attributes of the
    * path and for all child elements of the path, such as the path geometry.
    * @param    matrix Render transform matrix.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool getRenderTransform(FMatrix &matrix) const = 0;

    /**
    * @brief    Sets the render transform matrix.
    * The render transform matrix establishes a new coordinate frame for all attributes of the
    * path and for all child elements of the path, such as the path geometry.
    * @param    matrix The new render transform matrix.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setRenderTransform(const FMatrix &matrix) = 0;

    /**
    * @brief    Retrieves the opacity value of the path.
    * The opacity value defines the uniform transparency of the path. 
    * This is a number between 0 (fully transparent) and 1 (fully opaque).
    * Default value 1.0
    *
    * @return   @b float The opacity value
    */
    virtual float getOpacity() const = 0;

    /**
    * @brief    Sets the opacity value of the path.
    * The opacity value defines the uniform transparency of the path. 
    * This is a number between 0 (fully transparent) and 1 (fully opaque).
    * @param    opc The new opacity value.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setOpacity(float opc) = 0;

    /**
    * @brief    Get the blend mode to be used for rendering this path.
    * @return   @b eBlendMode The blend mode.
    */
    virtual eBlendMode getBlendMode() const = 0;

    /**
    * @brief    Set the blend mode to be used for rendering this path.
    *           Note: modes other than Normal are not directly
    *           representable in XPS.
    * @param    blendMode The desired blend mode.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setBlendMode(eBlendMode blendMode) = 0;

    /**
    * @brief    Retrieves render options edge mode of the path.
    * 
    * Render options edge mode controls how the edges of the path are painted.
    * The only valid value for edge mode is Aliased. 
    * Omitting this attribute causes the edges to be rendered in the consumer's default manner
    *
    * @return   \link eEdgeMode \endlink The function returns the edge mode of the path.
    */
    virtual eEdgeMode getEdgeMode () const = 0;

    /**
    * @brief    Sets render the options edge mode of the path.
    * 
    * The EdgeMode property controls how the edges of the path are rendered.
    * The only valid value is Aliased. Omitting this attribute causes the edges to be rendered in the 
    * consumers default manner. The EdgeMode property can be set in a path to 
    * instruct anti-aliasing consumers to render the path without performing antialiasing.
    *
    * @param    em The new edge mode for the canvas.
    */
    virtual void setEdgeMode (eEdgeMode em) = 0;

    /**
    * @brief    Retrieves the stroke thickness. 
    * The stroke thickness specifies the thickness 
    * of a stroke, in units of the effective coordinate space including the path's render transform. 
    * The stroke is drawn on top of the boundary of the geometry specified by the path geometry 
    * information. Half of the stroke thickness extends outside of the geometry and the other half 
    * extends inside of the geometry.
    * @return   @b double Stroke thickness (1.0 default)
    */
    virtual double getStrokeThickness() const = 0;

    /**
    * @brief    Sets stroke thickness
    * The stroke thickness specifies the thickness 
    * of a stroke, in units of the effective coordinate space including the path's render transform. 
    * The stroke is drawn on top of the boundary of the geometry specified by the path geometry 
    * information. Half of the stroke thickness extends outside of the geometry and the other half 
    * extends inside of the geometry.
    * @param    st Stroke thickness (1.0 default)
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setStrokeThickness(double st) = 0;
    
    /**
    * @brief    Should zero width strokes be visible as cosmetic lines?
    *           Zero width lines in XPS files are not visible, but in PDF and
    *           PostScript, such lines are rendered as a single pixel cosmetic line.
    *           This routine returns true if a zero width stroke for this path
    *           should be rendered.
    * @return   @b bool True if a zero width stroke in this path should be
    *           visible as a cosmetic line.
    */
    virtual bool getShouldZeroWidthLinesBeVisible() const = 0;
    
    /**
    * @brief    Sets whether or not zero width strokes in this path should be
    *           rendered visibly as a cosmetic line.
    * @param    visible
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setShouldZeroWidthLinesBeVisible(bool visible) = 0;

    /**
    * @brief    Retrieves the stroke miter limit. 
    * The stroke miter limit is the ratio between the maximum miter length and half of the
    * stroke thickness. This value must be equal to or greater than 1.0. The value is significant only if
    * the StrokeLineJoin attribute specifies mitered joins.
    * @return   @b double Stroke miter limit (10.0 default).
    */
    virtual double getStrokeMiterLimit() const = 0;

    /**
    * @brief    Sets the stroke miter limit.
    * The stroke miter limit is the ratio between the maximum miter length and half of the
    * stroke thickness. This value must be equal to or greater than 1.0. The value is significant only if
    * the StrokeLineJoin attribute specifies mitered joins.
    * @param    sml Stroke miter limit (10.0 default).
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setStrokeMiterLimit(double sml) = 0;

    /**
    * @brief    Retrieves the miter limit treatment for this path. 
    * The stroke miter treatment specifies how miters extending beyond the limit should be treated.
    * @return   \link eStrokeMiterLimitTreatment \endlink Stroke miter treatment. (ClipLongMiters default).
    */
    virtual eStrokeMiterLimitTreatment getStrokeMiterLimitTreatment() const = 0;

    /**
    * @brief    Sets the stroke miter limit treatment.
    * The stroke miter treatment specifies how miters extending beyond the limit should be treated.
    * @param    treatment The desired stroke miter treatment.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setStrokeMiterLimitTreatment(eStrokeMiterLimitTreatment treatment) = 0;

    /**
    * @brief    Retrieves the stroke dash offset value.
    * This adjusts the start point for 
    * repeating the dash array pattern. If this value is omitted, the dash array aligns 
    * with the origin of the stroke. Values are specified as multiples of the stroke thickness.
    * @return   @b double Stroke dash offset (0.0 default)
    */
    virtual double getStrokeDashOffset() const = 0;

    /**
    * @brief    Sets the stroke dash offset value.
    * This adjusts the start point for 
    * repeating the dash array pattern. If this value is omitted, the dash array aligns 
    * with the origin of the stroke. Values are specified as multiples of the stroke thickness.
    * @param    sdo Stroke dash offset (0.0 default)
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setStrokeDashOffset(double sdo) = 0;

    /**
    * @brief    Retrieves the stroke line join value.
    * The stroke line join specifies how a stroke is drawn at a corner of a path. Valid values are
    * specified by eStrokeLineJoin. If mitered joins are selected, the value of StrokeMiterLimit is
    * used in drawing the stroke.
    * @see eStrokeLineJoin
    * @return   \link eStrokeLineJoin \endlink Stroke line join (Miter default)
    */
    virtual eStrokeLineJoin getStrokeLineJoin() const = 0;

    /**
    * @brief    Sets the stroke line join value.
    * The stroke line join specifies how a stroke is drawn at a corner of a path. Valid values are
    * specified by eStrokeLineJoin. If mitered joins are selected, the value of StrokeMiterLimit is
    * used in drawing the stroke.
    * @see eStrokeLineJoin
    * @param    slj The stroke line join (Miter default)
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setStrokeLineJoin(eStrokeLineJoin slj) = 0;

    /**
    * @brief    Retrieves the line cap type for the start of the stroke.
    * @see      eStrokeLineCap
    * @return   \link eStrokeLineCap \endlink The type of line cap to use at the start of the stroke.
    */
    virtual eStrokeLineCap getStrokeStartLineCap() const = 0;

    /**
    * @brief    Sets the line cap type for the start of the stroke.
    * @see        eStrokeLineCap
    * @param    slc The stroke start line cap (Flat default)
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setStrokeStartLineCap(eStrokeLineCap slc) = 0;

    /**
    * @brief    Retrieves the line cap type for the end of the stroke.
    * @see        eStrokeLineCap
    * @return   \link eStrokeLineCap \endlink The type of line cap to use at the end of the stroke.
    */
    virtual eStrokeLineCap getStrokeEndLineCap() const = 0;

    /**
    * @brief    Sets the line cap type for the end of the stroke.
    * @see        eStrokeLineCap
    * @param    slc The stroke end line cap (Flat default)
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setStrokeEndLineCap(eStrokeLineCap slc) = 0;

    /**
    * @brief    Gets the stroke dash line cap.
    * @see        eStrokeLineCap
    * @return   \link eStrokeLineCap \endlink Stroke dash line cap (Flat default)
    */
    virtual eStrokeLineCap getStrokeDashLineCap() const = 0;

    /**
    * @brief    Sets the stroke dash line cap.
    * @see        eStrokeLineCap
    * @param    slc The stroke dash line cap (Flat default)
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setStrokeDashLineCap(eStrokeLineCap slc) = 0;

    /**
    * @brief    Retrieves the snapsToDevicePixels setting.
    *
    * The ignorable attribute snapsToDevicePixels can be set to allow consumers or viewers that perform
    * anti-aliasing to "snap" any path control points that are situated on the path bounding box 
    * to whole device pixels.
    * @return   @b bool True if control points snap to the nearest device pixels.
    */
    virtual bool getSnapsToDevicePixels() const = 0;

    /**
    * @brief    Sets snapsToDevicePixels.
    *
    * The ignorable attribute snapsToDevicePixels can be set to allow consumers or viewers that perform
    * anti-aliasing to "snap" any path control points that are situated on the path bounding box 
    * to whole device pixels.
    * @param    s2dp New value of snapsToDevicePixels.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setSnapsToDevicePixels(bool s2dp) = 0;

    /**
    * @brief    Retrieves the default language of the path node and any of its children.
    *
    * English is defined as en_GB and American English as en_US. There is no default setting. 
    * If the language is not known it is set to und (undetermined). For further information see
    * http://www.w3.org/International/articles/language-tags/.
    *
    * The language is specified according to RFC 3066.
    *
    * @param    lang Default language
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool getLanguage (EDLString &lang) const = 0;

    /**
    * @brief    Sets default language of the \<Path\> element and any of its children
    *
    * English is defined as en_GB and American English as en_US. There is no default setting. 
    * If the language is not known it is set to und (undetermined). For further information see
    * http://www.w3.org/International/articles/language-tags/.
    *
    * The language is specified according to RFC 3066.
    *
    * @param    lang Default language
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setLanguage (const EDLString &lang) = 0;

    /**
    * @brief    Retrieves the target of a hyperlink.
    * @param    target A smart pointer to receive the target of the hyperlink.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool getNavigateLink (IDOMTargetPtr &target) const = 0;

    /**
    * @brief    Sets the target of a hyperlink.
    * @param    target The target for the hyperlink.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setNavigateLink (const IDOMTargetPtr &target) = 0;

    /**
    * @brief    Retrieves the automation properties name of the path.
    * 
    * The automation properties name is a brief description of the path's content, used
    * for accessibility purposes, particularly if the path is filled with an image brush, 
    * or a set of vector graphics and text elements intended to comprise a single vector graphic.
    *
    * @param    propname Automation properties name 
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool getAutomationPropertiesName (EDLString &propname) const = 0;

    /**
    * @brief    Sets the automation properties name of the path.
    * 
    * Automation properties help text is a detailed description of the path's content, used
    * for accessibility purposes, particularly if the path is filled with an image brush, 
    * or a set of vector graphics and text elements intended to comprise a single vector graphic.
    *
    * @param    propname The new automation properties name.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setAutomationPropertiesName (const EDLString &propname) = 0;

    /**
    * @brief    Retrieves the automation properties help text of the path.
    * 
    * Automation properties help text is a detailed description of the path's content, used
    * for accessibility purposes, particularly if the path is filled with an image brush, 
    * or a set of vector graphics and text elements intended to comprise a single vector graphic.
    *
    * @param    helptext Automation properties help text 
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool getAutomationPropertiesHelpText (EDLString &helptext) const = 0;

    /**
    * @brief    Sets the automation properties help text of the path.
    * 
    * Automation properties help text is a detailed description of the path's content, used
    * for accessibility purposes, particularly if the path is filled with an image brush, 
    * or a set of vector graphics and text elements intended to comprise a single vector graphic.
    *
    * @param    helptext Automation properties help text 
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setAutomationPropertiesHelpText (const EDLString &helptext) = 0;
    
    /**
    * @brief    Checks to see if the path is dashed. Even if a dash array is provided
    *           it may still effectively represent a plain un-dashed line. In this case
    *           this member will return false.
    * @return   @b bool True if the stroke is affected by dashing, false otherwise.
    */
    virtual bool getIsDashed() = 0;

    /**
    * @brief    Retrieves an enumerator - navigable list interface - of the stroke dash array.
    *
    * The stroke dash array specifies the length of dashes and gaps of the outline stroke. 
    * The dash and gap lengths are specified as multiples of the stroke thickness and are stored
    * in an array containing an even number of non-negative values.
    *
    * The first argument in the array is the width of the first dash. The second is the width of the gap
    * following the first dash. The third argument is the second dash width, followed by another gap
    * width, and so on. If you specify an odd number of elements, the elements are repeated to produce
    * an even number.
    *
    * When a stroke is drawn, the dashes and gaps specified by these values are repeated to 
    * cover the length of the stroke. If this attribute is omitted, the stroke is drawn solid, without any gaps.
    * @return   @b IDoubleCollectionEnumPtr The enumerator of the stroke dash array
    */
    virtual IDoubleCollectionEnumPtr getStrokeDashCollectionEnum() = 0;

    /**
    * @brief    Retrieves the length of the stroke dash array.
    * @return   @b uint32 The number of entries in the stroke dash array.
    */
    virtual uint32 getStrokeDashsCount() = 0;

    /**
    * @brief    Clears the stroke dash array.
    */
    virtual void clearStrokeDashCollection() = 0;

    /**
    * @brief    Append a stroke dash to the stroke dash array.
    * @param    value The new stroke dash array item.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool addStrokeDash(double value) = 0;

    /**
    * @brief    Retrieves the fill brush for the path.
    * @param    ptrFill Reference parameter to receive a smart pointer to the fill brush.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool getFill(IDOMBrushPtr &ptrFill) const = 0;

    /**
    * @brief    Sets the fill brush for the path.
    * @param    ptrFill Smart pointer to the new fill brush.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setFill(const IDOMBrushPtr &ptrFill) = 0;

    /**
    * @brief    Retrieves the stroke brush for the path.
    * @param    ptrStroke Reference parameter to receive a smart pointer to the stroke brush
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool getStroke(IDOMBrushPtr &ptrStroke) const = 0;

    /**
    * @brief    Sets the stroke brush for the path.
    * @param    ptrStroke Smart pointer to the new stroke brush.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setStroke(const IDOMBrushPtr &ptrStroke) = 0;

    /**
    * @brief    Retrieves the opacity mask for the path.
    *
    * The opacity mask specifies a mask of alpha values that is applied to the path in the same fashion
    * as the simple opacity setting, but allowing different alpha values for different area of the canvas.
    * With an opacity mask, you can combine an object with one or more other objects to define transparent
    * areas of the shape.
    *
    * Opacity masks use the values of one object or image to define the transparent areas of another.
    * The objects that define the transparency can be any type of visual element - solid objects, strokes, 
    * gradients, raster images, text or combinations of all of the above.
    * @param    ptrOpacityMask Smart pointer to opacity mask
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool getOpacityMask(IDOMBrushPtr &ptrOpacityMask) const = 0;

    /**
    * @brief    Sets the opacity mask for the path.
    *
    * The opacity mask specifies a mask of alpha values that is applied to the path in the same fashion
    * as the simple opacity setting, but allowing different alpha values for different area of the canvas.
    * With an opacity mask, you can combine an object with one or more other objects to define transparent
    * areas of the shape.
    *
    * Opacity masks use the values of one object or image to define the transparent areas of another.
    * The objects that define the transparency can be any type of visual element - solid objects, strokes, 
    * gradients, raster images, text or combinations of all of the above.
    * @param    ptrOpacityMask Smart pointer to brush
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setOpacityMask(const IDOMBrushPtr &ptrOpacityMask) = 0;

    /**
    * @brief    Retrieves a smart pointer to the path geometry node.
    * @param    ptrPathData Reference parameter to receive a pointer to the path geometry node.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool getPathData(IDOMPathGeometryPtr &ptrPathData) const = 0;

    /**
    * @brief    Sets the path geometry node.
    * @param    ptrPathData Smart pointer to the new path geometry node.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setPathData(const IDOMPathGeometryPtr &ptrPathData) = 0;

    /**
    * @brief    Retrieves a smart pointer to the clip geometry node.
    *
    * The clip geometry node specifies a clipping region which describes
    * the geometric area to be preserved. The remainder is not rendered.
    * @param    ptrClip Reference parameter to recieve a smart pointer to the clip geometry node.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool getClip(IDOMPathGeometryPtr &ptrClip) const = 0;

    /**
    * @brief    Sets the clip geometry
    *
    * The clip geometry node specifies a clipping region which describes
    * the geometric area to be preserved. The remainder is not rendered.
    * @param    ptrClip Smart pointer to the new clip geometry node.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setClip(const IDOMPathGeometryPtr &ptrClip) = 0;
    
    /**
    * @brief   If the path represents both a fill and a stroke, separate
    *          the fill and stroke into separate paths.
    *          Throws an IEDLError on failure.
    * @param   pFactory The factory to use.
    * @return  @b IDOMNodePtr Either an IDOMGroup, IDOMTransparencyGroup or
    *          canvas representing the split results, or the path itself
    *          if no splitting is required.
    */
    virtual IDOMNodePtr split(IEDLClassFactory *pFactory) = 0;
    
    /**
     *  @brief Get the scan-converted shape of this path
     *  @param ptrShape   Reference parameter to receive a smart pointer to the shape.
     *  @param transform  The transform that should be applied before scan conversion.
     *  @param resolution The resolution that should be used for scan conversion.
     *  @param pFactory   The factory to use.
     *  @return @b bool True on success, false if the call fails
     */
    virtual bool getShape(IDOMShapePtr &ptrShape, const FMatrix &transform, float resolution, IEDLClassFactory *pFactory) = 0;

};
#define edlobj2IDOMPathNode(src) edl_cast((IDOMPathNode *)NULL, src) 


_END_EDL_DOM_NAMESPACE

#endif /* EDLIDOMPATH_H */
