/*
* Copyright (C) 2007-2016 Global Graphics Software Ltd. All rights reserved.
*/

/**
* @file     idompathgeometry.h
* @ingroup  IDom
* @brief    DOM interfaces for path geometry, figures and segments.
*/

#ifndef EDLIDOMPATHGEOMETRY_H
#define EDLIDOMPATHGEOMETRY_H

#include <edl/edltypes.h>
#include <edl/edlfwd.h>
#include <edl/edlgeom.h>
#include <edl/iedlobject.h>
#include <edl/edlnamespaces.h>
#include <edl/idomshape.h>
_BEGIN_EDL_DOM_NAMESPACE

/**
* @class    IDOMPathSegment
* @see      IDOMPathNode
* @see      IDOMPathGeometry
* @see      IDOMPathFigure
* @ingroup  IDom
* @brief    Interface to path segment element. 
* The path segment is the smallest unit in a path geometry.
*
* Segments may be lines or curves. One or more segments are combined into a PathFigure definition. 
* A PathFigure is a single shape comprised of continuous segments. 
* One or more PathFigures collectively define an entire path geometry.
* 
*/
class IDOMPathSegment : public IEDLObject
{
public:
    /**
    * @brief    Retrieves the value for IsStroked.
    * IsStroked specifies whether the stroke for this segment of the path
    * is drawn. Can be true or false.
    * @return   bool. Returns true if this line segment is stroked, false if it is not.
    */
    virtual bool getIsStroked() const = 0;
    /**
    * @brief    Sets the value of IsStroked.
    * IsStroked specifies whether the stroke for this segment of the path
    * is drawn. Can be true or false.
    * @param    isStroked New value of IsStroked.
    * @return   bool. Returns true on success, false if the call fails.
    */
    virtual bool setIsStroked(bool isStroked) = 0;
    /**
    * @brief    Gets the conservative bounding box of the segment given the start point.
    * @param    startPoint The start point of the segment.
    * @param    bounds Reference parameter to receive the bounding box.
    * @return   bool. Returns true on success, false if the call fails.
    */
    virtual bool getBounds(FPoint &startPoint, FRect &bounds) = 0;
    /**
    * @brief    Gets the end point of the segment.
    * @param    endPoint Reference parameter to receive the end point of the segment.
    * @return   bool. Returns true on success, false if the call fails.
    */
    virtual bool getEndPoint(FPoint &endPoint) = 0;
};
DECL_SMART_PTR(IDOMPathSegment);
#define edlobj2IDOMPathSegment(src) edl_cast((IDOMPathSegment *)NULL, src) 

/**
* @class    IDOMArcSegment
* @ingroup  IDom
* @brief    Interface to Arc Segment element.
*
* An ArcSegment describes an elliptical arc. An arc segment is defined by the intersection of
* two identical ellipses. The ellipses are defined by their x-radius and y-radius, and their
* angle of rotation relative to the current coordinate system.
* The intersection between the ellipses is defined by the start and end points of the arc.
* The correct line segment of the four specified by the intersection of two ellipses is
* determined by a combination of sweepDirection and isLargeArc.
*
* Note: The starting point for the arc is defined by the end point of the previous path segment
* in the path figure (see IDOMPathFigure), or by the starting point defined in the path figure
* if this is the first path segment in the figure.
*
* Note: It is possible to specify an ellipse that is incapable of intersecting simultaneously
* with the start and end points of the desired arc. An example of this would be an ellipse
* with a total height that is less than the vertical distance between the two points.  In 
* this case, no intersection is possible and no arc will be generated.
*/
class IDOMArcSegment : public IDOMPathSegment
{
public:
#define IDOMArcSegmentClassID 0xFC8F943E, 0x23E94b1a, 0x8A078F67, 0x63F188CA
    /**
    * @brief    Retrieves the class id of IDOMArcSegment
    * @return   CClassID. The function returns the class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMArcSegmentClassID);
        return id;
    }
    /**
    * @enum     SweepDirection
    * @ingroup  IDom
    * @brief    The direction of the sweep-clockwise or counter clockwise-between the start and end points.
    *
    * The intersection of two ellipses produces four line segments.  Generally speaking, two segments will 
    * have a sweep of more than 180 degrees, and two segments will have a sweep of less than 180 degrees. 
    * Of each pair of segments, one will be drawn in a clockwise direction from the starting point
    * the other in a counter clockwise direction.  SweepDirection is used in conjunction with the
    * Boolean member isLargeArc to specify which of the four arcs is required (see getIsLargeArc()).
    */
    enum SweepDirection 
    {
        Clockwise,
        Counterclockwise
    };
    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        Data(): isStroked(true)
        {}
        FPoint point;
        double radiusX;
        double radiusY;
        double rotationAngle;
        bool isLargeArc;
        bool isStroked;
        enum SweepDirection sweepDirection;
    };
    /**
    * @brief    Retrieves the end point of the arc.
    *
    * Note: the start point of the arc is specified by the end point of the previous path segment 
    * in the path figure or by the start point specified in the path figure if this is the first 
    * path segment in the figure (see IDOMPathFigure).
    * @return   FPoint.  The function returns the end point of the arc.
    */
    virtual const FPoint &getPoint() const = 0;
    /**
    * @brief    Sets the end point of the arc.
    * @param    pt The new end point of the arc.
    * @return   bool. The function returns true on success, false on failure.
    */
    virtual bool setPoint(const FPoint &pt) = 0;
    /**
    * @brief    Retrieves the x-radius of the arc.
    * @return   double. The function returns the x-radius of the arc.
    */
    virtual double getRadiusX() const = 0;
    /**
    * @brief    Sets the x-radius of the base ellipses.
    * @param    radiusX The new x-radius of the base ellipses.
    * @return   bool. The function returns true on success, false on failure.
    */
    virtual bool setRadiusX(double radiusX) = 0;
    /**
    * @brief    Retrieves the y-radius of the base ellipses.
    * @return   double.  The function returns the y-radius of the base ellipses.
    */
    virtual double getRadiusY() const = 0;
    /**
    * @brief    Sets the y-radius of the base ellipses.
    * @param    radiusY The new y-radius of the base ellipses.
    * @return   bool. The function returns true on success, false on failure.
    */
    virtual bool setRadiusY(double radiusY) = 0;
    /**
    * @brief    Retrieves the rotation angle of the base ellipses.
    *
    * The rotation angle indicates how the ellipses are rotated relative to the
    * current coordinate system. Positive values indicate clockwise rotation and 
    * negative values indicate counter clockwise rotation.
    * @return   double.  The function returns the angle of rotation in degrees, where positive 
    * values indicate clockwise rotation and negative values indicate counter clockwise  rotation.
    */
    virtual double getRotationAngle() const = 0;
    /**
    * @brief    Sets rotation angle of the arc.
    *
    * @param    rA New rotation angle. Positive values indicate clockwise rotation and 
    * negative values indicate counter clockwise rotation. The rotated ellipses 
    * must still be capable of intersecting with the specified start and end points, as before.
    * @return   bool. The function returns true on success, false on failure.
    */
    virtual bool setRotationAngle(double rA) = 0;
    /**
    * @brief    Retrieves isLargeArc.
    *
    * The member isLargeArc determines whether the specified arc is one of the large
    * arcs (sweep 180 degrees or greater) or one of the small arcs (sweep less than 
    * 180 degrees) produced by the intersection of the two ellipses. In conjunction with
    * the sweep direction, this determines which of the four arcs produced by intersecting
    * ellipses is required. See also getSweepDirection().
    * @return   bool. The function returns true if the arc is large
    */
    virtual bool getIsLargeArc() const = 0;
    /**
    * @brief    Sets isLargeArc.
    * See getIsLargeArc() for a description of how isLargeArc determines the arc to be drawn.
    * @param    isLA New value for isLargeArc.
    * @return   bool. The function returns true on success, false on failure.
    */
    virtual bool setIsLargeArc(bool isLA) = 0;
    /**
    * @brief    Retrieves the sweep direction.
    *
    * SweepDirection specifies the direction in which the arc is drawn from the start point.
    * At the start point, all four line segments converge.  Two of these will be short 
    * arcs and two will be long arcs.  One long arc and one short arc will move away from the start
    * point following a clockwise path; the other two arcs will follow a counter clockwise path.
    * In conjunction with isLargeArc, the sweep direction determines which of the four line
    * segments is the specified arc.
    * @return   SweepDirection. The return value specifies whether the stroke direction is
    * clockwise or counter clockwise from the start point.
    */
    virtual enum SweepDirection getSweepDirection() const = 0;
    /**
    * @brief    Sets sweep direction
    * @param    sd SweepDirection The direction in which the arc is drawn from the start point.
    * @return   bool. The function returns true on success, false on failure.
    */
    virtual bool setSweepDirection(enum SweepDirection sd) = 0;
    /**
    * @brief    Create a segment that represents this arc using a simpler segment type.
    *
    * This member will convert this arc path segment to a simpler segment type.
    * If the arc segment is degenerate, the result will be a single IDOMPolyLineSegment
    * representing this. Otherwise, the arc segment will be decomposed to cubic beziers
    * in an IDOMPolyBezierSegment.
    * @param    newSegment Smart pointer to receive the resulting segment
    * @param    startPoint The start point for the arc
    * @param    factory Pointer to the class factory to be used
    * @return   bool. The function returns true on success, false on failure.
    */
    virtual bool convertToSimpleSegment(IDOMPathSegmentPtr &newSegment, FPoint &startPoint, IEDLClassFactory *factory) = 0;
};
DECL_SMART_PTR(IDOMArcSegment);
#define edlobj2IDOMArcSegment(src) edl_cast((IDOMArcSegment *)NULL, src) 

/**
* @class    IDOMPolyLineSegment
* @ingroup  IDom
* @brief    Interface to a polyline segment node.
* A polyline segment describes a polygonal drawing containing an arbitrary number of individual
* vertices. The Points attribute defines the vertices.
*/
class IDOMPolyLineSegment : public IDOMPathSegment
{
#define IDOMPolyLineSegmentClassID 0x858ed3ce, 0xc14b4bdf, 0x8f1fb58b, 0xa046afd2
public:
    /**
    * @brief    Retrieves class id of IDOMPolyLineSegment
    * @return   CClassID class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMPolyLineSegmentClassID);
        return id;
    };
    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        Data(): isStroked(true)
        {}
        bool isStroked;
    };
    /**
    * @brief    Retrieves a navigable list-an enumerator-of the points in the polyline segment.
    * @return   IFPointCollectionEnumPtr.  Returns a pointer to the points list enumerator.
    */
    virtual IFPointCollectionEnumPtr getPointEnum() = 0;
    /**
    * @brief    Retrieves the number of points in the segment.
    * @return   uint32. Returns the size of the points list.
    */
    virtual uint32 getPointsCount() = 0;
    /**
    * @brief    Clears the points list.
    */
    virtual void clearPoints() = 0;
    /**
    * @brief    Appends a point to the list.
    * @param    FPoint Point to append
    * @return   bool. Returns true on success, false if the call fails.
    */
    virtual bool addPoint(const FPoint &point) = 0;
};
DECL_SMART_PTR(IDOMPolyLineSegment);
#define edlobj2IDOMPolyLineSegment(src) edl_cast((IDOMPolyLineSegment *)NULL, src) 

/**
* @class    IDOMPolyBezierSegment
* @ingroup  IDom
* @brief    Interface to a path segment node describing a set of cubic B&eacute;zier curves.
*
* B&eacute;zier curves are drawn from the previous point in the path figure or the previous B&eacute;zier curve in the 
* segment and terminate at the third point (x3n,y3n) in the Points attribute (where n is the curve being 
* drawn). The tangents and curvature of each B&eacute;zier curve are controlled by the first two control points 
* (x3n-2,y3n-2 and x3n-1,y3n-1) in the Points attribute. The Points attribute contains a multiple of three 
* whitespace delimited pairs of comma delimited x,y values.
*/
class IDOMPolyBezierSegment : public IDOMPathSegment
{
#define IDOMPolyBezierSegmentClassID 0xb2c0876c, 0x5f1b4e0a, 0xb6f0babb, 0x23061b1e
public:
    /**
    * @brief    Retrieves class id of IDOMPolyBezierSegment
    * @return   CClassID class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMPolyBezierSegmentClassID);
        return id;
    };
    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        Data(): isStroked(true)
        {}
        bool isStroked;
    };
    /**
    * @brief    Retrieves a navigable list-an enumerator-of the points in the B&eacute;zier segment.
    * The points list specifies control points for multiple B&eacute;zier segments.
    * @return   IFPointCollectionEnumPtr. Returns the enumerator for the points list.
    */
    virtual IFPointCollectionEnumPtr getPointEnum() = 0;
    /**
    * @brief    Retrieves the number of points in the points list.
    * @return   uint32. Returns the number of points in the points list.
    */
    virtual uint32 getPointsCount() = 0;
    /**
    * @brief    Clears the points list.
    */
    virtual void clearPoints() = 0;
    /**
    * @brief    Append a point to the list.
    * @param    FPoint Point to append.
    * @return   bool. Returns true on success, false if the call fails.
    */
    virtual bool addPoint(const FPoint &point) = 0;
};
DECL_SMART_PTR(IDOMPolyBezierSegment);
#define edlobj2IDOMPolyBezierSegment(src) edl_cast((IDOMPolyBezierSegment *)NULL, src) 

/**
* @class    IDOMPolyQuadraticBezierSegment
* @ingroup  IDom
* @brief    Interface to a polyqualdratic B&eacute;zier segment.
* A polyqualdratic B&eacute;zier segment describes a set of quadratic B&eacute;zier curves from the
* starting point defined in the IDOMPathFigure, or from the end point of the previous segment,
* through a set of vertices, using specified control points. The Points attribute stores an 
* off-curve control point (x2n-1, y2n-1) followed by the end point (x2n, y2n) for each quadratic B&eacute;zier 
* curve (where n represents the quadratic B&eacute;zier curve).
*/
class IDOMPolyQuadraticBezierSegment : public IDOMPathSegment
{
#define IDOMPolyQuadraticBezierSegmentClassID 0x8c8ae8b7, 0x7ad64100, 0x8e55774e, 0x7e06ba34
public:
    /**
    * @brief    Retrieves the class id of IDOMPolyQuadraticBezierSegment.
    * @return   CClassID. Returns the class id of the element.
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMPolyQuadraticBezierSegmentClassID);
        return id;
    };
    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        Data(): isStroked(true)
        {}
        bool isStroked;
    };
    /**
    * @brief    Retrieves a navigable list-an enumerator-of the points in the polyquadratic B&eacute;zier segment.
    * @return   IFPointCollectionEnumPtr. Returns the enumerator to the points list.
    */
    virtual IFPointCollectionEnumPtr getPointEnum() = 0;
    /**
    * @brief    Retrieves the number of points in the polyquadratic B&eacute;zier segment.
    * @return   uint32. Returns the size of the points list.
    */
    virtual uint32 getPointsCount() = 0;
    /**
    * @brief    Clears the points list.
    */
    virtual void clearPoints() = 0;
    /**
    * @brief    Append a point to the list.
    * @param    FPoint Point to append
    * @return   bool. Returns true on success, false if the call fails.
    */
    virtual bool addPoint(const FPoint &point) = 0;
    /**
    * @brief    Create a segment that represents this curve using a cubic bezier.
    * @param    newSegment Smart pointer to receive the resulting cubic bezier
    * @param    startPoint The start point for the curve
    * @param    factory Pointer to the class factory to be used
    * @return   bool. The function returns true on success, false on failure.
    */
    virtual bool convertToCubicBezierSegment(IDOMPolyBezierSegmentPtr &newSegment, FPoint &startPoint, IEDLClassFactory *pFactory) = 0;    
};
DECL_SMART_PTR(IDOMPolyQuadraticBezierSegment);
#define edlobj2IDOMPolyQuadraticBezierSegment(src) edl_cast((IDOMPolyQuadraticBezierSegment *)NULL, src) 

typedef IInputEnumRC<IDOMPathSegmentPtr> IDOMSegmentCollectionEnum;
DECL_SMART_PTR(IDOMSegmentCollectionEnum);

/**
* @class    IDOMPathFigure
* @see      IDOMPathNode
* @see      IDOMPathSegment
* @see      IDOMPathGeometry
* @ingroup  IDom
* @brief    Interface to the path figure element. A path figure is a single shape comprised of
* continuous path segments. One or more path figures collectively define an entire path geometry. A
* path geometry may define the fill algorithm to be used on the component PathFigures.
*/
class IDOMPathFigure : public IEDLObject
{
public:
#define IDOMPathFigureClassID 0x69B82E30, 0xA5D24512, 0xB44599CE, 0x21AF0BFC
    /**
    * @brief    Retrieves the class id of IDOMPathFigure.
    * @return   CClassID. Returns the class id of the element.
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMPathFigureClassID);
        return id;
    }
    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        Data(): isFilled(true), isClosed(false)
        {}
        FPoint startPoint;
        bool isClosed;
        bool isFilled;
    };
    /**
    * @brief    Retrieves IsClosed for the path figure. 
    * IsClosed specifies whether the path is closed, that is, whether
    * the last point in the last segment of the path figure should be connected to the start point
    * of the figure, otherwise the stroke is drawn open, and the last point is not connected to the
    * start point. This is only applicable if the path figure is used in a Path that specifies a stroke.
    * 
    * The default is false.
    * @return   bool. Returns true if the path is closed.
    */
    virtual bool getIsClosed() const = 0;
    /**
    * @brief    Sets IsClosed for the path figure. 
    * IsClosed specifies whether the path is closed. When this is set to true it specifies that the 
    * path is closed-thatis, the last point in the PathFigure is connected to the first.
    * 
    * The default is false.
    * @param    closed New value of IsClosed 
    * @return   bool. Returns true on success, false if the call fails.
    */
    virtual bool setIsClosed(bool closed) = 0;
    /**
    * @brief    Retrieves the start point for the first segment in the figure.
    * @return   FPoint. Returns the start point for the first segment in the figure.
    */
    virtual const FPoint &getStartPoint() const = 0;
    /**
    * @brief    Sets the start point for the first segment in the figure.
    * @param    sp New value of the start point of the first segment in the figure.
    * @return   bool. Returns true on success, false if the call fails.
    */
    virtual bool setStartPoint(const FPoint &sp) = 0;
    /**
    * @brief    Retrieves IsFilled for the figure. 
    * IsFilled specifies whether the path figure is used in computing the area of the containing path geometry.
    * When set to false, the path figure is considered only for stroking.
    * @return   bool. Returns true if the path is filled, false otherwise.
    */
    virtual bool getIsFilled() const = 0;
    /**
    * @brief    Sets IsFilled for the figure. 
    * IsFilled specifies whether the path figure is used in computing the area of the containing path geometry.
    * When set to false, the path figure is considered only for stroking.
    * @param    filled New value of IsFilled.
    * @return   bool Returns true on success, false if the call fails.
    */
    virtual bool setIsFilled(bool filled) = 0;

    /**
    * @brief    Retrieves a navigable list of the path segments in the figure.
    * @return   IDOMSegmentCollectionEnumPtr. Returns a smart pointer to the navigable list.
    */
    virtual IDOMSegmentCollectionEnumPtr getSegmentCollectionEnum() = 0;
    /**
    * @brief    Retrieves the number of path segments in the figure.
    * @return   uint32. Returns the number of path segments in the figure.
    */
    virtual uint32 getSegmentsCount() = 0;
    /**
    * @brief    Removes all path segments from the figure.
    */
    virtual void clearSegmentCollection() = 0;
    /**
    * @brief    Append a path segment to the figure.
    * @param    pathSegment Smart pointer to the path segment to add.
    * @return   bool Returns true on success, false if the call fails.
    */
    virtual bool addSegment(const IDOMPathSegmentPtr &ptrSegment) = 0;
    /**
    * @brief    Finds the conservative bounding box of the figure.
    * @param    bounds Reference variable to receive the bounding box.
    * @return   bool Returns true on success, false if the call fails.
    */
    virtual bool getBounds(FRect &bounds) = 0;
};
DECL_SMART_PTR(IDOMPathFigure);
#define edlobj2IDOMPathFigure(src) edl_cast((IDOMPathFigure *)NULL, src) 

typedef IInputEnumRC<IDOMPathFigurePtr> IDOMFigureCollectionEnum;
DECL_SMART_PTR(IDOMFigureCollectionEnum);

class IDOMPathGeometry;
DECL_SMART_PTR(IDOMPathGeometry);

/**
* @class    IDOMPathGeometry
* @see      IDOMPathFigure
* @see      IDOMPathNode
* @ingroup  IDom
* @brief    Interface to a path geometry node.
*
* A path geometry node constitutes a complete geometry definition. The smallest unit in a geometry
* is a segment. One or more segments are combined into a path figure definition. A path figure is a
* single shape comprised of continuous segments. One or more path figures collectively define an 
* entire path geometry. A path geometry may define the fill algorithm to be used on the component path figures.
*
* A single path geometry node may be used in the data property of the path node to describe its overall
* geometry. A path geometry node may also be used in the clip property of the Canvas, Path, or Glyphs
* nodes to describe a clipping region.
*/
class IDOMPathGeometry : public IEDLObject
{
public:
#define IDOMPathGeometryClassID 0xCC10E2AC, 0x4F8946d6, 0xA99ACB94, 0xC0379682
    
    /**
    * @brief    Simplified creator for a rectangular path geometry.
    *           The geometry will consist of a path beginning at x,y value of the
    *           rect, proceeding in a clockwise direction.
    *           Throws an IEDLError on failure.
    * @param    pFactory The factory to use.
    * @param    rect The rect to use.
    * @param    close Whether or not the rectangular path should be closed
    * @return   IDOMPathGeometryPtr The new geometry.
    */
    static EDL_API IDOMPathGeometryPtr create(IEDLClassFactory *pFactory, const FRect &rect, bool close = true);

    /**
    * @brief    Retrieves class id of IDOMPathGeometry
    * @return   CClassID class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMPathGeometryClassID);
        return id;
    }
    /**
    * @enum     eFillRule
    * @ingroup  IDom
    * @brief    Specifies the algorithm to determine whether or not a point is inside a shape on the canvas.
    *
    * "Insideness" is determined by drawing a ray from the point to infinity in any direction and counting 
    * the number of segments from the given shape that the ray crosses.
    *
    * The EvenOdd algorithm determines the "insideness" of a point on the canvas by drawing a ray
    * from the point to infinity in any direction and counting the number of segments from the given
    * shape that the ray crosses. If this number is odd, the point is inside; if it is even, the point is outside. 
    * This is the default rule.
    *
    * The NonZero rule determines the "insideness" of a point on the canvas by drawing a ray from the 
    * point to infinity in any direction and then examining the places where a segment of the shape
    * crosses the ray. Starting with a count of zero, add one each time a segment crosses the ray from
    * left to right and subtract one each time a path segment crosses the ray from right to left. After
    * counting the crossings, if the result is zero then the point is outside the path; otherwise, it is inside.
    */
    typedef enum
    {
        eEvenOdd,  //!< Use the even/odd fill algorithm
        eNonZero   //!< Use the non-zero fill algorithm
    } eFillRule;

    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        Data(): fillRule(eEvenOdd)
        {}
    public:
        eFillRule fillRule;
        FMatrix renderTransform;
    };
    /**
    * @brief    Retrieves the fill rule for the path. The valid values are specified by eFillRule.
    *
    * The FillRule attribute specifies a fill algorithm. The fillable area of the geometry is 
    * defined by taking all of the contained PathFigures and applying the fill algorithm to 
    * determine the enclosed area. Fill algorithms determine how the intersecting areas of 
    * geometric shapes are combined to form a region.
    * @return   eFillRule Fill rule (EvenOdd default)
    */
    virtual eFillRule getFillRule() const = 0;
    /**
    * @brief    Sets the fill rule for the path. The valid values are specified by eFillRule.
    *
    * The FillRule attribute specifies a fill algorithm. The fillable area of the geometry is 
    * defined by taking all of the contained PathFigures and applying the fill algorithm to 
    * determine the enclosed area. Fill algorithms determine how the intersecting areas of 
    * geometric shapes are combined to form a region.
    * @param    fr Fill rule
    * @return   bool Returns true on success.
    */
    virtual bool setFillRule(eFillRule fr) = 0;
    /**
    * @brief    Retrieves the render transform matrix for the path.
    * @param    matrix Reference parater to receive the render transform matrix.
    * @return   bool. Returns true on success, false if the call fails.
    */
    virtual bool getRenderTransform(FMatrix &matrix) const = 0;
    /**
    * @brief    Sets the render transform matrix for the path.
    * @param    matrix The new render transform matrix.
    * @return   bool. Returns true on success, false if the call fails.
    */
    virtual bool setRenderTransform(const FMatrix &matrix) = 0;
    /**
    * @brief    Retrieves a navigable list of the figures within the path.
    * @return   IDOMFigureCollectionEnumPtr. Returns a smart pointer to the list of path figures.
    */
    virtual IDOMFigureCollectionEnumPtr getFigureCollectionEnum() = 0;
    /**
    * @brief    Retrieves the number of figures in the path.
    * @return   uint32. Returns the number of figures in the path collection.
    */
    virtual uint32 getFiguresCount() = 0;
    /**
    * @brief    Removes all the figures from the path.
    */
    virtual void clearFigureCollection() = 0; 
    /**
    * @brief    Appends a figure to the path figure collection.
    * @param    pathFigure Path figure to add
    * @return   bool. Returns true on success, false if the call fails.
    */
    virtual bool addFigure(IDOMPathFigurePtr &ptrFigure) = 0;
    /**
    * @brief    Finds the conservative bounding box of the geometry.
    * @param    bounds Reference parameter to receive the bounding box.
    * @param    applyTransform Controls whether or not the receiver's
    *           transform is applied to the bounds (if it has one).
    *           Passing true (default) gives you results in the coordinate
    *           space of the object enclosing the receiver, while passing
    *           false will give you results in the coordinate space active
    *           inside the object.
    * @return   bool. Returns true on success, false if the call fails.
    */
    virtual bool getBounds(FRect &bounds, bool applyTransform = true) = 0;
    
    /**
    * @brief   Determines if the geometry is a simple rectangle.
    *          NB: It does not check to see if the path is closed,
    *          only that its shape is a rectangle that is orthogonal
    *          to a regular cartesian axis.
    * @param   rect Reference to receive the dimensions of the rectangle
    *          if the geometry indeed represents a rectangle.
    * @return  bool. True if the geometry is a simple orthogonal rectangle.
    */
    virtual bool getIsRect(FRect &rect) = 0;
    
    /**
    * @brief   Get the scan-converted shape of this path geometry
    * @param   ptrShape Reference parameter to receive a smart pointer to the shape.
    * @param   transform The transform that should be applied before scan conversion.
    * @param   resolution The resolution that should be used for scan conversion.
    * @param   pFactory A pointer to the EDL class factory.
    * @return  bool. Returns true on success, false if the call fails.
    */
    virtual bool getShape(IDOMShapePtr &ptrShape, FMatrix &transform, float resolution, IEDLClassFactory *pFactory) = 0;
    
    /**
    * @brief   Get a simplified version of this path geometry, with certain segment
    *          types reduced to simpler (or more common) types as directed.
    *          This may provide the called object if the path is already simple.
    * @param   simpleGeometry Reference parameter to receive the simple geometry.
    * @param   simplifyQuads If true, any quadratic segments will be converted to
    *          regular cubic segments in the simplified geometry.
    * @param   simplifyArcs If true, any arc segments will be converted to regular
    *          cubic segments or line segments as appropriate.
    * @param   pFactory A pointer to the EDL class factory.
    * @return  bool Returns true on success.
    */
    virtual bool getSimplifiedGeometry(IDOMPathGeometryPtr &simpleGeometry, bool simplifyQuads, bool simplifyArcs, IEDLClassFactory *pFactory) = 0;
    
    /**
    * @brief   Get a flattened version of the path; that is, with any curves converted
    *          to straight line approximations.
    *          This may provide the called object if the geometry has no curved segments.
    *          And exception of type IEDLError is thrown on failure.
    * @param   transform The external transformation applied to the geometry. Needed to
    *          determine the visible size of the geometry, and hence influences the level
    *          of curve approximation.
    * @param   resolution The intended display resolution. The flattened geometry will be
    *          computed such that the curved approximation is not noticeable when viewed
    *          at that resolution.
    * @param   pFactory A pointer to the EDL class factory.
    * @return  IDOMPathGeometryPtr The resulting flattened geometry, or this geometry if no
    *          flattening is required.
    */
    virtual IDOMPathGeometryPtr getFlattenedGeometry(const FMatrix &transform, float resolution, IEDLClassFactory *pFactory) = 0;
};
#define edlobj2IDOMPathGeometry(src) edl_cast((IDOMPathGeometry *)NULL, src)

_END_EDL_DOM_NAMESPACE

#endif /* __EDLIDOMPATHGEOMETRY_H__ */
