/*
 * Copyright (C) 2007 Global Graphics Software Ltd. All rights reserved.
 */

/**
 * @file     idomresources.h
 * @ingroup  IDom
 * @brief    IDOMResources interface
 */

#ifndef __IDOMRESOURCES_H__
#define __IDOMRESOURCES_H__

#include <edl/edltypes.h>
#include <edl/edlstream.h>
#include <edl/idomnode.h>
#include <edl/idomhashable.h>

_BEGIN_EDL_DOM_NAMESPACE

/**
* @class    IDOMResource
* @ingroup  IDom
* @brief    Provides an interface to an EDL DOM node representing a generalised resource.
* A resource represents non-markup document content such as images, fonts and profiles. 
* Resources are generally stream based. This class provides the base class for interfaces 
* to more specialized resource node types.
*/
class IDOMResource : public IEDLObject
{
public:
    
    /**
    * @brief    Retrieves the resource stream.
    * @param    stream Reference parameter to recieve a smart pointer to the resource stream.
    * @return   bool. Returns true on success, false if the call fails.
    */
    virtual bool getStream(IInputStreamPtr &stream) const = 0;

    /**
    * @brief    Sets the resource stream for the node.
    * @param    stream Smart pointer to the new resource stream.
    * @return   bool. Returns true on success, false if the call fails.
    */
    virtual bool setStream(const IInputStreamPtr &stream) = 0;

    /**
    * @brief    Retrieves the stream length, if it is available.
    * @param    length Reference parameter to receive the stream length.
    * @return   bool. Returns true on success, false if the call fails.
    */
    virtual bool getStreamLength(uint64 &length) const = 0;

    /**
    * @brief    Retrieves the resource URI.
    * @param    uri Reference parameter to receive the resource URI.
    * @return   bool. Returns true on success, false if the call fails.
    */
    virtual bool getUri(EDLSysString &uri) const = 0;

    /**
    * @brief    Sets the resource URI.
    * @param    uri The new resource URI.
    * @return   bool. Returns true on success, false if the call fails.
    */
    virtual bool setUri(const EDLSysString &uri) = 0;

};

DECL_SMART_PTR(IDOMResource);
#define edlobj2IDOMResource(src) edl_cast((IDOMResource *)NULL, src) 


class IDOMICCProfile;
DECL_SMART_PTR(IDOMICCProfile);

/**
* @class    IDOMICCProfile
* @ingroup  IDom
* @brief    IDOMICCProfile interface
*/
class IDOMICCProfile : public IDOMHashable, public IDOMResource
{
#define IDOMICCProfileClassID 0xcf1b9ba2, 0x237a4d37, 0x8b5e3b59, 0x68b7cfca
public:

    virtual ~IDOMICCProfile ()
    {}

    /**
    * @brief    Creation function for an IDOMICCProfile
    *           Throws an IEDLError exception on failure.
    * @param    pFactory The class factory.
    * @param    stream The ICC profile stream.
    * @param    length The length of the profile stream. If 0, an attempt will be made to
    *           determine the length of the stream.
    * @param   uri The URI that should be associated with this profile, if required.
    * @return   IDOMICCProfilePtr The new profile.
    */
    static EDL_API IDOMICCProfilePtr create(IEDLClassFactory *pFactory, const IInputStreamPtr &stream, uint32 length = 0, const EDLSysString &uri = EDLSysString());

    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        Data() : streamLength(0), numComponents(0)
        {}
        EDLSysString uri;
        IInputStreamPtr stream;
        uint64 streamLength;
        int32 numComponents;
    };

    /**
    * @brief    Retrieves class id of IDOM
    * @return   CClassID Class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMICCProfileClassID);
        return id;
    };
    
    /**
    * @brief    Get the ICC profile version.
    *           Throws an exception of type IEDLError on failure.
    * @param    majorVersion Reference to receive the major version.
    * @param    minorVersion Reference to receive the minor version.
    */
    virtual void getProfileVersion(uint8 &majorVersion, uint8 &minorVersion) = 0;
    
    
};

#define edlobj2IDOMICCProfile(src) edl_cast((IDOMICCProfile *)NULL, src) 


/**
* @class    IDOMPrintTicket
* @ingroup  IDom
* @brief    IDOMPrintTicket interface
*/
class IDOMPrintTicket : public IDOMResource
{
#define IDOMPrintTicketClassID 0xe05c10f8, 0xad9f4eec, 0x839a1ad6, 0xed5d73b9
public:

    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        Data() : streamLength(0)
        {}
        EDLSysString uri;
        IInputStreamPtr stream;
        uint64 streamLength;
    };

    /**
    * @brief    Retrieves the class id of IDOMPrintTicket.
    * @return   CClassID. Returns the class id of the element.
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMPrintTicketClassID);
        return id;
    };

};

DECL_SMART_PTR(IDOMPrintTicket);
#define edlobj2IDOMPrintTicket(src) edl_cast((IDOMPrintTicket *)NULL, src) 


/**
 * @class    IDOMAudioFile
 * @ingroup  IDom
 * @brief    IDOMAudioFile interface
 */
class IDOMAudioFile : public IDOMResource
{
#define IDOMAudioFileClassID 0x50f160ca, 0xa6ff41c8, 0xabf99ec0, 0xafac3702
public:
    
    typedef enum { eAudioFormatUnknown, eAudioFormatWavRaw, eAudioFormatWavPCM, eAudioFormatWavALaw, eAudioFormatWavMulaw } eAudioFormat;
    
    /**
     * @class    Data
     * @ingroup  IDom
     * @brief    Initialization data
     */
    class Data : public CClassParams
    {
    public:
        Data() : streamLength(0)
        {}
        EDLSysString uri;
        IInputStreamPtr stream;
        uint64 streamLength;
        
        eAudioFormat format;
        uint32 sampleRate;
        uint32 channelCount;
        uint32 bitsPerChannel;
        bool doGenerateHeader;
    };
    
    /**
     * @brief    Retrieves class id of IDOM
     * @return   CClassID Class id of the element
     */
    static const CClassID &classID()
    {
        static CClassID id(IDOMAudioFileClassID);
        return id;
    };
};

DECL_SMART_PTR(IDOMAudioFile);
#define edlobj2IDOMAudioFile(src) edl_cast((IDOMAudioFile *)NULL, src)


/**
 * @class    IDOMRawDataFile
 * @ingroup  IDom
 * @brief    IDOMRawDataFile interface
 */
class IDOMRawDataFile : public IDOMResource
{
#define IDOMRawDataFileClassID 0x562734ed, 0x40d64549, 0x8df0e51c, 0xef677cba
public:
    
    /**
     * @class    Data
     * @ingroup  IDom
     * @brief    Initialization data
     */
    class Data : public CClassParams
    {
    public:
        Data() : streamLength(0)
        {}
        EDLSysString uri;
        IInputStreamPtr stream;
        uint64 streamLength;
        EDLSysString mimeType;
    };
    
    /**
     * @brief    Retrieves class id of IDOM
     * @return   CClassID Class id of the element
     */
    static const CClassID &classID()
    {
        static CClassID id(IDOMRawDataFileClassID);
        return id;
    };
    
    /**
    * @brief Get the mime type of this raw data file.
    * return EDLSysString the mime type
    */
    virtual const EDLSysString & getMimeType() const = 0;
};

DECL_SMART_PTR(IDOMRawDataFile);
#define edlobj2IDOMRawDataFile(src) edl_cast((IDOMRawDataFile *)NULL, src) 


/**
* @class    IDOMResourceDictionary
* @ingroup  IDom
* @brief    Interface to the EDL DOM's resource dictionary.
* The resource dictionary is a document resource that is shared between page markups. 
* It holds a reference list of non-markup
* content that is shared between multiple pages of the document.
*/
class IDOMResourceDictionary : public IDOMResource
{
#define IDOMResourceDictionaryClassID 0x8dac1ddf, 0x84c34ec8, 0x83e40e64, 0x0830d186
public:

    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        Data() : streamLength(0)
        {}
        EDLSysString uri;
        IInputStreamPtr stream;
        uint64 streamLength;
    };

    /**
    * @brief    Retrieves the class id of IDOMResourceDictionary.
    * @return   CClassID. Returns the class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMResourceDictionaryClassID);
        return id;
    };

    /**
    * @brief    Retrieves a smart pointer to a resource stored in the resource dictionary.
    * @param    name The name of the resource to be retrieved.
    * @param    element Reference parameter to receive a smart pointer to the resource (as an IEDLObject pointer).
    * @return   bool. Returns true on success, false if the call fails.
    */
    virtual bool get(const EDLSysString &name, IEDLObjectPtr &element) const = 0;

    /**
    * @brief    Registers a resource element in the resource dictionary.
    * @param    name The name of the resource to add.
    * @param    element Smart pointer to an IEDLObject representing the resource.
    * @return   bool. Returns true on success, false if the call fails.
    */
    virtual bool put(const EDLSysString &name, const IEDLObjectPtr &element) = 0;

};

DECL_SMART_PTR(IDOMResourceDictionary);
#define edlobj2IDOMResourceDictionary(src) edl_cast((IDOMResourceDictionary *)NULL, src) 

/**
 * @class    IDOMMatrix
 * @ingroup  IDom
 * @brief    Defines the render transform matrix.
 */

class IDOMMatrix : public IEDLObject
{
#define IDOMMatrixClassID 0x635ea123, 0x1c5d4e35, 0xb174a419, 0x0deb4eab
public:
    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        FMatrix renderTransform;
    };

    /**
    * @brief    Retrieves the class id of IDOMMatrix
    * @return   CClassID. Returns the class id of the element.
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMMatrixClassID);
        return id;
    };

    /**
    * @brief    Retrieves the render transform matrix
    * @param    matrix Render transform matrix
    * @return   bool. Returns true on success, false if the call fails.
    */
    virtual bool getRenderTransform(FMatrix &matrix) const = 0;

    /**
    * @brief    Sets the render transform matrix
    * @param    matrix Render transform matrix
    * @return   bool. Returns true on success, false if the call fails.
    */
    virtual bool setRenderTransform(const FMatrix &matrix) = 0;

};

DECL_SMART_PTR(IDOMMatrix);
#define edlobj2IDOMMatrix(src) edl_cast((IDOMMatrix *)NULL, src) 


_END_EDL_DOM_NAMESPACE

#endif /* __IDOMRESOURCES_H__ */
