/*
* Copyright (C) 2007 Global Graphics Software Ltd. All rights reserved.
*/

/**
* @file    idomtarget.h
* @ingroup IDOMTarget
* @brief   IDOMTarget Interface
*/

#ifndef __EDLIDOMTARGET_H__
#define __EDLIDOMTARGET_H__

#include <assert.h>
#include <edl/iedlobject.h>
#include <edl/edltypes.h>
#include <edl/iedlenum.h>
#include <edl/edlstring.h>
#include <edl/idomid.h>

_BEGIN_EDL_DOM_NAMESPACE


/**
* @class    IDOMTarget
* @ingroup  IDom
* @brief    Base class for defining hyperlink targets in a document.

* Several subclasses 
* exist, defining more specialized types of target. For example, internal targets appear 
* in the same document as the working page, whereas external targets appear either in a 
* different document, or on the web.
*/
class IDOMTarget : public IEDLObject
{
public:

    /**
    * @enum     eTargetType
    * @ingroup  IDom
    * @brief    An enumeration of target types
    */
    typedef enum 
    {
        eExternal,              //!< (PDF URI Action) Resolve a uniform resource identifier.
        eInternal,              //!< XPS internal target
        ePage,                  //!< XPS page target
        ePageRect,              //!< (PDF Goto Action) Go to a destination in the current document.
        eActionGoToR,           //!< ("Go-to remote") Go to a destination in another document.
        eActionGoToE,           //!< ("Go-to embedded"; PDF 1.6) Go to a destination in an embedded file.
        eActionLaunch,          //!< Launch an application, usually to open a file.
        eActionThread,          //!< Begin reading an article thread.
        eActionSound,           //!< (PDF 1.2) Play a sound.
        eActionMovie,           //!< (PDF 1.2) Play a movie.
        eActionHide,            //!< (PDF 1.2) Set an annotation's Hidden flag.
        eActionNamed,           //!< (PDF 1.2) Execute an action predefined by the viewer application.
        eActionSubmitForm,      //!< (PDF 1.2) Send data to a uniform resource locator.
        eActionResetForm,       //!< (PDF 1.2) Set fields to their default values.
        eActionImportData,      //!< (PDF 1.2) Import field values from a file.
        eActionJavaScript,      //!< (PDF 1.3) Execute a JavaScript script.
        eActionSetOCGState,     //!< (PDF 1.5) Set the states of optional content groups.
        eActionRendition,       //!< (PDF 1.5) Controls the playing of multimedia content.
        eActionTrans,           //!< (PDF 1.5) Updates the display of a document, using a transition dictionary.
        eActionGoTo3DView,      //!< (PDF 1.6) Set the current view of a 3D annotation
        eActionArray            //!< Array of actions
    } eTargetType;

    /**
    * @brief    Retrieves the target type
    * @return   eTargetType The target type
    */
    virtual eTargetType getTargetType() const = 0;

};

DECL_SMART_PTR(IDOMTarget);
#define edlobj2IDOMTarget(src) edl_cast((IDOMTarget *)NULL, src) 


/**
* @class    IDOMExternalTarget
* @ingroup  IDOMTarget
* @brief    IDOMExternalTarget interface
*
** This interface describes the existence of links to documents and resources that are not 
* part of the present document.
*
*/
class IDOMExternalTarget : public IDOMTarget
{
#define IDOMExternalTargetClassID 0x45192b05, 0xa73c447f, 0xbd0ac312, 0x882e6a7d
public:

    DECLARE_CLASS_ID(IDOMExternalTargetClassID);

    /**
    * @brief    Retrieves the target URI.
    * @return   EDLSysString. The function retrieves the target URI.
    */
    virtual EDLSysString getTargetUri() const = 0;

    /**
    * @brief    Sets the target URI.
    * @param    uri The new target URI.
    */
    virtual void setTargetUri(const EDLSysString &uri) = 0;

    /**
    * @brief    Retrieves the value of the IsMap flag.
    *           This is a flag specifying whether to track the mouse position when the URI is resolved.
    * @return   bool. The function returns the target IsMap flag value.
    */
    virtual bool getIsMap() const = 0;

    /**
    * @brief    Sets the value of the IsMap flag.
    * @param    isMap The new value of IsMap flag.
    */
    virtual void setIsMap(bool isMap) = 0;

    /**
    * @brief    Retrieves the target URI type.
    * @return   eTargetType. The target type specified by the eTargetType enumeration.
    */
    virtual eTargetType getTargetType() const
    {
        return eExternal;
    }

};

DECL_SMART_PTR(IDOMExternalTarget);
#define edlobj2IDOMExternalTarget(src) edl_cast((IDOMExternalTarget *)NULL, src) 


/**
* @class    IDOMInternalTarget
* @ingroup  IDOMTarget
* @brief    The IDOMInternalTarget interface describes the targets of hyperlinks that are in the same document but not on the current page.
*/
class IDOMInternalTarget : public IDOMTarget
{
#define IDOMInternalTargetClassID 0xfcf28aeb, 0xf977476b, 0x884a1ab2, 0xfac79443
public:

    DECLARE_CLASS_ID(IDOMInternalTargetClassID);

    /**
    * @brief    Retrieves the internal target id
    * @return   DOMid. Returns the internal target id
    */
    virtual DOMid getTargetId() const = 0;

    /**
    * @brief    Sets the internal target id.
    * @param    id. The internal target id.
    */
    virtual void setTargetId(DOMid id) = 0;

    /**
    * @brief    Implementation of getTargetType for IDOMInternalTarget
    * @return   eTargetType Returns eInternal;
    */
    virtual eTargetType getTargetType() const
    {
        return eInternal;
    }

};

DECL_SMART_PTR(IDOMInternalTarget);
#define edlobj2IDOMInternalTarget(src) edl_cast((IDOMInternalTarget *)NULL, src) 

template <typename basetype>
class TypeWithNull
{
public:

    TypeWithNull() : m_isNull(true)
    {}

    TypeWithNull(basetype value) : m_value(value), m_isNull(false)
    {}

    TypeWithNull(const TypeWithNull<basetype>& that) : m_isNull(that.m_isNull), m_value(that.m_value)
    {}

    basetype getValue() const
    {
        assert(! m_isNull);
        return m_value;
    }
    
    basetype getValueOrDefault(basetype defaultVal = basetype())
    {
        if (m_isNull)
        {
            return defaultVal;
        }
        else
        {
            return m_value;
        }
    }

    bool isNull() const
    {
        return m_isNull;
    }

    void setValue(basetype value)
    {
        m_value = value;
        m_isNull = false;
    }

    void setNull()
    {
        m_isNull = true;
    }

    const TypeWithNull & operator =(const TypeWithNull &another)
    {
        m_isNull = another.m_isNull;
        if (!m_isNull)
            m_value = another.m_value;
        return *this;
    }
    
    bool equals(const TypeWithNull &another)
    {
        if (m_isNull != another.m_isNull)
        {
            return false;
        }
        if (!m_isNull)
        {
            if (m_value != another.m_value)
            {
                return false;
            }
        }
        return true;
    }

private:
    basetype m_value;
    bool m_isNull;
};

typedef TypeWithNull<double> doubleWithNull;


/**
* @class    IDOMPageTarget
* @ingroup  IDOMTarget
* @brief    IDOMPageTarget nodes are used to describe hyperlinks on a page to targets on the same page.
*/
class IDOMPageTarget : public IDOMTarget
{
#define IDOMPageTargetClassID 0x2cb17446, 0xac5741d3, 0xb9ba405e, 0x616c7cee
public:

    DECLARE_CLASS_ID(IDOMPageTargetClassID);

    /**
    * @brief    Retrieves the target page number for a page target.
    * A page target has the whole page as the target. It refers to a page
    * using the absolute page number within the document sequence, where the very first page in the
    * first document is number 1.
    * @return   uint32 The target page number.
    */
    virtual uint32 getTargetPage() const = 0;

    /**
    * @brief    Sets the target page number for a page target.
    * A page target has the whole page as the target. It refers to a page
    * using the absolute page number within the document sequence, where the very first page in the
    * first document is number 1.
    * @param    page The new target page number.
    */
    virtual void setTargetPage(uint32 page) = 0;

    /**
    * @brief    Implementation of getTargetType for IDOMPageTarget.
    * @return   eTargetType. Returns "ePage".
    */
    virtual eTargetType getTargetType() const
    {
        return ePage;
    }

};

DECL_SMART_PTR(IDOMPageTarget);
#define edlobj2IDOMPageTarget(src) edl_cast((IDOMPageTarget *)NULL, src) 

class IDOMPageRectTarget;
DECL_SMART_PTR(IDOMPageRectTarget);

/**
* @class    IDOMPageRectTarget
* @ingroup  IDOMTarget
* @brief    IDOMPageRectTarget nodes are used to describe hyperlinks on a page rectangle to targets on the same page.
*/
class IDOMPageRectTarget : public IDOMTarget
{
#define IDOMPageRectTargetClassID 0xd21f80cf, 0x124c495b, 0x9eb05a87, 0x2fa73ef6
public:

    DECLARE_CLASS_ID(IDOMPageRectTargetClassID);

    /**
    * @enum     eFitType
    * @ingroup  IDOMTarget
    * @brief    Destination fit types enumeration
    */
    typedef enum 
    {
        eFTInvalid,
        eFTXyz,
        eFTFit,
        eFTFitH,
        eFTFitV,
        eFTFitR,
        eFTFitB,
        eFTFitBH,
        eFTFitBV
    } eFitType;
    
    /**
    * @brief    Simplified creator for a page rect target.
    *           Throws an exception of type IEDLError on failure.
    * @param    pFactory The EDL factory to use
    * @param    pageId The DOM id of the target page
    * @param    fitType The fit type
    * @param    zoom The desired zoom level. Zero indicates that the zoom should be unchanged
    *           when navigating to this target.
    * @param    left The left of the target rect. For certain fit types, this will be ignored.
    * @param    top The top of the target rect. For certain fit types, this will be ignored.
    * @param    right The right of the target rect. For certain fit types, this will be ignored.
    * @param    bottom The left of the target rect. For certain fit types, this will be ignored.
    * @return   IDOMPageRectTarget The new target
    */
    static EDL_API IDOMPageRectTargetPtr create(IEDLClassFactory *pFactory,
                                                DOMid pageId,
                                                eFitType fitType = eFTFit,
                                                double zoom = 0.0,
                                                const doubleWithNull &left = doubleWithNull(),
                                                const doubleWithNull &top = doubleWithNull(),
                                                const doubleWithNull &right = doubleWithNull(),
                                                const doubleWithNull &bottom = doubleWithNull());

    /**
    * @brief    Retrieves the target page DOM id
    * @return   DOMid. The target page id
    */
    virtual DOMid getPageId() const = 0;

    /**
    * @brief    Sets the target page DOM id.
    * @param    id. The new target page id
    */
    virtual void setPageId(DOMid id) = 0;

    /**
    * @brief    Gets the fit type.
    * @see      eFitType
    * @return   eFitType. Returns the fit type.
    */
    virtual eFitType getFitType() const = 0;

    /**
    * @brief    Sets the fit type.
    * @see      eFitType
    * @return   eFitType. The new fit type.
    */
    virtual void setFitType(eFitType fitType) = 0;

    /**
    * @brief    Gets the zoom value.
    * @return   double. Returns the zoom value.
    * Zero zoom value specifies that the current value of that parameter is to be retained unchanged
    */
    virtual double getZoom() const = 0;

    /**
    * @brief    Sets for the zoom value.
    * @param    zoom The new zoom value.  Zero zoom value specifies that the current value of that 
    * parameter is to be retained unchanged.
    */
    virtual void setZoom(double zoom) = 0;

    /**
    * @brief    Gets the left coordinate value of the target.
    * @return   doubleWithNull Returns the left coordinate value of the target.
    */
    virtual doubleWithNull getLeft() const = 0;

    /**
    * @brief    Sets the left coordinate value of the target.
    * @param    left The new left coordinate value.
    */
    virtual void setLeft(const doubleWithNull &left) = 0;

    /**
    * @brief    Gets the top coordinate value of the target.
    * @return   doubleWithNull Returns the top coordinate value of the target.
    */
    virtual doubleWithNull getTop() const = 0;

    /**
    * @brief    Sets the top coordinate value of the target.
    * @param    left The new top coordinate value.
    */
    virtual void setTop(const doubleWithNull &top) = 0;

    /**
    * @brief    Gets the right coordinate value of the target.
    * @return   doubleWithNull Returns the right coordinate value of the target.
    */
    virtual doubleWithNull getRight() const = 0;

    /**
    * @brief    Sets the right coordinate value of the target.
    * @param    left The new right coordinate value.
    */
    virtual void setRight(const doubleWithNull &right) = 0;

    /**
    * @brief    Gets the bottom coordinate value of the target.
    * @return   doubleWithNull Returns the bottom coordinate value of the target.
    */
    virtual doubleWithNull getBottom() const = 0;

    /**
    * @brief    Sets the bottom coordinate value of the target.
    * @param    left The new bottom coordinate value.
    */
    virtual void setBottom(const doubleWithNull &bottom) = 0;

    /**
    * @brief    Implementation of getTargetType for IDOMPageTarget.
    * @return   eTargetType. Returns "ePage".
    */
    virtual eTargetType getTargetType() const
    {
        return ePageRect;
    }

};

#define edlobj2IDOMPageRectTarget(src) edl_cast((IDOMPageRectTarget *)NULL, src)


/**
* @class    IDOMActionLaunch
* @ingroup  IDOMTarget
* @brief    IDOMActionLaunch interface
*/
class IDOMActionLaunch : public IDOMTarget
{
#define IDOMActionLaunchClassID 0x877b883f, 0x5dcf4c04, 0xa469d7f5, 0xf637b394
public:

    DECLARE_CLASS_ID(IDOMActionLaunchClassID);

    /**
    * @brief    Retrieves the file specification
    * @return   EDLSysString The file specification
    */
    virtual EDLString getFile() const = 0;

    /**
    * @brief    Sets the file specification
    * @param    file File specification
    */
    virtual void setFile(const EDLString &file) = 0;

    /**
    * @brief    Retrieves the Windows-specific launch parameters
    * @return   EDLSysString The launch parameters
    */
    virtual EDLString getWinParameters() const = 0;

    /**
    * @brief    Sets the Windows-specific launch parameters
    * @param    params The launch parameters
    */
    virtual void setWinParameters(const EDLString &params) = 0;

    /**
    * @brief    Retrieves the Mac OS-specific launch parameters
    * @return   EDLSysString The launch parameters
    */
    virtual EDLString getMacParameters() const = 0;

    /**
    * @brief    Sets the Mac OS-specific launch parameters
    * @param    params The launch parameters
    */
    virtual void setMacParameters(const EDLString &params) = 0;

    /**
    * @brief    Retrieves the UNIX-specific launch parameters
    * @return   EDLSysString The launch parameters
    */
    virtual EDLString getUnixParameters() const = 0;

    /**
    * @brief    Sets the UNIX-specific launch parameters
    * @param    params The launch parameters
    */
    virtual void setUnixParameters(const EDLString &params) = 0;

    /**
    * @brief    Getter for NewWindow flag
    * @return   bool Returns the value of the NewWindow flag
    */
    virtual bool getNewWindow() const = 0;

    /**
    * @brief    Setter for NewWindow flag
    * @param    bNewWindow New value of the NewWindow flag
    */
    virtual void setNewWindow(bool bNewWindow) = 0;

    /**
    * @brief    Implementation of geTargetType for IDOMActionLaunch
    * @return   eTargetType Returns eActionLaunch;
    */
    virtual eTargetType getTargetType() const
    {
        return eActionLaunch;
    }

};

DECL_SMART_PTR(IDOMActionLaunch);
#define edlobj2IDOMActionLaunch(src) edl_cast((IDOMActionLaunch *)NULL, src) 

typedef IInputEnumRC<IDOMTargetPtr> IDOMTargetCollectionEnum;
DECL_SMART_PTR(IDOMTargetCollectionEnum);

/**
* @class    IDOMActionArray
* @ingroup  IDom
* @brief    IDOMActionArray interface
*/
class IDOMActionArray : public IDOMTarget
{
#define IDOMActionArrayClassID 0x52205412, 0xfaf74a65, 0x9d1d5e19, 0x9d15ece6
public:

    DECLARE_CLASS_ID(IDOMActionArrayClassID);

    /**
    * @brief    Retrieves enumerator of the action collection
    *           Action collection is analog of entry "Next" of PDF action dictionary
    * @return   IDOMActionCollectionEnumPtr Actions enumerator
    */
    virtual IDOMTargetCollectionEnumPtr getActionsEnum() = 0;

    /**
    * @brief    Retrieves count of the actions
    * @return   uint32 Size of action collection
    */
    virtual uint32 getActionsCount() = 0;

    /**
    * @brief    Clears collection of actions
    */
    virtual void clearActions() = 0;

    /**
    * @brief    Append a action to the action collection
    * @param    ptrAction The smart pointer to the target interface
    * @return   bool Returns true on success
    */
    virtual bool addAction(const IDOMTargetPtr &ptrAction) = 0;

    /**
    * @brief    Implementation of geTargetType for IDOMActionArray
    * @return   eTargetType Returns eActionArray;
    */
    virtual eTargetType getTargetType() const
    {
        return eActionArray;
    }

};

DECL_SMART_PTR(IDOMActionArray);
#define edlobj2IDOMActionArray(src) edl_cast((IDOMActionArray *)NULL, src) 

//TODO: implement classes:
//    IDOMActionGoToR
//    IDOMActionGoToE
//    IDOMActionThread
//    IDOMActionSound
//    IDOMActionMovie
//    IDOMActionHide
//    IDOMActionNamed
//    IDOMActionSubmitForm
//    IDOMActionResetForm
//    IDOMActionImportData
//    IDOMActionJavaScript
//    IDOMActionSetOCGState
//    IDOMActionRendition
//    IDOMActionTrans
//    IDOMActionGoTo3DView

_END_EDL_DOM_NAMESPACE

#endif // __EDLIDOMTARGET_H__
