/*
 * Copyright (C) 2010-2018 Global Graphics Software Ltd. All rights reserved.
 */

/**
 * @file iedltempstore.h
 * @ingroup IController
 * @brief A mechanism for storing and accessing temporary data for use with EDL.
 */

#ifndef __IEDLTEMPSTORE_H__
#define __IEDLTEMPSTORE_H__

#include <edl/edlstream.h>

_BEGIN_EDL_NAMESPACE

/**
* @class    IEDLTempStoreObject
* @ingroup  IController
* @brief    A temporary, file-like object, stored with an IEDLTempStore.
*/
class IEDLTempStoreObject : public IRCObject
{
  public:
  
    virtual ~IEDLTempStoreObject() {};
    
    /**
    * @brief    Create a writer that may be used to write to this temporary object.
    *           Multiple writers writing to a single temporary store object is not
    *           supported, and the writer may not be cloned.
    * @return   IRAOutputStreamPtr The writer. 
    */
    virtual IRAOutputStreamPtr createWriter() = 0;
    
    /**
    * @brief    Create a reader that may be used to read from this temporary object
    *           Many readers can simultaneously perform reads.
    * @param    bool buffered - If set to false, the resulting reader will not buffer
    *           read data. Useful if a reader and writer are being used on the same
    *           IEDLTempStoreObject, as it allows changes made by a writer to be
    *           immediately available to users of the reader.
    * @return   IRAInputStreamPtr The reader. 
    */
    virtual IRAInputStreamPtr createReader(bool buffered = true) = 0;
    
    /**
    * @brief    Create a stream that may be used to simulatneously read and write
    *           from this temporary object.
    *           Multiple writers writing to a single temporary object store is not
    *           supported. Clones of this object type will be read only.
    *           This is convenient for some cases that require simultaneous reading
    *           and writing. Reads from this stream will be unbuffered, so for best
    *           performance, please consider creating a reader writer pair instead.
    */
    virtual IRAInputOutputStreamPtr createReaderWriter() = 0;
};
DECL_SMART_PTR(IEDLTempStoreObject);

/**
* @class    IEDLTempStore
* @ingroup  IController
* @brief    A self-cleaning area for storage of temporary data in the
*           form of streams. One per session, obtainable from an ISession.
*/
class IEDLTempStore : public IEDLObject
{
#define IEDLTempStoreClassID 0x3BABAADA, 0x3838435F, 0x8E576C41, 0xB8F94150
  public:
    
    virtual ~IEDLTempStore () {};
    
    /**
    * @brief    Retrieves class id of IEDLTempStore
    * @return   CClassID class id of the object
    */
    static const CClassID &classID()
    {
        static CClassID id(IEDLTempStoreClassID);
        return id;
    };
    
    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
      public:
        EDLSysString      pathToTempDir; // Must exist.
        IEDLClassFactory *pFactory;
        uint64            maxMemorySize;
        uint32            blockSize;
        uint64            maxDiskSize;
    };
    
    /**
    * @brief    Create a temporary object, which is similar in concept
    *           to a file.
    * @return   IEDLTempStoreObjectPtr The new temporary object. 
    */
    virtual IEDLTempStoreObjectPtr createTemporaryObject() = 0;
    
    /**
    * @brief    Create a temporary object with the contents of an existing
    *           input stream.
    * @param    stream Reference to the source stream.
    * @return   IEDLTempStoreObjectPtr The new temporary object, or null on failure.
    */
    virtual IEDLTempStoreObjectPtr createTemporaryObjectWithContents(const IInputStreamPtr &stream) = 0;
    
    /**
    * @brief    Copy the given stream into a temporary store object and return a
    *           buffered reader. Convenience. Throws an IEDLError on failure.
    * @param    stream Reference to the source stream.
    * @return   IRAInputStreamPtr The result stream.
    */
    virtual IRAInputStreamPtr createTemporaryStreamWithContents(const IInputStreamPtr &stream) = 0;
    
    /**
    * @brief    Create a temporary object and provide a reader/writer pair.
    *           Convenience method. Throws an IEDLError on failure.
    * @param    IRAInputStreamPtr reference to receive a reader for the temporary object.
    * @param    IRAOutputStreamPtr reference to receive a writer for the temporary object.
    */
    virtual void createTemporaryReaderWriterPair(IRAInputStreamPtr &reader, IRAOutputStreamPtr &writer) = 0;

    /**
    * @brief    Create a temporary object and provide a consolidated reader/writer.
    *           Convenience method. Throws an IEDLError on failure.
    * @return   IRAInputOutputStreamPtr The consolidated reader/writer for the temporary object.
    */
    virtual IRAInputOutputStreamPtr createTemporaryReaderWriter() = 0;
    
    /**
    * @brief    Clone - not available for objects of this type.
    */
    virtual bool clone(IEDLObjectPtr &ptrObject, IEDLClassFactory *pFactory)
    {
        UNREFERENCED_PARAMETER(ptrObject);
        UNREFERENCED_PARAMETER(pFactory);
        return false;
    }
};
DECL_SMART_PTR(IEDLTempStore);
#define edlobj2IEDLTempStore(src) edl_cast((IEDLTempStore *)NULL, src) 

_END_EDL_NAMESPACE

#endif
