/*
* Copyright (C) 2008-2017 Global Graphics Software Ltd. All rights reserved.
*/

/**
* @file    ifilespec.h
* @ingroup IController
* @brief   Interface to File specification
*/

#ifndef IFILESPEC_H
#define IFILESPEC_H

#include <edl/edltypes.h>
#include <edl/edlstring.h>
#include <edl/iedlobject.h>
#include <edl/edlstream.h>
#include <edl/edlvector.h>

_BEGIN_EDL_NAMESPACE

class IFileSpec : public IEDLObject
{
public:
    
    typedef enum
    {
        eFileSpecExternal,
        eFileSpecEmbedded,
        eFileSpecUrl
    } eFileSpecType;
    
    /**
     * @brief    Retrieves the file spec type.
     * @return   eFileSpecType Returns the file spec type.
     */
    virtual eFileSpecType getFileSpecType() const = 0;
};


DECL_SMART_PTR(IFileSpec);
#define edlobj2IFileSpec(src) edl_cast((IFileSpec *)NULL, src)
typedef CEDLVector<IFileSpecPtr> CFileSpecVect;


class IFileSpecAsEmbeddedData : public IFileSpec
{
#define IFileSpecAsEmbeddedDataClassID 0x363ca3a4, 0xa2b548e1, 0xb4246b67, 0xf7a5b7dc
public:
    
    class Data : public CClassParams
    {
    public:
        Data() : fileSize(0) {}
        EDLSysString      fileName;
        EDLSysString      description;
        EDLSysString      mimeType;
        EDLSysString      checksum;
        EDLSysString      creationDate;
        EDLSysString      modificationDate;
        uint64            fileSize;
        IRAInputStreamPtr fileDataStream;
    };
    
    /**
     * @brief    Retrieves class id of IFileSpecAsEmbeddedData
     * @return   CClassID Class id of the element
     */
    static const CClassID &classID()
    {
        static CClassID id(IFileSpecAsEmbeddedDataClassID);
        return id;
    };

    /**
     * @brief    Retrieves the file name.
     * @param    fileName The file name.
     * @return   bool Returns true on success
     */
    virtual bool getFileName(EDLSysString &fileName) const = 0;
    
    /**
     * @brief    Sets the file name.
     * @param    fileName The file name.
     * @return   bool Returns true on success
     */
    virtual bool setFileName(const EDLSysString &fileName) = 0;

    /**
     * @brief    Retrieves the description.
     * @param    description The description.
     * @return   bool Returns true on success
     */
    virtual bool getDescription(EDLSysString &description) const = 0;
    
    /**
     * @brief    Sets the description.
     * @param    description The description.
     * @return   bool Returns true on success
     */
    virtual bool setDescription(const EDLSysString &description) = 0;

    /**
     * @brief    Retrieves the file mime type.
     * @param    mimeType The file mime type.
     * @return   bool Returns true on success
     */
    virtual bool getMimeType(EDLSysString &mimeType) const = 0;

    /**
     * @brief    Sets the file mime type.
     * @param    mimeType The file mime type.
     * @return   bool Returns true on success
     */
    virtual bool setMimeType(const EDLSysString &mimeType) = 0;
    
    /**
     * @brief    Retrieves the date and time when the annotation was created.
     * @param    date The data and time
     * @return   bool Returns true on success
     */
    virtual bool getCreationDate(EDLSysString &date) const = 0;
    
    /**
     * @brief    Sets the annotation creation date
     * @param    date the data and time
     * @return   bool Returns true on success
     */
    virtual bool setCreationDate(const EDLSysString &date) = 0;
    
    /**
     * @brief    Retrieves the date and time when the annotation was most recently modified.
     * @param    date The date and time
     * @return   bool Returns true on success
     */
    virtual bool getModificationDate(EDLSysString &date) const = 0;
    
    /**
     * @brief    Sets the annotation modification date
     * @param    date the date and time
     * @return   bool Returns true on success
     */
    virtual bool setModificationDate(const EDLSysString &date) = 0;
    
    /**
     * @brief    Retrieves the file size
     * @return   uint64 The size of the file.
     */
    virtual uint64 getFileSize() const = 0;
    
    /**
     * @brief    Sets the file size
     * @param    fileSize the file size.
     * @return   bool Returns true on success
     */
    virtual bool setFileSize(uint64 fileSize) = 0;
    
    /**
     * @brief    Retrieves the checksum for the file data.
     * @param    checksum The data checksum
     * @return   bool Returns true on success
     */
    virtual bool getChecksum(EDLSysString &checksum) const = 0;
    
    /**
     * @brief    Sets the checksum for the file data.
     * @param    checksum The data checksum
     * @return   bool Returns true on success
     */
    virtual bool setChecksum(const EDLSysString &date) = 0;

    /**
     * @brief    Retrieves the stream associated with the file data.
     * @param    stream The file data stream
     * @return   bool Returns true on success
     */
    virtual bool getFileDataStream(IRAInputStreamPtr &stream) const = 0;
    
    /**
     * @brief    Sets the stream associated with the file data.
     * @param    stream The file data stream
     * @return   bool Returns true on success
     */
    virtual bool setFileDataStream(const IRAInputStreamPtr &stream) = 0;
};

DECL_SMART_PTR(IFileSpecAsEmbeddedData);
#define edlobj2IFileSpecAsEmbeddedData(src) edl_cast((IFileSpecAsEmbeddedData *)NULL, src)

typedef CEDLVector<IFileSpecAsEmbeddedDataPtr> CFileSpecAsEmbeddedDataVect;

class IFileSpecAsFileReference : public IFileSpec
{
#define IFileSpecAsFileReferenceClassID 0xa5f832aa, 0x385544ad, 0xae875b19, 0x23204e25
public:
    
    class Data : public CClassParams
    {
      public:
        EDLSysString strFileSpec;
    };
    /**
    * @brief    Retrieves class id of IFileSpecAsFileReference
    * @return   CClassID Class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IFileSpecAsFileReferenceClassID);
        return id;
    };
    
    /**
    * @brief    Sets file specification string
    * @param    file specification string
    * @return   bool Returns true on success
    */
    virtual bool getFileSpec (EDLSysString &strFileSpec) const = 0;
    
    /**
    * @brief    Gets file specification string
    * @param    file specification string
    * @return   bool Returns true on success
    */
    virtual bool setFileSpec (const EDLSysString &strFileSpec) = 0;
    
    
    
    /**
    * @brief    Static utility function that attempts to resolve a file specification
    *           string to a URI. 
    *
    *           This may involve searching local and potentially remote filesystems
    *           for the existance of the file. Should the file not be unambiguously
    *           or directly found, resolved will be set to false and a best-effort
    *           result URI will be returned.
    *
    *           Note: Some filespecs may be already present in URI form (such
    *           file://path/to/file). For such cases, the URI will be returned
    *           unmodified (albeit escaped with percent encoding if required),
    *           and it will be assumed that the URI can be resolved (resolved
    *           will be set to true).
    *
    * @param    strUri Reference to receive the resolved URI
    * @param    strFileSpec The file spec to be resolved
    * @param    resolved Reference to receive the status of the resolution attempt.
    * @param    strDocumentDir The full path to the directory containing the current
    *           file, for resolving relative paths. Pass an empty string to use the
    *           process' current working directory. This string must be in the
    *           standard form used by the host operating system. 
    * @param    allowNetworkLookup Allow the resolver to check for the existance of
    *           a shared volume that matches the filespec.
    * @return   bool returns false on failure.
    */
    static EDL_API bool resolveToURI(EDLSysString &strUri, EDLSysString &strFileSpec, bool &resolved, EDLString &strDocumentDir, bool allowNetworkLookup = false);
    
    /**
    * @brief    Static utility function to convert a file URI to a filespec string.
    * @param    strFileSpec A reference to receive the file specification
    * @param    uriStr the URI to convert.
    * @return   bool returns true on success
    */
    static EDL_API bool convertURIToFileSpecString(EDLSysString &strFileSpec, const EDLSysString &strUri);
    
};
DECL_SMART_PTR(IFileSpecAsFileReference);
#define edlobj2IFileSpecAsFileReference(src) edl_cast((IFileSpecAsFileReference *)NULL, src)


class IFileSpecAsUrl : public IFileSpec
{
#define IFileSpecAsUrlClassID 0x00f5e473, 0x571c4A5B, 0xae78305a, 0xdB714e41
public:
    
    class Data : public CClassParams
    {
      public:
        EDLSysString url;
    };
    
    /**
     * @brief    Retrieves class id of IFileSpecAsUrlClassID
     * @return   CClassID Class id of the element
     */
    static const CClassID &classID()
    {
        static CClassID id(IFileSpecAsUrlClassID);
        return id;
    };
    
    /**
     * @brief    Gets the URI
     */
    virtual EDLSysString getUrl() const = 0;
    
    /**
     * @brief    Sets the URI
     */
    virtual void setUrl(const EDLSysString &url) = 0;
};
DECL_SMART_PTR(IFileSpecAsUrl);
#define edlobj2IFileSpecAsUrl(src) edl_cast((IFileSpecAsUrl *)NULL, src)

_END_EDL_NAMESPACE

#endif // IFILESPEC_H
