/*
* Copyright (C) 2007-2019 Global Graphics Software Ltd. All rights reserved.
*/

/**
 * @file iimagecodec.h
 * @ingroup IController
 * @brief 
 */

#ifndef __IIMAGECODEC_H
#define __IIMAGECODEC_H

#include <edl/isession.h>
#include <edl/edlstream.h>
#include <edl/edlgeom.h>
#include <edl/iedlobject.h>
#include <edl/idomresources.h>
#include <edl/idomcolorspace.h>

_BEGIN_EDL_NAMESPACE


   /**
    * @enum     ImageExtraChannel_t
    * @ingroup  IDom
    * @brief    type used to uniquely identify the type of extra image channel.
    */
    
typedef enum 
{
    ImageExtraChannel_None,               // No extra channel present.
    ImageExtraChannel_Alpha,              // Extra channel is an alpha mask.
    ImageExtraChannel_PremultipliedAlpha, // Extra channel is an alpha, and the color samples have been premultiplied
    ImageExtraChannel_Mask,               // Extra channel is an binary mask.
    ImageExtraChannel_NoisyMask,          // Extra channel is an alpha mask with binary values.
    ImageExtraChannel_ImageMask,          // Extra channel is an image mask.
    ImageExtraChannel_PatternImageMask    // Extra channel is a pattern image mask.
    
} ImageExtraChannel_t;

/**
 * @class IImageFrame
 * @brief IImageFrame encapsulates an EDL image with its details.
 */
class IImageFrame : public IEDLObject 
{
protected:    
        double                  m_xResolution;
        double                  m_yResolution;
        ImageExtraChannel_t     m_extraChannelType;

public:        
        /// Constructor
        IImageFrame ()
        {
            m_xResolution       = 96.0;
            m_yResolution       = 96.0;
            m_extraChannelType  = ImageExtraChannel_None;
        }

        /// Destructor
        virtual ~IImageFrame ()
        {}

public:    
    
        /**
         * @brief    Gets the width of the image.
         * @return   int32. Returns the image width.
        */
        
        virtual int32 getWidth () = 0;

        /**
         * @brief    Gets the height of the image.
         * @return   int32. Returns the image height.
        */
        
        virtual int32 getHeight () = 0;
        
        
        /**
         * @brief    Gets the bits per sample of the image.
         * @return   int32. Returns the image bits from sample.
        */
        
        virtual int32 getBPS () = 0;
        
        
        /**
         * @brief    Gets the color space.
         * @param    colorSpace The colorspace to be passed back by reference.
         * @return   bool. Returns an indicator of success.
        */
        
        virtual bool getColorSpace (IDOMColorSpacePtr &colorSpace) = 0;
       
       
        /**
          * @brief    Gets the type of information contained in the extra channel.
          * @return   ImageExtraChannel_t. Returns the type of information in the extra channel.
         */ 
         
        virtual ImageExtraChannel_t getExtraChannelType ()
        {
            return m_extraChannelType;
        }

        
        /**
          * @brief    Gets the number of image extra channels (for example an alpha or mask)
          * @return   int32. Returns the number of image extra channels.
          */
          
        virtual int32 getNumExtraChannels ()
        {
            ImageExtraChannel_t     channelType = getExtraChannelType ();

            return (channelType == ImageExtraChannel_None) ? 0 : 1;
        }

        
        /**
          * @brief    Gets the total number of image channels (including extra channels)
          * @return   int32. Returns the total number of image channels.
          */
          
        virtual int32 getNumChannels ()
        {
            IDOMColorSpacePtr   colorSpace;
            int32               numChannels;

            numChannels = getNumExtraChannels ();

            if (getColorSpace (colorSpace))
            {
                numChannels += colorSpace->getNumComponents ();
            }
            return numChannels;
        }


        /**
          * @brief    Indicates if the image has an alpha mask
          * @return   bool. Returns an indication of alpha mask present in the image.
          */
                
        virtual bool  getHasAlphaChannel ()
        {
            return (getExtraChannelType () == ImageExtraChannel_Alpha ||
                    getExtraChannelType () == ImageExtraChannel_PremultipliedAlpha);
        }


        /**
          * @brief    Indicates if the image has a mask (i.e. a binary alpha channel)
          * @param    isNoisy. An indicator that the alpha channel has binary values and so is acting as a mask.
          * @return   bool. Returns an indication of mask present in the image
          */
          
        virtual bool getHasMask (bool &isNoisy)
        {
            ImageExtraChannel_t     channelType = getExtraChannelType ();

            isNoisy = false;

            if (channelType == ImageExtraChannel_NoisyMask)
            {
                isNoisy = true;
                return true;
            }
            else if (channelType == ImageExtraChannel_Mask)
                return true;
            else if (channelType == ImageExtraChannel_ImageMask)
                return true;
            else if (channelType == ImageExtraChannel_PatternImageMask)
                return true;
            else
                return false;
        }


        /**
         * @brief    Indicates if the image has an ICC profile.
         * @return   bool. Returns an indication of ICC profile present in the image
         */
         
        virtual bool getHasICCProfile() = 0;
    
    
       /**
         * @brief    Gets the ICC profile for the image.
         * @param    profile The image ICC profile to be passed back.
         * @return   bool. Returns an indication success.
         */
  
        virtual bool getICCProfile (IDOMICCProfilePtr &profile) = 0;
        
        
        /// Please note:  getXResolution() and getYResolution() are temporary APIs.
        ///     Because it is not certain what the interprtation of resolution means for all image types.
        ///     At the moment both APIs return values that are to be interpreted as pixels per inch
        ///     and the default value of 96.0 (XPS resolution) is returned for any image (type)
        ///     for which there is currently no other meaningful resolution value.
        
        virtual double getXResolution ()            { return m_xResolution;}

        virtual double getYResolution ()            { return m_yResolution;}


      /**
        * @brief    Gets the transform for the image.
        * @return   FMatrix*. Returns the image transform.
        */
        
        virtual FMatrix *getMatrix() const = 0;
        
        
       /**
         * @brief    Gets the image row size in bytes
         * @return   int32. The image row size in bytes
         */

        virtual int32 getRawBytesPerRow () = 0;
        
        
        /**
         * @brief    Gets an image row scanline (of size given by getRawBytesPerRow())
         * @param    pRow The buffer in which the row scanline will be copied into.
         * @return   bool. An indicator of success.
         */
         
        virtual bool readScanLine (void *pRow) = 0;

        /**
         * @brief    Skips the given number of scanline rows, which may be
         *           fast for some image types.
         * @param    skipCount The number of scanlines to skip..
         * @return   bool. An indicator of success.
         */
         
        virtual bool skipScanLines (uint32 skipCount)
        {
            uint8 *scanlineBuffer = new uint8[getRawBytesPerRow()];
            for (uint32 r = 0; r < skipCount; r++)
            {
                if (!readScanLine(scanlineBuffer))
                {
                    delete [] scanlineBuffer;
                    return false;
                }
            }
            delete [] scanlineBuffer;
            return true;
        }
};
DECL_SMART_PTR(IImageFrame);
#define edlobj2IImageFrame(src) edl_cast((IDOM *)NULL, src) 


/**
 * @class IImageFrameReader
 * @brief IImageFrameReader reads an image into an imageframe. 
 */
 
class IImageFrameReader : public IImageFrame
{
public:        
        /// Constructor
        IImageFrameReader ()
        {
            m_xResolution = 96.0;
            m_yResolution = 96.0;
        }

        /// Destructor
        virtual ~IImageFrameReader ()
        {}

public:    

       /**
         * @brief    Writes an image row ino the image frame.
         * @param    pScanLine A buffer that contains the scanline.
         * @return   bool. An indicator of success.
         */
         
        virtual bool writeScanLine (void *pScanLine) { UNREFERENCED_PARAMETER(pScanLine); return false; }
        
        /**
          * @brief    Gets the transform for the image.
          * @return   FMatrix*. Returns the image transform.
          */
           
        virtual FMatrix *getMatrix() const { return (FMatrix *) NULL; }

};
DECL_SMART_PTR(IImageFrameReader);


/**
 * @class IImageFrameWriter
 * @brief IImageFrameWriter writes an image from an imageframe. 
 */

class IImageFrameWriter : public IImageFrame
{
public:        
        /// Constructor
        IImageFrameWriter ()
        {
        }

        /// Destructor
        virtual ~IImageFrameWriter ()
        {}
public:    
        virtual bool writeScanLine (void *pScanLine) = 0;
        virtual bool setWidth (int32 width) = 0;
        virtual bool setHeight (int32 height) = 0;
        virtual bool setBPS (int32 bps) = 0;
        virtual bool setSamples (int32 samples) = 0;

        virtual bool setImageExtraChannelType (ImageExtraChannel_t extraChannelType)
        {
            m_extraChannelType = extraChannelType;
            return true;
        }

        virtual bool setXResolution (double xres)       { m_xResolution = xres; return true;}
        virtual bool setYResolution (double yres)       { m_yResolution = yres; return true;}
        virtual bool setColorSpace (IDOMColorSpacePtr &colorSpace) = 0;

        virtual int32 getRawBytesPerRow ()              { return 0; }

        /// reads a scanline
        virtual bool readScanLine (void *pScanLine) { UNREFERENCED_PARAMETER(pScanLine); return false; }

        /* this prepares the imageframe for receiving scanline data, all the other parameters
         * must have been set already before calling this
         */
        virtual bool prepareForData () = 0;

        /* this flushes the imageframe to the image data */
        virtual bool flushData () = 0;
};
DECL_SMART_PTR(IImageFrameWriter);


/**
 * @class IImageDecoder
 * @brief IImageDecoder returns IImageFrame objects as requested by the client.
 * This object knows about the imageformat internals and knows how to unpack the
 * image.
 */
 
class IImageDecoder : public IEDLObject 
{
public:    
        /// Constructor
        IImageDecoder()
        {
        }


        /// Destructor
        virtual ~IImageDecoder()
        {}

public:

        /// Returns the number of frames for this image file
    virtual int32 getNumFrames () { return 0; }

    virtual bool getFrame (int32 frameNo, IImageFramePtr &imageFrame) = 0;
};
DECL_SMART_PTR(IImageDecoder);

/**
 * @class IImageEncoder
 * @brief IImageEncoder accepts IImageFrame objects and streams it out to an image format.
 */
class IImageEncoder : public IEDLObject 
{
public:    
        /// Constructor
        IImageEncoder()
        {
        }

        /// Destructor
        virtual ~IImageEncoder()
        {}

public:
        virtual bool createFrame (IImageFrameWriterPtr &imageFrame) = 0;
};
DECL_SMART_PTR(IImageEncoder);

_END_EDL_NAMESPACE


#endif //__IIMAGECODEC_H

