/*
* Copyright (C) 2007 Global Graphics Software Ltd. All rights reserved.
*/

/**
* @file     ircobject.h
* @ingroup  IController
* @brief    Interface for Reference Counted Object
*
* In order to allow applications and filters to not have to worry about
* remembering to release/free/delete EDL allocated objects when they are no longer needed
* EDL implements so-called "smart pointers" around the EDL objects.
* As the smart pointers are created/destroyed the underlying object's
* reference count is automatically incremented/decremented
* Only when the underlying object's reference count returns to zero
* that it is actually released/deleted.
*/

#ifndef __IRCOBJECT_H__
#define __IRCOBJECT_H__

#include "smartptr.h"
#include "edltypes.h"

_BEGIN_EDL_NAMESPACE


/**
* @class    IRCObject
* @ingroup  IRCObject
* @brief    Base class Interface for all Reference Counted objects
*/
class IRCObject 
{
public:
    /**
    * @brief    Increases the reference count of the actual object pointed to. This would take place during 
    * an assignment or copying.
    */
    virtual void addRef() = 0; 

    /**
    * @brief    Decreases the reference count of the actual object pointed to. When the reference count
    * falls to Zero, it deletes the actual object pointed to.
    * @return   bool Returns true on success
    */
    virtual bool decRef() = 0;

protected:
    /**
    * @brief    Virtual destructor
    */
    virtual ~IRCObject()
    {
    }
};

DECL_SMART_PTR(IRCObject);

_END_EDL_NAMESPACE

#endif /* __IRCOBJECT_H__ */
