/*
 * Copyright (C) 2007-2019 Global Graphics Software Ltd. All rights reserved.
 */

/**
 * @file objclassid.h
 * @ingroup IFactory
 * @brief An object class ID is a 128-bit globally unique ID
 * (i.e. a GUID). In EDL it is abstracted within a CClassID class
 * that provides methods for constructing GUIDs from 4 unsigned 32-bit integers
 * or a string or even another CClassID and a method for comparing GUIDs
 */

#ifndef __EDLOBJECTCLASSID_H__
#define __EDLOBJECTCLASSID_H__

#include "edltypes.h"
#include "edlstring.h"
#include "memutils.h"

_BEGIN_EDL_NAMESPACE

/**
 * @class CClassID
 * @brief An object to represent a 128-bit globally unique ID
 */

class CClassID
{
public:
    union Data
    {
        /// bytes presentation - for converting
        uint8 b[16];
        /// hashvalue
        uint32 hv;
        /// double word presentation - for copying, comparing
        uint32 dw[4];
    } m_ID;
    
	/**
	* @brief Constructor
	*/
    CClassID()
    {
        m_ID.dw[0]=0;
        m_ID.dw[1]=0;
        m_ID.dw[2]=0;
        m_ID.dw[3]=0;
    }
    
	/**
	* @brief	Converts hexadecimal representation of a CClassID to a CClassID
	* @param	str String containing the hexadecimal value
	* @return   CClassID The CClassID
	*/
    CClassID(const EDLSysString &str)
    {
        for( int i = 0; i < 16; i++ )
        {
            m_ID.b[i] = sxtob( str[i*2] ) * 16;
            m_ID.b[i] += sxtob( str[i*2+1] );
        }
    }
    
	/**
	* @brief	Copy to another CClassID
	* @param	other CClassID to copy to
	*/
    CClassID(const CClassID & other)
    {
        EDLmemcpy(&m_ID, &other.m_ID, sizeof(m_ID));
    }

	/**
	* @brief	Construct CClassID from 4 x uint32
	* @param	dw0 First unsigned, 32-bit integer
	* @param	dw1 Second unsigned, 32-bit integer
	* @param	dw2 Third unsigned, 32-bit integer
	* @param	dw3 Fourth unsigned, 32-bit integer
	* @return   CClassID The new CClassID
	*/
    CClassID( uint32 dw0, uint32 dw1, uint32 dw2, uint32 dw3)
    {
        m_ID.dw[0]=dw0;
        m_ID.dw[1]=dw1;
        m_ID.dw[2]=dw2;
        m_ID.dw[3]=dw3;
    }
	
	/**
	* @brief	Compare to another CClassID
	* @param	id CClassID with which to compare
	* @return   <b>bool</b> True if equal, false if not
	*/
    bool equal( const CClassID &id ) const
    {
        if((m_ID.dw[0]==id.m_ID.dw[0]) && 
        (m_ID.dw[1]==id.m_ID.dw[1]) &&
        (m_ID.dw[2]==id.m_ID.dw[2]) && 
        (m_ID.dw[3]==id.m_ID.dw[3])) 
            return true;
        return false;
    }
};

/**
* @brief	Compare two CClassID (for the hashtable)
* @param	id1 First CClassID
* @param    id2 Second CClassID
* @return   <b>bool</b> True if equal, false if not
*/
inline bool equal( const CClassID& id1, const CClassID& id2 ) 
{ 
    return id1.equal( id2 ); 
}

/**
* @brief	Obtain hash value for given CClassID
* @param	id The CClassID
* @return   <b>uint32</b> Unsigned integer hash value
*/
inline uint32 hashValue( const CClassID& id ) 
{ 
    return id.m_ID.hv; 
}

#define DECLARE_CLASS_ID(class_id) \
    static const CClassID &classID() \
    { \
        static CClassID id(class_id); \
        return id; \
    }

#define IMPLEMENT_CLASS_ID(Type) \
    virtual const CClassID &getClassID() const \
    { \
        return Type::classID(); \
    }

_END_EDL_NAMESPACE

#endif /* __EDLOBJECTCLASSID_H__ */

