/*
 * Copyright (C) 2007 Global Graphics Software Ltd. All rights reserved.
 */

/**
 * @file platform.h
 * @ingroup PlatformDependencies
 * @brief Platform-dependent defines, enumerations, types etc.
 * that are visible through the EDL API.
 */

#ifndef __PLATFORM_H__
#define __PLATFORM_H__


/***********************************************************
 * The target platforms currently supported by EDL consist of:
 *
 *  Operating System            Processor Architecture
 *  ----------------            ----------------------
 *  Windows XP                  x86
 *  Windows Server 2003         x86
 *  Windows Vista               x86
 *  Windows Server 2008         x86
 *
 *  Windows XP                  x64
 *  Windows Server 2003         x64
 *  Windows Vista               x64
 *  Windows Server 2008         x64
 *
 *  OS/X                        x64
 *  OS/X                        x86
 *  iOS                         armv7
 *  iOS                         x86 (simulator)
 *
 *  Linux                       x86
 *
 * In order to be able to resolve these platforms at compilation time
 * we are expecting the compilation environment to supply one (or more)
 * of the following preprocessor symbols:
 *
 *  _WIN32
 *  _WIN64
 *  __linux
 *  __APPLE__
 *  __ppc__
 *  __i386__
 *  __ppc64__
 *  __amd64__
 *  __x86_64__
 *  __arm__
 *  __arm64__
 *
 *
 * For some of these we additionally define a number of parallel symbols
 * that are needed by header files provided by other packages whose header files we compile against
 * and whose libraries we lint to to build EDL
 *
 *  WIN32
 *  WIN64
 *  LINUX
 ***********************************************************/

#ifdef _WIN32
#ifndef WIN32
#define WIN32
#endif
#endif

#ifdef _WIN64
#ifndef WIN64
#define WIN64
#endif
#endif

#ifdef __linux
#ifndef LINUX
#define LINUX
#endif
#endif


/***********************************************************
 * In addition to these operating-system/processor-architecture dependent
 * switches, where supported by the target platform EDL can also be build
 * with the "endianness" configured by the setting of the symbol
 *
 *  EDLENDIAN
 *
 * to either eEndianBig or eEndianLittle
 ***********************************************************/

typedef enum { eEndianBig, eEndianLittle } eEDLEndian;

#if defined(_WIN32) || defined(_WIN64)
// Windows only runs on little endian systems
#define EDLENDIAN   eEndianLittle
#elif defined(__i386__) || defined(__amd64__) || defined(__x86_64__) || defined(__arm__) || defined(__arm64__)
#define EDLENDIAN   eEndianLittle
#elif defined(__ppc__) || defined(__ppc64__)
#define EDLENDIAN   eEndianBig
#else
#error "Don't know the endianness for this architecture!"
#endif

/***********************************************************
 * EDLPTRSIZE indicates the size in bytes of a pointer for this architecture
 ***********************************************************/
 
 #if defined(_WIN64) || defined(__amd64__) || defined(__x86_64__) || defined(__ppc64__) || defined(__arm64__)
 #define EDLPTRSIZE 8
 #elif defined(_WIN32) || defined(__ppc__) || defined(__i386__) || defined(__arm__)
 #define EDLPTRSIZE 4
 #else
 #error "Don't know the pointer size for this architecture"
 #endif


/**********************************************************
 * In addition to the target-platform-dependent "switches"
 * we also define a number of symbols whose values are themselves platform-dependent
 * These include:
 *
 * PATH_SEP_CHAR which defines the character that represents a filestore path separator.
 *
 * DRIVE_DELIM which defines the character which separates/identifies
 * a drive letter at the start of a (Windows) path.
 *
 * And the various file permission "masks" like S_IRWXU
 *********************************************************/

#if defined(_WIN32) || defined(_WIN64)

// Windows dependencies
#include <limits.h>

#define PATH_SEP_CHAR    '\\'
#if _MSC_VER < 1900
#define snprintf _snprintf
#endif
#define getcwd   _getcwd

#ifndef MAXPATHLEN
#define MAXPATHLEN _MAX_PATH
#endif /* _WIN32 || _WIN64 */

#if _MSC_VER < 1800 
#define roundf(v) round(v)
#endif

#endif /* _WIN32 || _WIN64 */


#ifdef __APPLE__

// OSX OS and iOS dependencies

#include <errno.h>
#include <unistd.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/stat.h>    // for the file creation mask symbols S_IRWXU etc. and for mkdir()
#include <semaphore.h>  // for semaphores 
#include <sys/socket.h> // for sockets
#include <netinet/in.h>   // for sockaddr_in
#include <arpa/inet.h>   // for inet_addr

#define PATH_SEP_CHAR     '/'

extern const char **environ;

#endif /* __APPLE__ */


#ifdef __linux

    // Linux OS dependencies

#include <errno.h>
#include <unistd.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/stat.h>    // for the file creation mask symbols S_IRWXU etc. and for mkdir()
#include <semaphore.h>  // for semaphores 
#include <sys/socket.h> // for sockets
#include <netinet/in.h>   // for sockaddr_in
#include <arpa/inet.h>   // for inet_addr

#define PATH_SEP_CHAR     '/'
#define DRIVE_DELIM       ''

#endif /* __linux */


#endif   /* __PLATFORM_H__ */
