/*
 * Copyright (C) 2007-2019 Global Graphics Software Ltd. All rights reserved.
 */

/**
 * @file platform_utils.h
 * @ingroup PlatformDependencies
 * @brief Platform-dependent functions
 * that are visible through the EDL API.
 */

#ifndef __PLATFORM_UTILS_H__
#define __PLATFORM_UTILS_H__

#include <stdio.h>
#include <stdarg.h>
#include <edl/edlnamespaces.h>

_BEGIN_EDL_NAMESPACE

#ifdef __cplusplus
extern "C" {
#endif

    /**
     * @brief    Creates a subdirectory path (including parents) in an existing directory tree.
     * @param    dir Specifies the path to be created.
     *           If the parameter is specified as /foo/bar/dir, and the path /foo/bar already exists,
     *           the directory dir will be created within bar. If the parameter is specified as
     *           /foo/bar and /foo already exists, then both bar and bar/dir are created, and so
     *           forth.
     *           The path is treated as a UTF8 string.
     * @return   @b int Non-zero on success, zero if the call fails.
     */
    int  edlMkdir (const char *dir);
	
    /**
     * @brief    Removes a subdirectory, if it is empty.
     * @param    dir Specifies the path to, and name of, the directory to be removed.
     *           The path is treated as a UTF8 string.
     * @return   @b int Non-zero on success, zero if the call fails.
     */
    int  edlRmdir (const char *dir);
    
    /**
     * @brief    Get the platform-specific temporary directory path.
     *           This is created by consulting platform specific APIs or through the TEMP or TMP
     *           environment variables.
     * @return   A pointer to a string containing the temporary directory path. If the call fails
     *           NULL is returned. It is the responsibility of the caller to release the memory
     *           pointed to by the return value by calling free().
     */
    char   *edlGetTemporaryDirectory(void);
	
    /**
     * @brief    Creates an EDL temporary directory as specified by the two parameters.
     *           The temporary directory will be created as a subdirectory of the root temporary
     *           directory, which is specified by either the @c TEMP or @c TMP environment variables
     *           on most platforms, with platform specific methods used on iOS and UWP instead. For
     *           example, if @c TMP is set to c:\\tmp and you invoke @c edlMakeTempDir("edl", 2) then it
     *           will create c:\\tmp\\edl\\2.
     *
     * @param    subdir Specifies the first part of the path to the temporary directory to be created.
     *                  The path is treated as a UTF8 string.
     * @param    pidNo  Specifies the second part of the path to the temporary directory to be created. This
     *                  parameter allows multiple instances of the application to run on the same machine 
     *                  whilst maintaining independent temporary directories.
     * @return   A pointer to a string containing the path of the created directory.
     *           If the call fails, the pointer will be NULL.  It is the responsibility of the 
     *           caller to release the memory pointed to by the return value by calling @c free().
     */
    char    *edlMakeTempDir (const char *subdir, int pidNo);
	
    /**
     * @brief As edlMakeTempDir, but fails if the directory exists.
	 * @param subdir Specifies the first part of the path to the temporary directory to be created.
     *               The path is treated as a UTF8 string.
     * @param pidNo  Specifies the second part of the path to the temporary directory to be created. This
     *               parameter allows multiple instances of the application to run on the same machine 
     *               whilst maintaining independent temporary directories.
     * @return   A pointer to a string containing the path of the created directory.
     *           If the call fails, the pointer will be NULL.  It is the responsibility of the 
     *           caller to release the memory pointed to by the return value by calling @c free().
     */
    char    *edlExclusiveMakeTempDir (const char *subdir, int pidNo);
	
    /**
     * @brief As edlMakeTempDir, but also provides the UTF8 path to the subdir to the pointer at
     *        address fullSubDir. This must be freed by the caller using @c free(). This is
     *        useful if the user wishes to delete that directory. If the subdir parameter was an
     *        empty string or nothing but path separators, then fullSubDir will be NULL on exit.
     * @param subdir Specifies the first part of the path to the temporary directory to be created.
     *        The path is treated as a UTF8 string.
     * @param pidNo  Specifies the second part of the path to the temporary directory to be created. This
     *        parameter allows multiple instances of the application to run on the same machine 
     *             whilst maintaining independent temporary directories.
     * @param fullSubDir The pointer to the address of a UTF8 path to the subdir
     * @return A pointer to a string containing the path of the created directory.
     *         If the call fails, the pointer will be NULL.  It is the responsibility of the 
     *         caller to release the memory pointed to by the return value by calling @c free().
     */
    char    *edlMakeTempDirProvidingSubDirPath(const char *subdir, int pidNo, char **fullSubDir);
    
    /**
     * @brief    Gets the process id of the calling process.
                 This is useful if the user wants to provide separate temporary directories for each instance of EDL.
     * @return   @b int The process id of the calling process.
     */
	int     edlGetProcessId ();
    
    /**
     * @brief    Open a file as per fopen, handling UTF8 file names on all platforms.
     * @param    filename The path to the file.
     * @param    mode An fopen mode string.
     * @return   @b FILE A file pointer, or NULL on failure.
     */
    FILE *edlFopen(const char *filename, const char *mode);
   
    /**
     * @brief    Implementation of vsnprintf that always uses the C locale,
	 *           to write formatted data from variable argument list to sized buffer
     *           Parameters as per vsnprintf()
	 * @param buffer C string to accept the result
	 * @param n Buffer size
	 * @param format Format string
	 * @param ap Variable argument list
     * @return   @b int Either the number of characters printed (not including
     *           the null terminator) or the number of characters that would
     *           have been printed if the target buffer was unlimited in size.
     */
    int  edlVsnprintf(char *buffer, size_t n, const char *format, va_list ap);

    /**
     * @brief    Implementation of snprintf that always uses the C locale.
     *           Parameters as per snprintf().
	 * @param buffer C string to accept the result
	 * @param n Buffer size
	 * @param format Format string
     * @return   @b int Either the number of characters printed (not including
     *           the null terminator) or the number of characters that would
     *           have been printed if the target buffer was unlimited in size.
     */
    int  edlSnprintf(char *buffer, size_t n, const char *format, ...);
    
    /**
     * @brief    As edlSnprintf(), but throws an exception if the operation fails.
	 * @param buffer C string to accept the result
	 * @param n Buffer size
	 * @param format Format string
     * @return   @b int Either the number of characters printed (not including
     *           the null terminator) or the number of characters that would
     *           have been printed if the target buffer was unlimited in size.
     */
    void edlSnprintfE(char *buffer, size_t n, const char *format, ...);
    
#ifdef __cplusplus
}
#endif

_END_EDL_NAMESPACE

#endif   /* __PLATFORM_H__ */

