/*
* Copyright (C) 2013-2019 Global Graphics Software Ltd. All rights reserved.
*/
 
#ifndef _JAWSMAKO_INTERACTIVE_H_
#define _JAWSMAKO_INTERACTIVE_H_

/**
* @file     interactive.h
* @ingroup  JawsMako
* @brief    JawsMako interactive features.
*
* This header contains definitions of interactive objects
* of JawsMako, such as annotations and forms.
*/

#include <jawsmako/types.h>
#include <edl/idomform.h>

namespace JawsMako
{
    using namespace EDL;
    
    // Forward declarations
    class ILinkAnnotation;
    DECL_SMART_PTR(ILinkAnnotation);
    class IFreeTextAnnotation;
    DECL_SMART_PTR(IFreeTextAnnotation);
    class ICaretAnnotation;
    DECL_SMART_PTR(ICaretAnnotation);
    class IInkAnnotation;
    DECL_SMART_PTR(IInkAnnotation);
    class IPolyAnnotation;
    DECL_SMART_PTR(IPolyAnnotation);
    class IPopupAnnotation;
    DECL_SMART_PTR(IPopupAnnotation);
    class ILineAnnotation;
    DECL_SMART_PTR(ILineAnnotation);
    class IShapeAnnotation;
    DECL_SMART_PTR(IShapeAnnotation);
    class ISoundAnnotation;
    DECL_SMART_PTR(ISoundAnnotation);
    class IStampAnnotation;
    DECL_SMART_PTR(IStampAnnotation);
    class ITextAnnotation;
    DECL_SMART_PTR(ITextAnnotation);
    class ITextMarkupAnnotation;
    DECL_SMART_PTR(ITextMarkupAnnotation);
    class IRedactionAnnotation;
    DECL_SMART_PTR(IRedactionAnnotation);
    class IWidgetAnnotation;
    DECL_SMART_PTR(IWidgetAnnotation);
    class IAnnotationReference;
    DECL_SMART_PTR(IAnnotationReference);
    typedef CEDLVector<IAnnotationReferencePtr> CAnnotationReferenceVect;
    class IFormField;
    DECL_SMART_PTR(IFormField);
    typedef CEDLVector<IFormFieldPtr> CFormFieldVect;
    class IForm;
    DECL_SMART_PTR(IForm);
    class IAnnotationAppearance;
    DECL_SMART_PTR(IAnnotationAppearance);
    typedef CEDLVector<IAnnotationAppearancePtr> CAnnotationAppearanceVect;
    class IAnnotation;
    DECL_SMART_PTR(IAnnotation);
    class IMarkupAnnotation;
    DECL_SMART_PTR(IMarkupAnnotation);
    typedef CEDLVector<IAnnotationPtr> CAnnotationVect;
    class IThreads;
    DECL_SMART_PTR(IThreads);
    
    /**
    * @ingroup  JawsMako
    * @class    IAnnotationReference
    * @brief    A generic reference to an annotation. The target annotation
    *           might not be loaded. Chiefly used to refer to annotations
    *           from a Form.
    */
    class IAnnotationReference : public IRCObject
    {
      public:
        virtual ~IAnnotationReference() {}
        
        /**
        * @brief  Determine if another annotation reference refers to the same annotation.
		* @param other The other annotation reference 
		* @return @b bool True if @a other refers to the same annotation 
        */
        virtual bool equals(const IAnnotationReferencePtr &other) const = 0;
    };
    
    /**
    * @ingroup  JawsMako
    * @brief    The type of a form field. These map to field types present in
    *           the pdf specification.
    */
    typedef enum
    {
        eFTInherited,
        eFTButton,
        eFTText,
        eFTChoice,
        eFTSignature
    } eFieldType;

    /**
    * @ingroup  JawsMako
    * @class    IFormField
    * @brief    An interface class for a form field. A form field may have multiple child
    *           fields and widget annotations, arranged in a tree.
    */
    class IFormField : public IRCObject
    {
      public:
        virtual ~IFormField() {}
        
        /**
         *  @brief  Clone the form field, which includes a deep clone of any child IFormFields.
		 *  @return @b IFormFieldPtr The cloned field
		 */
		virtual IFormFieldPtr clone() const = 0;
        
        /**
         *  @brief  Get the type of the field.
         *          If eFTInherited is returned, the parent field should be consulted to determine
         *          the type.
         *  @return @b eFieldType The field type
         */
		 virtual eFieldType getFieldType() const = 0;
        
        /**
         *  @brief  Get the field flags, if present.
         *  @param  flags Reference to receive the flags
         *  @return @b bool False if the flags have not been set for this field,
         *           otherwise true. If false, the parent field should be consulted.
         */
        virtual bool getFieldFlags(uint32 &flags) const = 0;
        
        /**
         *  @brief Get the child fields in a vector
         *  @return @b CFormFieldVect The vector of child fields
         */
        virtual CFormFieldVect getChildFields() const = 0;
        
        /**
         *  @brief Get references to the child widgets in a vector
         *  @return @b CAnnotationReferenceVect The vector of child widgets' references
         */
        virtual CAnnotationReferenceVect getChildWidgets() const = 0;

        /**
         *  @brief Get the unique id for this field.
         *          This id is preserved across cloning, and therefore all clones of this 
         *          IFormField will have the same id. This id is particularly useful when merging 
         *          fields from one form to another. 
         *  @return @b DOMid The unique id for this field
         */
        virtual DOMid getFieldId() const = 0;

        /**
         *  @brief Clear the child fields list
         */
        virtual void clearChildFields() = 0;
        
        /**
         *  @brief  Add a child field.
         *          An exception will be thrown if the field to be inserted has the same 
         *          partial name or id as an existing field. 
         *  @param field Field to be added
         */
        virtual void addChildField(const IFormFieldPtr &field) = 0;
        
        /**
         *  @brief Remove a child field
         *  @param field Field to be removed
         */
        virtual void removeChildField(const IFormFieldPtr &field) = 0;
        
        /**
         *  @brief Search the IFormField subtree for a field that has the same id as the given field.
         *          The search recurses through the children. 
         *  @param field Field to search for
         *  @return @b bool True if found, otherwise false
         */
        virtual bool fieldInSubtree(const IFormFieldPtr &field) const = 0;

        /**
         *  @brief Clear the child widgets list.
         */
        virtual void clearChildWidgets() = 0;
		
        /**
         *  @brief  Add a child widget.
         *          An exception will be thrown if an annotation with the same reference is already 
         *          present at this level. 
         *  @param widget The widget to be added
         */
        virtual void addChildWidget(const IWidgetAnnotationPtr &widget) = 0;
        
        /**
         *  @brief Add a child widget by reference.
         *          An exception will be thrown if an annotation with the same reference is already 
         *          present at this level. 
         *  @param widgetReference The widget, by reference, to be added
         */
        virtual void addChildWidget(const IAnnotationReferencePtr &widgetReference) = 0;

        /**
         *  @brief Remove the given child widget from the field
         *  @param widget The widget to be removed
         */
        virtual void removeChildWidget(const IWidgetAnnotationPtr &widget) = 0;
        
        /**
         *  @brief Remove the given child widget from the field, by reference
         *  @param widgetReference The widget, by reference, to be removed
         */
        virtual void removeChildWidget(const IAnnotationReferencePtr &widgetReference) = 0;

        /**
         *  @brief  Search the IFormField subtree for a widget
         *          The search recurses through the children. 
         *  @param  widget The widget to search for
         *  @return @b bool True if found, otherwise false
         */
        virtual bool widgetInSubtree(const IWidgetAnnotationPtr &widget) const = 0;
        
        /**
         *  @brief  Search the IFormField subtree for a widget, by reference
         *          The search recurses through the children. 
         *  @param  widgetReference The widget, by reference, to search for
         *  @return @b bool True if found, otherwise false
         */
        virtual bool widgetInSubtree(const IAnnotationReferencePtr &widgetReference) const = 0;

        /**
         *  @brief  Get the partial name of the field, if defined.
         *           If not present, an empty string will be returned. 
         *  @return @b U8String The partial name of the field
         */        
		virtual U8String getPartialName() const = 0;
        
        /**
         *  @brief Set the partial name of the field
         *  @param name The partial name
         */
        virtual void setPartialName(const U8String &name) = 0;
    };

    /**
     * @ingroup  JawsMako
     * @brief A class encapsulating an entry in an XFA array
     */
    class CXFAPacket
    {
      public:
        U8String        name; ///< If empty and this is the only packet in the array, it represents the entire XFA package
        IInputStreamPtr stream; ///< The stream containing the XFA packet
    };
    typedef CEDLVector<CXFAPacket> CXFAPacketVect;
    
    /**
    * @ingroup  JawsMako
    * @class    IForm
    * @brief    An interface class for an interactive form, which tracks
    *           a tree of IFormFields and widgets.
    */
    class IForm : public IRCObject
    {
      public:
        virtual ~IForm() {}
        
        /**
         *  @brief Clone the form field, which includes a deep clone of any child IFormFields.
         *  @return @b IFormPtr The cloned form field
         */
        virtual IFormPtr clone() const = 0;
        
        /**
         *  @brief  Get the top level fields in a vector.
         *  @return @b CFormFieldVect The vector of fields
         */
        virtual CFormFieldVect getFields() const = 0;
        
        /**
         *  @brief  Get references to the top level widgets in a vector.
         *  @return @b CAnnotationReferenceVect The vector of annotation references
         */
        virtual CAnnotationReferenceVect getWidgets() const = 0;
        
        /**
         *  @brief Clear the top level fields list.
         */
        virtual void clearFields() = 0;
        
        /**
         *  @brief Add a field at the top level.
         *          An exception will be thrown if the field to be inserted has the same 
         *          partial name or id as an existing field at the top level. 
         *  @param field Field too be added
         */
        virtual void addField(const IFormFieldPtr &field) = 0;
        
        /**
         *  @brief Remove a top level field.
         *  @param field Field to be removed
         */
        virtual void removeField(const IFormFieldPtr &field) = 0;
        
        /**
         *  @brief Clear the top level widgets list.
         */
        virtual void clearWidgets() = 0;

        /**
         *  @brief Add a top level widget.
         *          An exception will be thrown if an annotation with the same reference is already 
         *  @param widget The widget to be added
         */
        virtual void addWidget(const IWidgetAnnotationPtr &widget) = 0;
        
        /**
         *  @brief Add a top level widget by reference.
         *          An exception will be thrown if an annotation with the same reference is already 
         *          present at this level. 
         *  @param widgetReference The widget, by reference, to be added
         */
        virtual void addWidget(const IAnnotationReferencePtr &widgetReference) = 0;

        /**
         *  @brief Remove the given top level widget
         *  @param widget The widget to be removed
         */
        virtual void removeWidget(const IWidgetAnnotationPtr &widget) = 0;
        
        /**
         *  @brief Remove the given top level widget, by reference.
         *  @param widgetReference The widget, by reference, to be removed
         */
        virtual void removeWidget(const IAnnotationReferencePtr &widgetReference) = 0;
        
        /**
         *  @brief Search the field tree for a field that has the same id as the given field.
         *          The search recurses through the children.
         *  @param field The field to be matched
         *  @return @b bool True if field was found, otherwise false
         */
        virtual bool fieldInTree(const IFormFieldPtr &field) const = 0;
        
        /**
         *  @brief Find the path to a field with the same id that is present in the field tree.
         *          
         *  @details This returns a vector of IFormFields representing the branches of
         *            the field tree that lead to the given field. That is, the last
         *            entry in the returned vector is the immediate parent of the desired
         *            field, the second last entry is the grandparent of the desired field, 
         *            and so forth leading back to the top level.
         *            
         *            This is useful in particular for determining the fully qualified name
         *            of a field.
         *            
         *            An empty vector is returned if the requested field is at the top
         *            level of the form and therefore has no parents.
         *            
         *            An exception is thrown if the field is not present in the tree.
         *  @param field The field to be matched
         *  @return @b CFormFieldVect A vector of IFormFields
         */
        virtual CFormFieldVect getPathToField(const IFormFieldPtr &field) const = 0;

        /**
         *  @brief  Search the field sub for a widget.
         *          The search recurses through the child fields.
         *  @param widget The widget to be matched
         *  @return @b bool True if widget was found, otherwise false
         */
        virtual bool widgetInTree(const IWidgetAnnotationPtr &widget) const = 0;
        
        /**
         *  @brief  Search the field sub for a widget, by reference
         *          The search recurses through the child fields.
         *  @param widgetReference The widget, by reference, to be matched
         *  @return @b bool True if widget was found, otherwise false
         */
        virtual bool widgetInTree(const IAnnotationReferencePtr &widgetReference) const = 0;

        /**
         *  @brief  Find the path to a widget with the same reference as the given widget.
         *          
         *  @details  This returns a vector of IFormFields representing the branches of
         *            the field tree that lead to the given widget. That is, the last
         *            entry in the returned vector is the immediate parent of the desired
         *            widget, the second last entry is the grandparent of the desired widget, 
         *            and so forth leading back to the top level.
         *            
         *            This is useful in particular for determining the fully qualified name
         *            of a field.
         *            
         *            An empty vector is returned if the requested widget is at the top
         *            level of the form and therefore has no parents.
         *            
         *            An exception is thrown if the widget is not present in the tree.
         *  @param widget The widget to be matched
         *  @return @b CFormFieldVect A vector of IFormFields
         */
        virtual CFormFieldVect getPathToWidget(const IWidgetAnnotationPtr &widget) const = 0;

        /**
         *  @brief     Find the path to a widget with the given reference.
         *             
         *  @details   This returns a vector of IFormFields representing the branches of
         *             the field tree that lead to the given widget. That is, the last
         *             entry in the returned vector is the immediate parent of the desired
         *             widget, the second last entry is the grandparent of the desired widget,
         *             and so forth leading back to the top level.
         *             
         *             This is useful in particular for determining the fully qualified name
         *             of a field.
         *             
         *             An empty vector is returned if the requested widget is at the top
         *             level of the form and therefore has no parents.
         *             
         *             An exception is thrown if the widget is not present in the tree.
         *  @param widgetReference The widget, by reference, to be matched
         *  @return @b CFormFieldVect A vector of IFormFields
         */
        virtual CFormFieldVect getPathToWidget(const IAnnotationReferencePtr &widgetReference) const = 0;
        
        /**
         *  @brief  Determine the type for the given widget field, resolving inherited values.
         *          Convenience. An exception is thrown if the widget is not present in the form.
         *  @param  widget The widget to inspect
         *  @return @b fieldType The field type
         */
        virtual eFieldType getWidgetFieldType(const IWidgetAnnotationPtr &widget) const = 0;
        
        /**
         *  @brief  Determine the flags for the given widget field, resolving inherited values.
         *          Convenience. An exception is thrown if the widget is not present in the form.
         *  @param  widget The widget to inspect
         *  @return uint32 The field flags
         */
        virtual uint32 getWidgetFieldFlags(const IWidgetAnnotationPtr &widget) const = 0;
        
        /**
         *  @brief  Get whether a viewer should construct its own appearances for the
         *          widgets in this form.
         *  @return @b bool True if viewer should construct its own appearances
         */
        virtual bool getNeedAppearances() const = 0;

        /**
         *  @brief  Set whether a viewer should construct its own appearances for the
         *          widgets in this field.
         *  @param needAppearances True or false
         */
        virtual void setNeedAppearances(bool needAppearances) = 0;
        
        /**
         *  @brief  Get the XFA packet data, if present, for this form.
         *          This information is not currently used by Mako, but is preserved.
         *  @return @b CXFAPacketVect The XFA packet data, or an empty vector if
         *          not present.
         */
        virtual CXFAPacketVect getXfaPacketData() const = 0;
        
        /**
         * @brief  Set the XFA packet data for this form.
         * @param  xfaPacketData The packet data to use, or an empty vector to clear.
         */
        virtual void setXfaPacketData(const CXFAPacketVect &xfaPacketData) = 0;
    };
    
    /**
    * @ingroup  JawsMako
    * @brief    The usage scenario for an annotation appearance.
    */
    typedef enum
    {
        eAUNormal,    ///< The normal appearance is used when the annotation is not interacting with the user. This appearance is also used for printing the annotation.
        eAURollover,  ///< The rollover appearance is used when the user moves the cursor into the annotation’s active area without pressing the mouse button.
        eAUDown,      ///< The down appearance is used when the mouse button is pressed or held down within the annotation’s active area.
    } eAppearanceUsage;

    /**
    * @ingroup  JawsMako
    * @class    IAnnotationAppearance
    * @brief    An interface class for an annotation appearance, describing the graphical
    *           content of an annotation in a given usage and state.
    *           Annotation appearances are immutable.
    */
    class IAnnotationAppearance : public IRCObject
    {
      public:
        virtual ~IAnnotationAppearance() {}
        
        /**
         *  @brief Create a new annotation appearance
         *  @param jawsMako The JawsMako instance
         *  @param form     The graphical contents as an IDOMForm
         *  @param usage    The usage for this appearance
         *  @param state    The appearance state for this appearance
         *  @return @b IAnnotationAppearancePtr A smart pointer to the new annotation appearance
         */
        static JAWSMAKO_API IAnnotationAppearancePtr create(const IJawsMakoPtr &jawsMako, const IDOMFormPtr &form, eAppearanceUsage usage, const U8String &state = U8String());
      
        /**
         *  @brief Clone the appearance. This is a deep clone.
         *  @return @b IAnnotationAppearancePtr A smart pointer to the cloned annotation appearance
         */
        virtual IAnnotationAppearancePtr clone() const = 0;

        /**
         *  @brief Get the usage for this appearance
         *  @return @b eAppearanceUsage The usage for this appearance
         */
        virtual eAppearanceUsage getUsage() const = 0;

        /**
         *  @brief  Get the appearance state for this appearance.
         *          Will return an empty string if there is no state associated with the appearance.
         *  @return @b U8String The appearance state for this appearance
         */
        virtual U8String getState() const = 0; 
        
        /**
         *  @brief  Get the graphical contents as an IDOMForm.
         *          Do not edit this form.
         *  @return @b IDOMFormPtr The graphical contents as an IDOMForm
         */
        virtual IDOMFormPtr getForm() const = 0;
        
        /**
         *  @brief  Get DOM for the annotation scaled appropriately to the given rect.
         *          Useful for preparing an annotation appearance for display or rendering.
         *          A clone of the contents will be returned.
         *  @param annotRect An FRect that defines the scale of the returned DOM node
         *  @return @b IDOMNodePtr The DOM for the annotation 
         */
        virtual IDOMNodePtr getScaledAppearance(const FRect &annotRect) const = 0;
    };
    
    /**
    * @ingroup  JawsMako
    * @class    CAnnotationBorder
    * @brief    A class representing an annotation's border (described in the
    *           PDF Specification as BorderStyle).
    *           The meaning of a border style depends on the annotation type
    *           and not all annotation types will support all attributes of
    *           this class, and neither will all PDF versions support all
    *           attributes.. Please refer to the PDF 1.7 specification for the
    *           required styles. JawsMako will only store the attributes that
    *           are valid for the given type, but will not signal errors for
    *           this case.
    */
    class CAnnotationBorder
    {
      public:
        /**
        * @brief Types of border
        */
        typedef enum
        {
            eBTSolid,     ///< A solid rectangle surrounding the annotation
            eBTDashed,    ///< A dashed rectangle surrounding the annotation
            eBTBeveled,   ///< A simulated embossed rectangle that appears to be raised above the surface of the page
            eBTInset,     ///< A simulated engraved rectangle that appears to be recessed below the surface of the page
            eBTUnderline  ///< A single line along the bottom of the annotation rectangle
        } eBorderType;
        
		/**
		 *  @brief Constructor. 
		 *  @param inWidth Border width. Default width is 1 PDF unit, or 1/72nd of an inch
		 *  @param inType  Border type. Default is @ref eBTSolid
		 *  @param inDash  A vector of floats describing a @ref eBTDashed dash pattern
		 */
		CAnnotationBorder(float inWidth = 1.0f / 72.0f * 96.0f, const eBorderType &inType = eBTSolid, const CEDLVector<float> &inDash = CEDLVector<float>())
        {
            width = inWidth;
            type = inType;
            dash = inDash;
        }
        
        float width;             ///< Width of the border in standard JawsMako units (96ths of an inch);
                                 ///< a width of zero indicates that the border is not drawn.
        eBorderType type;        ///< The type of border
        CEDLVector<float> dash;  ///< If the border type is @ref eBTDashed, this is the dash pattern,
                                 ///< expressed in standard JawsMako units (96ths of an inch)
    };
    
    /**
    * @ingroup  JawsMako
    * @class    IAnnotation
    * @brief    An interface class for an annotation.
    *           It is intended that future releases of JawsMako will extend this interface.
    */
    class IAnnotation : public IRCObject
    {
      public:
        virtual ~IAnnotation() {}
        
        /**
        * @brief Types of annotations, listed with the earliest version of PDF that supported them.
        */
        typedef enum
        {
			eAT3D,             ///< (PDF 1.6) 3D annotation 
			eATCaret,          ///< (PDF 1.5) Caret annotation
			eATCircle,         ///< (PDF 1.3) Circle annotation
			eATFileAttachment, ///< (PDF 1.3) File attachment annotation
			eATFreeText,       ///< (PDF 1.3) Free text annotation
			eATHighlight,      ///< (PDF 1.3) Highlight annotation
			eATInk,            ///< (PDF 1.3) Ink annotation
			eATLine,           ///< (PDF 1.3) Line annotation
			eATLink,           ///< (PDF 1.2) Link annotation
			eATMovie,          ///< (PDF 1.2) Movie annotation
			eATPolygon,        ///< (PDF 1.5) Polygon annotation
			eATPolyLine,       ///< (PDF 1.5) PolyLine annotation
			eATPopup,          ///< (PDF 1.3) Popup annotation
			eATPrinterMark,    ///< (PDF 1.4) Printer's mark annotation
			eATProjection,     ///< (PDF 1.7) Projection annotation
			eATRedact,         ///< (PDF 1.7) Redact annotation
			eATRichMedia,      ///< (PDF 1.7) RichMedia annotation
			eATScreen,         ///< (PDF 1.5) Screen annotation
			eATSound,          ///< (PDF 1.2) Sound annotation
			eATSquare,         ///< (PDF 1.3) Square annotation
			eATSquiggly,       ///< (PDF 1.4) Squiggly underline annotation
			eATStamp,          ///< (PDF 1.3) Stamp annotation
			eATStrikeOut,      ///< (PDF 1.3) Strikeout annotation
			eATText,           ///< (PDF 1.2) Text annotation
			eATTrapNet,        ///< (PDF 1.3) Trap network annotation
			eATUnderline,      ///< (PDF 1.3) Underline annotation
			eATWatermark,      ///< (PDF 1.6) Watermark annotation
			eATWidget,         ///< (PDF 1.2) Widget annotation
			eATOther,          ///< Other annotation
        } eAnnotationType;
        
        /**
         *  @brief Get the type of the annotation
         *  @return @b eAnnotationType The annotation type
         */
        virtual eAnnotationType getType() const = 0;
        
        /**
         *  @brief Get the rect in which the appearances should be displayed
         *  @return @b FRect The display rect
         */
        virtual const FRect &getRect() const = 0;

		/**
		 *  @brief  Set the rect in which the appearances should be displayed
		 *  @param rect The display rect, which must be non-empty
		 */
		virtual void setRect(const FRect &rect) = 0;
        
        /**
         *  @brief  Get the Contents entry in UTF-8.
         *  @return @b U8String The Contents entry, or an empty string if there is no Contents entry.
         */
        virtual U8String getContents() const = 0;
        
        /**
         *  @brief Set the Contents entry in UTF-8
         *  @param contents The Contents entry
         */        
		virtual void setContents(const U8String &contents) = 0;
        
        /**
         *  @brief  Get the color.
         *          The use of this color depends on the annotation type. See
         *          the PDF 1.7 specification for details.
         *  @return @b IDOMColorPtr The annotation color
         */
        virtual IDOMColorPtr getColor() const = 0;
        
        /**
         *  @brief  Set the color. The use of this color depends on the annotation type.
         *          See the PDF 1.7 specification for details. 
         *  @param color The required color. Setting a NULL value will
         *          remove any existing color. If a color is provided it must use
         *          a DeviceRGB, DeviceGray or DeviceCMYK color space.
         */
        virtual void setColor(const IDOMColorPtr &color) = 0;
        
        /**
         *  @brief  Get the Modification date and time of the annotation, if present
         *  @return @b IEDLTimePtr The modification date. If the date is not of the expected format,
         *  		 or is missing, a NULL object will be returned
         */
        virtual IEDLTimePtr getModificationTime() const = 0;
        
        /**
         *  @brief  Set the Modification date and time of the annotation
         *  @param modificationTime the Modification date and time. Must not be NULL.
         */
        virtual void setModificationTime(const IEDLTimePtr &modificationTime) = 0;
        
        /**
         *  @brief Get the annotation's border. See @ref CAnnotationBorder for details
         *  @return @b CAnnotationBorder The annotation's border
         */
        virtual CAnnotationBorder getBorder() const = 0;
        
        /**
         *  @brief  Set the annotation's border.
         *  @param border The border. See @ref CAnnotationBorder for details
         *          as not all attributes will be supported for all annotation types
         *          and all versions of PDF.
         */
        virtual void setBorder(const CAnnotationBorder &border) = 0;
        
        /**
         *  @brief  Get the annotation flags.
         *          To interpret these flags please see section 8.4.2 "Annotation Flags"
         *          in the PDF 1.7 specification.
         *  @return @b uint32 The annotation flags
         */
        virtual uint32 getFlags() const = 0;
        
        /**
         *  @brief  Set the annotation flags.
         *          Please see section 8.4.2 "Annotation Flags" for details about
         *          these flags.
         *  @param flags The annotation flags
         */
        virtual void setFlags(uint32 flags) = 0;
        
        /**
         *  @brief  Rotate the annotation clockwise as if the page
         *          was rotated by the same amount.
         *  
         *          NB If an annotation has the NoRotate (bit 4) flag set,
         *          then the annotation will not be rotated, merely repositioned
         *          accordingly.
         *  @param  rotate The angle to rotate. Must be 0, 90, 180 or 270 degrees.
         *  @param  pageWidth The width of the parent page.
         *  @param  pageHeight The height of the parent page.
         */
        virtual void rotate(uint16 rotate, double pageWidth, double pageHeight) = 0;
        
        /**
         *  @brief Return all the annotation appearances in a vector.
         *  @return @b CAnnotationAppearanceVect The vector of annotation appearances
         */
        virtual CAnnotationAppearanceVect getAppearances() const = 0;
        
        /**
         *  @brief Remove all annotation appearances
         */
        virtual void removeAppearances() = 0;
        
        /**
         *  @brief  Get the current annotation state
         *  @return @b U8String The current annotation state, or an empty string
         *                       if there is no state applicable
         */
        virtual U8String getState() const = 0;
        
        /**
         *  @brief  Set the current annotation state
         *  @param state The annotation state, or an empty string if the state should be removed
         */
        virtual void setState(const U8String &state) = 0;
        
        /**
         *  @brief  Fetch the annotation appearance that should be used for the
         *          given annotation usage and state according to the following rules:
         *          - If an exact match is found, it will be returned.
         *          - If an exact match is not found, but there exists a Normal appearance with
         *            the same state, it will be returned.
         *          - If none of the above yield an appearance, an exception results.
         *  
         *  @param usage The annotation usage
         *  @param state The annotation state
         *  @return @b IAnnotationAppearancePtr A smart pointer to the annotation appearance object
         */
		virtual IAnnotationAppearancePtr getAppearance(eAppearanceUsage usage, const U8String &state = U8String()) const = 0;
        
        /**
         *  @brief  Add or replace an appearance.
         *  
         *          If the insert would result in two or more appearances with the same usage
         *          but different states, then all appearances with the same usage must have
         *          a defined state (an invalid scenario). If not, an exception will be thrown.
         *  @param appearance The annotation appearance
         */
        virtual void addAppearance(const IAnnotationAppearancePtr &appearance) = 0;

        /**
         *  @brief  Does the annotation have a normal appearance?
         *          Convenience utility function.
         *  @return @b bool True if the annotation has a normal appearance
         */
        virtual bool hasNormalAppearance() const = 0;
        

        /**
         *  @brief  Clone the annotation. This is a deep clone.
         *          The annotation reference will remain the same.
         *  @return @b IAnnotationPtr A smart pointer to the cloned annotation object
         */
        virtual IAnnotationPtr clone() const = 0;
        
        /**
         *  @brief Does this annotation match the given IAnnotationReference?
         *  @param reference The annotation reference to be matched
         *  @return @b bool True if a match is found, otherwise false
         */
        virtual bool matchesReference(const IAnnotationReferencePtr &reference) const = 0;
        
        /**
         *  @brief Get a reference that can be used to refer to this annotation
         *  @return @b IAnnotationReferencePtr The annotation reference
         */
        virtual IAnnotationReferencePtr getReference() const = 0;
        
    };

    /**
    * @ingroup  JawsMako
    * @class    IMarkupAnnotation
    * @brief    An interface class for markup annotations.
    *           It is intended that future releases of JawsMako will extend this interface.
    */
    class IMarkupAnnotation : public IAnnotation
    {
      public:
        /**
         *  @brief   Get the Author of the markup annotation.
         *           This is the "T" entry in the annotation, which by convention is the author.
         *  @return @b U8String The author. An empty string is returned if the entry is missing.
         */
        virtual U8String getAuthor() const = 0;

        /**
         *  @brief  Set the Author of the markup annotation.
         *          This is the "T" entry in the annotation, which by convention is the author.
         *  @param author The author to set. An empty string will cause the author to be removed entirely.
         */
        virtual void setAuthor(const U8String &author) = 0;
        
        /**
         *  @brief  Get the creation date and time of the annotation, if present.
         *  @return @b IEDLTimePtr The annotation's creation date and time.
         *  @details If the date is not of the expected format, or is missing,
         *            a NULL object will be returned.
         */
        virtual IEDLTimePtr getCreationTime() const = 0;
        
        /**
         *  @brief  Set the creation date and time of the annotation.
         *  @param creationTime The creation date and time. Must not be NULL.
         */
        virtual void setCreationTime(const IEDLTimePtr &creationTime) = 0;
        
        /**
         *  @brief Get the opacity of the markup annotation
         *  @return @b float The opacity value
         */
        virtual float getOpacity() const = 0;
        
        /**
         *  @brief Set the opacity of the markup annotation
         *  @param opacity A value between 0.0 (fully transparent) and 1.0 (fully opaque)
         */
        virtual void setOpacity(float opacity) = 0;
        
        /**
         *  @brief  Get a reference to the popup, if present.
         *  @return @b IAnnotationReferencePtr The annotation reference to the popup.
         *  		 If there is no popup, a NULL reference will be returned.
         */
        virtual IAnnotationReferencePtr getPopupReference() const = 0;

        /**
         *  @brief  Set or clear the popup, by reference.
         *  @param popupReference A reference to the popup. Set to NULL to remove the current popup.
         *  @details For full functionality, the popup must be added to the current page's
         *            annotations.
         */
        virtual void setPopup(const IAnnotationReferencePtr &popupReference) = 0;

        /**
         *  @brief  Set a reference to a popup, if present.
         *  @param popup The popup. Set to NULL to remove the current popup.
         *  @details For full functionality, the popup must be added to the current page's
         *            annotations.
         */
        virtual void setPopup(const IPopupAnnotationPtr &popup) = 0;

    };
    #define obj2IMarkupAnnotation(obj) IMarkupAnnotationPtr(dynamic_cast<IMarkupAnnotation *>((IRCObject *) obj), true)

    /**
    * @ingroup  JawsMako
    * @class    CQuadPoint
    * @brief    A representation of a PDF Quadpoint, in DOM coordinates.
    */
    class CQuadPoint
    {
      public:
        CQuadPoint() {}
        CQuadPoint(const FPoint &inP1, const FPoint &inP2, const FPoint &inP3, const FPoint &inP4)
        {
            p1 = inP1;
            p2 = inP2;
            p3 = inP3;
            p4 = inP4;
        }
        FPoint p1;
        FPoint p2;
        FPoint p3;
        FPoint p4;
    };
    typedef CEDLVector<CQuadPoint> CQuadPointVect;
    
    typedef CEDLVector<CFPointVect> CFPointVectVect;
    

    /**
    * @ingroup  JawsMako
    * @class    CRectInset
    * @brief    A class which specifies an inset from a rectangle.
    */
    class CRectInset
    {
      public:
        CRectInset(double _left = 0.0, double _top = 0.0, double _right = 0.0, double _bottom = 0.0)
        {
            left = _left;
            top = _top;
            right = _right;
            bottom = _bottom;
        }
        double left;
        double top;
        double right;
        double bottom;
    };
    
    /**
    * @ingroup  JawsMako
    * @class    IWidgetAnnotation
    * @brief    An interface class for a widget annotation
    *           It is intended that future releases of JawsMako will extend this interface.
    */
    class IWidgetAnnotation : public IAnnotation
    {
      public:
        virtual ~IWidgetAnnotation() {}
        
        /**
         *  @brief Get the field type
         *  @return @b eFieldType The field type
         */
        virtual eFieldType getFieldType() const = 0;
        
        /**
         *  @brief  Get the partial name of the widget.
         *  @return @b U8String The partial name. If no partial name is present,
         *  		 an empty string is returned
         */
        virtual U8String getPartialName() const = 0;
        
        /**
         *  @brief Set the partial name of the widget
         *  @param name The partial name to be set
         */
        virtual void setPartialName(const U8String &name) = 0;
        
        /**
         *  @brief  Get the field flags. Please see the PDF specification for
         *          the definition of the flags. Note that the significance of
         *          of the flags changes depending on type, and that the value
         *          may be inheritable. 
         *  @param flags Reference to receive the field flags
         *  @return @b bool False if the flags have not been set for this
         *                   widget, otherwise true
         */
        virtual bool getFieldFlags(uint32 &flags) const = 0;
    };
    #define obj2IWidgetAnnotation(obj) IWidgetAnnotationPtr(dynamic_cast<IWidgetAnnotation *>((IRCObject *) obj), true)

    /**
    * @ingroup  JawsMako
    * @class    ITextMarkupAnnotation
    * @brief    A generic interface class for a text markup annotation
    *           It is intended that future releases of JawsMako will extend this interface.
    */
    class ITextMarkupAnnotation : public IMarkupAnnotation
    {
      public:
        virtual ~ITextMarkupAnnotation() {}
        
        /**
         *  @brief Get the markup annotation's quad points if present
         *  @return @b CQuadPointVect The markup annotation's quad points
         */
        virtual CQuadPointVect getQuadPoints() const = 0;
        
        /**
         *  @brief Set the markup annotation's quad points
         *  @param quadPoints The markup annotation's quad points
         */
        virtual void setQuadPoints(const CQuadPointVect &quadPoints) = 0;
    };
    #define obj2ITextMarkupAnnotation(obj) ITextMarkupAnnotationPtr(dynamic_cast<ITextMarkupAnnotation *>((IRCObject *) obj), true)

    /**
    * @ingroup  JawsMako
    * @class    ILinkAnnotation
    * @brief    A generic interface class for a link annotation
    *           It is intended that future releases of JawsMako will extend this interface.
    */
    class ILinkAnnotation : public IAnnotation
    {
      public:
        virtual ~ILinkAnnotation() {}
        
        /**
         *  @brief  Get the link annotation's quad points if present.
         *          The points are relative to the annotation rect.
         *  @return @b CQuadPointVect The link annotation's quad points
         */
        virtual CQuadPointVect getQuadPoints() const = 0;
        
        /**
         *  @brief Set the link annotation's quad points
         *  @param quadPoints The link annotation's quad points
         */
        virtual void setQuadPoints(const CQuadPointVect &quadPoints) = 0;
    };
    #define obj2ILinkAnnotation(obj) ILinkAnnotationPtr(dynamic_cast<ILinkAnnotation *>((IRCObject *) obj), true)

    /**
    * @ingroup  JawsMako
    * @class    IFreeTextAnnotation
    * @brief    A generic interface class for a free text annotation
    *           It is intended that future releases of JawsMako will extend this interface.
    */
    class IFreeTextAnnotation : public IMarkupAnnotation
    {
      public:
        virtual ~IFreeTextAnnotation() {}
        
        /**
         *  @brief  Get the rect inset describing, relative to the annotation rect, the free text
         *          area within the annotation.
         *          The points are relative to the annotation rect.
         *  @return @b CRectInset The rect inset
         */
        virtual CRectInset getRectInset() const = 0;
        
        /**
         *  @brief  Get the points that comprise the callout line of the free text annotation.
         *  @return @b CFPointVect A vector of the points that comprise the callout line. An empty vector will be returned if there is no callout line
         */
        virtual CFPointVect getCalloutLine() const = 0;
    };
    #define obj2IFreeTextAnnotation(obj) IFreeTextAnnotationPtr(dynamic_cast<IFreeTextAnnotation *>((IRCObject *) obj), true)
    
    /**
    * @ingroup  JawsMako
    * @class    ICaretAnnotation
    * @brief    A generic interface class for a caret annotation
    *           It is intended that future releases of JawsMako will extend this interface.
    */
    class ICaretAnnotation : public IMarkupAnnotation
    {
      public:
        virtual ~ICaretAnnotation() {}
        
        /**
         *  @brief  Get the rect inset describing, relative to the annotation rect, the caret
         *          area within the annotation.
         *  @return @b CRectInset The rect inset
         */
        virtual CRectInset getRectInset() const = 0;
    };
    #define obj2ICaretAnnotation(obj) ICaretAnnotationPtr(dynamic_cast<ICaretAnnotation *>((IRCObject *) obj), true)
        
    /**
    * @ingroup  JawsMako
    * @class    IShapeAnnotation
    * @brief    A generic interface class for circle and square annotations.
    *           It is intended that future releases of JawsMako will extend this interface.
    */
    class IShapeAnnotation : public IMarkupAnnotation
    {
      public:
        virtual ~IShapeAnnotation() {}
        
        /**
         *  @brief Create a shape (Rectangle or Circle) annotation with the given parameters.
         *         An appearance for the new annotation will be created based on the parameters
         *         using @ref createNormalAppearance().
         *  @param jawsMako The JawsMako instance
         *  @param rect     The annotations bounds. Must not be empty.
         *  @param type     The type of annotation to create. Must be IAnnotation::eATSquare or
         *                  IAnnotation::eATCircle
         *  @param color    Optional; The desired annotation color. If NULL, no line will be visible.
         *  @param border   Optional; The border to use. If not provided, a default will be used.
         *  @param interior Optional; the fill color to use. If NULL, no fill will be visible.
         *  @param inset    Optional; The amount to inset the line from the annotation rectangle.
         *
         *  @return @b IShapeAnnotationPtr A smart pointer to the new shape annotation
         */
        static JAWSMAKO_API IShapeAnnotationPtr create(const IJawsMakoPtr &jawsMako, const FRect &rect,
                                                       IAnnotation::eAnnotationType type,
                                                       const IDOMColorPtr &color = IDOMColorPtr(),
                                                       const CAnnotationBorder &border = CAnnotationBorder(),
                                                       const IDOMColorPtr &interior = IDOMColorPtr(),
                                                       const CRectInset &inset = CRectInset());
        
        /**
         *  @brief  Create an appearance given the current annotation's parameters.
         *          This can then be installed by using IAnnotation::addAppearance().
         *
         *          Note that when Inset, Beveled and Underline border styles will be
         *          represented as Solid lines.
         *  @return IAnnotationAppearancePtr A smart pointer to the resulting appearance
         */
        virtual IAnnotationAppearancePtr createNormalAppearance() const = 0;

        /**
         *  @brief  Get the rect inset describing, relative to the annotation rect, the shape
         *          area within the annotation.
         *  @return @b CRectInset The rect inset
         */
        virtual CRectInset getRectInset() const = 0;

        /**
         *  @brief  Get the interior color of the fill used for the shape.
         *  @return @b IDOMColorPtr The fill color. NULL is returned if there is no such color
         */
        virtual IDOMColorPtr getInteriorColor() const = 0;
        
        /**
         *  @brief   Set the interior color to be used to fill the shapes
         *  @param color The color definition, or pass NULL to remove the color
         */
        virtual void setInteriorColor(const IDOMColorPtr &color) = 0;
    };
    #define obj2IShapeAnnotation(obj) IShapeAnnotationPtr(dynamic_cast<IShapeAnnotation *>((IRCObject *) obj), true)

    /**
    * @ingroup  JawsMako
    * @class    IPolyAnnotation
    * @brief    An interface class for a polygon or polyline annotation.
    *           It is intended that future releases of JawsMako will extend this interface.
    */
    class IPolyAnnotation : public IMarkupAnnotation
    {
      public:
        virtual ~IPolyAnnotation() {}
        
        /**
         *  @brief  Get the points comprising the vertices of the polygon.
         *          The points are relative to the annotation rect.
         *  @return @b CFPointVect The vector of points
         */
        virtual CFPointVect getPoints() const = 0;

        /**
         *  @brief  Set the points comprising the vertices of the polygon.
         *          The points are relative to the annotation rect.
         *  @param points The vector of points
         */
        virtual void setPoints(const CFPointVect &points) = 0;
    };
    #define obj2IPolyAnnotation(obj) IPolyAnnotationPtr(dynamic_cast<IPolyAnnotation *>((IRCObject *) obj), true)
    
    /**
    * @ingroup  JawsMako
    * @class    ILineAnnotation
    * @brief    An interface class for a line annotation.
    *           It is intended that future releases of JawsMako will extend this interface.
    */
    class ILineAnnotation : public IMarkupAnnotation
    {
      public:
        virtual ~ILineAnnotation() {}
        
        /**
         *  @brief   Get the line end points
         *          The points are relative to the annotation rect.
         *  @param start Start point
         *  @param end   End point
         */
        virtual void getLineEndpoints(FPoint &start, FPoint &end) const = 0;
        
        /**
         *  @brief  Set the line end points.
         *          The points are relative to the annotation rect.
         *  @param start Start point
         *  @param end   End point
         */
        virtual void setLineEndpoints(const FPoint &start, const FPoint &end) = 0;
        
        /**
         *  @brief Get the length of the leader line
         *  @return @b float The leader line length
         */
        virtual float getLeaderLineLength() const = 0;
        
        /**
         *  @brief Get the length of the leader line extensions
         *  @return @b float The leader line extensions length
         */
        virtual float getLeaderLineExtensionsLength() const = 0;
        
        /**
         *  @brief Get the leader line offset
         *  @return @b float The leader line offset
         */
        virtual float getLeaderLineOffset() const = 0;
        
        /**
         *  @brief Get the caption offset
         *  @param xOffset The xOffset value
         *  @param yOffset The yOffset value
         */
        virtual void getCaptionOffset(float &xOffset, float &yOffset) const = 0;
        
        /**
         *  @brief  Get the interior color of the fill used for the line endings.
         *  @return @b IDOMColorPtr The fill color. NULL is returned if there is no such color
         */
        virtual IDOMColorPtr getInteriorColor() const = 0;
        
        /**
         *  @brief   Set the interior color to be used to fill the line endings.
         *  @param color The color definition, or pass NULL to remove the color
         */
        virtual void setInteriorColor(const IDOMColorPtr &color) = 0;

    };
    #define obj2ILineAnnotation(obj) ILineAnnotationPtr(dynamic_cast<ILineAnnotation *>((IRCObject *) obj), true)

    /**
    * @ingroup  JawsMako
    * @class    IInkAnnotation
    * @brief    A generic interface class for a ink annotation
    *           It is intended that future releases of JawsMako will extend this interface.
    */
    class IInkAnnotation : public IMarkupAnnotation
    {
    public:
        virtual ~IInkAnnotation() {}
        
		/**
		 *  @brief Create an ink annotation
		 *  @param jawsMako The JawsMako instance
		 *  @param rect     The annotations bounds. Must not be empty.
		 *  @param color    Optional; The desired annotation color. If NULL, RGB yellow will be chosen.    
		 *                  If provided, the color space must be DeviceRGB, DeviceCMYK or DeviceGray
		 *  @param inkList  Optional; the ink list to use
		 *  @return @b IInkAnnotationPtr A smart pointer to the new ink annotation
		 */		
		 static JAWSMAKO_API IInkAnnotationPtr create(const IJawsMakoPtr &jawsMako, const FRect &rect,
                                                     const IDOMColorPtr &color = IDOMColorPtr(),
                                                     const CFPointVectVect &inkList = CFPointVectVect());
        
        /**
         *  @brief  Get the annotation's ink list.
         *          All coordinates are relative to the annotation rectangle.
         *  @return @b CFPointVectVect The ink list (a collection of quadpoint data)
         */
        virtual CFPointVectVect getInkList() const = 0;
        
        /**
         *  @brief  Get the annotation's ink list.
         *          All coordinates are relative to the annotation rectangle
         *  @param inkList The ink list (a collection of quadpoint data)
         */
        virtual void setInkList(const CFPointVectVect &inkList) = 0;
    };
    #define obj2IInkAnnotation(obj) IInkAnnotationPtr(dynamic_cast<IInkAnnotation *>((IRCObject *) obj), true)

    /**
     * @ingroup  JawsMako
     * @class    IPopupAnnotation.
     * @brief    An interface class for a popup annotation, which should not exist as a
     *           standalone, but is associated with a Markup Annotation.
     *           No appearances can be added to this annotation type.
     *           It is intended that future releases of JawsMako will extend this interface.
     */
    class IPopupAnnotation : public IAnnotation
    {
      public:
        /**
         *  @brief Create a popup annotation.
         *  @param jawsMako The JawsMako instance
         *  @param rect     The annotations bounds. Must not be empty.
         *  @param open     Optional; whether or not the note is open. Default is false.
         *  @return @b IPopupAnnotationPtr A smart pointer to the new popup annotation
         */
        static JAWSMAKO_API IPopupAnnotationPtr create(const IJawsMakoPtr &jawsMako, const FRect &rect,
                                                       bool open = false);
        
        /**
         *  @brief Get the annotation's open status
         *  @return @b bool True if open, false if closed
         */
        virtual bool getOpen() const = 0;
        
        /**
         *  @brief Set the annotation's open status
         *  @param open Set to true for open, false for closed
         */
        virtual void setOpen(bool open) = 0;
    };
    #define obj2IPopupAnnotation(obj) IPopupAnnotationPtr(dynamic_cast<IPopupAnnotation *>((IRCObject *) obj), true)

    /**
    * @ingroup  JawsMako
    * @class    ISoundAnnotation
    * @brief    An interface class for a sound annotation. Allows access to the sound
    *           as a WAV stream if the stream is embedded.
    *           It is intended that future releases of JawsMako will extend this interface.
    */
    class ISoundAnnotation : public IMarkupAnnotation
    {
      public:
        virtual ~ISoundAnnotation() {}
        
        /**
         *  @brief  Get the embedded sound (if present) as a WAV stream.
         *  @return @b IInputStreamPtr The WAV stream, or NULL if there is no embedded stream
         */
        virtual IInputStreamPtr getSoundAsWav() const = 0;
    };
    #define obj2ISoundAnnotation(obj) ISoundAnnotationPtr(dynamic_cast<ISoundAnnotation *>((IRCObject *) obj), true)
    
    /**
    * @ingroup  JawsMako
    * @class    ITextAnnotation
    * @brief    A generic interface class for a text (sticky note) annotation
    */
    class ITextAnnotation : public IMarkupAnnotation
    {
      public:
        virtual ~ITextAnnotation() {}
        
		/**
		 *  @brief Create a text annotation
		 *  @param jawsMako The JawsMako instance
		 *  @param rect     The annotations bounds. Must not be empty.
		 *  @param contents The textual contents of the text annotation.
		 *  @param open     Optional; whether or not the note is open. Default is false.
		 *  @param iconName Optional; the name of the icon to use for displaying the annotation.
		 *                  The default is "Note". Must not be empty.
		 *  @param popup    The associated popup. The associated popup must be added to the annotations
		 *                   of the same page.
		 *  @return @b ITextAnnotationPtr A smart pointer to the new text annotation
		 */        
		static JAWSMAKO_API ITextAnnotationPtr create(const IJawsMakoPtr &jawsMako, const FRect &rect,
                                                      const U8String &contents, bool open = false,
                                                      const U8String &iconName = U8String("Note"),
                                                      const IPopupAnnotationPtr &popup = IPopupAnnotationPtr());
        
        /**
         *  @brief Get the annotation's open status
         *  @return @b bool True if open, false if closed
         */
        virtual bool getOpen() const = 0;
        
        /**
         *  @brief Set the annotation's open status
         *  @param open Set to true for open, false for closed
         */
        virtual void setOpen(bool open) = 0;
        
        /**
         *  @brief Get the annotation's icon name. Must not be empty.
         *  @return @b U8String The annotation's icon name
         */
        virtual U8String getIconName() const = 0;
        
        /**
         *  @brief Set the annotation's icon name
         *  @param iconName the annotation's icon name. Cannot be empty
         */
        virtual void setIconName(const U8String &iconName) = 0;
        
    };
    #define obj2ITextAnnotation(obj) ITextAnnotationPtr(dynamic_cast<ITextAnnotation *>((IRCObject *) obj), true)

    /**
    * @ingroup  JawsMako
    * @class    IRedactionAnnotation
    * @brief    A generic interface class for a redaction annotation.
    */
    class IRedactionAnnotation : public IMarkupAnnotation
    {
      public:
        virtual ~IRedactionAnnotation() {}
        
        /**
         *  @brief Create a redaction annotation
         *  @param jawsMako      The JawsMako instance
         *  @param rect          The annotations bounds. Must not be empty.
         *  @param quadPoints    The quadrilaterals defining the redaction area.
         *                        May be empty, in which case the annotation rect is used.
         *  @param color         The outline color for the annotation when viewed
         *                        in the user interface. Pass NULL for the default black
         *  @param interiorColor The color used to fill the annotation when it is applied.
         *                        An empty colour will result in white being used. The 
         *                        color, if provided, must use the DeviceRGB color space.
         *  @return @b IRedactionAnnotationPtr A smart pointer to the new redaction annotation
         */
        static JAWSMAKO_API IRedactionAnnotationPtr create(const IJawsMakoPtr &jawsMako, const FRect &rect,
                                                           const CQuadPointVect &quadPoints = CQuadPointVect(),
                                                           const IDOMColorPtr &color = IDOMColorPtr(),
                                                           const IDOMColorPtr &interiorColor = IDOMColorPtr());
        /**
         *  @brief  Get the redaction annotation's quad points if present.
         *          The points are relative to the annotation rect.
         *  @return @b CQuadPointVect The annotation's quad points
         */
        virtual CQuadPointVect getQuadPoints() const = 0;
        
        /**
         *  @brief Set the redaction annotation's quad points.
         *  @param quadPoints The annotation's desired quad points
         */
        virtual void setQuadPoints(const CQuadPointVect &quadPoints) = 0;
        
        /**
         *  @brief  Get the interior color of the fill used for the shape.
         *  @return @b IDOMColorPtr The interior color, or NULL if there is no such color
         */
        virtual IDOMColorPtr getInteriorColor() const = 0;
        
        /**
         *  @brief  Set the interior color to be used to fill the line endings.
         *  @param color The desired color. Pass NULL to remove the color (default is white). If a
         *               color is passed, it must use the DeviceRGB color space. 
         */
        virtual void setInteriorColor(const IDOMColorPtr &color) = 0;
    };
    #define obj2IRedactionAnnotation(obj) IRedactionAnnotationPtr(dynamic_cast<IRedactionAnnotation *>((IRCObject *) obj), true)

    /**
    * @ingroup  JawsMako
    * @class    IStampAnnotation
    * @brief    A generic interface class for a stamp annotation.
    */
    class IStampAnnotation : public IMarkupAnnotation
    {
      public:
        virtual ~IStampAnnotation() {}
        
        /**
         *  @brief Create a stamp annotation
         *  @param jawsMako The JawsMako instance
         *  @param rect     The annotations bounds. Must not be empty
         *  @param name     The "name" of the stamp
         *  @return @b IStampAnnotationPtr A smart pointer to the new stamp annotation
         */
        static JAWSMAKO_API IStampAnnotationPtr create(const IJawsMakoPtr &jawsMako, const FRect &rect,
                                                       const U8String &name = U8String("Draft"));
        /**
         *  @brief Get the stamp annotation's name
         *  @return @b U8String The stamp annotation's name
         */
        virtual U8String getName() const = 0;
        
        /**
         *  @brief Set the stamp annotation's name
         *  @param name The desired name
         */
        virtual void setName(const U8String &name) = 0;
        
        /**
         *  @brief Get the stamp annotation's intent
         *  @return @b U8String The stamp annotation's intent
         *  @see IStampAnnotation::getIntent
         */
        virtual U8String getIntent() const = 0;
        
        /**
         *  @brief  Set the stamp annotation's intent.
         *          Note: This is a PDF 2.0 feature. If intent is set to anything
         *          other than Stamp (the default), then the name of the stamp
         *          annotation will be dropped.
         *  @param intent The intent
         */
        virtual void setIntent(const U8String &intent) = 0;
        
    };
#define obj2IStampAnnotation(obj) IStampAnnotationPtr(dynamic_cast<IStampAnnotation *>((IRCObject *) obj), true)

    /**
    * @ingroup  JawsMako
    * @class    IThreads
    * @brief    An interface class for document threads,
    *           Currently a stub interface.
    */
    class IThreads : public IRCObject
    {
      public:
        virtual ~IThreads() {}
        
        virtual IThreadsPtr clone() const = 0;
    };
    
    /**
    * @ingroup JawsMako
    * @class   IAnnotationUtils
    *          Utility class for dealing with XML representations of annotations
    *          and forms.
    */
    class IAnnotationUtils
    {
      public:

        /**
        * @brief Simple class for tracking streams associated with XML generated by
        *        generateXMLForDocument()
        */
        class CXMLResource
        {
          public:
            U8String       name;
            U8String       mimeType;
            IInputStreamPtr resource;
        };
        typedef CEDLVector<CXMLResource> CXMLResourceVect;

		/**
		 *  @brief  Generate an XML version of the Annotations and Forms content in
		 *          the given IDocument.
		 *  @param jawsMako  The JawsMako instance
		 *  @param document  The document to generate the XML stream for
		 *  @param resources  A reference to a vector to receive any streams that hold
		 *                    sounds, embedded files, or other embedded streams referenced from
		 *                    annotations
		 *  @return @b IRAInputStreamPtr A smart pointer to a stream containing the XML 
		 *                                representation of the Annotations and Form content
		 */        
		static JAWSMAKO_API IRAInputStreamPtr generateXMLForDocument(const IJawsMakoPtr &jawsMako, const IDocumentPtr &document, CXMLResourceVect &resources);
        
        /**
         *  @brief Generate an IAnnotationReference from an Annotation Tag attribute found in the XML.
         *  @param tag Annotation Tag
         *  @return @b IAnnotationReferencePtr A smart pointer to the new annotation reference
         */
        static JAWSMAKO_API IAnnotationReferencePtr createAnnotationReferenceFromTag(const U8String &tag);
    };
    
    /**
    * @ingroup  JawsMako
    * @class INamedDestination
    * @brief A named destination in a PDF Document
    */
    class INamedDestination : public IRCObject
    {
      public:
        virtual ~INamedDestination() {}
        
        /**
         *  @brief  Create a named destination with the given name to the given target.
         *  @param  jawsMako The JawsMako instance
         *  @param  name     The name of the destination
         *  @param  target   The target of the destination
         *  @return INamedDestinationPtr A smart pointer to the new named destination
         */
        static JAWSMAKO_API INamedDestinationPtr create(IJawsMakoPtr &jawsMako, const U8String &name, const IDOMPageRectTargetPtr &target);
        
        /**
         *  @brief Clone the named destination
         *  @return @b INamedDestinationPtr A smart pointer to the cloned named destination
         */
        virtual INamedDestinationPtr clone() const = 0;
        
        /**
         *  @brief Get the name of this named destination
         *  @return @b U8String The name of this named destination
         */
        virtual const U8String &getName() const = 0;
        
        /**
         *  @brief   Get the destination.
         *  @return  @b IDOMPageRectTargetPtr A smart pointer to a page rect target
         *            that represents the destination of this named destination, or
         *            NULL if the destination is not understood
         *  @details Do not edit this destination
         */
        virtual IDOMPageRectTargetPtr getTarget() const = 0;
    };

}

#endif
