/*
 * jawsmakotext.h
 *
 * Copyright (C) 2012-2015 Global Graphics Software Ltd. All rights reserved.
 */

#ifndef JAWSMAKOTEXT_H
#define JAWSMAKOTEXT_H

#include <jawsmako/jawsmako.h>

namespace JawsMako
{
    using namespace EDL;

    /**
    * @brief Search options
    */
    typedef enum 
    {
        eFTCaseSensitive = 0x01,
        eFTIgnoreSpace = 0x10
    } eFindTextFlag;

    /**
    * @class    IFindText
    * @brief    A class to return the results of search. It includes a unicode string and a bounding box 
    *           for the string. 
    */
    class IFindText : public IRCObject
    {
    public:
        virtual ~IFindText() {}

        /**
        * @brief  Get the Unicode string for the run as available.
        */
        virtual const String& getUnicode() const = 0;

        /**
        * @brief  Get the Unicode string for the run as available, in UTF-8.
        */
        virtual U8String getUTF8() const = 0;

        /**
        * @brief  Get the bounding box as quadpoint for the text.
        *         The quadpoint is in closewise direction.
        */
        virtual CQuadPoint getQuadPoint() const = 0;
    };

    DECL_SMART_PTR(IFindText);
    typedef CEDLVector<IFindTextPtr> CFindTextVect;

    /**
    * @class    JawsMakoText
    * @brief    A wrapper class to handle text manipulations using JawsMako technology.
    *           At the moment, the class only handles text search.
    */
    class JawsMakoText
    {
    public:
        /**
        * @brief   Search for a given string on a page based on the given options and returns
        *          all the results;  
        * @param   page [Input] The page object to search for the text;
        * @param   searchText [Input] The string to search against;
        * @param   flag [Input] A bit-array for search options. see eFindTextFlag;
        * @param   findTextVect [Output] An vector to return all the search results;
        */
        static JAWSMAKO_API void findText(const IPagePtr &page, const U8String &searchText, uint32 flag, CFindTextVect &findTextVect);

    private:
        /**
        * @brief   Search for a given string in a textrun based on the given options and returns
        *          all the results;
        * @param   textRun [Input] The TextRun object to search for the text;
        * @param   searchText [Input] The string to search against;
        * @param   flag [Input] A bit-array for search options. see eFindTextFlag;
        * @param   findTextVect [Output] An vector to return all the search results;
        */
        static void match(const ITextRunPtr &textRun, const String &searchText, uint32 flag, CFindTextVect &findTextVect);

        /**
         * @brief   Building a mapping table for all searchable chars in the string, ie, chars that are not combination
         *          chars and are not space chars (if that is the option), to their indices in the original string
         *          so that once a string is found, we could find the string as they were in original textrun and
         *          return that and its bounding box.
         * @param   textToMatch [Input] The original string;
         * @param   ignoreSpace [Input] Whether space chars should be checked;
         * @param   indices [Output] An vector to give the indices of all non-space chars in the original string .
         */
        static void findSearchableCharsIndices(const String textToMatch, bool ignoreSpace, CEDLVector<uint32> &indices);

        /**
        * @brief   Normalise the string and also replace the ligatures that Unicode list as normal chars.
        *          Use JawsMako to normalse the string. There are a group of "ligatures" such as "" that
        *          Unicode considers as normal chars. For text search purpose, we need to treat them as
        *          ligatures.
        *          Refer to https://en.wikipedia.org/wiki/List_of_words_that_may_be_spelled_with_a_ligature for details.
        * @param   string string to be processed;
        * @return  result string.
        */
        static String normaliseAndReplaceNonListedLigatures(const String string);
    };
}

#endif // JAWSMAKOTEXT_H
