/*
 * Copyright (C) 2016 Global Graphics Software Ltd. All rights reserved.
 */

#ifndef JAWSMAKO_OPTIONALCONTENT_H
#define JAWSMAKO_OPTIONALCONTENT_H

/**
*  @file    optionalcontent.h
*  @ingroup JawsMako
*  @brief   Declarations of interfaces for querying and manipulating optional content.
*           Optional content is a PDF feature allowing sections of graphical content
*           to be made visible or visible when certain conditions are met. Most often
*           this is used to add visible layers to a PDF document.
*
*           Optional content is very powerful, and also complex. However, simple
*           layering features can be added simply. In this header, complex aspects
*           of the optional content scheme have been separated out.
*/

#include <jawsmako/types.h>

namespace JawsMako
{
    using namespace EDL;
    
    // Forward declarations
    class IOptionalContentGroupUsage;
    DECL_SMART_PTR(IOptionalContentGroupUsage);
    class IOptionalContentGroupUsageApplication;
    DECL_SMART_PTR(IOptionalContentGroupUsageApplication);
    class IOptionalContentConfiguration;
    DECL_SMART_PTR(IOptionalContentConfiguration);
    class IOptionalContentVisibilityExpression;
    DECL_SMART_PTR(IOptionalContentVisibilityExpression);
    typedef CEDLVector<IOptionalContentGroupPtr> COptionalContentGroupVect;
    typedef CEDLVector<IOptionalContentGroupUsageApplicationPtr> COptionalContentGroupUsageApplicationVect;
    typedef CEDLVector<IOptionalContentConfigurationPtr> COptionalContentConfigurationVect;
    typedef CEDLVector<IOptionalContentVisibilityExpressionPtr> COptionalContentVisibilityExpressionVect;

    typedef enum
    {
        eOCVVisible,
        eOCVInvisible,
        eOCVUnchanged
    } eOptionalContentVisibility;
    
    typedef enum
    {
        eOCCUnknown,
        eOCCView,
        eOCCPrint,
        eOCCExport,
        eOCCZoom,
        eOCCUser,
        eOCCLanguage
    } eOptionalContentCategory;
    typedef CEDLVector<eOptionalContentCategory> COCCategoryVect;

    /**
    * @ingroup JawsMako
    * @class   IOptionalContent
    * @brief   Root level optional content information for an entire document.
    */
    class IOptionalContent : public IRCObject
    {
      public:
        virtual ~IOptionalContent() {}
        
        /**
        * @brief Create a new optional content object.
        *        If no default configuration is provided, one will be created.
        */
        static JAWSMAKO_API IOptionalContentPtr create(const IJawsMakoPtr &jawsMako,
                                                       const IOptionalContentConfigurationPtr &defaultConfiguration = IOptionalContentConfigurationPtr());
        
        /**
        * @brief Clone the optional content.
        *        This is a deep clone.
        */
        virtual IOptionalContentPtr clone() = 0;
        
        /*
         *
         * Making content optional - convenience routines.
         *
         */
        
        /**
        * @brief Convenience to create a new optional content group with the given
        *        visibility status, and add. The group will also be added to the ui
        *        by adding to the default configuration's order.
        */
        virtual IOptionalContentGroupPtr makeNewGroup(const U8String &name, bool visible) = 0;
        
        /**
        * @brief Make the given DOM node optional as part of an existing
        *        optional content group.
        *
        *        If the node is an IDOMGroup, the group will have it's optional content
        *        information set accordingly and the returned result will be that node.
        *        However, if the node is some other graphical node, it will be moved
        *        inside an IDOMGroup with the optional content information set. If the
        *        node is in a tree, the node will be replaced with the group. The
        *        group will be returned.
        *
        *        Convenience.
        */
        virtual IDOMNodePtr makeNodeOptional(const IDOMNodePtr &node, const IOptionalContentGroupPtr &group) = 0;
        
        /**
        * @brief Make the given DOM node optional with a new optional content group.
        *
        *        Convenience.
        */
        IDOMNodePtr makeNodeOptionalWithNewGroup(const IDOMNodePtr &node, const U8String &newGroupName, bool visible)
        {
            IOptionalContentGroupPtr group = makeNewGroup(newGroupName, visible);
            return makeNodeOptional(node, group);
        }
        
        /*
         *
         * Visibilty Queries
         *
         */
        
        /**
        * @brief Determine if the given optional content group is visible
        *        in the given circumstances.
        * @param group The group to check.
        * @param event The event representing the intended use. Pass eOCEUnknown if
        *        the automatic state IOptionalContentGroupUsageApplication should not
        *        be checked.
        * @param configuration The configuration to use. Pass NULL to use the
        *        default configuration.
        */
        virtual bool groupIsVisible(const IOptionalContentGroupPtr &group,
                                    eOptionalContentEvent event = eOCEView,
                                    IOptionalContentConfigurationPtr configuration = IOptionalContentConfigurationPtr()) = 0;
        
        /**
        * @brief Determine if the given optional content group is visible
        *        in the given circumstances, using the group's reference.
        * @param groupRef Reference to the group to check.
        * @param event The event representing the intended use. Pass eOCEUnknown if
        *        the automatic state IOptionalContentGroupUsageApplication should not
        *        be checked.
        * @param configuration The configuration to use. Pass NULL to use the
        *        default configuration.
        */
        virtual bool groupIsVisible(const IOptionalContentGroupReferencePtr &groupRef,
                                    eOptionalContentEvent event = eOCEView,
                                    IOptionalContentConfigurationPtr configuration = IOptionalContentConfigurationPtr()) = 0;
        
        /*
         *
         * Manipulation and Interrogation
         *
         */
        
        /**
        * @brief Is a group with the given reference present?
        */
        virtual bool groupIsPresent(const IOptionalContentGroupReferencePtr &groupRef) const = 0;
        
        /**
        * @brief Get the optional content group with the given reference.
        *        An exception will be thrown if the group is not found.
        */
        virtual IOptionalContentGroupPtr getGroup(const IOptionalContentGroupReferencePtr &groupRef) const = 0;
        
        /**
        * @brief Force the given group to be always on or off in the default
        *        configuration. Convenience.
        *
        *        This will ensure the group is in or removed from the default
        *        visible and invisible list, ensure the group has the same
        *        intent as the configuration, and remove any usage information
        *        from the group.
        */
        virtual void forceGroupState(const IOptionalContentGroupPtr &group, bool visible) = 0;
        
        /**
        * @brief Get all the document's optional content groups in a vector.
        */
        virtual COptionalContentGroupVect getGroups() const = 0;
        
        /**
        * @brief Add an optional content group. An error will be thrown
        *        if a group with the same reference is already present.
        */
        virtual void addGroup(const IOptionalContentGroupPtr &group) = 0;
        
        /**
        * @brief Add an optional content group from another document.
        *
        *        This will attempt to maintain the context of the optional
        *        content group from the source document, adding to the
        *        default configuration in order to maintain its user-visible
        *        behaviour as closely as possible.
        */
        virtual void addGroup(const IOptionalContentGroupPtr &group, const IDocumentPtr &sourceDocument) = 0;
        
        /**
        * @brief Remove an optional content group.
        *        This will also purge the group from any mention in the
        *        default or alternate configurations.
        */
        virtual void removeGroup(const IOptionalContentGroupPtr &group) = 0;
        
        /**
        * @brief Set the default optional content configuration.
        *
        *        Note: There are restrictions on the intent attribute of
        *        an optional content configuration for it to be the default,
        *        which must be "View". Also the base state must be visible.
        *        This routine will modify the configuration to suit.
        *
        *        If the optional content configuration is not present in the
        *        configuration list, it will be added.
        */
        virtual void setDefaultConfiguration(const IOptionalContentConfigurationPtr &configuration) = 0;
        
        /**
        * @brief Get the default optional content configuration.
        */
        virtual IOptionalContentConfigurationPtr getDefaultConfiguration() const = 0;
        
        /**
        * @brief Add to the list of configurations for this optional content.
        *        If the configuration is already present, no action will be
        *        taken.
        */
        virtual void addConfiguration(const IOptionalContentConfigurationPtr &configuration) = 0;
        
        /**
        * @brief Remove the given configuration.
        *        This configuration must not be the current default or an
        *        exception will be thrown.
        */
        virtual void removeConfiguration(const IOptionalContentConfigurationPtr &configuration) = 0;
        
        /**
        * @brief Get all the configurations.
        */
        virtual COptionalContentConfigurationVect getConfigurations() const = 0;
        
    };
    

    /**
    * @ingroup JawsMako
    * @class   IOptionalContentDetails
    * @brief   Interface for objects used to tag content as optional.
    *          Instances of this class are set in IDOMGroup instances to make those
    *          objects optional, linking them to one or more optional content groups.
    */
    class IOptionalContentDetails : public IRCObject, public IHashable
    {
      public:
        virtual ~IOptionalContentDetails() {}
        
        /**
        * @brief The visibility policy controlling how the dependent optional
        *        content groups affect the visibility of this content.
        */
        typedef enum
        {
            eVPAllOn,  ///< This content is visible if all the dependent groups are visible
            eVPAnyOn,  ///< This content is visible if any dependent group is visible
            eVPAnyOff, ///< This content is visible if any dependent group is invisible
            eVPAllOff, ///< This content is visible if all the dependent groups are invisible
        } eVisibilityPolicy;

        /**
        * @brief Create using the given (optional) group.
        */
        static JAWSMAKO_API IOptionalContentDetailsPtr create(const IJawsMakoPtr jawsMako,
                                                              const IOptionalContentGroupReferencePtr &groupRef = IOptionalContentGroupReferencePtr(),
                                                              const IOptionalContentPtr &optionalContent = IOptionalContentPtr());
        
        /**
        * @brief Create using the given groups, policy, and (optional)
        * visibility expression.
        */
        static JAWSMAKO_API IOptionalContentDetailsPtr create(const IJawsMakoPtr jawsMako,
                                                              const COptionalContentGroupReferenceVect &groups,
                                                              eVisibilityPolicy policy,
                                                              const IOptionalContentVisibilityExpressionPtr &visibilityExpression = IOptionalContentVisibilityExpressionPtr(),
                                                              const IOptionalContentPtr &optionalContent = IOptionalContentPtr());
        
        /**
        * @brief Clone.
        */
        virtual IOptionalContentDetailsPtr clone() const = 0;
        
        /**
        * @brief Determine the visibility of this content.
        *        Pass in the Optional Content for this document to obtain current
        *        results. Otherwise, the visibility returned will be either:
        *        - The optional content state at the time this object was created,
        *          if the IOptionalContent was passed at the time of creation
        *          (always the case for objects created directly from reading from
        *          a PDF, or if it was created using an IOptionalContent API function)
        *        - Otherwise, true is returned.
        */
        virtual bool getIsVisible(eOptionalContentEvent event = eOCEView,
                                  const IOptionalContentPtr &optionalContent = IOptionalContentPtr()) const = 0;
        
        /**
        * @brief Make the optional content dependent on the given group
        *        (if it is not already so).
        *        Optional content can be a member of multiple groups.
        *        The visibility policy or visibility expression (see below)
        *        controls how the visibility of this piece of optional
        *        content is calculated based on the groups upon which it
        *        depends.
        */
        virtual void addGroup(const IOptionalContentGroupReferencePtr &groupRef) = 0;
        
        /**
        * @brief Remove the dependence of the content on the given group.
        */
        virtual void removeGroup(const IOptionalContentGroupReferencePtr &groupRef) = 0;
        
        /**
        * @brief Get all the groups that this content depends on.
        */
        virtual COptionalContentGroupReferenceVect getGroupReferences() const = 0;

        /**
        * @brief Set the visibility policy.
        *        If present, the visibility expression will take precedence.
        *        The default is eAnyOn.
        */
        virtual void setVisibilityPolicy(eVisibilityPolicy policy) = 0;
        
        /**
        * @brief Get the visibility policy.
        */
        virtual eVisibilityPolicy getVisibilityPolicy() const = 0;
        
        /**
        * @brief Set the visibility expression, which affords more powerful
        *        control of the visibilty of this content via a logical
        *        expression. This takes precedence over the visibility
        *        policy for consumers that support visibility expressions.
        *        Pass NULL to remove any existing expression.
        */
        virtual void setVisibilityExpression(const IOptionalContentVisibilityExpressionPtr &expression) = 0;
        
        /**
        * @brief Get the visibility expression.
        */
        virtual IOptionalContentVisibilityExpressionPtr getVisibilityExpression() = 0;
        
    };
    
    /**
    * @ingroup JawsMako
    * @class   IOptionalContentGroupReference
    * @brief   A reference to an optional content group.
    */
    class IOptionalContentGroupReference : public IRCObject, public IHashable
    {
      public:
        virtual ~IOptionalContentGroupReference() {}
        
        /**
        * @brief Does the given reference point to the same group as this reference?
        */
        virtual bool equals(const IOptionalContentGroupReferencePtr &reference) const = 0;
    };
    
    /**
    * @ingroup JawsMako
    * @class   IOptionalContentGroup
    * @brief   Interface for an optional content group
    */
    class IOptionalContentGroup : public IRCObject
    {
      public:
        virtual ~IOptionalContentGroup() {}
        
        /**
        * @brief Create an optional content group.
        *        A non-zero length name must be provided.
        */
        static JAWSMAKO_API IOptionalContentGroupPtr create(const IJawsMakoPtr &jawsMako, const U8String &name);
        
        /**
        * @brief Clone the group.
        *        Note that cloning the group will *not* result in a new reference
        *        being created.
        */
        virtual IOptionalContentGroupPtr clone() = 0;
        
        /**
        * @brief Get the reference to this group.
        *        This is used with several APIs to maintain a link to this group.
        */
        virtual IOptionalContentGroupReferencePtr getReference() const = 0;
        
        /**
        * @brief Set the name of this group.
        *        Must not be an empty string.
        */
        virtual void setName(const U8String &name) = 0;

        /**
        * @brief Get the name of this group.
        */
        virtual U8String getName() const = 0;
        
        /**
        * @brief Set the intent of the group, or an empty string to
        *        reset to the default.
        */
        virtual void setIntent(const U8String &intent) = 0;
        
        /**
        * @brief Set an array of intents for this group.
        */
        virtual void setIntents(const CU8StringVect &intents) = 0;
        
        /**
         * @brief Get the intents for this group.
         *        Usually this will be a single entry for most real world cases.
         *        A default of "View" will be returned if the entry is not present.
         */
        virtual CU8StringVect getIntents() const = 0;
        
        /**
        * @brief Set the Usage information for this group.
        *        Pass NULL to remove the usage information.
        */
        virtual void setUsage(const IOptionalContentGroupUsagePtr &usage) = 0;
        
        /**
        * @brief Get the Usage information for this group. May be NULL.
        */
        virtual IOptionalContentGroupUsagePtr getUsage() const = 0;
    };
    
    
    
    /*
     *
     *
     *   Advanced API
     *
     *
     */
    
    
    /**
    * @ingroup JawsMako
    * @class   IOptionalContentGroupUsage
    * @brief   Usage information for an optional content group, providing context
    *          that an application can use to automatically show or hide content
    *          in the optional content group. This is optional.
    *
    *          These are only consulted when making visibility decisions if
    *          the document's IOptionalContent specifies that the usage for this
    *          group should be used, and even then only some attributes may
    *          be consulted depending on how the usages are configured. See
    *          #IOptionalContentGroupUsageApplication below.
    */
    class IOptionalContentGroupUsage : public IRCObject
    {
      public:
        ~IOptionalContentGroupUsage() {}
        
        /**
        * @brief  Create usage information.
        */
        static JAWSMAKO_API IOptionalContentGroupUsagePtr create(const IJawsMakoPtr &jawsMako);
        
        /**
        * @brief  Clone the usage information.
        */
        virtual IOptionalContentGroupUsagePtr clone() const = 0;
        
        /**
        * @brief  Set the language of the content controlled by this optional
        *         content group, and whether or not it should be considered
        *         "preferred" if there is no exact match for the language used
        *         by the viewer's.
        * @param  language A "Natural Language Specification" string representing
        *         the language and possible a locale, such as "en-US". Pass an
        *         empty string to specify no particular language.
        * @param  preferred Whether this should be considered preferred if no
        *         exact match for the user's viewing language is found.
        */
        virtual void setLanguage(const U8String &language, bool preferred = false) = 0;
        
        /**
        * @brief  Get the language for the group, which may be an empty string
        *         if no language has been set.
        */
        virtual U8String getLanguage() const = 0;
        
        /**
        * @brief  Get whether or not the language of this optional content
        *         group should be considered "preferred".
        */
        virtual bool getLanguageIsPreferred() const = 0;
        
        /**
        * @brief  Set if the content in the group should be present when the
        *         content is "exported" to a format that does not support
        *         optional content.
        */
        virtual void setExportVisibility(eOptionalContentVisibility visibility) = 0;
        
        /**
        * @brief  Get whether the content in the group should be present when
        *         the content is "exported" to a format that does not support
        *         optional content.
        */
        virtual eOptionalContentVisibility getExportVisibility() const = 0;
        
        /**
        * @brief  Set if the content should be visible between the given zoom
        *         levels. Setting maximumZoom to a negative value indicates
        *         infinity.
        */
        virtual void setZoomVisibility(float minimumZoom, float maximumZoom) = 0;
        
        /**
        * @brief  Get the zoom levels between which this content is visible.
        *         A negative maximumZoom indicates infinity..
        */
        virtual void getZoomVisibility(float &minimumZoom, float &maximumZoom) const = 0;
        
        /**
        * @brief  Set if the content in the group should be present when the
        *         content is printed, and optionally an informative string
        *         describing the kind of content.
        */
        virtual void setPrintVisibility(eOptionalContentVisibility visibility, const U8String &description = U8String()) = 0;
        
        /**
        * @brief  Get whether the content in the group should be present when
        *         the content is printed.
        */
        virtual eOptionalContentVisibility getPrintVisibility() const = 0;
        
        /**
        * @brief  Get the print visibility description, which may be an empty
        *         string.
        */
        virtual U8String getPrintVisibilityDescription() const = 0;
        
        /**
        * @brief  Set if the content in the group should be visible when the
        *         content is viewed.
        */
        virtual void setViewVisibility(eOptionalContentVisibility visibility) = 0;
        
        /**
        * @brief  Get whether the content in the group should be visible when
        *         the content is viewed.
        */
        virtual eOptionalContentVisibility getViewVisibility() const = 0;

        /**
        * @brief  Get the type and users for which this content is primarily
        *         intended.
        * @param  type The type of user, which should be either "Ind" (Individual),
        *         "Ttl" (Title), or "Org" (Organisation). Must not be empty
        *         unless the users vector is also empty (which will cause the
        *         user information to be ignored for visibility determination)
        * @param  users The user names.
        */
        virtual void setUsers(const U8String &type, const CU8StringVect &users) = 0;
        
        /**
        * @brief  Get the type and users for which this content is primarily
        *         intended.
        * @param  type Reference to receive the type of user.
        * @return The user names, or an empty array if none are present.
        */
        virtual CU8StringVect getUsers(U8String &type) const = 0;
        
        /**
        * @brief  Sets a description of the page element that is represented
        *         by the optional content group.
        * @param  type Should be either "HF" (Header or footer), "FG"
        *         (foreground content), "BG" (Background content), "L" (Logo)
        *         or an empty string indicating no particular page element is
        *         represented.
        */
        virtual void setPageElement(const U8String &type) = 0;
        
        /**
        * @brief  Gets a description of the page element that is represented
        *         by the optional content group. May be an empty string.
        */
        virtual U8String getPageElement() const = 0;
        
        /**
        * @brief  Deteremine the recommended state from this usage information
        *         for the given usage categories.
        *         Uses the algorithm on page 383 of the PDF 1.7 specification,
        *         with the following modifications:
        *         - Zoom is not considered and always assumed ON
        *         - User is not considered and always assumed ON
        *         - Language is considered ON if the language is preferred.
        *         May return Unchanged.
        */
        virtual eOptionalContentVisibility recommendedVisibilityForCategories(const COCCategoryVect &categories) const = 0;
    };
    
    
    /**
    * @ingroup JawsMako
    * @class   IOptionalContentGroupUsageApplication
    * @brief   Interface for controlling how IOptionalContentGroupUsage is
    *          applied, and for what groups.
    */
    class IOptionalContentGroupUsageApplication : public IRCObject
    {
      public:
        virtual ~IOptionalContentGroupUsageApplication() {}
        
        /**
        * @brief  Create usage application information.
        */
        static JAWSMAKO_API IOptionalContentGroupUsageApplicationPtr create(const IJawsMakoPtr &jawsMako,
                                                                            eOptionalContentEvent event = eOCEView);
        
        /**
        * @brief  Clone the usage application information.
        */
        virtual IOptionalContentGroupUsageApplicationPtr clone() const = 0;
        
        /**
        * @brief  Set the event for which the the optional content usage should
        *         be applied. Must not be eOCEUnknown.
        */
        virtual void setEvent(eOptionalContentEvent event) = 0;
        
        /**
        * @brief  Get the event for which the optional content usage should be
        *         applied.
        */
        virtual eOptionalContentEvent getEvent() const = 0;
        
        /**
        * @brief Set the optional content groups (by reference) for which the
        *        usage information should be applied.
        */
        virtual void setAffectedOptionalContentGroups(const COptionalContentGroupReferenceVect &groups) = 0;
        
        /**
        * @brief Get the optional content groups for which the usage information
        *        should be applied.
        */
        virtual COptionalContentGroupReferenceVect getAffectedOptionalContentGroups() const = 0;
        
        /**
        * @brief Add the given group to the list of affected groups.
        *        It should not already be present.
        */
        virtual void addGroup(const IOptionalContentGroupReferencePtr &groupRef) = 0;
        
        /**
        * @brief Is the given group affected by this usage application?
        */
        virtual bool groupIsAffected(const IOptionalContentGroupReferencePtr &groupRef) const = 0;
        
        /**
        * @brief Set the categories from the optional content groups that should
        *        be consulted to determine visibility.
        */
        virtual void setCategories(const COCCategoryVect &categories) = 0;
        
        /**
        * @brief Get the categories from the optional content groups that should
        *        be consulted to determine visibility.
        */
        virtual COCCategoryVect getCategories() const = 0;
        
        /**
        * @brief Does the usage categories include the given category?
        */
        virtual bool categoryInCategories(eOptionalContentCategory category) const = 0;
        
        /**
        * @brief Does the usage categories include all the given categories?
        *        Convenience.
        */
        virtual bool categoriesInCategories(const COCCategoryVect &categories) const = 0;
    };
    
    /**
    * @ingroup JawsMako
    * @class   IOptionalContentConfiguration
    * @brief   A configuration for optional content.
    */
    class IOptionalContentConfiguration : public IRCObject
    {
      public:
        virtual ~IOptionalContentConfiguration() {}
        
        /**
        * @brief Create a new configuration.
        */
        static JAWSMAKO_API IOptionalContentConfigurationPtr create(const IJawsMakoPtr &jawsMako);
        
        /**
        * @brief Clone the configuration.
        */
        virtual IOptionalContentConfigurationPtr clone() const = 0;
        
        /**
        * @brief Set the name of the configuration.
        *        May be an empty string.
        */
        virtual void setName(const U8String &name) = 0;
        
        /**
        * @brief Get the name of the configuration.
        *        An empty string is returned if no name is present.
        */
        virtual U8String getName() const = 0;
        
        /**
        * @brief Set the "creator" of the configuration.
        *        May be an empty string.
        */
        virtual void setCreator(const U8String &creator) = 0;
        
        /**
        * @brief Get the "creator" of the configuration.
        *        An empty string is returned if no name is present.
        */
        virtual U8String getCreator() const = 0;
        
        /**
        * @brief Get the base visibility state of the optional content groups
        *        in the document.
        */
        virtual void setBaseState(eOptionalContentVisibility visibility) = 0;
        
        /**
        * @brief Get the base visibility state of the optional content groups
        *        in the document.
        */
        virtual eOptionalContentVisibility getBaseState() const = 0;
        
        /**
        * @brief Set the vector of optional content group references that should
        *        be visible by default.
        */
        virtual void setDefaultVisibleGroups(const COptionalContentGroupReferenceVect &groups) = 0;
        
        /**
        * @brief Get the vector of optional content groups references that should
        *        be visible by default.
        */
        virtual COptionalContentGroupReferenceVect getDefaultVisibleGroups() const = 0;
        
        /**
        * @brief Is the given group present in the visible groups?
        */
        virtual bool groupInVisibleGroups(const IOptionalContentGroupReferencePtr &groupRef) const = 0;
        
        /**
        * @brief Set the vector of optional content group references that should
        *        be invisible by default.
        */
        virtual void setDefaultInvisibleGroups(const COptionalContentGroupReferenceVect &groups) = 0;
        
        /**
        * @brief Get the vector of optional content groups references that should
        *        be invisible by default.
        */
        virtual COptionalContentGroupReferenceVect getDefaultInvisibleGroups() const = 0;
        
        /**
        * @brief Is the given group present in the invisible groups?
        */
        virtual bool groupInInvisibleGroups(const IOptionalContentGroupReferencePtr &groupRef) const = 0;
        
        /**
        * @brief Set the intent for this configuration
        *        Pass an empty string to remove the intent.
        */
        virtual void setIntent(const U8String &intent) = 0;
        
        /**
        * @brief Set an array of intents for this configuration.
        *        If set to an empty array, all groups will be considered
        *        visible, regardless of any other status information.
        *        To remove the intents entry, use #setIntent above with
        *        an empty string.
        */
        virtual void setIntents(const CU8StringVect &intents) = 0;
        
        /**
        * @brief Get the intents for this configuration.
        *        Usually this will be a single entry for most real world cases.
        *        A default of "View" will be returned if the entry is not present.
        */
        virtual CU8StringVect getIntents() const = 0;
        
        /**
        * @brief Determine if the given intent is present in the list of intents.
        *        If the intents specify All then all intents will match.
        *        intent must not be an empty string.
        */
        virtual bool intentInIntents(const U8String &intent) const = 0;
        
        /**
        * @brief Set the usage applications that apply to this configuration.
        */
        virtual void setAutoStates(const COptionalContentGroupUsageApplicationVect &applications) = 0;
        
        /**
        * @brief Get the usage applications that apply to this configuraiton.
        */
        virtual COptionalContentGroupUsageApplicationVect getAutoStates() const = 0;
        
        class COrderEntry;
        DECL_SMART_PTR(COrderEntry);
        typedef CEDLVector<COrderEntryPtr> COrderEntryVect;

        /**
        * @brief Class for presenting the order that groups should be
        *        displayed in a user interface. May be arranged in a tree.
        */
        class COrderEntry : public IRCObject
        {
          public:
            static JAWSMAKO_API COrderEntryPtr create();
            
            COrderEntryPtr clone(); ///< Create a deep copy
            
          public:
            // If true, this indicates that this entry merely represents a group,
            // in which case only the groupRef entry is valid.
            // If false, this entry represents an array of groups, that may have
            // an optional name for the UI.
            bool                              isGroup;

            IOptionalContentGroupReferencePtr groupRef; ///< Only valid if isGroup is true
            
            U8String                          name; ///< May be an empty string. Only valid if isGroup is false
            COrderEntryVect                   children; ///< Only valid if isGroup is false
        };
        
        /**
        * @brief Set the order that groups should be shown in a user interface.
        */
        virtual void setOrder(const COrderEntryVect &order) = 0;
        
        /**
        * @brief Get the order that groups should be shown in a user interface.
        *        An independent copy will be returned.
        */
        virtual COrderEntryVect getOrder() const = 0;
        
        typedef enum
        {
            eLMAllPages,
            eLMVisiblePages
        } eListMode;
        
        /**
        * @brief Set which optional content groups should be shown in the
        *        consuming application's user interface.
        */
        virtual void setListMode(eListMode listMode) = 0;
        
        /**
        * @brief Get which optional content groups should be shown in the
        *        consuming application's user interface.
        */
        virtual eListMode getListMode() const = 0;

        /**
        * @brief Set the "Radio Button" optional content groups for this
        *        configuration.
        *        This is a vector of vectors of group references. For each
        *        of the vectors of references, setting a group to Visible
        *        should result in all of the others to become invisible.
        *        Pass an empty vector to remove this feature.
        */
        virtual void setRadioButtonGroups(const CEDLVector<COptionalContentGroupReferenceVect> &rbGroups) = 0;
        
        /**
        * @brief Get the "Radio Button" optional content groups for this
        *        configuration.
        *        This is a vector of vectors of group references. For each
        *        of the vectors of references, setting a group to Visible
        *        should result in all of the others to become invisible.
        */
        virtual CEDLVector<COptionalContentGroupReferenceVect> getRadioButtonGroups() const = 0;
        
        /**
        * @brief Set the optional content groups that should be locked in
        *        the user interface.
        */
        virtual void setLockedGroups(const COptionalContentGroupReferenceVect &lockedGroups) = 0;
        
        /**
        * @brief Get the optional content groups that should be locked in
        *        the user interface.
        */
        virtual COptionalContentGroupReferenceVect getLockedGroups() const = 0;
    };
    
    
    /**
    * @ingroup JawsMako
    * @class   IOptionalContentVisibilityExpression
    * @brief   An interface representing a PDF 1.6+ visibility expression.
    *          Please refer to table 4.4.9 of the PDF 1.7 specification for
    *          background and detail.
    */
    class IOptionalContentVisibilityExpression : public IRCObject, public IHashable
    {
      public:
        virtual ~IOptionalContentVisibilityExpression() {}
        
        /**
        * @brief Clone this expression.
        */
        virtual IOptionalContentVisibilityExpressionPtr clone() = 0;
        
        /**
        * @brief The operation of this expression.
        */
        typedef enum
        {
            eVEOGroupState, ///< The result of this expression is the state of the single group
            eVEONot,        ///< The result of this expression is the inverse of the sub-expression
            eVEOAnd,        ///< The result of this expression is the logical and of the result of the sub-expressions
            eVEOOr,         ///< The result of this expression is the logical or of the result of the sub-expressions
        } eVisibilityExpressionOperation;
        
        /**
        * @brief Create a "Group State" expression.
        *        That is, the result of this expression is true if the given group is visible,
        *        and false otherwise.
        */
        static JAWSMAKO_API IOptionalContentVisibilityExpressionPtr createGroupState(const IOptionalContentGroupReferencePtr &groupRef);

        /**
        * @brief Create a "Not" expression.
        *        That is, the result of this expression is the inverse of the
        *        visibility of the given sub-expression.
        */
        static JAWSMAKO_API IOptionalContentVisibilityExpressionPtr createNot(const IOptionalContentVisibilityExpressionPtr &subExpression);
        
        /**
        * @brief Create an "And" expression.
        *        That is, the result of this expression is the logical "and" of the result of
        *        all the sub-expressions.
        */
        static JAWSMAKO_API IOptionalContentVisibilityExpressionPtr createAnd(const COptionalContentVisibilityExpressionVect &subExpressions);
        
        /**
        * @brief Create an "Or" expression.
        *        That is, the result of this expression is the logical "or" of the result of
        *        all the sub-expressions.
        */
        static JAWSMAKO_API IOptionalContentVisibilityExpressionPtr createOr(const COptionalContentVisibilityExpressionVect &subExpressions);
        
        /**
        * @brief  Get the type of operation.
        */
        virtual eVisibilityExpressionOperation getOperation() const = 0;
        
        /**
        * @brief Get the group for eVEOGroupState operations. If the operation
        *        is something else, an exception will be thrown.
        */
        virtual IOptionalContentGroupReferencePtr getGroup() const = 0;
        
        /**
        * @brief Get the subexpressions if the operation is eVEONot, eVEOAnd or eVEOOr.
        *        If the operation is something else, an exception will be thrown.
        */
        virtual COptionalContentVisibilityExpressionVect getSubExpressions() const = 0;
        
        /**
        * Determine the result of the expression.
        * The document's optional content must be provided.
        */
        virtual bool evaluate(const IOptionalContentPtr &optionalContent, eOptionalContentEvent event) const = 0;
        
    };

}

#endif /* JAWSMAKO_OPTIONALCONTENT_H */
