/*
* Copyright (C) 2018-2019 Global Graphics Software Ltd. All rights reserved.
*/
 
#ifndef JAWSMAKO_PCL5INPUT_H
#define JAWSMAKO_PCL5INPUT_H


/**
* @file     pcl5input.h
* @ingroup  JawsMako
* @brief    PCL5 Input-Specific Features.
*
* This header describes the IPCL5Input interface.
* These are a subclass of IInput. Instances of this type are returned by
* IInput::create() when eFFPCL5 is specified. This allows access to PCL5
* specific input features and modes of operation.
*/

#include <jawsmako/jawsmako.h>
#include <jawsmako/pjl.h>

namespace JawsMako
{
    using namespace EDL;

    class IPCL5Input;
    DECL_SMART_PTR(IPCL5Input);

    /**
    * @ingroup  JawsMako
    * @class    IPCL5Input
    * @brief    An instance of the JawsMako PCL5 input class.
    */
    class IPCL5Input : public IInput
    {
      public:
        virtual ~IPCL5Input() {}

        /**
         * @brief   Create an input for reading source documents in PCL5 format.
         *
         * @return  IPCL5InputPtr the PCL5 input
         */
        static JAWSMAKO_API IPCL5InputPtr create(const IJawsMakoPtr &jawsMako, const IAbortPtr &abort = IAbortPtr ());
        
        /**
         * @brief    Set the resolution to be used when flattening ROPs.
         *
         *           PCL5, as PCL/XL features a method of using bitwise operations
         *           to composite pixels from patterns and images with content
         *           already on the page. These are known as ROPs (Raster
         *           OPerations). These are not compatible with the DOM and
         *           most other PDLs, and in many situations require rendering
         *           at the input stage.
         *
         *           This parameter controls the rendering resolution. The default
         *           is 600dpi.
         *
         * @param    resolution The ROP rendering resolution to use.
         */
        virtual void setRopResolution(uint32 resolution) = 0;
        
        /**
         * @brief    Set the default paper size
         *
         *           The paper size is set using a string. To take effect the paper
         *           size string must match one of the followingstandard strings:
         *
         *           - letter
         *           - legal
         *           - a4
         *           - exec
         *           - executive
         *           - ledger
         *           - a3
         *           - monarchenvelope
         *           - c5
         *           - dl
         *           - jisb4
         *           - jisb5
         *           - a5
         *           - a6
         *           - tabloid
         *           - ofuku
         *
         *           The matching is case-insensitive.
         *
         *           The default page size is a4
         */
        virtual void setDefaultPaperSize(const U8String &paperSize) = 0;
        
        /**
         * @brief    Set the default orientation.
         *
         *           Pass true to set landscape, or false to set portrait.
         *           The default is portrait.
         */
        virtual void setDefaultLandscape(bool landscape) = 0;
        
        /**
         * @brief    Set the default number of per-page copies.
         *
         *           The default is 1. Must be greater than zero.
         */
        virtual void setDefaultCopies(uint32 copies) = 0;
        
        /**
         * @brief    Set whether or not duplex should be set by default.
         *
         *           The default is false.
         */
        virtual void setDefaultDuplex(bool duplex) = 0;
        
        /**
         * @brief    Set the default binding edge for duplexing.
         *
         *           The default is long edge.
         */
        virtual void setDefaultDuplexBindingMode(IPJLParser::eDuplexBindingMode bindingMode) = 0;
        
        /**
         * @brief    Set whether or not manual feed should be set by default.
         *
         *           The default is false.
         */
        virtual void setDefaultManualFeed(bool manualFeed) = 0;
        
        /**
         * @brief    Take initialisation data from the given PJL parser.
         *
         *           Uses the environment in the given PJL parser to set
         *           defaults in the PCL5 input. Any previously set
         *           defaults will be ignored.
         *
         *           This is especially useful with unencapsulated mode,
         *           where PJL parsing happens externally to the input.
         */
        virtual void setDefaultsFromPjl(IPJLParserPtr &pjlParser) = 0;
        
        /**
         * @brief    Enable or disable unencapsulated mode.
         *
         *           Setting this mode to true changes the parser behaviour
         *           as follows.
         *
         *           - The input stream is not opened; it is assumed to point to
         *             the start of a PCL5 stream.
         *           - The PJL parser is not used. If PJL is encountered an
         *             error will be thrown.
         *           - The PCL5 interpreter will not consume data after the point the
         *             interpreter either exhausts the stream or returns to PJL.
         *             After requesting a document after the last document
         *             in the session, or by requesting the number of documents) the
         *             stream will be positioned at the point that the PCL5
         *             interpreter exited.
         *
         *           Whereas the default (false) will:
         *
         *           - open() the input stream.
         *           - use the built-in PJL parser when PJL is encountered.
         *           - process all PCL5 sessions present in the stream, even
         *             if they are interupted by returning to PJL.
         *
         */
        virtual void enableUnencapsulatedMode(bool unencapsulated) = 0;
        

    };
    #define obj2IPCL5Input(obj) IPCL5InputPtr(dynamic_cast<IPCL5Input *>((IRCObject *) obj), true)
}

#endif
