/*
 * Copyright (C) 2014-2019 Global Graphics Software Ltd. All rights reserved.
 */

#ifndef _JAWSMAKO_PCL5OUTPUT_H_
#define _JAWSMAKO_PCL5OUTPUT_H_

/**
* @file     pcl5output.h
* @ingroup  JawsMako
* @brief    JawsMako PCL5 Output.
*/

#include <jawsmako/jawsmako.h>

namespace JawsMako
{
    using namespace EDL;
    
    class IPCL5Output;
    DECL_SMART_PTR(IPCL5Output);

    /**
     * @ingroup  JawsMako
     * @class    IPCL5Output
     * @brief    Interface for the PCL5 IOutput class
     */
    class IPCL5Output : public IOutput
    {
    public:
        virtual ~IPCL5Output() {}
        
        /**
         * @brief   Create a PCL5 Output instance.
         */
        static IPCL5OutputPtr create(const IJawsMakoPtr &jawsMako, const IAbortPtr &abort = IAbortPtr ());
        
        /*
         * PCL5 Output-specific configuration
         */
        
        /**
         * @brief Supported versions
         */
        typedef enum
        {
            ePCL5e,
            ePCL5c
        } ePCL5Version;

        /**
         * @brief Set the PCL version.
         *        The default is ePCL5c.
         *        Equivalent to calling setParameter() with the parameter name
         *        "PCL5Version" using the string value "pcl5e" or "pcl5c"
         * @param version The version to use.
         */
        virtual void setVersion(ePCL5Version version) = 0;
        
        /**
         * @brief Set the PCL resolution
         *        The default is 600.
         *        Equivalent to calling setParameter() with the parameter name
         *        "Resolution".
         * @param version The version to use.
         */
        virtual void setResolution(uint32 resolution) = 0;
        
        /**
         * @brief Available image compression schemes.
         */
        typedef enum
        {
            ePICNone,
            ePICRLE,
            ePICDeltaRow
        } ePCL5ImageCompression;
        
        /**
         * @brief Set the desired image compression method.
         *        The default is delta row.
         *        Equivalent to calling setParameter() with the parameter name
         *        "ImageCompression" using the string value "None", "RLE", or
         *        "DeltaRow"
         * @param compression The compression method to use.
         */
        virtual void setImageCompression(ePCL5ImageCompression compression) = 0;
        
        /**
         * @brief Set the desired media source
         *        The default is 7.
         *        Equivalent to calling setParameter() with the parameter name
         *        name "MediaSource".
         * @param source The media source to use.
         */
        virtual void setMediaSource(uint32 mediaSource) = 0;
        
        /**
         * @brief Set whether the output stream should be opened or not.
         *
         *        If true, the output stream will be opened and closed by the
         *        PCL5 output. If false, the stream will assumed to be opened
         *        and will not be closed.
         *
         *        Setting this to false allows the PCL5 stream to be written
         *        to an existing stream or channel.
         *
         *        The default is true.
         *
         *        Equivalent to calling setParameter() with the parameter name
         *        "OpenStream", the value "true" or "false".
         */
        virtual void setOpenStream(bool open) = 0;
        
        /**
         * @brief Set whether or not a PJL header or end of job should be emitted.
         *
         *        The default is true.
         *
         *        Equivalent to calling setParameter() with the parameter name
         *        "EmitPJL", the value "true" or "false".
         */
        virtual void setEmitPjl(bool emitPjl) = 0;
 
		/**
	   * @brief Set whether or not a TrueType font's .notdef glyph is displayed.
	   *
	   *        The default is false.
	   *
	   *        Equivalent to calling setParameter() with the parameter name
	   *        "EnableTrueTypeNotDef", the value "true" or "false".
	   */
		virtual void setEnableTrueTypeNotDef(bool enableTrueTypeNotDef) = 0;

    };
#define obj2IPCL5Output(obj) IPCL5OutputPtr(dynamic_cast<IPCL5Output *>((IRCObject *) obj), true)
}

#endif

