/*
 * Copyright (C) 2014-2019 Global Graphics Software Ltd. All rights reserved.
 */
 
#ifndef JAWSMAKO_PDFOUTPUT_H
#define JAWSMAKO_PDFOUTPUT_H

/**
 * @file     pdfoutput.h
 * @ingroup  JawsMako
 * @brief    JawsMako PDF Output.
 */

#include <jawsmako/jawsmako.h>
#include <jawsmako/transforms.h>

namespace JawsMako
{
    using namespace EDL;

    class IPDFOutput;
    DECL_SMART_PTR(IPDFOutput);

    /**
     * @ingroup  JawsMako
     * @class    IPDFOutput
     * @brief    Interface for the PDF IOutput class.
     *
     *           Supported presets (use \link IOutput::setPreset \endlink) are:
     *           - "Preserve" which will attempt to produce output as close
     *             to the input as possible for the output format.
     *           - "PDF1.3" which will configure the output for PDF 1.3.
     *             In addition to setting the output version, this will also
     *             cause all objects to be colour converted to DeviceRGB, but
     *             this may be overridden with setTargetSpace().
     *           - "General" - a general purpose PDF output for general document
     *             exchange and viewing. PDF 1.7, RGB.
     *           - "Print" - a higher quality/larger file size for print purposes.
     *             Uses PDF 1.4 for wide support.
     *           - "Web" - a general purpose PDF output for distributing documents
     *             online. PDF 1.7, RGB.
     *           - "PDF/A-1b" - A preset for generating PDF/A-1b, RGB, with
     *             sensible defaults for resolution and downsampling.
     *           - "PDF/A-2b" - A preset for generating PDF/A-2b, RGB output
     *             intent, with sensible defaults for resolution and downsampling.
     *           - "PDF/X-1a" - A preset for generating PDF/X-1a, CMYK (SWOP) with
     *             everything converted to CMYK.
     *           - "PDF/X-4" - A preset for generating PDF/X-4, CMYK (SWOP) with
     *             sensible defaults.
     *
     *           Note that when writing PDF output with multiple documents,
     *           only the forms and other top level metadata from the first
     *           document is used. Multiple documents should be combined
     *           before writing an assembly if required.
     *
     *           Note that there is a limit on the number of PDF and PS
     *           output operations that may be in progress simultaneously.
     *           These are:
     *           - iOS - 1 PDF or PS output operation at a time
     *           - Android - 1 PDF or PS output operation at a time
     *           - Windows/MacOS/X 32 bit - 8 PDF or PS output operations at a time, subject to available memory
     *           - Windows/MacOS/X 64 bit - 96 PDF or PS output operations at a time for most tool chains
     *                                    - VS2015 Static builds are currently limited to 48
     *           - Windows UWP - 1 PDF or PS output operation at a time
     */
    class IPDFOutput : public IOutput
    {
      public:
        virtual ~IPDFOutput() {}
        
        /**
         * @brief   Create a PDF Output instance.
         */
        JAWSMAKO_API static IPDFOutputPtr create(const IJawsMakoPtr &jawsMako, const IAbortPtr &abort = IAbortPtr ());
        
        /*
         * PDF Output-specific configuration
         */
        
        /* Bit positions for ePDFVersion so that we can store a PDF version number
         * plus the flavor (like PDF/A or PDF/X). We use a total of 28 bits so it
         * fits in a 32-bit int.
         *
         *      (field:size)        (offset)
         *      variantType:8       20
         *      variantVer:4        16
         *      variantVerSub:8     8
         *      verMajor:4          4
         *      verMinor:4          0
         */

        #define MAKE_PDF_VERSION(major,minor)                               ( ((major) << 4) | (minor) )
        #define MAKE_PDF_VARIANT(variantType,variantVer,variantVerSub)      ( ((variantType) << 20) | ((variantVer) << 16) | ((variantVerSub) << 8) )

        #define GET_PDF_VERSION_MAJOR(pdfVersion)                   (((pdfVersion) >> 4) & 0x0f)
        #define GET_PDF_VERSION_MINOR(pdfVersion)                   (((pdfVersion) >> 0) & 0x0f)

        #define GET_PDF_VARIANT_TYPE(pdfVersion)                    (((pdfVersion) >> 20) & 0x0ff)
        #define GET_PDF_VARIANT_VER(pdfVersion)                     (((pdfVersion) >> 16) & 0x00f)
        #define GET_PDF_VARIANT_VERSUB(pdfVersion)                  (((pdfVersion) >>  8) & 0x0ff)

        #define GET_PDFA_VERSION(pdfVersion) ( (GET_PDF_VARIANT_TYPE(pdfVersion) == 'A') ? GET_PDF_VARIANT_VER(pdfVersion) : 0)
        #define GET_PDFX_VERSION(pdfVersion) ( (GET_PDF_VARIANT_TYPE(pdfVersion) == 'X') ? GET_PDF_VARIANT_VER(pdfVersion) : 0)

        /**
         * @brief Supported versions
         */
        typedef enum
        {
            ePDF1_3 = MAKE_PDF_VERSION(1,3) ,  ///< PDF 1.3
            ePDF1_4 = MAKE_PDF_VERSION(1,4) ,  ///< PDF 1.4
            ePDF1_5 = MAKE_PDF_VERSION(1,5) ,  ///< PDF 1.5
            ePDF1_6 = MAKE_PDF_VERSION(1,6) ,  ///< PDF 1.6
            ePDF1_7 = MAKE_PDF_VERSION(1,7) ,  ///< PDF 1.7
            ePDF2_0 = MAKE_PDF_VERSION(2,0) ,  ///< PDF 2.0 (EXPERIMENTAL)
            ePDFA1b = MAKE_PDF_VERSION(1,4) | MAKE_PDF_VARIANT('A',1,'b') ,     ///< PDF/A-1b, based on PDF 1.4
            ePDFA2b = MAKE_PDF_VERSION(1,7) | MAKE_PDF_VARIANT('A',2,'b') ,     ///< PDF/A-2b, based on PDF 1.7
            ePDFX1a = MAKE_PDF_VERSION(1,4) | MAKE_PDF_VARIANT('X',1,'a') ,     ///< PDF/X-1a:2003, based on PDF 1.4
            ePDFX4  = MAKE_PDF_VERSION(1,6) | MAKE_PDF_VARIANT('X',4,0) ,       ///< PDF/X-4, based on PDF 1.6
        } ePDFVersion;

        /**
         * @brief Set the PDF version to generate.
         *
         *        Valid versions supported by this release are:
         *        - 1.3
         *        - 1.4
         *        - 1.6
         *        - 1.7
         *        - PDF/A-1b
         *        - PDF/X-1a
         *        - PDF/X-2b
         *        - PDF/X-4
         *
         *        Equivalent to calling setParameter with the parameter
         *        name "PDFVersion" with the value of the version as a string
         *        (i.e. "1.3", "1.7", "PDF/A-1b", "PDF/X-1a"). The default is 1.7.
         *        Some features in the content being written may be dropped.
         *
         *        For PDF 1.3 it is recommended that the "PDF1.3" preset be
         *        used. This will ensure correct-looking results on diverse
         *        consumers with different color management schemes, and will
         *        also set the version to 1.3.
         *
         */
        virtual void setVersion(ePDFVersion version) = 0;
        
        /**
         * @brief Set the Producer name for the output PDF.
         *
         *        Equivalent to calling setParameter with the parameter
         *        name "Producer" with the value as a string.
         *
         *        The default is "Mako VX.X.X"
         *
         *        Note that this will also be used for the PDF's Creator
         *        if the metadata for the document being written does not
         *        specify a Creator.
         *
         *        If an empty string is provided, the default will be used.
         */
        virtual void setProducer(const U8String &producer) = 0;
         
        /**
         * @brief Set whether or not incremental output should be enabled.
         *
         *        The default is false (that is, a full save is performed).
         *        Incremental output is only possible when all of the following
         *        are true:
         *        - writeAssembly() is used (rather than the IOutputWriter
         *          interface,
         *        - The assembly has only one document,
         *        - The assembly was opened from a PDF, and
         *        - The PDF was either a file or opened from a random-access
         *          stream.
         *        - The PDF version specified is the same version or higher than
         *          the input PDF.
         *
         *        If all are true, only modified content is written and appended
         *        to the output (actually the source PDF is copied, and then the
         *        new content is appended to the copied PDF). This is usually much
         *        faster than a full save when the assembly has only been lightly
         *        modified. Note that the original PDF will be recoverable and so
         *        this should not be used when redacting for example.
         *
         *        Otherwise, a regular full save is performed.
         *
         *        Equivalent to calling setParameter with the parameter name
         *        "IncrementalOutput".
         *
         *        For PDF/X and PDF/A, this setting is ignored.
         */
        virtual void setEnableIncrementalOutput(bool enable = true) = 0;
        
        /**
         * @brief Set the encryption for the output PDF.
         *        The default is no encryption.
         *
         *        Please note:
         *
         *        - Setting a key length of zero disables encryption.
         *        - If encryption is enabled, the encryption algorithm will be
         *          chosen based on the output PDF version. Not all key lengths are
         *          available for all versions, and an exception will be thrown if
         *          an unavailable key length is chosen.
         *        - If encryption is enabled, at least one password must be supplied.
         *          If only a user password is supplied, it will also be used for
         *          the owner password.
         *        - Permissions are only used if encryption is enabled.
         *          That is, permissions information is only stored if
         *          the PDF is to be encrypted.
         *        - If 40 bit encryption is used, then encryptMetadata must be true,
         *          or an exception will result.
         *        - Some permissions flags are ignored for 40 bit key lengths.
         *          For 40 bit output, eFillFormAllowed, eExtractionAllowed,
         *          eAssemblyAllowed and eHighQualityPrintAllowed are ignored.
         *        - If encryption is enabled, incremental output is disabled.
         *
         *        Equivalent to calling setParameter with the parameter names
         *        "KeyLength", "Permissions", "EncryptMetadata", "OwnerPassword"
         *        and "UserPassword".
         *
         *        For PDF/A and PDF/X, encryption is not allowed and an exception
         *        will be thrown if output is attempted with Encryption enabled.
         *
         * @param keyLength The key length to use. Must be 0, 40, 128 or 256.
         *        0 indicates that encryption will not be performed.
         * @param permissions The permissions bit mask for the output PDF.
         *        (see IDOMStandardPDFSecurityInfo::ePermissionsFlags).
         * @param encryptMetadata Whether or not to encrypt metadata.
         *        The default is false. Must be set to true if a 40 bit key length
         *        is specified.
         * @param ownerPassword The owner password to use.
         * @param userPassword The user password to use.
         */
        virtual void setEncryption(uint32 keyLength, uint32 permissions, bool encryptMetadata = false,
                                   const U8String &ownerPassword = "", const U8String &userPassword = "") = 0;
        
        /**
         * @brief  Set the target color space for the output.
         *
         *         The default is DeviceRGB.
         *
         *         By default, the target color space is used if anything must be rendered
         *         or color-converted. The default behaviour is to apply this only when
         *         required. Use convertAllColors() if converting everything is required.
         *
         *         Equivalent to calling setParameter with the param name "TargetColorSpace"
         *         with appropriate values (please refer to documentation).
         *
         *         For PDF/A-1b output, an OutputIntents dictionary will be created for
         *         this color space. LAB color spaces may not be used as a destination
         *         color space for PDF/A.
         *
         *         For PDF/X-1a output, only DeviceCMYK is supported; this setting is ignored.
         *         If you wish to change the profile to be used, please set the DeviceCMYK
         *         intercept in the color manager to your desired profile (see
         *         IColorManager::setDeviceCMYKIntercept()). As for PDF/A-1b an OutputIntents
         *         dictionary will be created.
         *
         *         For PDF/X-4 and PDF/A-2b output, an OutputIntents dictionary will be
         *         created for this color space. LAB colour spaces may not be used as a
         *         destination color space for PDF/X-4 or PDF/A-2. Please note that content
         *         using a device space with the same number of components as the output intent
         *         will be assumed to be compatible with this space. For example, if a CMYK
         *         ICCBased colour space is used for the target space, then any DeviceCMYK
         *         content will remain as DeviceCMYK in the output even if the current
         *         DeviceCMYK intercept is set to some other profile. Note also that if the
         *         output intent is a CMYK color space, then DeviceGray similarly will also
         *         be considered equivalent to the K channel of the output intent.
         *
         *         Additionally for PDF/A-2b output, it is possible to set a NULL output
         *         intent, in which case no OutputIntent will be produced.
         *
         * @param  targetSpace The desired color space. Must be a simple or ICC space. scRGB is not supported for output.
         */
        virtual void setTargetColorSpace(const IDOMColorSpacePtr &targetSpace) = 0;
        
        /**
         * @brief  Set the target color space for the output using an ICC profile.
         *
         *         By default, the target color space is used if anything must be rendered
         *         or color-converted. The default behaviour is to apply this only when
         *         required. Use convertAllColors() if converting everything is required.
         *
         *         Equivalent to calling setParameter() with the param name "TargetProfile"
         *         with the value as the path to the profile.
         *
         *         For PDF/A-1b, PDF/X-1a and PDF/X-4 output, the restrictions described in
         *         setTargetColorSpace() above apply.
         *
         * @param  profile The desired profile.
         */
        virtual void setTargetProfile(const IDOMICCProfilePtr &profile) = 0;
        
        /**
         * @brief  Set an explicit output intent, or NULL to clear it.
         *
         *         Note: This is only currently honoured for PDF/X-4 and PDF/A-2b output.
         *         Please note that as a result of this, PDF 2.0 features are ignored.
         *
         *         For PDF/X-4 and PDF/A-2b the output intent must provide a profile.
         *         The target color space is set to this profile.
         *
         *         For PDF/A-2b setting a NULL output intent causes no output intent to
         *         be emitted.
         */
        virtual void setOutputIntent(const IOutputIntentPtr &outputIntent) = 0;

        /**
         * @brief  Set whether or not all content should be color converted to the
         *         target space set by setTargetColorSpace().
         *
         *         The default is false.
         *
         *         This parameter is ignored and assumed true when writing to PDF/A-1b
         *         or PDF/X-1a format output.
         *
         *         Note that if incremental output is used, the content of unedited
         *         pages is not colour converted.
         *
         *         Equivalent to calling setParameter() with the param name "ConvertAllColors"
         *         with a boolean string ("true" or "false").
         */
        virtual void setConvertAllColors(bool convert) = 0;
        
        /**
         * @brief  Set whether or not gray colors should be subject to color conversion.
         *         That is, colors using DeviceGray, sGray, or single-component ICC
         *         colorspaces.
         *
         *         The default is true.
         *
         *         This parameter is ignored unless setConvertAllColors() is true.
         *
         *         For PDF/X output, this parameter only applies to objects using
         *         the DeviceGray colour space. All other gray spaces are subject to
         *         the setting of setConvertAllColors().
         *
         *         Note that if incremental output is used, the content of unedited
         *         pages is not colour converted.
         *
         *         Equivalent to calling setParameter() with the param name "ConvertGray"
         *         with a boolean string ("true" or "false").
         */
        virtual void setConvertGray(bool convert) = 0;
        
        /**
         * @brief  Set the desired maximum resolution, threshold and downsampling method
         *         for colour images.
         *
         *         Note that if incremental output is used, the content of unedited
         *         pages is not affected.
         *
         *         The default behaviour is leave the image resolution unchanged.
         *         Equivalent to calling setParameter() with the param names
         *         "ColorImageDownsamplingResolution", "ColorImageDownsamplingThreshold"
         *         and "ColorImageDownsamplingMethod" with the respective values.
         *
         * @param  resolution The desired output resolution, in dpi. Pass 0 to leave
         *         images unchanged, which is the default.
         * @param  threshold The threshold above which images will be reduced to the
         *         desired resolution. Pass 0 to use the resolution.
         * @param  method The method to use when downsampling. The default is bicubic
         *         for colour images.
         */
        virtual void setColorImageMaxResolution(float resolution, float threshold = 0.0f, IDOMImageDownsamplerFilter::eDownsamplingMethod method = IDOMImageDownsamplerFilter::eBicubic) = 0;
        
        /**
         * @brief  Set the desired maximum resolution, threshold and downsampling method
         *         for gray images.
         *
         *
         *         Note that if incremental output is used, the content of unedited
         *         pages is not affected.
         *
         *         The default behaviour is leave the image resolution unchanged.
         *         Equivalent to calling setParameter() with the param names
         *         "GrayImageDownsamplingResolution", "GrayImageDownsamplingThreshold"
         *         and "GrayImageDownsamplingMethod" with the respective values.
         *
         * @param  resolution The desired output resolution, in dpi. Pass 0 to leave
         *         images unchanged, which is the default.
         * @param  threshold The threshold above which images will be reduced to the
         *         desired resolution. Pass 0 to use the resolution.
         * @param  method The method to use when downsampling. The default is bicubic
         *         for gray images.
         */
        virtual void setGrayImageMaxResolution(float resolution, float threshold = 0.0f, IDOMImageDownsamplerFilter::eDownsamplingMethod method = IDOMImageDownsamplerFilter::eBicubic) = 0;
        
        /**
         * @brief  Set the desired maximum resolution, threshold and downsampling method
         *         for monochrome images.
         *
         *         Note that if incremental output is used, the content of unedited
         *         pages is not affected.
         *
         *         The default behaviour is leave the image resolution unchanged.
         *         Equivalent to calling setParameter() with the param names
         *         "MonoImageDownsamplingResolution", "MonoImageDownsamplingThreshold"
         *         and "MonoImageDownsamplingMethod" with the respective values.
         *
         * @param  resolution The desired output resolution, in dpi. Pass 0 to leave
         *         images unchanged, which is the default.
         * @param  threshold The threshold above which images will be reduced to the
         *         desired resolution. Pass 0 to use the resolution.
         * @param  method The method to use when downsampling. The default is subsample
         *         for monochrome images; using any other method will result in grayscale
         *         output.
         */
        virtual void setMonoImageMaxResolution(float resolution, float threshold = 0.0f, IDOMImageDownsamplerFilter::eDownsamplingMethod method = IDOMImageDownsamplerFilter::eSubsample) = 0;
        
        /**
         * @brief Set whether or not to downsample masked images
         *
         *        This applies to cases where an image is masked by a separate
         *        masked image, such as types of PDF masked or soft-masked images.
         *        These are represented in the DOM using IDOMMaskedBrush, where the
         *        sub-brush is an image.
         *
         *        Has no effect if downsampling is not enabled
         *
         *        The default is true.
         *
         *        Equivalent to calling setParameter() with the param names
         *        "DownsampleMaskedImages" with the values "true" or "false".

         */
        virtual void setDownsampleMaskedImages(bool downsampleMaskedImages) = 0;
        
        /**
         * @brief Set whether or not the mask resolution setting should be applied
         *        to the image portion of a masked image.
         *
         *        This applies to cases where an image is masked by a separate
         *        masked image, such as types of PDF masked or soft-masked images.
         *        These are represented in the DOM using IDOMMaskedBrush, where the
         *        sub-brush is an image.
         *
         *        If false, then the mask and the image are evaluated separately and
         *        a downsampling resolution and threshold are chosen. For the mask,
         *        this is normally either grayscale or monochrome. The image data
         *        can be anything. In this mode it is possible for the downsampled
         *        image and mask to be downsampled to different resolutions.
         *
         *        If true, then whatever target resolution and threshold is applied
         *        to the mask will also be applied to the image samples. If these
         *        images have the same effective resolution before downsampling,
         *        then they will also share the same effective resolution after
         *        downsampling.
         *
         *        Has no effect if downsampling is not enabled
         *
         *        The default is true.
         *
         *        Equivalent to calling setParameter() with the param names
         *        "UseMaskResolutionForMaskedImages" with the values "true" or "false".
         */
        virtual void setUseMaskResolutionForMaskedImages(bool useMaskResolutionSettingForMaskedImages) = 0;

        /**
         * @brief  Set the resolution to use if page content requires rendering in
         *         order to be output.
         *         The default is 300dpi. This is affected also by the maximum image
         *         resolution parameters.
         *         Equivalent to calling setParameter() with param name "RenderResolution"
         *         with the value as the desired resolution as value.
         * @param  resolution The desired resolution in dpi.
         */
        virtual void setRenderResolution(uint32 resolution) = 0;
        
        /**
         * @brief  Enumeration for image compression formats.
         */
        typedef enum
        {
            eICAuto,
            eICFlate,
            eICDCT, ///< Jpeg
            eICLZW,
            eICRLE, ///< Run length encoding
            eICCCITT,
        } eImageCompression;
        
        /**
         * @brief  Set the desired image compression for color images that need to be
         *         reencoded. The default is eICAuto.
         *
         *         CCITT is not allowed for colour images.
         *
         *         Note: this is advisory only and may not be honoured in all
         *         cases.
         *
         *         Note also that if incremental output is used, the content of unedited
         *         pages is not affected. For example, PDF/A does not allow LZW.
         *
         *         Equivalent to calling setParameter() with the parameter name
         *         "ColorImageCompression" with appropriate values.
         * @param  compression The desired compression.
         */
        virtual void setPreferredColorImageCompression(eImageCompression compression) = 0;
        
        /**
         * @brief  Set the desired image compression for gray images that need to be
         *         reencoded. The default is eICAuto.
         *
         *         CCITT is not allowed for gray images.
         *
         *         Note: this is advisory only and may not be honoured in all
         *         cases. For example, PDF/A does not allow LZW.
         *
         *         Note also that if incremental output is used, the content of unedited
         *         pages is not affected.
         *
         *         Equivalent to calling setParameter() with the parameter name
         *         "GrayImageCompression" with appropriate values (please refer to
         *         documentation).
         * @param  compression The desired compression.
         */
        virtual void setPreferredGrayImageCompression(eImageCompression compression) = 0;
        
        /**
         * @brief  Set the desired image compression for monochrome images that need to be
         *         reencoded. The default is eICCCITT. JPEG/DCT is not allowed for mono
         *         images, and auto will be set to CCITT.
         *
         *         Note: this is advisory only and may not be honoured in all
         *         cases. For example, PDF/A does not allow LZW.
         *
         *         Note also that if incremental output is used, the content of unedited
         *         pages is not affected.
         *
         *         Equivalent to calling setParameter() with the parameter name
         *         "MonoImageCompression" with appropriate values (please refer to
         *         documentation).
         * @param  compression The desired compression.
         */
        virtual void setPreferredMonoImageCompression(eImageCompression compression) = 0;
        
        /**
         * @brief  Set the desired image compression for images that are the result of rendering
         *         The default is eICFlate. If set to auto, flate will be used.
         *
         *         Note: this is advisory only and may not be honoured in all
         *         cases. For example, PDF/A does not allow LZW.
         *
         *         Note also that if incremental output is used, the content of unedited
         *         pages is not affected.
         *
         *         Equivalent to calling setParameter() with the parameter name
         *         "RenderedImageCompression" with appropriate values (please refer to
         *         documentation).
         * @param  compression The desired compression.
         */
        virtual void setPreferredRenderedImageCompression(eImageCompression compression) = 0;
        
        /**
         * @brief  Set the JPEG quality to use when compressing images in DCT format.
         *         Equivalent to calling setParameter() with the parameter name
         *         "JPEGQuality" and the value being the required quality.
         *         The default is 5 - highest quality.
         * @param  quality The desired quality level, with 1 being lowest quality
         *         and 5 being highest quality.
         */
        virtual void setJPEGQuality(uint8 quality) = 0;
        
        /**
         * @brief  Set whether images should be reencoded.
         *
         *         PDF Output will attempt to embed images in the output PDF as is,
         *         if possible. To force recompression, set to true.
         *
         *         The default is false.
         *
         *         Note that if incremental output is used, the content of unedited
         *         pages is not affected.
         */
        virtual void setReencodeImages(bool reencode) = 0;
        
        /**
         * @brief  Set whether or not compression should be applied to page content.
         *         The default is true.
         *
         *         Note that if incremental output is used, the content of unedited
         *         pages is not affected.
         *
         *         Equivalent to calling setParameter() with the parameter name
         *         "CompressPages" with a boolean string ("true" or "false").
         */
        virtual void setCompressPages(bool compressPages) = 0;
        
        /**
         * @brief  Set whether or not to compress individual objects.
         *
         *         In addition to the compression of individual objects, setting
         *         this to true allows the creation of cross reference streams, which
         *         allow the generation of PDF files larger than 9999999999 (~9.3GB).
         *
         *         This setting is advisory only. In particular:
         *
         *         - If performing incremental update, the cross reference format
         *           of the original PDF will be retained.
         *         - If writing to PDF 1.4 or earlier, object compression will not be
         *           used at all.
         *
         *         The default is true.
         */
        virtual void setCompressObjects(bool compressObjects) = 0;
        
        /**
         * @brief  Set whether or not pages should be automatically rotated.
         *         The default is false.
         *
         *         Note that if incremental output is used, the content of unedited
         *         pages is not affected.
         *
         *         If true, pages are automatically rotated based on the prevalent
         *         text direction on the page. If incremental output is used,
         *         non-modified pages will not be subject to this setting.
         *         Equivalent to calling setParameter() with the parameter name
         *         "AutoRotatePages" with a boolean string ("true" or "false").
         *
         *         The rotation is not a physical rotation of content, merely the
         *         PDF Page's view rotate is set appropriately. So, in PDF viewers
         *         all annotations and page content will continue to appear
         *         correctly.
         */
        virtual void setAutoRotatePages(bool autoRotate) = 0;
        
        /**
         * @brief  Set whether embedded fonts should be subset or not.
         *         The default is true.
         *
         *         This is a preference only. Some font types may require subsetting
         *         based on context.
         *
         *         Note that if incremental output is used, the content of unedited
         *         pages is not affected.
         *
         *         Equivalent to calling setParameter() with the parameter name
         *         "SubsetFonts" with a boolean string ("true" or "false").
         */
        virtual void setSubsetFonts(bool subset) = 0;
        
        /**
         * @brief  Set whether fonts should be forcibly embedded. If true, this 
         *         overrides the embedding settings in each IDOMFontOpenType. The
         *         standard 14 fonts are not affected by this; use setEmbedBaseFonts() 
         *         instead.
         *
         *         Note that if incremental output is used, the content of unedited
         *         pages is not affected.
         *
         *         The default is false. This setting is ignored for PDF/A and PDF/X
         *         where it is forced to true.
         *
         *         Equivalent to calling setParameter() with the parameter name
         *         "EmbedFonts" with a boolean string ("true" or "false").
         */
        virtual void setEmbedFonts(bool embed) = 0;
        
        /**
         * @brief  Set whether the base 14 PDF fonts should be forcibly embedded.
         *         If true, this overrides the embedding settings in IDOMFontOpenType.
         *         Only the standard 14 fonts are not affected by this; use setEmbedFonts()
         *         to affect other fonts.
         *
         *         Note that if incremental output is used, the content of unedited
         *         pages is not affected.
         *
         *         The default is false. This setting is ignored for PDF/A and PDF/X
         *         where it is forced to true.
         *
         *         Equivalent to calling setParameter() with the parameter name
         *         "EmbedBase14Fonts" with a boolean string ("true" or "false").
         */
        virtual void setEmbedBase14Fonts(bool embed) = 0;
        
        /**
         * @brief  Provide a list of name of fonts that should be always embedded,
         *         regardless of the settings for setEmbedFonts() or setEmbedBase14Fonts().
         *         This setting is ignored for PDF/A.
         *
         *         Note that if incremental output is used, the content of unedited
         *         pages is not affected.
         *
         *         Equivalent to calling setParameter() with the parameter name
         *         "AlwaysEmbed" and a string with the fonts delimited with semicolons
         *         (backslashes are used as an escape character).
         */
        virtual void setAlwaysEmbedFonts(CU8StringVect &fontNames) = 0;
        
        /**
         * @brief  Provide a list of name of fonts that should never be embedded,
         *         regardless of the settings for setEmbedFonts() or setEmbedBase14Fonts().
         *         This setting is ignored for PDF/A.
         *
         *         Equivalent to calling setParameter() with the parameter name
         *         "AlwaysEmbed" and a string with the fonts delimited with semicolons
         *         (backslashes are used as an escape character).
         */
        virtual void setNeverEmbedFonts(CU8StringVect &fontNames) = 0;
        
        /**
         * @brief  Allow restricted fonts to be embedded in the output.
         *
         *         If true, fonts that flag that they should not be embedded
         *         will be allowed to be embedded. If false, then such fonts
         *         will not be embedded.
         *
         *         Set this to true if you have sufficient permissions to do so.
         *
         *         Note that for PDF/X-4 and PDF/A-2b output, the output will instead
         *         fail if this setting is false and a restricted font is encountered.
         *
         *         The default is false.
         *
         *         Equivalent to calling setParameter() with the parameter name
         *         "AllowRestrictedFonts" and the value "true" or "false".
         */
        virtual void setAllowRestrictedFonts(bool allowRestrictedFonts) = 0;

        /**
         * @brief  Flags controlling PDF/X-4 and PDF/A-2 DeviceN handling
         *         Allows some control on what the PDF writer is allowed
         *         to invent, or what inconsistencies are considered safe
         *         to ignore.
         *
         *         Should an exception need to be raised, an IError with code
         *         EDL_ERR_INCOMPATIBLE_PDFX or EDL_ERR_INCOMPATIBLE_PDFA
         *         (depending on output version) will be thrown, where the error
         *         description will contain a short message outlining the problem
         *         that was encountered.
         */
        typedef enum
        {
            ePDNHDefault = 0,                                             ///< Default; make a best effort attempt to produce
                                                                          //   conformant output
            ePDNHFailOnMissingColorantInformation = 0x1,                  ///< Throw an exception if a spot colorant has
                                                                          //   missing colorant or tint transform information
            ePDNHFailOnInconsistentColorantInformation = 0x2,             ///< Throw an exception if inconsistent spot colorant
                                                                          //   information is found
            ePDNHFailIfBetterColorantInformationFoundAfterFirstUse = 0x4, ///< Throw an exception if better spot colorant
                                                                          //   information is encountered after a color space
                                                                          //   has already been written using that colorant
            ePDNHFailIfDotGainFound = 0x8,                                ///< Throw an exception if a dotgain is encountered
                                                                          //   (these are forbidden for PDF/X-4). Normal behaviour
                                                                          //   is to drop DotGain entries.
            ePDNFailOnMismatchedDeviceNProcessSpace = 0x10,               ///< Throw an exception if a DeviceN Process Property does
                                                                          //   not match the output intent. If this flag is not set,
                                                                          //   any DeviceN Process space found will be replaced
                                                                          //   with the device space corresponding to the output
                                                                          //   intent. For PDF/A-2 output
            
        } ePdfDeviceNHandling;

        /**
         * @brief  Set flags controlling PDF/X-4 and PDF/A-2 DeviceN handling.
         *         See #ePdfDeviceNHandling# for details.
         *
         *         Equivalent to calling setParameter() with the parameter name
         *         "DeviceNErrorHandling" and an unsigned integer string
         *         containing the flags value.
         *
         *         Ignored if the output version is anything other than PDF/A-2b or PDF/X-4.
         */
        virtual void setDeviceNErrorHandling(uint32 flags) = 0;
        
        /**
         * @brief  Flags controlling PDF/X-4 and PDF/A-2 Optional content
         *         Allows some control on what the PDF writer is allowed
         *         to invent, or what inconsistencies are considered safe
         *         to ignore.
         *
         *         Should an exception need to be raised, an IError with code
         *         EDL_ERR_INCOMPATIBLE_PDFX or EDL_ERR_INCOMPATIBLE_PDFA
         *         (depending on output version) will be thrown, where the error
         *         description will contain a short message outlining the problem
         *         that was encountered.
         */
        typedef enum
        {
            ePOCHDefault = 0,                               ///< Default; make a best effort attempt to produce
                                                            //   conformant output
            ePOCHFailOnInconsistentConfigurationName = 0x1, ///< Throw an exception if an optional content configuration
                                                            //   either has no name or whose name clashes with another
                                                            //   configuration. If not set, configurations will be named
                                                            //   with a default value or edited on a clash
            ePOCHFailOnInvalidOrder = 0x2,                  ///< Throw if the Optional Content order information, if present
                                                            //   present, does not contain all the groups. If not set, any
                                                            //   groups missing from the order will be added to the end, in
                                                            //   a sub-group labelled "Additional Layers".
            ePOCHFailIfAutoStateFound = 0x4,                ///< Throw an exception if an optional content configuration contains
                                                            //   one or more Auto States. These are prohibited for PDF/X-4 and
                                                            //   PDF/A-2. If this flag is not set the auto state entries will
                                                            //   simply be dropped.
        } ePdfOptionalContentHandling;
        
        /**
         * @brief  Set flags controlling PDF/X-4 and PDF/A-2 Optional Content handling.
         *         See #ePdfOptionalContentHandling# for details.
         *
         *         Equivalent to calling setParameter() with the parameter name
         *         "OptionalContentErrorHandling" and an unsigned integer string
         *         containing the flags value.
         */
        virtual void setOptionalContentErrorHandling(uint32 flags) = 0;

        /**
         * @brief  Flags controlling PDF/X-4 and PDF/A-2 Extended Graphics State information
         *         Allows some control on what the PDF writer is allowed
         *         to invent, or what inconsistencies are considered safe
         *         to ignore.
         *
         *         Should an exception need to be raised, an IError with code
         *         EDL_ERR_INCOMPATIBLE_PDFX or EDL_ERR_INCOMPATIBLE_PDFA
         *         (depending on output version) will be thrown, where the error
         *         description will contain a short message outlining the problem
         *         that was encountered.
         */
        typedef enum
        {
            ePEGSHDefault = 0,                      ///< Default; make a best effort attempt to produce
                                                     //   conformant output
            ePEGSHFailOnTransferFunction = 0x1,     ///< Throw an exception if a non-default transfer function is encountered.
                                                     //    If not set, transfer functions will be silently dropped.
        } ePdfExtendedGraphicsStateHandling;
        
        /**
         * @brief  Set flags controlling PDF/X-4 and PDF/A-2 Extended Graphics State handling.
         *         See #ePdfExtendedGraphicsStateHandling# for details.
         *
         *         Equivalent to calling setParameter() with the parameter name
         *         "ExtendedGraphicsStateErrorHandling" and an unsigned integer string
         *         containing the flags value.
         */
        virtual void setExtendedGraphicsStateErrorHandling(uint32 flags) = 0;
        
        /**
         * @brief  Flags controlling PDF/X-4 and PDF/A-2 Alternate image handling
         *         Allows control on what the PDF/X writer is allowed
         *         to invent, or what inconsistencies are considered safe
         *         to ignore.
         *
         *         For PDF/X-4, Alternate images are allowed, but no alternate image
         *         is allowed to specify that it is the default for printing (via the
         *         DefaultForPrinting key). By default, if an image with
         *         DefaultForPrinting set to true is present, then this image will be
         *         selected and the other alternate images shall be dropped.
         *
         *         For PDF/A-2b, Alternate images are not allowed at all. By default, we
         *         will apply the optional content rules for alternate images to select
         *         a single image as if the PDF was being printed.
         *
         *         Instead of this default, the following flags allow processing to stop
         *         in these situations.
         *
         *         Should an exception need to be raised, an IError with code
         *         EDL_ERR_INCOMPATIBLE_PDFX or EDL_ERR_INCOMPATIBLE_PDFA
         *         (depending on output version) will be thrown, where the error
         *         description will contain a short message outlining the problem
         *         that was encountered.
         */
        typedef enum
        {
            ePAIHDefault = 0,                         ///< Default; see the comment above
            ePAIHFailIfAlternatesFound = 0x1,         ///< Throw an exception if an alternate image is encountered
            ePAIHFailIfDefaultForPrintingFound = 0x2, ///< Throw an exception if an alternate image marked as default for printing is encountered
        } ePdfAlternateImageHandling;
        
        /**
         * @brief  Set flags controlling PDF/X-4 and PDF/A-2b alternate image handling
         *         See #ePdfAlternateImageHandling# for details.
         *
         *         Equivalent to calling setParameter() with the parameter name
         *         "AlternateImageErrorHandling" and an unsigned integer string
         *         containing the flags value.
         */
        virtual void setAlternateImageErrorHandling(uint32 flags) = 0;

        /**
         * @brief A CJK character set.
         */

        typedef enum
        {
            eFCSJapanese,
            eFCSKorean,
            eFCSSimplifiedChinese,
            eFCSTraditionalChinese,
        } eFormCJKCharacterSet;
        
        /**
         * @brief Set the default CJK character set for forms.
         *        Used when generating form appearance streams, and is only
         *        used when the character set cannot be determined from the content
         *        of the form. The default is Japanese.
         */
        virtual void setDefaultFormCJKLanguage(eFormCJKCharacterSet characterSet) = 0;

		/**
		 * @brief  Enable the use of a True Type font's /.notdef glyph in the output.
		 *
		 *         If true, a character that does not have a glyph or that has been linked to the /.notdef glyph
		 *         will be replaced by the /.notdef glyph.
		 *         
		 *         If false, a character that does not have a glyph or that has been linked to the /.notdef glyph
		 *         will be ignored.
		 *
		 *         Note that for PDF/X-4 and PDF/A-2b output, the flag is set to true anyway.
		 *
		 *         The default is false.
		 *
		 *         Equivalent to calling setParameter() with the parameter name
		 *         "EnableTrueTypeNotDef" and the value "true" or "false".
		 */
		virtual void setEnableTrueTypeNotDef(bool enableTrueTypeNotDef) = 0;

        /**
         * @brief Allow embedded files to be retained in PDF/A and PDF/X-4 output.
         *
         *        Applies only to PDF/A and PDF/X-4.  If true, any embedded files
         *        will be retained in the output.  If false embedded files will be
         *        dropped.
         *
         *        The default is false.
         *
         *        Equivalent to calling setParameter() with the parameter name
         *        "RetainEmbeddedFiles" and the value "true" or "false".
         */
        virtual void setRetainEmbeddedFiles(bool retainEmbeddedFiles) = 0;
    };
    #define obj2IPDFOutput(obj) IPDFOutputPtr(dynamic_cast<IPDFOutput *>((IRCObject *) obj), true)
}

#endif

