/*
 * Copyright (C) 2019 Global Graphics Software Ltd. All rights reserved.
 */

#ifndef JAWSMAKO_PJL_H
#define JAWSMAKO_PJL_H

#include <jawsmako/types.h>

/**
 * @file     pjl.h
 * @ingroup  JawsMako
 * @brief    A PJL (Printer Job Language) parser for Mako
 *
 * This header describes the IPJPParser interface.
 */

#include <jawsmako/jawsmako.h>

namespace JawsMako
{
    using namespace EDL;
    
    /**
     * @ingroup  JawsMako
     * @class    IPJLParser
     * @brief    An instance of the Mako PJL Parser
     */
    class IPJLParser : public IRCObject
    {
      public:
        virtual ~IPJLParser() {}
        
        /**
         * @brief   Create a PJL parser instance
         *
         * @return  IPJLParserPtr the PCL/XL input
         */
        static JAWSMAKO_API IPJLParserPtr create(const IJawsMakoPtr &jawsMako);
        
        /**
         * @brief    Clone the parser and the PJL environment
         */
        virtual IPJLParserPtr clone() = 0;
        
        /**
         * @brief    Result code for PJL parsing.
         */
        typedef enum
        {
            ePREnterPclXl,
            ePREnterPcl,
            ePREndOfFile, // An end of file was encountered.
        } ePjlResult;
        
        /**
         * @brief   Parse from the given stream, until another language is encountered
         *          or the end of the stream is reached.
         *
         *          Exceptions of type IError are thrown on error.
         *
         * @param   stream The stream to parse. This must be a pushback-capable stream, as the
         *          parser must be able to sniff ahead. The stream must be open; the parser
         *          will neither open nor close the stream.
         * @return  ePjlResult The result of parsing.
         */
        virtual ePjlResult parse(const IInputPushbackStreamPtr &stream) = 0;
        
        /**
         * @brief   A captured PJL attribute.
         */
        class CPjlAttributeValue
        {
          public:
            RawString modifier; /**<  The parsed PJL modifier, if present.
                                 *    Always presented as upper case without spaces.
                                 */
            RawString key;      /**<  The parsed PJL key, if present.
                                 *    Always presented as upper case.
                                 */
            RawString value;    /**<  The PJL value, if present. Unmodified. */
        };
        typedef CEDLVector<CPjlAttributeValue> CPjlAttributeVect;

        /**
         * @brief   Retrieves all attributes set with the given PJL command and key.
         *
         *          If the key is empty, all attributes set with the given command
         *          will be returned.
         */
        virtual CPjlAttributeVect getAttributes(const RawString &command, const RawString &key = RawString()) = 0;

        /*
         *          Functions to manipulate the PJL environment
         */
        
        /**
         * @brief    Set the default paper size in the PJL Environment.
         *           Equivalent to having parsed "@PJL SET PAPER = <papersize>"
         */
        virtual void setDefaultPaperSize(const U8String &paperSize) = 0;
        
        /**
         * @brief    Set the default orientation in the PJL Environment.
         *           Equivalent to having parsed "@PJL SET ORIENTATION = <PORTRAIT or LANDSCAPE>"
         */
        virtual void setDefaultLandscape(bool defaultLandscape) = 0;
        
        /**
         * @brief    Set the default number of copies in the PJL Environment.
         *           Equivalent to having parsed "@PJL SET COPIES = <default copies>"
         */
        virtual void setDefaultCopies(uint32 defaultCopies) = 0;
        
        /**
         * @brief    Set the default duplex in the PJL Environment.
         *           Equivalent to having parsed "@PJL SET DUPLEX = <ON or OFF>"
         */
        virtual void setDefaultDuplex(bool defaultDuplex) = 0;
        
        /**
         * @brief    Duplex binding mode enumeration
         */
        typedef enum
        {
            eDBMLongEdge,
            eDBMShortEdge,
        } eDuplexBindingMode;
        
        /**
         * @brief    Set the default binding mode in the PJL Environment.
         *           Equivalent to having parsed "@PJL SET BINDING = <LONGEDGE or SHORTEDGE>"
         */
        virtual void setDefaultDuplexBindingMode(eDuplexBindingMode defaultDuplexBinding) = 0;
        
        /**
         * @brief    Set the default manual feed mode in the PJL Environment.
         *           Equivalent to having parsed "@PJL SET MANUALFEED = <ON or OFF>"
         */
        virtual void setDefaultManualFeed(bool defaultManualFeed) = 0;
    };
    #define obj2IPJLParser(obj) IPJLParserPtr(dynamic_cast<IPJLParser *>((IRCObject *) obj), true)
}

#endif
