/*
* Copyright (C) 2014-2015 Global Graphics Software Ltd. All rights reserved.
*/
 
#ifndef _JAWSMAKO_XPSINPUT_H_
#define _JAWSMAKO_XPSINPUT_H_

/**
* @file     XPSinput.h
* @ingroup  JawsMako
* @brief    XPS Input-Specific Features.
*
* This header describes the IXPSInput interface.
* These are a subclass of IInput. Instances of this type are returned by
* IInput::create() when eFFXPS is specified. This allows access to XPS
* specific input features and modes of operation.
*/

#include <jawsmako/jawsmako.h>

namespace JawsMako
{
    using namespace EDL;

    class IXPSInput;
    DECL_SMART_PTR(IXPSInput);

    /**
    * @class    IXPSInput
    * @brief    An instance of the JawsMako XPS input class.
    */
    class IXPSInput : public IInput
    {
      public:
        virtual ~IXPSInput() {}

        /**
         * @brief   Create an input for reading source documents in XPS format.
         *
         * @return  IXPSInputPtr the xps input
         */
        static JAWSMAKO_API IXPSInputPtr create(const IJawsMakoPtr &jawsMako, const IAbortPtr &abort = IAbortPtr ());
        
        /**
         * @brief    Open a stream, returning the IDocumentAssembly representing the
         *           contents, in streaming mode.
         *
         *           This is designed to support cases where we wish to begin working
         *           with the XPS before the entire stream has been received, such as
         *           within a Windows XPS print driver or if the XPS is being
         *           downloaded from a remote server.
         *
         *           The assembly returned by this function is opened once the most
         *           basic parts of the XPS stream have become available; for a
         *           properly interleaved XPS stream this should not required much
         *           data to be received.
         *
         *           Access to the assembly can still be in a random fashion, but a
         *           request for a document or page will block until that document
         *           becomes available in the incoming stream.
         *
         *           Note that there are several operations that will cause the
         *           a wait until the entire XPS input has been received.
         *           These include:
         *           - Document manipulation (any attempt to insert or remove a document)
         *           - Page manipulation (any attempt to insert or remove a page)
         *           - Requesting the number of pages or documents.
         *           - Searching for a target in a document or assembly.
         *
         *           Instead of requesting the number of pages or documents, it is
         *           reccommended that the documents and pages are requested
         *           sequentially, and if a page or document is not available an
         *           IError exception with error code JM_ERR_PAGE_NOT_FOUND or
         *           JM_ERR_DOCUMENT_NOT_FOUND will be thrown.
         *           These can be caught and appropriately handled.
         *
         * @return   IDocumentAssemblyPtr the document assembly.
         */
        virtual IDocumentAssemblyPtr openStreaming(const IInputStreamPtr &inputStream) = 0;

    };
    #define obj2IXPSInput(obj) IXPSOutputPtr(dynamic_cast<IXPSInput *>((IRCObject *) obj), true)
}

#endif
