/*
* Copyright (C) 2013-2015 Global Graphics Software Ltd. All rights reserved.
*/
 
#ifndef _JAWSMAKO_XPSOUTPUT_H_
#define _JAWSMAKO_XPSOUTPUT_H_

/**
* @file     xpsoutput.h
* @ingroup  JawsMako
* @brief    JawsMako XPS Output.
*/

#include <jawsmako/jawsmako.h>
#include <jawsmako/transforms.h>

namespace JawsMako
{
    using namespace EDL;

    class IXPSOutput;
    DECL_SMART_PTR(IXPSOutput);

    /**
    * @ingroup  JawsMako
    * @class    IXPSOutput
    * @brief    Interface for the XPS IOutput class
    */
    class IXPSOutput : public IOutput
    {
      public:
        virtual ~IXPSOutput() {}
        
        /**
        * @brief   Create an XPS Output instance.
        */
        JAWSMAKO_API static IXPSOutputPtr create(const IJawsMakoPtr &jawsMako, const IAbortPtr &abort = IAbortPtr ());
        
        /*
         * XPS Output-specific configuration
         */

        /**
        * @brief   Set whether or not entire XPS parts should be copied from the
        *          original assembly source when writing that assembly if the
        *          part remains unchanged. This is almost always faster.
        *          This defaults to true, and is set to true when using the
        *          "Preserve" preset. However, any change to configuration will
        *          cause this to be set to false in order to force reprocessing
        *          for the configuration to take effect.
        *          However this setting may be set to true after other configuration
        *          has been changed.
        */
        virtual void setCopyExistingXPSPartsWherePossible(bool copy) = 0;
        
        /**
        * @brief   Set whether fonts should be subset in the output.
        *
        *          The default is false. Equivalent to calling setParameter with
        *          "SubsetFonts" as the parameter name.
        *          
        *          Note that all fonts may not be subsetted; for some subsetting may be
        *          forbidden, and for others there may be little gain in subsetting.
        */
        virtual void setSubsetFonts(bool subset) = 0;
        
        /**
        * @brief   Set whether or not an attempt will be made to merge disparate
        *          subsets of a font into a single font.
        * 
        *          The default is false. Equivalent to calling setParameter with
        *          "MergeFonts" as the parameter name.
        *
        *          Some formats such as PostScript and XPS tend to include many
        *          font subsets and for output it is often avantageous to attempt
        *          to merge these fonts into a single font where possible.
        *
        *          Note that it is possible to enable both font subsetting and
        *          merging at the same time. In this case merging happens first,
        *          followed by subsetting of the merged results.
        */
        virtual void setMergeFonts(bool merge) = 0;
        
        /**
        * @brief   Set if the XPS output should attempt to merge adjacent images.
        *          Equivalent to calling setParameter with "MergeAdjacentImages" as the
        *          parameter name.
        *          The default is true.
        */
        virtual void setMergeImages(bool merge = true) = 0;
        
        /**
        * @brief  Set the desired maximum resolution, threshold and downsampling method
        *         for colour images.
        *
        *         The default behaviour is leave the image resolution unchanged.
        *         Equivalent to calling setParameter() with the param names
        *         "ColorImageDownsamplingResolution", "ColorImageDownsamplingThreshold"
        *         and "ColorImageDownsamplingMethod" with the respective values.
        * @param  resolution The desired output resolution, in dpi. Pass 0 to leave
        *         images unchanged, which is the default.
        * @param  threshold The threshold above which images will be reduced to the
        *         desired resolution. Pass 0 to use the resolution.
        * @param  method The method to use when downsampling. The default is bicubic
        *         for colour images.
        */
        virtual void setColorImageMaxResolution(float resolution, float threshold = 0.0f, IDOMImageDownsamplerFilter::eDownsamplingMethod method = IDOMImageDownsamplerFilter::eBicubic) = 0;
        
        /**
        * @brief  Set the desired maximum resolution, threshold and downsampling method
        *         for gray images.
        *
        *         The default behaviour is leave the image resolution unchanged.
        *         Equivalent to calling setParameter() with the param names
        *         "GrayImageDownsamplingResolution", "GrayImageDownsamplingThreshold"
        *         and "GrayImageDownsamplingMethod" with the respective values.
        * @param  resolution The desired output resolution, in dpi. Pass 0 to leave
        *         images unchanged, which is the default.
        * @param  threshold The threshold above which images will be reduced to the
        *         desired resolution. Pass 0 to use the resolution.
        * @param  method The method to use when downsampling. The default is bicubic
        *         for gray images.
        */
        virtual void setGrayImageMaxResolution(float resolution, float threshold = 0.0f, IDOMImageDownsamplerFilter::eDownsamplingMethod method = IDOMImageDownsamplerFilter::eBicubic) = 0;
        
        /**
        * @brief  Set the desired maximum resolution, threshold and downsampling method
        *         for monochrome images.
        *
        *         The default behaviour is leave the image resolution unchanged.
        *         Equivalent to calling setParameter() with the param names
        *         "MonoImageDownsamplingResolution", "MonoImageDownsamplingThreshold"
        *         and "MonoImageDownsamplingMethod" with the respective values.
        * @param  resolution The desired output resolution, in dpi. Pass 0 to leave
        *         images unchanged, which is the default.
        * @param  threshold The threshold above which images will be reduced to the
        *         desired resolution. Pass 0 to use the resolution.
        * @param  method The method to use when downsampling. The default is subsample
        *         for monochrome images; using any other method will result in grayscale
        *         output.
        */
        virtual void setMonoImageMaxResolution(float resolution, float threshold = 0.0f, IDOMImageDownsamplerFilter::eDownsamplingMethod method = IDOMImageDownsamplerFilter::eSubsample) = 0;
        
        /**
        * @brief  Set the target color space for the output.
        *         The default behaviour is to, where possible, leave the color space
        *         of objects unchanged.
        *         Equivalent to calling setParameter with the param name "TargetColorSpace"
        *         with appropriate values (please refer to documentation).
        * @param  colorSpace The desired color space. Must be a simple or ICC space.
        */
        virtual void setTargetColorSpace(const IDOMColorSpacePtr &targetSpace) = 0;
        
        /**
        * @brief  Set the target color space for the output using an ICC profile.
        *         The default behaviour is to, where possible, leave the color space
        *         of objects unchanged.
        *         Equivalent to calling setParameter() with the param name "TargetProfile"
        *         with the value as the path to the profile.
        * @param  profile The desired profile.
        */
        virtual void setTargetProfile(const IDOMICCProfilePtr &profile) = 0;
        
        /**
        * @brief  Apply the given color converter transform to the contents before
        *         writing to XPS. This supersedes the target color space parameters
        *         described above. This allows for more advanced configuration of
        *         the color spaces of the output.
        */
        virtual void applyColorConverterTransform(const IColorConverterTransformPtr &transform) = 0;
        
        /**
        * @brief  Set the resolution to use if page content requires rendering in
        *         order to be output as XPS.
        *         The default is 150dpi. This is affected also by the maximum image
        *         resolution parameters.
        *         Equivalent to calling setParameter() with param name "RenderResolution"
        *         with the value as the desired resolution as value.
        * @param  resolution The desired resolution in dpi.
        */
        virtual void setRenderResolution(uint32 resolution) = 0;
        
        /**
        * @brief  Apply the given renderer transform to the contents before
        *         writing to XPS. This supersedes the target color space parameters
        *         described above. This allows for more advanced configuration of
        *         rendering.
        */
        virtual void applyRendererTransform(const IRendererTransformPtr &transform) = 0;
        
        /**
        * @brief  Set the desired image format for color images that need to be
        *         reencoded for XPS output. The default is eEFAuto.
        *         Note: this is advisory only and may not be honoured in all
        *         cases if the image cannot be represented in XPS in the desired
        *         form.
        *         Equivalent to calling setParameter() with the parameter name
        *         "ColorImageFormat" with appropriate values (please refer to
        *         documentation).
        * @param  format The desired format.
        */
        virtual void setPreferredColorImageFormat(IImageEncoderTransform::eEncodeFormat format) = 0;
        
        /**
        * @brief  Set the desired image format for gray images that need to be
        *         reencoded for XPS output. The default is eEFAuto.
        *         Note: this is advisory only and may not be honoured in all
        *         cases if the image cannot be represented in XPS in the desired
        *         form.
        *         Equivalent to calling setParameter() with the parameter name
        *         "GrayImageFormat" with appropriate values (please refer to
        *         documentation).
        * @param  format The desired format.
        */
        virtual void setPreferredGrayImageFormat(IImageEncoderTransform::eEncodeFormat format) = 0;
        
        /**
        * @brief  Set the desired image format for monochrome images that need to be
        *         reencoded for XPS output. The default is eEFAuto.
        *         Note: this is advisory only and may not be honoured in all
        *         cases if the image cannot be represented in XPS in the desired
        *         form.
        *         Equivalent to calling setParameter() with the parameter name
        *         "MonoImageFormat" with appropriate values (please refer to
        *         documentation).
        * @param  format The desired format.
        */
        virtual void setPreferredMonoImageFormat(IImageEncoderTransform::eEncodeFormat format) = 0;
        
        /**
        * @brief  Set the JPEG quality to use when encoding images in JPEG format.
        *         Equivalent to calling setParameter() with the parameter name
        *         "JPEGQuality" and the value being the required quality.
        * @param  quality The desired quality level, with 1 being lowest quality
        *         and 5 being highest quality.
        */
        virtual void setJPEGQuality(uint8 quality) = 0;
        
        /**
        * @brief  Apply the given image encoder transform to the contents before
        *         writing to XPS. This supersedes the image encoding parameters
        *         described above. This allows for more advanced configuration of
        *         image encoding.
        */
        virtual void applyEncoderTransform(const IImageEncoderTransformPtr &transform) = 0;
        
    };
    #define obj2IXPSOutput(obj) IXPSOutputPtr(dynamic_cast<IXPSOutput *>((IRCObject *) obj), true)
}

#endif

