﻿// --------------------------------------------------------------------------------------------------------------------
// <copyright file="MakoDocument.cs" company="Global Graphics Software Ltd">
//   Copyright (c) 2017 Global Graphics Software Ltd. All rights reserved.
// </copyright>
// <summary>
//   
// </summary>
// --------------------------------------------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Linq;

namespace Mako.Wrapper
{
    public class MakoDocument : IMakoDocument
    {
        private readonly IntPtr m_handle;

        public MakoDocument(IntPtr documentHandle)
        {
            m_handle = documentHandle;
        }

        public void UpdateRaster(IMakoRenderer renderer, int pageIndex, MakoSize destinationSize, MakoRect sourceRect, string[] separations, int nSeparation, ref MakoRaster raster)
        {
            renderer.GetRaster(m_handle, pageIndex, destinationSize, sourceRect, separations, nSeparation, ref raster);
        }

        public int GetPageCount()
        {
            return MakoDll.fnGetPageCount(m_handle);
        }

        public IEnumerable<IMakoLayer> GetLayers()
        {
            var makoLayers = new List<IMakoLayer>();

            var layerCount = MakoDll.fnGetLayerCount(m_handle);
            for (var layerIndex = 0; layerIndex < layerCount; layerIndex++)
            {
                var layerHandle = MakoDll.fnGetLayer(m_handle, layerIndex);
                makoLayers.Add(new MakoLayer(layerHandle));
            }

            return makoLayers;
        }

        public IMakoElement GetElement(int pageIndex, double x, double y, IMakoElement selectedElement)
        {
            var elementHandle = MakoDll.fnGetElement(m_handle, pageIndex, x, y, selectedElement?.GetElementHandle() ?? IntPtr.Zero);
            if (elementHandle == IntPtr.Zero)
                return null;

            return new MakoElement(elementHandle);
        }

        public IEnumerable<IMakoElement> GetAllElements(int pageIndex)
        {
            MakoDll.fnGetAllElements(m_handle, pageIndex, out var elements, out _);
            return elements.Select(o => new MakoElement(o));
        }

        public void AddLayerWithName(string layerName)
        {
            MakoDll.fnAddLayerWithName(m_handle, layerName);
        }

        public void AddElement(IMakoElement makoElement)
        {
            MakoDll.fnAddElement(m_handle, makoElement.GetElementHandle());
        }

        public IMakoElement CreateCutContour(IntPtr[] elementHandles, MakoContourSettings settings, MakoSeparation separation)
        {
            var contourHandle = MakoDll.fnCreateContour(m_handle, elementHandles, elementHandles.Length, settings, separation);
            return contourHandle == IntPtr.Zero ? null : new MakoElement(contourHandle);
        }

        public IMakoElement CreatePresetShapeContour(int pageIndex, MakoShapeType type, MakoPoint[] points, MakoContourSettings settings, MakoSeparation separation)
        {
            var contourHandle = MakoDll.fnCreatePresetShapeContour(m_handle, pageIndex, type, points, points.Length, settings, separation);

            return contourHandle == IntPtr.Zero ? null : new MakoElement(contourHandle);
        }

        public MakoSize GetPageSize(int pageIndex)
        {
            if (pageIndex >= MakoDll.fnGetPageCount(m_handle))
                throw new ArgumentOutOfRangeException("pageIndex");

            return new MakoSize
            {
                width = MakoDll.fnGetPageWidth(m_handle, pageIndex),
                height = MakoDll.fnGetPageHeight(m_handle, pageIndex)
            };
        }

        public IEnumerable<MakoSeparation> GetSeparations()
        {
            var makoSeparations = new List<MakoSeparation>();
            var nSeparations = MakoDll.fnGetSeparationCount(m_handle);
            for (var i = 0; i < nSeparations; i++)
            {
                var makoSeparation = new MakoSeparation();
                if (!MakoDll.fnGetSeparation(m_handle, i, ref makoSeparation))
                    throw new InvalidOperationException("Cannot retrieve separation No " + i);
                makoSeparations.Add(makoSeparation);
            }
            return makoSeparations;
        }

        public void LoadSeparations()
        {
            MakoDll.fnLoadSeparations(m_handle);
        }

        public bool AddSeparation(MakoSeparation makoSeparation)
        {
            return MakoDll.fnAddSeparation(m_handle, makoSeparation);
        }

        public bool ReplaceSeparation(int index, MakoSeparation makoSeparation)
        {
            return MakoDll.fnReplaceSeparation(m_handle, index, makoSeparation);
        }

        public bool CanUndo()
        {
            return MakoDll.fnCanUndo(m_handle);
        }

        public bool CanRedo()
        {
            return MakoDll.fnCanRedo(m_handle);
        }

        public IMakoStateChange Undo()
        {
            var changeType =  MakoStateChangeType.None;

            if (!MakoDll.fnUndo(m_handle, out var changedElements, out _, out var changedPageIndexes, out _, ref changeType))
                throw new InvalidOperationException("Failed to undo changes.");

            return new MakoStateChange(changeType, changedElements, changedPageIndexes);
        }

        public IMakoStateChange Redo()
        {
            var changeType = MakoStateChangeType.None;

            if (!MakoDll.fnRedo(m_handle, out var changedElements, out _, out var changedPageIndexes, out _, ref changeType))
                throw new InvalidOperationException("Failed to undo changes.");

            return new MakoStateChange(changeType, changedElements, changedPageIndexes);
        }

        public void StartChangeGroup()
        {
            MakoDll.fnStartChangeGroup(m_handle);
        }

        public void CompleteChangeGroup()
        {
            MakoDll.fnCompleteChangeGroup(m_handle);
        }
        
        public void Dispose()
        {
            MakoDll.fnCloseDocument(m_handle);
        }
    }
}