// -----------------------------------------------------------------------
//  <copyright file="MakoWrapperDll.cpp" company="Global Graphics Software Ltd">
//      Copyright (c) 2017 Global Graphics Software Ltd. All rights reserved.
//  </copyright>
// -----------------------------------------------------------------------

#include "stdafx.h"
#include <Windows.h>
#include <direct.h>
#include <objbase.h>
#include <iostream>
#include <vector>

#include <jawsmako/jawsmako.h>
#include <jawsmako/pdfinput.h>
#include <jawsmako/transforms.h>

#include "GLock.h"
#include "MakoAssembly.h"
#include "MakoRenderer.h"
#include "MakoWrapperDll.h"
#include "MakoLayer.h"
#include "APIHelper.h"
#include "IContourGenerator.h"
#include "LayerAddModifier.h"

using namespace EDL;
using namespace JawsMako;
using std::string;

IJawsMakoPtr gMako;

static GCRITICAL_SECTION gCriticalSection;

MAKOWRAPPER_API bool fnInitialize(wchar_t* libraryLocation)
{
    try
    {
        const String location(libraryLocation);

        // Create the gDocCore instance
        gMako = IJawsMako::create(StringToU8String(location));

        IJawsMako::enableAllFeatures(gMako);
    }
    catch (IEDLError &)
    {
        return false;
    }

    return true;
}

MAKOWRAPPER_API void fnShutdown()
{
    gMako = nullptr;
}

MAKOWRAPPER_API MakoResult fnOpenAssembly(wchar_t* filePath, MakoHandle* assemblyHandle)
{
    String documentFile(filePath);
    IDocumentAssemblyPtr assembly;

    *assemblyHandle = nullptr;

    try
    {
        IPDFInputPtr pdfInput = IPDFInput::create(gMako);
        assembly = pdfInput->open(filePath);
    }
    catch (IEDLError& err)
    {
        return err.getErrorCode();
    }

    // Lock the following section for thread safe
    GLock lock(&gCriticalSection);

    MakoAssembly *makoAssembly = new MakoAssembly(gMako, assembly, filePath);
    *assemblyHandle = reinterpret_cast<MakoHandle>(makoAssembly);

    return 0;
}

MAKOWRAPPER_API void fnCloseAssembly(MakoHandle assemblyHandle)
{
    MakoAssembly* assembly = reinterpret_cast<MakoAssembly*>(assemblyHandle);
    delete assembly;
}

MAKOWRAPPER_API void fnSaveAssembly(MakoHandle assemblyHandle, wchar_t* filePath, wchar_t* outputParams[], int parameterCount)
{
    MakoAssembly *assembly = reinterpret_cast<MakoAssembly*>(assemblyHandle);
    CEDLSysStringVect parameters;
    for (auto i = 0; i < parameterCount; i++)
    {
        parameters.append(EDLStringToEDLSysString(outputParams[i]));
    }
    assembly->save(filePath, parameters);
}

MAKOWRAPPER_API void fnSaveAssemblyAsPdf(MakoHandle assemblyHandle, wchar_t* filePath, bool incremental)
{
    MakoAssembly *assembly = reinterpret_cast<MakoAssembly*>(assemblyHandle);
    assembly->saveAsPdf(filePath, incremental);
}

MAKOWRAPPER_API MakoHandle fnOpenDocument(MakoHandle assemblyHandle, int documentIndex)
{
    MakoAssembly *assembly = reinterpret_cast<MakoAssembly*>(assemblyHandle);
    MakoDocument* document = assembly->openDocument(documentIndex);

    return reinterpret_cast<MakoHandle>(document);
}

MAKOWRAPPER_API void fnCloseDocument(MakoHandle documentHandle)
{
    MakoDocument* document = reinterpret_cast<MakoDocument*>(documentHandle);
    delete document;
}

MAKOWRAPPER_API MakoHandle fnCreateRenderer()
{
    MakoRenderer* makoRenderer = new MakoRenderer(gMako);
    return reinterpret_cast<MakoHandle>(makoRenderer);
}

MAKOWRAPPER_API void fnCloseRenderer(MakoHandle rendererHandle)
{
    MakoRenderer* renderer = reinterpret_cast<MakoRenderer*>(rendererHandle);
    delete renderer;
}

MAKOWRAPPER_API void fnGetRaster(MakoHandle rendererHandle, MakoHandle documentHandle, int pageIndex, MakoSize destinationSize, MakoRect sourceRect, wchar_t** separations, int nSeparation, MakoRaster* raster)
{
    MakoDocument* document = reinterpret_cast<MakoDocument*>(documentHandle);
    MakoRenderer* renderer = reinterpret_cast<MakoRenderer*>(rendererHandle);

    IPagePtr page = document->getPage(pageIndex);
    const IOptionalContentPtr optionalContent = document->getOptionalContent();

    try
    {
        renderer->getPageRaster(page, destinationSize, sourceRect, optionalContent, separations, nSeparation, raster);
    }
    catch (const std::exception& e)
    {
        std::cout << "Failed to update page raster: " << e.what();
    }
}

MAKOWRAPPER_API void fnFreeRaster(MakoRaster raster)
{
    MakoRenderer::freeRaster(raster);
}

MAKOWRAPPER_API int fnGetPageCount(MakoHandle documentHandle)
{
    MakoDocument* document = reinterpret_cast<MakoDocument*>(documentHandle);
    return document->getPageCount();
}

MAKOWRAPPER_API double fnGetPageWidth(MakoHandle documentHandle, int pageIndex)
{
    MakoDocument* document = reinterpret_cast<MakoDocument*>(documentHandle);
    return document->getPageWidth(pageIndex);
}

MAKOWRAPPER_API double fnGetPageHeight(MakoHandle documentHandle, int pageIndex)
{
    MakoDocument* document = reinterpret_cast<MakoDocument*>(documentHandle);
    return document->getPageHeight(pageIndex);
}

MAKOWRAPPER_API MakoHandle fnGetElement(MakoHandle documentHandle, int pageIndex, double x, double y, MakoHandle selectedElementHandle)
{
    MakoElement* selectedElement = reinterpret_cast<MakoElement*>(selectedElementHandle);
    MakoDocument* document = reinterpret_cast<MakoDocument*>(documentHandle);

    return reinterpret_cast<MakoHandle>(document->getPageElement(pageIndex, x, y, selectedElement ? selectedElement->getNode() : IDOMNodePtr()));
}

MAKOWRAPPER_API void fnGetAllElements(MakoHandle documentHandle, int pageIndex, MakoHandle** elements, int* elementCount)
{
    MakoDocument* document = reinterpret_cast<MakoDocument*>(documentHandle);

    CEDLVector<MakoElement*> pageElements = document->getAllPageElements(pageIndex);

    *elementCount = (int)pageElements.size();

    size_t byteCount = sizeof(MakoHandle) * pageElements.size();
    *elements = static_cast<MakoHandle*>(CoTaskMemAlloc(byteCount));

    if (*elementCount)
        memcpy(*elements, &pageElements[0], byteCount);
}

MAKOWRAPPER_API MakoHandle fnCreateContour(MakoHandle documentHandle, MakoHandle elementHandles[], const int count, const MakoContourSettings& settings, const MakoSeparation& separation)
{
    std::vector<MakoElement*> elements;
    elements.reserve(count);
    for (int elementIndex = 0; elementIndex < count; ++elementIndex)
    {
        elements.push_back(reinterpret_cast<MakoElement*>(elementHandles[elementIndex]));
    }

    MakoDocument* document = reinterpret_cast<MakoDocument*>(documentHandle);
    MakoLayerPtr targetLayer(settings.layerName ? document->getLayerWithName(settings.layerName) : nullptr);

    return MakoElement::createContour(gMako, elements, settings, separation, targetLayer);
}

MAKOWRAPPER_API void fnAddElement(MakoHandle documentHandle, MakoHandle elementHandle)
{
    MakoDocument* document = reinterpret_cast<MakoDocument*>(documentHandle);
    document->addElement(reinterpret_cast<MakoElement*>(elementHandle));
}

MAKOWRAPPER_API MakoHandle fnGetElementNode(MakoHandle elementHandle)
{
    MakoElement* element = reinterpret_cast<MakoElement*>(elementHandle);
    return element->getNode();
}

MAKOWRAPPER_API int fnGetElementPageIndex(MakoHandle elementHandle)
{
    MakoElement* element = reinterpret_cast<MakoElement*>(elementHandle);
    return element->getPageIndex();
}

MAKOWRAPPER_API void fnExtractElements(MakoHandle* elementHandles, int count)
{
    CEDLVector<MakoElement*> elements(count);
    for (int i = 0; i < count; i++)
        elements[i] = reinterpret_cast<MakoElement*>(elementHandles[i]);

    MakoElement::extractFromParent(elements);
}

MAKOWRAPPER_API void fnGetElementBoundingBox(MakoHandle elementHandle, MakoRect* rect)
{
    MakoElement* element = reinterpret_cast<MakoElement*>(elementHandle);
    element->getBounds(rect);
}

MAKOWRAPPER_API void fnSetElementBoundingBox(MakoHandle elementHandle, MakoRect rect, bool isInvertedHorizontally, bool isInvertedVertically)
{
    MakoElement* element = reinterpret_cast<MakoElement*>(elementHandle);
    element->setBounds(rect, isInvertedHorizontally, isInvertedVertically);
}

MAKOWRAPPER_API MakoHandle fnCopyElement(MakoHandle elementHandle, int newPageIndex)
{
    MakoElement* element = reinterpret_cast<MakoElement*>(elementHandle);
    return element->copy(gMako, newPageIndex);
}

MAKOWRAPPER_API void fnCloseElement(MakoHandle elementHandle)
{
    MakoElement* element = reinterpret_cast<MakoElement*>(elementHandle);
    delete element;
}

MAKOWRAPPER_API const wchar_t* fnGetLayerName(MakoHandle layerHandle)
{
    MakoLayer* layer = reinterpret_cast<MakoLayer*>(layerHandle);
    const String name = layer->getName();

    return APIHelper::createMarshalableStringFromString(name);
}

MAKOWRAPPER_API int fnGetLayerCount(MakoHandle documentHandle)
{
    MakoDocument* document = reinterpret_cast<MakoDocument*>(documentHandle);
    return document->getLayerCount();
}

MAKOWRAPPER_API MakoHandle fnGetLayer(const MakoHandle documentHandle, int layerIndex)
{
    MakoDocument* document = reinterpret_cast<MakoDocument*>(documentHandle);
    MakoLayer* layer = document->getLayerAtIndex(layerIndex);

    return reinterpret_cast<MakoHandle>(layer);
}

MAKOWRAPPER_API void fnFreeLayer(MakoHandle layerHandle)
{
    MakoLayer* layer = reinterpret_cast<MakoLayer*>(layerHandle);
    delete layer;
}

MAKOWRAPPER_API bool fnGetLayerIsVisible(MakoHandle layerHandle)
{
    MakoLayer* layer = reinterpret_cast<MakoLayer*>(layerHandle);
    return layer->getIsVisible();
}

MAKOWRAPPER_API void fnSetLayerIsVisible(MakoHandle layerHandle, bool isVisible)
{
    MakoLayer* layer = reinterpret_cast<MakoLayer*>(layerHandle);
    layer->setIsVisible(isVisible);
}

MAKOWRAPPER_API bool fnGetLayerIsLocked(MakoHandle layerHandle)
{
    MakoLayer* layer = reinterpret_cast<MakoLayer*>(layerHandle);
    return layer->getIsLocked();
}

MAKOWRAPPER_API void fnSetLayerIsLocked(MakoHandle layerHandle, bool isLocked)
{
    MakoLayer* layer = reinterpret_cast<MakoLayer*>(layerHandle);
    layer->setIsLocked(isLocked);
}

MAKOWRAPPER_API bool fnGetLayerIsPrintVisible(MakoHandle layerHandle)
{
    MakoLayer* layer = reinterpret_cast<MakoLayer*>(layerHandle);
    return layer->isPrintVisible();
}

MAKOWRAPPER_API void fnAddLayerWithName(MakoHandle documentHandle, const wchar_t* const layerName)
{
    MakoDocument* document = reinterpret_cast<MakoDocument*>(documentHandle);

    document->getStateTracker()->registerChange(LayerAddModifier::create(gMako, document, layerName));
}

MAKOWRAPPER_API bool fnCanUndo(MakoHandle documentHandle)
{
    MakoDocument* document = reinterpret_cast<MakoDocument*>(documentHandle);
    return document->getStateTracker()->canUndo();
}

MAKOWRAPPER_API bool fnCanRedo(MakoHandle documentHandle)
{
    MakoDocument* document = reinterpret_cast<MakoDocument*>(documentHandle);
    return document->getStateTracker()->canRedo();
}

MAKOWRAPPER_API bool fnUndo(MakoHandle documentHandle, MakoHandle** changedElements, int* changedElementsSize, int** changedPageIndexes, int* changedPageIndexesSize, eStateChangeType* changeType)
{
    MakoDocument* document = reinterpret_cast<MakoDocument*>(documentHandle);

    StateTrackerPtr stateTracker = document->getStateTracker();

    const ChangeTrackerPtr changeTracker = ChangeTracker::create();
    if (!stateTracker->canUndo() || !stateTracker->undo(changeTracker))
        return false;

    *changeType = changeTracker->getChangeType();

    std::vector<MakoElement*> makoElements = changeTracker->getChangedElements();
    std::vector<int> changedPages = changeTracker->getChangedPages();

    //Elements
    *changedElementsSize = (int)makoElements.size();
    const auto elementsSize = (*changedElementsSize) * sizeof(MakoHandle);

    *changedElements = static_cast<MakoHandle*>(CoTaskMemAlloc(elementsSize));
    memcpy(*changedElements, makoElements.data(), elementsSize);

    // Pages
    *changedPageIndexesSize = (int)changedPages.size();
    const auto pagesSize = (*changedPageIndexesSize) * sizeof(int);

    *changedPageIndexes = static_cast<int*>(CoTaskMemAlloc(pagesSize));
    memcpy(*changedPageIndexes, changedPages.data(), pagesSize);

    return true;
}

MAKOWRAPPER_API bool fnRedo(MakoHandle documentHandle, MakoHandle** changedElements, int* changedElementsSize, int** changedPageIndexes, int* changedPageIndexesSize, eStateChangeType* changeType)
{
    MakoDocument* document = reinterpret_cast<MakoDocument*>(documentHandle);

    StateTrackerPtr stateTracker = document->getStateTracker();

    const ChangeTrackerPtr changeTracker = ChangeTracker::create();
    if (!stateTracker->canRedo() || !stateTracker->redo(changeTracker))
        return false;

    *changeType = changeTracker->getChangeType();

    std::vector<MakoElement*> makoElements = changeTracker->getChangedElements();
    std::vector<int> changedPages = changeTracker->getChangedPages();

    // Elements
    *changedElementsSize = (int)makoElements.size();
    const auto elementsSize = (*changedElementsSize) * sizeof(MakoHandle);

    *changedElements = static_cast<MakoHandle*>(CoTaskMemAlloc(elementsSize));
    memcpy(*changedElements, makoElements.data(), elementsSize);

    // Pages
    *changedPageIndexesSize = (int)changedPages.size();
    const auto pagesSize = (*changedPageIndexesSize) * sizeof(int);

    *changedPageIndexes = static_cast<int*>(CoTaskMemAlloc(pagesSize));
    memcpy(*changedPageIndexes, changedPages.data(), pagesSize);

    return true;
}

MAKOWRAPPER_API void fnStartChangeGroup(MakoHandle documentHandle)
{
    MakoDocument* document = reinterpret_cast<MakoDocument*>(documentHandle);
    document->getStateTracker()->startChangeGroup();
}

MAKOWRAPPER_API void fnCompleteChangeGroup(MakoHandle documentHandle)
{
    MakoDocument* document = reinterpret_cast<MakoDocument*>(documentHandle);
    document->getStateTracker()->completeChangeGroup();
}

MAKOWRAPPER_API MakoHandle fnCreatePresetShapeContour(MakoHandle documentHandle, const int pageIndex, eShapeType type, MakoPoint points[], const int length, const MakoContourSettings& settings, const MakoSeparation& separation)
{
    MakoDocument* document = reinterpret_cast<MakoDocument*>(documentHandle);

    IPagePtr page = document->getPage(pageIndex);
    MakoLayerPtr targetLayer(settings.layerName ? document->getLayerWithName(settings.layerName) : nullptr);

    return MakoElement::createPresetShapeContour(gMako, page, pageIndex, document->getStateTracker(), type, points, length, settings, separation, targetLayer);
}
