﻿// -----------------------------------------------------------------------
//  <copyright file="ElementDeletionModifier.cpp" company="Global Graphics Software Ltd">
//      Copyright (c) 2018 Global Graphics Software Ltd. All rights reserved.
//  </copyright>
// -----------------------------------------------------------------------

#include "stdafx.h"
#include "ElementDeletionModifier.h"
#include "MakoElement.h"
#include <edl/idompath.h>

ElementDeletionModifier::ElementDeletionModifier(const IPagePtr parentPage, const int pageIndex, const CEDLVector<IDOMNodePtr>& nodesToRemove) : m_pageIndex(pageIndex), m_parentPage(parentPage)
{
    if (!parentPage)
        throw std::invalid_argument("The parameter 'parentPage' cannot be null.");

    if (pageIndex < 0)
        throw std::invalid_argument("The parameter 'pageIndex' must be equal or greater than 0.");

    if (nodesToRemove.empty())
        throw std::invalid_argument("The parameter 'nodesToRemove' must be not be an empty collection.");

    // Mark the parent page as changed so that it's kept in memory and save picks up the changes.
    m_parentPage->edit();

    m_nodeData.reserve(nodesToRemove.size());
    for (uint32 i = 0; i < nodesToRemove.size(); i++)
        m_nodeData.append(NodeData(nodesToRemove[i]));
}

IDocumentModifierPtr ElementDeletionModifier::create(const IPagePtr parentPage, const int pageIndex, const CEDLVector<IDOMNodePtr>& nodesToRemove)
{
    IDocumentModifier* documentModifier = new ElementDeletionModifier(parentPage, pageIndex, nodesToRemove);
    return IDocumentModifierPtr(documentModifier);
}

bool ElementDeletionModifier::undo(StateTrackerPtr stateTracker, ChangeTrackerPtr changeTracker)
{
    CEDLVector<bool> haveMoved(m_nodeData.size());

    while (!haveAllMoved(haveMoved))
    {
        for (uint32 i = 0; i < haveMoved.size(); i++)
        {
            if (haveMoved[i])
                continue;

            if (m_nodeData[i].isSiblingUnparented())
                continue;

            if (!m_nodeData[i].reinsertNode())
                throw std::exception("Unable to re-add node.");

            haveMoved[i] = true;
            changeTracker->registerChangedElement(new MakoElement(m_parentPage, m_pageIndex, m_nodeData[i].getNode(), stateTracker));
        }
    }

    changeTracker->registerChangeType(eSCPages);

    return true;
}

bool ElementDeletionModifier::redo(StateTrackerPtr stateTracker, ChangeTrackerPtr changeTracker)
{
    for (uint32 i = 0; i < m_nodeData.size(); i++)
        m_nodeData[i].extract();
    changeTracker->registerChangeType(eSCPages);

    return true;
}

bool ElementDeletionModifier::haveAllMoved(const CEDLVector<bool>& haveMoved)
{
    for (uint32 i = 0; i < haveMoved.size(); i++)
    {
        if (!haveMoved[i])
            return false;
    }

    return true;
}
