﻿// -----------------------------------------------------------------------
//  <copyright file="IContourGenerator.cpp" company="Global Graphics Software Ltd">
//      Copyright (c) 2018 Global Graphics Software Ltd. All rights reserved.
//  </copyright>
// -----------------------------------------------------------------------

#include "stdafx.h"
#include "IContourGenerator.h"
#include <jawsmako/jawsmako.h>
#include <edl/idomglyphs.h>
#include "clipper.hpp"
#include "IOffsetSource.h"
#include "CropboxTransformer.h"

using namespace JawsMako;

// The clipper library only works with integers, in order to be numerically robust. In order to support generating contours for 
// coordinate spaces that are close to zero, we need to scale before handing the coordinates to clipper.
const int SCALE_PRECISION = 256;

// The thickness specified in 96th of an inch.
const int CONTOUR_THICKNESS = 2;

IContourGenerator::~IContourGenerator() = default;

IContourGenerator::IContourGenerator(const IJawsMakoPtr& mako, const MakoContourSettings& settings, const MakoSeparation& separation) : m_mako(mako), m_settings(settings), m_separation(separation)
{
}

ClipperLib::IntPoint IContourGenerator::fPointToIntPoint(const FPoint point)
{
    return ClipperLib::IntPoint(static_cast<ClipperLib::cInt>(roundl(point.x * SCALE_PRECISION)), static_cast<ClipperLib::cInt>(roundl(point.y * SCALE_PRECISION)));
}

FPoint IContourGenerator::intPointToFPoint(const ClipperLib::IntPoint point)
{
    return FPoint(static_cast<double>(point.X) / static_cast<double>(SCALE_PRECISION), static_cast<double>(point.Y) / static_cast<double>(SCALE_PRECISION));
}

ClipperLib::JoinType IContourGenerator::getJointType() const
{
    return m_settings.alignment == eOutline ? ClipperLib::jtRound : ClipperLib::jtMiter;
}

void IContourGenerator::addNodeToOffsetter(IOffsetSourcePtr& offsetSource, ClipperLib::ClipperOffset& offsetter) const
{
    // Flatten the geometry so that we are only dealing with straight lines.
    const IDOMPathGeometryPtr flattenedGeometry = offsetSource->createFlattenedGeometry(m_mako, m_settings.alignment);

    // Get the transform back to page coordinates
    FMatrix geometryTransform;
    if (!flattenedGeometry->getRenderTransform(geometryTransform))
        throw std::runtime_error("Failed to get render transform for contour source geometry");

    FMatrix nodeTransform = offsetSource->getPathTransform();
    FMatrix pageTransform = nodeTransform.preMul(geometryTransform);

    FPoint transformedPoint;

    // Enumerate the figures
    IDOMFigureCollectionEnumPtr figures = flattenedGeometry->getFigureCollectionEnum();
    IDOMPathFigurePtr figure;
    ClipperLib::Paths closedPaths;
    while (figures->getNext(&figure))
    {
        ClipperLib::Path path;

        // Add the starting point for this figure
        const FPoint startPoint = figure->getStartPoint();
        pageTransform.transform(transformedPoint, startPoint);
        path << fPointToIntPoint(transformedPoint);

        // And the segments
        IDOMSegmentCollectionEnumPtr segments = figure->getSegmentCollectionEnum();
        IDOMPathSegmentPtr segment;
        while (segments->getNext(&segment))
        {
            // Should be a polyline, but one last check.
            IDOMPolyLineSegmentPtr polyLineSegment = edlobj2IDOMPolyLineSegment(segment);
            if (!polyLineSegment)
                continue;

            // Then add each point into the clipper lib path
            IFPointCollectionEnumPtr points = polyLineSegment->getPointEnum();
            FPoint point;
            while (points->getNext(&point))
            {
                pageTransform.transform(transformedPoint, point);
                path << fPointToIntPoint(transformedPoint);
            }
        }

        if (figure->getIsClosed())
        {
            // If it's closed, add it to the paths to be simplified and continue.
            closedPaths << path;
            continue;
        }

        if (path.size() > 2 && offsetSource->getIsFilled() && figure->getIsFilled())
        {
            // If it's not closed and there are more than two verticies, and it's filled
            // we'll want to treat it as closed, so add it to the rest of the closed paths.
            closedPaths << path;
            continue;
        }

        // Otherwise, it's a simple path, so add it.
        offsetter.AddPath(path, getJointType(), ClipperLib::etOpenRound);
    }

    // Simplify the closed paths to remove self intersection and other complexities that the offsetter can't handle.
    ClipperLib::Paths simplifiedClosedPaths;
    SimplifyPolygons(closedPaths, simplifiedClosedPaths, flattenedGeometry->getFillRule() == IDOMPathGeometry::eEvenOdd ?  ClipperLib::pftEvenOdd : ClipperLib::pftNonZero);

    // Offset all the paths.
    offsetter.AddPaths(simplifiedClosedPaths, getJointType(), ClipperLib::etClosedPolygon);
}

static IDOMColorSpaceDeviceN::CColorantInfo createDeviceNColorSpace(const MakoSeparation& separation)
{
    return IDOMColorSpaceDeviceN::CColorantInfo(
        EDLStringToEDLSysString(separation.name),
        3,
        B2F((separation.colorArgbValue & 0xFF0000) >> 16),
        B2F((separation.colorArgbValue & 0x00FF00) >> 8),
        B2F((separation.colorArgbValue & 0x0000FF)));
}

IDOMPathNodePtr IContourGenerator::createContourPath(const IDOMPathGeometryPtr geometry, const IDOMBrushPtr brush, const bool isPerfCut) const
{
    CEDLVector<double> dashPattern;

    if (isPerfCut)
    {
        dashPattern.append(3);
        dashPattern.append(1);
    }

    return IDOMPathNode::createStroked(
        m_mako,
        geometry,
        brush,
        FMatrix(),
        IDOMPathGeometryPtr(),
        1.0,
        10.0,
        IDOMPathNode::eMiterJoin,
        IDOMPathNode::eFlatCap,
        IDOMPathNode::eFlatCap,
        IDOMPathNode::eFlatCap,
        IDOMPathNode::eClipLongMiters,
        0.0,
        dashPattern);
}

IDOMPathNodePtr IContourGenerator::createPath() const
{
    // Define our spot color.
    IDOMColorSpaceDeviceN::CColorantInfoVect colorants;
    colorants.append(createDeviceNColorSpace(m_separation));

    const IDOMColorSpaceDeviceRGBPtr alternateColorSpace = IDOMColorSpaceDeviceRGB::create(m_mako);
    const IDOMColorSpaceDeviceNPtr space = IDOMColorSpaceDeviceN::create(m_mako, colorants, alternateColorSpace);
    const IDOMColorPtr idomColor = IDOMColor::create(m_mako, space, 1.0f, 1.0f);

    // Create the new path
    const IDOMPathGeometryPtr geometry = edlobj2IDOMPathGeometry(m_mako->createInstance(IDOMPathGeometry::classID()));
    const IDOMBrushPtr brush = IDOMSolidColorBrush::create(m_mako, idomColor);
    IDOMPathNodePtr path = createContourPath(geometry, brush, String(m_separation.name) == L"PerfCutContour");

    // Set the stroke thickness in page coordinates (96th of an inch)
    path->setStrokeThickness(CONTOUR_THICKNESS);

    // Ensure a round join rather than the defaul miter
    path->setStrokeLineJoin(IDOMPathNode::eRoundJoin);

    return path;
}

void IContourGenerator::addContour(IDOMPathGeometryPtr geometry, ClipperLib::Path path) const
{
    if (path.size() == 0)
        return;

    CEDLVector<FPoint> points;
    for (int pointIndex = 0; pointIndex < path.size(); ++pointIndex)
    {
        const FPoint point = intPointToFPoint(path[pointIndex]);
        points.append(point);
    }

    addPolygonGeometry(geometry, points);
}

void IContourGenerator::processChildren(IDOMPathGeometryPtr geometry, ClipperLib::PolyNodes nodes) const
{
    for (ClipperLib::PolyNode* node : nodes)
    {
        if (node->ChildCount() != 0)
            processChildren(geometry, node->Childs);

        if (m_settings.removeHoles && node->IsHole())
            continue;

        addContour(geometry, node->Contour);
    }
}

void IContourGenerator::fillPath(IDOMPathNodePtr& pathNode, const ClipperLib::PolyTree& offsetTree) const
{
    IDOMPathGeometryPtr geometry;
    if (!pathNode->getPathData(geometry))
        throw std::runtime_error("Failed to get offset path geometry for filling");
    
    addContour(geometry, offsetTree.Contour);
    processChildren(geometry, offsetTree.Childs);
}

/*
 * Creates a contour around the supplied node. The path data in the returned node is in page coordinate space,
 * Therefore, the node should be added to the IDOMFixedPage node of the source node's page.
 */
IDOMNodePtr IContourGenerator::createContour(const std::vector<IDOMNodePtr>& nodes) const
{
    ClipperLib::ClipperOffset offsetter;

    for (const IDOMNodePtr node : nodes)
    {
        IOffsetSourcePtr offsetNode = IOffsetSource::create(node);

        // Create an offsetter for the path in terms of the page coordinates
        addNodeToOffsetter(offsetNode, offsetter);
    }

    // Offset the path in terms of the page coordinates (96th of an inch)
    ClipperLib::PolyTree offsetTree;
    offsetter.Execute(offsetTree, m_settings.offsetInPoints * SCALE_PRECISION);

    // Create a new path node for it to sit in
    IDOMPathNodePtr offsetPath = createPath();
    fillPath(offsetPath, offsetTree);
    
    return offsetPath;
}

IContourGeneratorPtr IContourGenerator::create(const IJawsMakoPtr& mako, const MakoContourSettings& settings, const MakoSeparation& separation)
{
    return std::make_shared<IContourGenerator>(mako, settings, separation);
}

IDOMNodePtr IContourGenerator::createPresetShapeContour(const IPagePtr& page, eShapeType type, MakoPoint* const makoPoints, const int length) const
{
    IDOMPathGeometryPtr geometry;
    IDOMPathNodePtr pathNode = createPath();
    if (!pathNode->getPathData(geometry))
        throw std::runtime_error("Failed to get offset path geometry for filling");

    CropboxTransformer transformer(page);
    FMatrix pageTransform = transformer.getPageTransform();
    pageTransform.invert();
    CEDLVector<FPoint> points;
    for (int i = 0; i < length; i++)
    {
        FPoint point = FPoint(makoPoints[i].x, makoPoints[i].y);
        pageTransform.transform(point, point);
        points.append(point);
    }

    switch (type)
    {
    case eEllipse:
        addEllipseGeometry(geometry, points);
        break;

    case ePolygon:
        addPolygonGeometry(geometry, points);
        break;

    default:
        throwEDLError(JM_ERR_UNSUPPORTED);
    }
    return pathNode;
}

void IContourGenerator::addPolygonGeometry(const IDOMPathGeometryPtr& geometry, const CEDLVector<FPoint>& points) const
{
    if (points.size() == 0)
        return;

    IDOMPathFigurePtr pathFigure = edlobj2IDOMPathFigure(m_mako->createInstance(IDOMPathFigure::classID()));
    IDOMPolyLineSegmentPtr segment = edlobj2IDOMPolyLineSegment(m_mako->createInstance(IDOMPolyLineSegment::classID()));

    pathFigure->setIsClosed(true);
    pathFigure->setStartPoint(points[0]);

    for (uint32 pointIndex = 1; pointIndex < points.size(); ++pointIndex)
    {
        segment->addPoint(points[pointIndex]);
    }

    pathFigure->addSegment(segment);
    geometry->addFigure(pathFigure);
}

void IContourGenerator::addEllipseGeometry(const IDOMPathGeometryPtr& geometry, const CEDLVector<FPoint>& points) const
{
    // In case of Ellipse, we expect two and only two points representing the top-left and bottom-right of boundingbox.
    if (points.size() != 2)
        throwEDLError(EDL_ERR_BAD_ARGUMENTS);

    IDOMPathFigurePtr pathFigure = edlobj2IDOMPathFigure(m_mako->createInstance(IDOMPathFigure::classID()));
    const FPoint topLeft(points[0].x, points[0].y);
    const FPoint bottomRight(points[1].x, points[1].y);

    // We add two arc segments for the ellipse, each representing half ellipse. 
    // We cannot add just one single segment as the same start and end point will
    // cause the shape to degenerate.
    const FPoint startPoint(topLeft.x, (topLeft.y + bottomRight.y) / 2);
    const FPoint midPoint(bottomRight.x, (topLeft.y + bottomRight.y) / 2);

    pathFigure->setIsClosed(true);
    pathFigure->setStartPoint(startPoint);

    const double radiusX = (bottomRight.x - topLeft.x) / 2;
    const double radiusY = (bottomRight.y - topLeft.y) / 2;

    // First segment
    pathFigure->addSegment(createEllipseSegment(midPoint, radiusX, radiusY));

    // Second segment
    pathFigure->addSegment(createEllipseSegment(startPoint, radiusX, radiusY));

    geometry->addFigure(pathFigure);
}

IDOMArcSegmentPtr IContourGenerator::createEllipseSegment(const FPoint& endPoint, double radiusX, double radiusY) const
{
    IDOMArcSegmentPtr segment = edlobj2IDOMArcSegment(m_mako->createInstance(IDOMArcSegment::classID()));
    segment->setIsStroked(true);
    segment->setPoint(endPoint);
    segment->setRadiusX(radiusX);
    segment->setRadiusY(radiusY);
    segment->setRotationAngle(0);
    segment->setIsLargeArc(true);
    segment->setSweepDirection(IDOMArcSegment::Counterclockwise);
    return segment;
}
