﻿// -----------------------------------------------------------------------
//  <copyright file="MakoColorReplacementTransformImplementation.cpp" company="Global Graphics Software Ltd">
//      Copyright (c) 2018 Global Graphics Software Ltd. All rights reserved.
//  </copyright>
// -----------------------------------------------------------------------

#include "stdafx.h"
#include "MakoColorReplacementTransformImplementation.h"

MakoColorReplacementTransformImplementation::MakoColorReplacementTransformImplementation(const IJawsMakoPtr &jawsMako, const IRendererTransform::CInkInfoVect &allInks, const IRendererTransform::CInkInfo &oldInk, const IRendererTransform::CInkInfo &newInk) : m_inkList(allInks), m_mako(jawsMako), m_newInk(newInk)
{
    m_colorant = oldInk.inkName;
    m_colorSpaceMap.clear();
}

MakoColorReplacementTransformImplementation::~MakoColorReplacementTransformImplementation()
{
    m_colorSpaceMap.clear();
}

// Colors are referred by components of a color space. To replace a color, we need to replace the corresponding component of the color space.
IDOMColorSpacePtr MakoColorReplacementTransformImplementation::transformColorSpace(IImplementation *genericImplementation, const IDOMColorSpacePtr &colorSpace, const CTransformState &state)
{
    // Check if the color space is already mapped and if yes, return the mapped one.
    const MakoColorSpaceMap::iterator pos = m_colorSpaceMap.find(colorSpace);
    if (pos != m_colorSpaceMap.end())
        return pos->second;

    // Otherwise check if the color space contains the colorant being replaced and if so, create a new one to replace it.
    const IDOMColorSpacePtr newColorSpace = replaceColorSpace(colorSpace);

    if (newColorSpace)
    {
        m_colorSpaceMap[colorSpace] = newColorSpace;
        return newColorSpace;
    }

    return genericImplementation->transformColorSpace(genericImplementation, colorSpace, state);
}

// We actually need to handle transformShadingPatternBrush(), as the color space settings for Shading pattern brush is 
// not intercepted by transformColorSpace(). But that API is not called at the moment. So we hook on brush and update
// if the brush is of type shading pattern.
IDOMBrushPtr MakoColorReplacementTransformImplementation::transformBrush(IImplementation *genericImplementation, const IDOMBrushPtr &brush, eBrushUsage usage, const CTransformState &state)
{
    if (brush->getBrushType() == IDOMBrush::eType2ShadingPattern)
    {
        IDOMShadingPatternBrushPtr type2ShadingPatternbrush = edlobj2IDOMShadingPatternType2Brush(brush);
        IDOMColorSpacePtr colorSpace;
        type2ShadingPatternbrush->getColorSpace(colorSpace);

        const IDOMColorSpacePtr newColorSpace = replaceColorSpace(colorSpace);

        // If a color space has been replaced, create a new brush from the new color space to replace the old one.
        if (newColorSpace)
        {
            IDOMShadingPatternBrushPtr newPatternBrush = clone(type2ShadingPatternbrush, m_mako);
            newPatternBrush->setColorSpace(newColorSpace);
            return edlobj2IDOMBrush(newPatternBrush);
        }
    }

    return genericImplementation->transformBrush(genericImplementation, brush, usage, state);
}

// Check if a color space has the given colroant and if so, and replace it as appropriate.
// Only index color space and deviceN color space can have replacible spot colors.
IDOMColorSpacePtr MakoColorReplacementTransformImplementation::replaceColorSpace(IDOMColorSpacePtr colorSpace)
{
    IDOMColorSpacePtr newColorSpace = IDOMColorSpacePtr();
    switch (colorSpace->getColorSpaceType())
    {
    case IDOMColorSpace::eIndexed:
        newColorSpace = replaceIndexedColorSpace(edlobj2IDOMColorSpaceIndexed(colorSpace));
        break;

    case IDOMColorSpace::eDeviceN:
        newColorSpace = replaceDeviceNColorSpace(edlobj2IDOMColorSpaceDeviceN(colorSpace));

    default:
        break;
    }
    return newColorSpace;
}

// Check if the underlying color space of an indexed color space has the named colorant, and if so, create a new color space with it to
// replace the existing one.
IDOMColorSpacePtr MakoColorReplacementTransformImplementation::replaceIndexedColorSpace(const IDOMColorSpaceIndexedPtr& indexedColorSpace)
{
    // Recurse on the underlying
    IDOMColorSpacePtr underlyingColorSpace = indexedColorSpace->getUnderlyingColorSpace();
    if (underlyingColorSpace->getColorSpaceType() == IDOMColorSpace::eDeviceN)
    {
        // The underlying color space is DeviceN: need to create a replacment if applicable
        const IDOMColorSpacePtr newColorSpace = replaceDeviceNColorSpace(edlobj2IDOMColorSpaceDeviceN(underlyingColorSpace));
        if (newColorSpace)
            return IDOMColorSpaceIndexed::create(m_mako, newColorSpace, indexedColorSpace->getMappingFunction());
    }
    return IDOMColorSpacePtr();
}

// Check if a DeviceN color space has the named colorant, and if so, create a new color space with it to replace the existing one.
IDOMColorSpacePtr MakoColorReplacementTransformImplementation::replaceDeviceNColorSpace(const IDOMColorSpaceDeviceNPtr& deviceN)
{
    if (!colorSpaceHasComponent(deviceN, m_colorant))
        return IDOMColorSpacePtr();

    const int numComponents = deviceN->getNumComponents();
    const IDOMColorSpaceDeviceN::CColorantInfoVect colorantInfoVect(numComponents);
    for (int component = 0; component < numComponents; component++)
    {
        IDOMDeviceNColorantPtr colorant = deviceN->getColorant(component);
        const U8String colorantName = colorant->getName();
        if (colorantName == m_colorant)
        {
            getColorantInfoFromInkInfo(&colorantInfoVect[component], m_newInk, deviceN->getAlternateColorSpace());
        }
        else if (colorantName == "All")
        {
            getSpecialAllColorantInfo(&colorantInfoVect[component], deviceN->getAlternateColorSpace());
        }
        else if (colorantName == "None")
        {
            getSpecialNoneColorantInfo(&colorantInfoVect[component], deviceN->getAlternateColorSpace());
        }
        else
        {
            for (unsigned int i = 0; i < m_inkList.size(); i++)
            {
                if (m_inkList[i].inkName == colorantName)
                {
                    getColorantInfoFromInkInfo(&colorantInfoVect[component], m_inkList[i], deviceN->getAlternateColorSpace());
                    break;
                }
            }
        }
    }

    return IDOMColorSpaceDeviceN::create(m_mako, colorantInfoVect, deviceN->getAlternateColorSpace());
}

// Returns true if a color space contains the named colorant.
bool MakoColorReplacementTransformImplementation::colorSpaceHasComponent(const IDOMColorSpaceDeviceNPtr& colorSpace, const U8String& colorantToFind)
{
    const int numComponents = colorSpace->getNumComponents();
    for (int component = 0; component < numComponents; component++)
    {
        IDOMDeviceNColorantPtr colorant = colorSpace->getColorant(component);
        if (colorant->getName() == colorantToFind)
            return true;
    }
    return false;
}

// Create a CColorantInfo object from a CInkInfo object with color space set to colorSpace.
void MakoColorReplacementTransformImplementation::getColorantInfoFromInkInfo(IDOMColorSpaceDeviceN::CColorantInfo* pColorantInfo, const IRendererTransform::CInkInfo &inkInfo, const IDOMColorSpacePtr& colorSpace) const
{
    pColorantInfo->name = inkInfo.inkName;

    IEDLObjectPtr cloned;
    inkInfo.inkColor->clone(cloned, m_mako);
    IDOMColorPtr clonedColor = edlobj2IDOMColor(cloned);
    // Change the color space to the alternate color space
    clonedColor->setColorSpace(colorSpace, eRelativeColorimetric, eBPCDefault, m_mako);
    const int numComponents = colorSpace->getNumComponents();
    pColorantInfo->components.resize(numComponents);
    for (int i = 0; i < numComponents; i++)
    {
        clonedColor->getComponentValue(i, pColorantInfo->components[i]);
    }
}

// Create a special colorantInfo for special colorants "None", using white for its color
void MakoColorReplacementTransformImplementation::getSpecialNoneColorantInfo(IDOMColorSpaceDeviceN::CColorantInfo* pColorantInfo, const IDOMColorSpacePtr& colorSpace) const
{
    IRendererTransform::CInkInfo inkInfo;
    inkInfo.inkName = "None";
    // Use white color for "None"
    inkInfo.inkColor = IDOMColor::create(m_mako, IDOMColorSpaceDeviceCMYK::create(m_mako), 1.0, 0.0, 0.0, 0.0, 0.0);
    getColorantInfoFromInkInfo(pColorantInfo, inkInfo, colorSpace);
}

// Create a special colorantInfo for special colorants "All", using black for its color.
void MakoColorReplacementTransformImplementation::getSpecialAllColorantInfo(IDOMColorSpaceDeviceN::CColorantInfo* pColorantInfo, const IDOMColorSpacePtr& colorSpace) const
{
    IRendererTransform::CInkInfo inkInfo;
    inkInfo.inkName = "All";
    // Use black color for "All"
    inkInfo.inkColor = IDOMColor::create(m_mako, IDOMColorSpaceDeviceCMYK::create(m_mako), 1.0, 1.0, 1.0, 1.0, 1.0);
    getColorantInfoFromInkInfo(pColorantInfo, inkInfo, colorSpace);
}
