// -----------------------------------------------------------------------
//  <copyright file="MakoDocument.cpp" company="Global Graphics Software Ltd">
//      Copyright (c) 2017 Global Graphics Software Ltd. All rights reserved.
//  </copyright>
// -----------------------------------------------------------------------

#include "stdafx.h"
#include "MakoDocument.h"
#include <urlmon.h>
#include "MakoElement.h"

#include "CropboxTransformer.h"
#include "HitDetection.h"
#include "ElementInsertionModifier.h"

using namespace JawsMako;

MakoDocument::MakoDocument(IJawsMakoPtr mako, IDocumentPtr document, std::wstring sourceFilePath) : m_document(document), m_mako(mako), m_sourceFilePath(sourceFilePath)
{
    m_stateTracker = StateTracker::create();
}

IOptionalContentPtr MakoDocument::getOptionalContent() const
{
    return m_document->getOptionalContent();
}

double MakoDocument::getPageHeight(const int pageIndex) const
{
    const IPagePtr page = m_document->getPage(pageIndex);
    CropboxTransformer transformer(page);

    return transformer.getTransformedPageRect().dY;
}

double MakoDocument::getPageWidth(const int pageIndex) const
{
    const IPagePtr page = m_document->getPage(pageIndex);
    CropboxTransformer transformer(page);

    return transformer.getTransformedPageRect().dX;
}

MakoElement* MakoDocument::getPageElement(const int pageIndex, double x, double y, IDOMNodePtr selectedNode) const
{
    IPagePtr page = m_document->getPage(pageIndex);
    CropboxTransformer transformer(page);

    FMatrix pageTransform = transformer.getPageTransform();
    pageTransform.invert();

    FPoint transformedPoint;
    pageTransform.transform(transformedPoint, FPoint(x, y));

    const IDOMNodePtr pageContent = page->edit();
    HitDetectionResult result = {};
    if (HitDetection::findNode(pageContent, CTransformState(), transformedPoint, selectedNode, result))
    {
        // This element will be used by the application and freed when no longer required.
        return new MakoElement(page, pageIndex, result.node, m_stateTracker);
    }

    return nullptr; 
}

bool isNodeSelectable(eDOMNodeType nodeType)
{
    return nodeType == eDOMGlyphsNode || nodeType == eDOMPathNode;
}

static bool nodeSelectTreeWalker(void *priv, const IDOMNodePtr &node)
{
    if (isNodeSelectable(node->getNodeType()) && node->getBounds().dX > 0 && node->getBounds().dY > 0)
        ((CEDLVector<IDOMNodePtr>*)priv)->append(node);

    return true;
}

CEDLVector<MakoElement*> MakoDocument::getAllPageElements(int pageIndex) const
{
    IPagePtr page = m_document->getPage(pageIndex);

    CEDLVector<IDOMNodePtr> selectableNodes;
    page->getContent()->walkTree(nodeSelectTreeWalker, &selectableNodes);

    CEDLVector<MakoElement*> selectableElements(selectableNodes.size());
    for (uint32 i = 0; i < selectableNodes.size(); i++)
        selectableElements[i] = new MakoElement(page, pageIndex, selectableNodes[i], m_stateTracker);

    return selectableElements;
}

IPagePtr MakoDocument::getPage(const int pageIndex) const
{
    return m_document->getPage(pageIndex);
}

uint32 MakoDocument::getPageCount() const
{
    return m_document->getNumPages();
}

uint32 MakoDocument::getLayerCount() const
{
    const IOptionalContentPtr optionalContent = m_document->getOptionalContent();
    return optionalContent ? optionalContent->getGroups().size() : 0;
}

MakoLayer* MakoDocument::getLayerAtIndex(const int layerIndex) const
{
    const IOptionalContentPtr optionalContent = m_document->getOptionalContent();
    const IOptionalContentGroupPtr optionalContentGroup = optionalContent->getGroups()[layerIndex];
    
    return new MakoLayer(m_mako, optionalContent, optionalContentGroup, m_stateTracker);
}

MakoLayer* MakoDocument::getLayerWithName(const String& layerName) const
{
    if (layerName.empty())
        return nullptr;

    const IOptionalContentPtr optionalContent = m_document->getOptionalContent();
    COptionalContentGroupVect layerGroups = optionalContent->getGroups();

    for (uint32 i = 0; i < getLayerCount(); i++)
    {
        if (U8StringToString(layerGroups[i]->getName()) == layerName)
            return new MakoLayer(m_mako, optionalContent, layerGroups[i], m_stateTracker);
    }

    return nullptr;
}

void MakoDocument::addLayer(const MakoLayerPtr& layer) const
{
    IOptionalContentPtr optionalContent = m_document->getOptionalContent();
    IOptionalContentGroupPtr group = layer->getOptionalContentGroup();
    optionalContent->addGroup(group);
}

void MakoDocument::addLayerWithName(const String& layerName) const
{
    IOptionalContentPtr optionalContent = m_document->getOptionalContent();
    if (!optionalContent)
    {
        optionalContent = IOptionalContent::create(m_mako);
        m_document->setOptionalContent(optionalContent);
    }

    optionalContent->makeNewGroup(StringToU8String(layerName), true);
}

void MakoDocument::removeLayerWithName(const String& layerName) const
{
    const IOptionalContentPtr optionalContent = m_document->getOptionalContent();

    COptionalContentGroupVect layerGroups = optionalContent->getGroups();
    for (uint32 i = 0; i < getLayerCount(); i++)
    {
        if (U8StringToString(layerGroups[i]->getName()) == layerName)
        {
            optionalContent->removeGroup(layerGroups[i]);
            return;
        }
    }
}

StateTrackerPtr MakoDocument::getStateTracker() const
{
    return m_stateTracker;
}

uint32 MakoDocument::getPageIndex(const IPagePtr& page) const
{
    for (uint32 pageIndex = 0; pageIndex < getPageCount(); pageIndex++)
    {
        if (m_document->getPage(pageIndex)->getPageId() == page->getPageId())
            return pageIndex;
    }

    return 0;
}

void MakoDocument::addElement(const MakoElement* element) const
{
    IDOMNodePtr nodeToAdd = element->getGroupNode();
    if (!nodeToAdd)
        nodeToAdd = element->getNode();

    IPagePtr page = m_document->getPage(element->getPageIndex());
    m_stateTracker->registerChange(ElementInsertionModifier::create(page, element->getPageIndex(), nodeToAdd, page->edit()));
}
