// -----------------------------------------------------------------------
//  <copyright file="MakoElement.cpp" company="Global Graphics Software Ltd">
//      Copyright (c) 2017 Global Graphics Software Ltd. All rights reserved.
//  </copyright>
// -----------------------------------------------------------------------

#include "stdafx.h"
#include "MakoElement.h"
#include <jawsmako/transforms.h>
#include "CropboxTransformer.h"
#include "ElementTransformModifier.h"
#include "IContourGenerator.h"
#include "ElementInsertionModifier.h"
#include "ElementDeletionModifier.h"
#include <iostream>

using namespace JawsMako;

MakoElement::MakoElement(IPagePtr page, const int pageIndex, IDOMNodePtr node, StateTrackerPtr stateTracker) : m_node(node), m_page(page), m_stateTracker(stateTracker), m_pageIndex(pageIndex)
{
    if (page == nullptr)
        throw std::invalid_argument("Parameter 'page' cannot be null");

    if (node == nullptr)
        throw std::invalid_argument("Parameter 'page' cannot be null");

    if (stateTracker == nullptr)
        throw std::invalid_argument("Parameter 'stateTracker' cannot be null");

    if (pageIndex < 0)
        throw std::invalid_argument("The parameter 'pageIndex' must be equal or greater than 0.");
}

MakoElement::~MakoElement()
{
}

void MakoElement::getBounds(MakoRect* rect) const
{
    FRect bounds = m_node->getBounds(false);

    CTransformState state(m_node);
    state.transform.transformRect(bounds);

    // Transform the bounds
    CropboxTransformer transformer(m_page);
    FMatrix pageTransform = transformer.getPageTransform();
    pageTransform.transformRect(bounds);

    rect->x = bounds.x;
    rect->y = bounds.y;
    rect->width = bounds.dX;
    rect->height = bounds.dY;
}

void MakoElement::transformToPageCoordinates(FPoint& translation, IPagePtr page)
{
    // Get the transform for the current page
    CropboxTransformer transformer(page);
    FMatrix pageTransform = transformer.getPageTransform();

    // Invert it
    pageTransform.invert();

    // Transform our translation to the page coordinate space
    pageTransform.transform(translation, translation, true);
}

void MakoElement::transformToNodeCoordinates(FPoint& translation, IDOMNodePtr node)
{
    // Get the transform state between the root of the page up to (and including) the nodes parent
    const IDOMNodePtr nodeParent = node->getParentNode();
    const CTransformState state(nodeParent);

    // Invert it
    FMatrix parentNodeEffectiveTransform = state.transform;
    parentNodeEffectiveTransform.invert();

    // Transform our translation so it's in the coordinate space of the node we want to translate.
    parentNodeEffectiveTransform.transform(translation, translation, true);
}

FRect MakoElement::makoRectToRect(MakoRect makoRect)
{
    FRect rect;
    rect.x = makoRect.x;
    rect.y = makoRect.y;
    rect.dX = makoRect.width;
    rect.dY = makoRect.height;

    return rect;
}

void MakoElement::setBounds(const MakoRect& rect, const bool isInvertedHorizontally, const bool isInvertedVertically) const
{
    MakoRect originalBoundsMako;
    getBounds(&originalBoundsMako);

    const FRect originalBounds = makoRectToRect(originalBoundsMako);
    const FRect newBounds = makoRectToRect(rect);

    // Get the matrix to transform between the original bounds and the new bounds.
    FMatrix boundsTransform(originalBounds, newBounds);

    // Apply any required vertical flipping
    if (isInvertedVertically)
    {
        const FMatrix translate(1, 0, 0, 1, 0, -newBounds.y - newBounds.dY);
        const FMatrix flip(1, 0, 0, -1, 0, 0);
        const FMatrix translateBack(1, 0, 0, 1, 0, newBounds.y);
        boundsTransform.postMul(translate).postMul(flip).postMul(translateBack);
    }

    // Apply any required horizontal flipping
    if (isInvertedHorizontally)
    {
        const FMatrix translate(1, 0, 0, 1, -newBounds.x - newBounds.dX, 0);
        const FMatrix flip(-1, 0, 0, 1, 0, 0);
        const FMatrix translateBack(1, 0, 0, 1, newBounds.x, 0);
        boundsTransform.postMul(translate).postMul(flip).postMul(translateBack);
    }

    // Update the original node transform
    const FMatrix originalNodeTransform = getNodeRenderTransform(m_node);
    FMatrix newNodeTransform = originalNodeTransform;

    // Get the transform from the root of the page to the parent node, inclusive.
    CTransformState parentState(m_node->getParentNode());

    // Get the transform to transform between page coordinates and visual page coordinates 
    // (as seen on screen with the cropbox and rotation applied)
    CropboxTransformer transformer(m_page);
    FMatrix pageTransform = transformer.getPageTransform();

    // transform back to visual page coordinates
    newNodeTransform.postMul(parentState.transform);
    newNodeTransform.postMul(pageTransform);

    // Apply the transform between bounding boxes
    newNodeTransform.postMul(boundsTransform);

    // Invert the transforms
    parentState.transform.invert();
    pageTransform.invert();

    // And go back to node coordinate space
    newNodeTransform.postMul(pageTransform);
    newNodeTransform.postMul(parentState.transform);

    // Register the change in the state tracker
    const IDocumentModifierPtr modifier = ElementTransformModifier::create(m_page, m_pageIndex, m_node, originalNodeTransform, newNodeTransform);
    m_stateTracker->registerChange(modifier);
}

IDOMNodePtr MakoElement::getNode() const
{
    return m_node;
}

IDOMNodePtr MakoElement::getGroupNode() const
{
    return m_groupNode;
}

int MakoElement::getPageIndex() const
{
    return m_pageIndex;
}

MakoElement* MakoElement::copy(const IJawsMakoPtr& mako, int newPageIndex) const
{
    IDOMNodePtr toStore;
    IDOMNodePtr rootGroupNode;

    if (m_node->getParentNode())
    {
        // Clone the node and all direct groups.
        // (Parents needed to maintain clipping info.)
        IDOMNodePtr previousCloneNode;
        IDOMNodePtr currentNode = m_node;
        do
        {
            IDOMNodePtr cloned = currentNode->cloneNode(mako);
            if (!toStore)
                toStore = cloned;
            if (previousCloneNode)
                cloned->appendChild(previousCloneNode);

            previousCloneNode = cloned;
        } while ((currentNode = edlobj2IDOMGroup(currentNode->getParentNode())) != nullptr);

        rootGroupNode = edlobj2IDOMGroup(previousCloneNode);
    }
    else
    {
        toStore = m_node->cloneTree(mako);
    }

    MakoElement* makoElement = new MakoElement(m_page, newPageIndex, toStore, m_stateTracker);

    // Also store a reference to the top-level group node (if there was one).
    makoElement->m_groupNode = rootGroupNode;

    return makoElement;
}

MakoElement* MakoElement::createContour(const IJawsMakoPtr& mako, std::vector<MakoElement*>& elements, const MakoContourSettings& settings, const MakoSeparation& separation, MakoLayerPtr targetLayer)
{
    std::vector<IDOMNodePtr> nodes;
    for (MakoElement* element : elements)
        nodes.push_back(element->m_node);

    IDOMNodePtr contourNode;
    try
    {
        const IContourGeneratorPtr contourGenerator = IContourGenerator::create(mako, settings, separation);
        contourNode = contourGenerator->createContour(nodes);

        FRect rectTmpl = contourNode->getBounds();
        if (rectTmpl.isEmpty())
        {
            std::cerr << "Created contour has negative dimensions." << std::endl;
            return nullptr;
        }
    }
    catch (const std::exception& e)
    {
        std::cerr << "Failed to create the contour: " << e.what() << std::endl;
        return nullptr;
    }

    MakoElement* firstElement = elements[0];

    const IDocumentModifierPtr modifier = ElementInsertionModifier::create(firstElement->m_page, firstElement->getPageIndex(), contourNode, firstElement->m_page->edit(), targetLayer);
    firstElement->m_stateTracker->registerChange(modifier);

    return new MakoElement(firstElement->m_page, firstElement->getPageIndex(), contourNode, firstElement->m_stateTracker);
}

MakoElement* MakoElement::createPresetShapeContour(const IJawsMakoPtr& mako, const IPagePtr& page, const int pageIndex, const StateTrackerPtr& stateTracker, eShapeType type, MakoPoint* points, const int length, const MakoContourSettings& settings, const MakoSeparation& separation, MakoLayerPtr targetLayer)
{
    IDOMNodePtr contourNode;
    try
    {
        const IContourGeneratorPtr contourGenerator = IContourGenerator::create(mako, settings, separation);
        contourNode = contourGenerator->createPresetShapeContour(page, type, points, length);
    }
    catch (const std::exception& e)
    {
        std::cerr << "Failed to create the contour: " << e.what() << std::endl;
        return nullptr;
    }
    
    const IDocumentModifierPtr modifier = ElementInsertionModifier::create(page, pageIndex, contourNode, page->edit(), targetLayer);
    stateTracker->registerChange(modifier);

    return new MakoElement(page, pageIndex, contourNode, stateTracker);
}

void MakoElement::extractFromParent(const CEDLVector<MakoElement*>& elements)
{
    if (elements.empty())
        return;

    CEDLVector<IDOMNodePtr> nodesToRemove(elements.size());
    for (uint32 i = 0; i < elements.size(); i++)
        nodesToRemove[i] = elements[i]->m_node;

    const IDocumentModifierPtr modifier = ElementDeletionModifier::create(elements[0]->m_page, elements[0]->m_pageIndex, nodesToRemove);
    elements[0]->m_stateTracker->registerChange(modifier);
}

FMatrix MakoElement::getNodeRenderTransform(IDOMNodePtr node)
{
    FMatrix renderTransform;

    const IDOMPathNodePtr pathNode = edlobj2IDOMPathNode(node);
    if (pathNode)
    {
        (bool)pathNode->getRenderTransform(renderTransform);
        return renderTransform;
    }

    const IDOMGlyphsPtr glyphsNode = edlobj2IDOMGlyphs(node);
    if (glyphsNode)
    {
        glyphsNode->getRenderTransform(renderTransform);
        return renderTransform;
    }

    const IDOMGroupPtr groupNode = edlobj2IDOMGroup(node);
    if (groupNode)
    {
        (void)groupNode->getRenderTransform(renderTransform);
        return renderTransform;
    }

    return renderTransform;
}

void MakoElement::setNodeRenderTransform(IDOMNodePtr node, const FMatrix& m)
{
    const IDOMPathNodePtr pathNode = edlobj2IDOMPathNode(node);
    if (pathNode)
    {
        (bool)pathNode->setRenderTransform(m);
        return;
    }

    const IDOMGlyphsPtr glyphsNode = edlobj2IDOMGlyphs(node);
    if (glyphsNode)
        glyphsNode->setRenderTransform(m);
}
