// -----------------------------------------------------------------------
//  <copyright file="MakoRenderer.cpp" company="Global Graphics Software Ltd">
//      Copyright (c) 2017 Global Graphics Software Ltd. All rights reserved.
//  </copyright>
// -----------------------------------------------------------------------

#include "stdafx.h"
#include "MakoRenderer.h"
#include <jawsmako/jawsmako.h>
#include <jawsmako/transforms.h>
#include <edl/icolormanager.h>
#include <map>
#include "CropboxTransformer.h"

MakoRenderer::MakoRenderer(IJawsMakoPtr mako) : m_mako(mako)
{
    m_jawsRenderer = IJawsRenderer::create(m_mako);
    m_deviceCMYKColourSpace = IDOMColorSpaceDeviceCMYK::create(m_mako);
    m_sRGBColorSpace = IDOMColorSpacesRGB::create(m_mako);
    m_optionalContentFixer = IOptionalContentFixerTransform::create(m_mako);
}

MakoRenderer::~MakoRenderer()
{
}

// Apply a transformation based on page transform, this may be, for example, the cropbox and rotation of the page
void MakoRenderer::applyPageTransform(IPagePtr page) const
{
    CropboxTransformer transformer(page);

    IDOMFixedPagePtr fixedPage = page->edit();
    fixedPage->setWidth(transformer.getTransformedPageRect().dX);
    fixedPage->setHeight(transformer.getTransformedPageRect().dY);

    IDOMGroupPtr transformGroup = IDOMGroup::create(m_mako, transformer.getPageTransform());
    
    IDOMNodePtr node = fixedPage->getFirstChild();
    while (node)
    {
        fixedPage->extractChild(node);
        transformGroup->appendChild(node);

        node = fixedPage->getFirstChild();
    }

    fixedPage->appendChild(transformGroup);
}

void MakoRenderer::getPageRaster(const IPagePtr& page, MakoSize destinationSize, const MakoRect& sourceRect, IOptionalContentPtr optionalContent, wchar_t** separations, int nSeparation, MakoRaster* raster)
{
    IPagePtr clonedPage = page->clone();
    
    IRendererTransform::CInkInfoVect inks = IRendererTransform::findInks(m_mako, clonedPage->getContent());

    // Prune the CMYK colorants, and build the list of spot colour names for Jaws
    CSpotColorNames spotColours;
    if (inks.size() > 0)
    {
        IRendererTransform::CInkInfoVect inksCopy = inks;
        inks.resize(0);
        for (uint32 i = 0; i < inksCopy.size(); i++)
        {
            if (inksCopy[i].inkName == "Cyan" ||
                inksCopy[i].inkName == "Magenta" ||
                inksCopy[i].inkName == "Yellow" ||
                inksCopy[i].inkName == "Black")
            {
                continue;
            }
            inks.append(inksCopy[i]);
            spotColours.append(inksCopy[i].inkName);
        }
    }

    FRect bounding;
    bounding.x = sourceRect.x;
    bounding.y = sourceRect.y;
    bounding.dX = sourceRect.width;
    bounding.dY = sourceRect.height;

    applyPageTransform(clonedPage);

    // Render the bounding rect to the vector of images, including all spot colors, even some of them are off.
    // The reason is that if we do not list them, they are still rendered but converted to CMYK. We will turn
    // them off when we should them on screen.
    CEDLVector<IDOMImagePtr> images = m_jawsRenderer->renderSeparations(
        clonedPage->getContent(),
        m_deviceCMYKColourSpace,
        true, bounding,
        (uint32)destinationSize.width,
        (uint32)destinationSize.height,
        spotColours,
        optionalContent,
        eOCEView);

    for (uint32 i = 0; i < inks.size(); i++)
    {
        inks[i].inkColor->setColorSpace(m_sRGBColorSpace, eRelativeColorimetric, eBPCDefault, m_mako);
    }

    // Whether the separation is on 
    bool* separationOn = new bool[inks.size() + 4];
    separationOn[0] = showSeparation(L"Cyan", separations, nSeparation);
    separationOn[1] = showSeparation(L"Magenta", separations, nSeparation);
    separationOn[2] = showSeparation(L"Yellow", separations, nSeparation);
    separationOn[3] = showSeparation(L"Black", separations, nSeparation);
    for (uint32 i = 0; i < inks.size(); i++)
        separationOn[i + 4] = showSeparation(U8StringToString(inks[i].inkName), separations, nSeparation);

    // The frame index if it is the single separation. Note the single separation might not be
    // on this page. We only render as single separation if it is.
    int iSingleSeparation = -1;
    if (nSeparation == 1)
    {
        for (uint32 j = 0; j < inks.size() + 4; j ++)
        {
            if (separationOn[j])
            {
                iSingleSeparation = j;
                break;
            }
        }
    }

    IColorManagerPtr cmm = IColorManager::get(m_mako);

    // Get the image frames
    CEDLVector<IImageFramePtr> frames;
    frames.resize(images.size());

    for (uint32 sep = 0; sep < images.size(); sep++)
    {
        images[sep]->getFirstImageFrame(m_mako, frames[sep]);
    }

    // Get the measurements of the generated raster and make sure they match the supplied buffer
    uint32 tileWidth = frames[0]->getWidth();
    uint32 tileHeight = frames[0]->getHeight();

    raster->width = tileWidth;
    raster->height = tileHeight;

    raster->samplePerPixel = iSingleSeparation != -1 ? 1 : 3;
    uint32 tileStride = tileWidth * raster->samplePerPixel;
    raster->bufferStride = tileStride;

    // The output buffer, passed in by the caller.
    uint8* outputBuffer = new uint8[raster->bufferStride * raster->height];
    raster->buffer = outputBuffer;

    // If only a single separation is turned on, copy its raster to output directly.
    if (iSingleSeparation != -1)
    {
        // A single separation is turned on: copy the corresponding image to output.
        for (uint32 y = 0; y < tileHeight; y++)
        {
            // Load the raster directly into output
            frames[iSingleSeparation]->readScanLine(&outputBuffer[y * tileStride]);

            // Invert the color
            for (uint32 x = 0; x < tileWidth; x++)
            {
                outputBuffer[y * tileStride + x] = 0xFF - outputBuffer[y * tileStride + x];
            }
        }
        delete[] separationOn;
        return;
    }

    // And buffers for each component thereof
    uint8* cmykChannels[4];
    for (uint32 i = 0; i < 4; i++)
    {
        cmykChannels[i] = new uint8[tileWidth];
    }

    // We need a CMYK scanline buffer
    uint8* cmykScanline = new uint8[tileWidth * 4];
    // And buffers for handling spots
    uint8* rawScanline = new uint8[tileWidth];

    // Ok. Loop over the height.
    for (uint32 y = 0; y < tileHeight; y++)
    {
        // The output scanline is here
        uint8 *outPtr = &outputBuffer[y * tileStride];

        // First assemble the CMYK. Read the scanlines
        for (uint32 i = 0; i < 4; i++)
        {
            frames[i]->readScanLine(&cmykChannels[i][0]);
        }

        // Assemble
        for (uint32 x = 0; x < tileWidth; x++)
        {
            // CMYK channels
            for (uint32 i = 0; i < 4; i++)
                cmykScanline[x * 4 + i] = separationOn[i] ? cmykChannels[i][x] : 0;
        }

        // Convert to RGB
        cmm->convertColors(tileWidth, false, m_deviceCMYKColourSpace, m_sRGBColorSpace, eRelativeColorimetric, eBPCDefault, &cmykScanline[0], outPtr, 8, 8);

        // Now, the spots
        for (uint32 i = 4; i < frames.size(); i++)
        {
            // Skip the frame if its channel is off
            if (!separationOn[i])
                continue;

            // Read the scanline
            frames[i]->readScanLine(&rawScanline[0]);

            // Get the RGB for the colorant
            float rgbColorant[3];
            inks[i - 4].inkColor->getComponentValue(0, rgbColorant[0]);
            inks[i - 4].inkColor->getComponentValue(1, rgbColorant[1]);
            inks[i - 4].inkColor->getComponentValue(2, rgbColorant[2]);

            // Pseudo-additive (CMY)
            rgbColorant[0] = 1.0f - rgbColorant[0];
            rgbColorant[1] = 1.0f - rgbColorant[1];
            rgbColorant[2] = 1.0f - rgbColorant[2];

            // Now apply this
            for (uint32 x = 0; x < tileWidth; x++)
            {
                float inkValue = rawScanline[x] / 255.0f;

                // TODO: We probably ought to be using a flat colour space here
                // to minimise colour shifts, then apply a gamma after the fact.
                // Thinking... actually applying a gamma function scaled RGB
                // value is perhaps good enough (if not completely correct)?
                outPtr[x * 3 + 0] = (uint8)(outPtr[x * 3 + 0] * (1.0f - inkValue * rgbColorant[0]));
                outPtr[x * 3 + 1] = (uint8)(outPtr[x * 3 + 1] * (1.0f - inkValue * rgbColorant[1]));
                outPtr[x * 3 + 2] = (uint8)(outPtr[x * 3 + 2] * (1.0f - inkValue * rgbColorant[2]));
            }
        }
    }

    // Cleanup the temporary buffers we created
    delete[] cmykScanline;
    delete[] rawScanline;
    delete[] separationOn;

    // And buffers for each component thereof
    for (uint32 i = 0; i < 4; i++)
    {
        delete[] cmykChannels[i];
    }

    // The buffer allocated in the MakoRaster structure will need to be freed by the caller.

    clonedPage->release();
}

bool MakoRenderer::showSeparation(String separation, wchar_t** allShownSeparations, int nSeparations)
{
    for (int i = 0; i < nSeparations; i++)
    {
        String name(allShownSeparations[i]);
        if (separation == name)
            return true;
    }
    return false;
}

void MakoRenderer::freeRaster(MakoRaster& raster)
{
    uint8* buffer = reinterpret_cast<uint8*>(raster.buffer);
    delete[] buffer;

    raster.buffer = nullptr;
    raster.width = 0;
    raster.height = 0;
    raster.bufferStride = 0;
    raster.samplePerPixel = 0;
}
