﻿// -----------------------------------------------------------------------
//  <copyright file="StateTracker.cpp" company="Global Graphics Software Ltd">
//      Copyright (c) 2018 Global Graphics Software Ltd. All rights reserved.
//  </copyright>
// -----------------------------------------------------------------------

#include "stdafx.h"
#include "StateTracker.h"
#include "MakoElement.h"

StateTracker::StateTracker() : m_position(0), m_transactionalModifier(nullptr)
{
}

StateTrackerPtr StateTracker::create()
{
    return StateTrackerPtr(new StateTracker());
}

StateTracker::~StateTracker()
{
    resize(0);
}

bool StateTracker::canUndo() const
{
    return m_position > 0;
}

bool StateTracker::canRedo() const
{
    return m_position < m_modifierStack.size();
}

bool StateTracker::redo(const ChangeTrackerPtr changeTracker)
{
    if (!canRedo())
        throw std::runtime_error("Attempted to redo, where there is no stack to redo.");

    IDocumentModifierPtr modifier = m_modifierStack[m_position];
    m_position++;

    return modifier->redo(shared_from_this(), changeTracker);
}

bool StateTracker::undo(const ChangeTrackerPtr changeTracker)
{
    if (!canUndo())
        throw std::runtime_error("Attempted to undo, where there is no stack to undo.");

    m_position--;

    IDocumentModifierPtr modifier = m_modifierStack[m_position];
    return modifier->undo(shared_from_this(), changeTracker);
}

void StateTracker::registerChange(IDocumentModifierPtr documentModifier)
{
    // If we're in a transaction, add the change and continue
    if (m_transactionalModifier != nullptr)
    {
        m_transactionalModifier->registerChange(documentModifier);
        return;
    }

    // Do the change once
    const ChangeTrackerPtr changeTracker = ChangeTracker::create();
    documentModifier->redo(shared_from_this(), changeTracker);

    // If we've been given new elements, we must free them.
    for (MakoElement* makoElement : changeTracker->getChangedElements())
        delete makoElement;

    // Drop the undo redo stack from this point onwards, as we've made a change to a page.
    resize(m_position);

    // Pushback, the change and update our position in the stack
    m_modifierStack.push_back(documentModifier);
    m_position = m_modifierStack.size();
}

void StateTracker::startChangeGroup()
{
    if (m_transactionalModifier != nullptr)
        throw std::runtime_error("A change group cannot be started when a group is already in progress.");

    m_transactionalModifier = new TransactionalModifier();
}

void StateTracker::completeChangeGroup()
{
    if (m_transactionalModifier == nullptr)
        return;

    const IDocumentModifierPtr modifier(m_transactionalModifier);
    m_transactionalModifier = nullptr;

    registerChange(modifier);
}

void StateTracker::resize(const size_t position)
{
    m_modifierStack.resize(position);
}


